#!/usr/local/bin/perl
#
# $Id: analyse_tb_configdata.pl,v 1.6 2006/11/04 14:40:48 ruben Exp $
#
# All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of the author not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

#------------------------------------------------------------------------------
# Module Header
# Filename          : analyse_tb_configdata.pl
# Purpose           : analyse test boxes relation group configuration file
# Author            : Maximo Alves
# Date              : 26 April 2002
# Description       : it first determines changes in the relation group config files
#                     doing a diff between the recently collected files and the files 
#                     collectde before. Currently there're two types of changes with 
#                     related actions listed bellow:
#                     1 - port number: * update ttreg entry with new value
#                                      * update all measurement relations which target 
#                                        changed its port number 
#                     2 - relation group: * add new measurement relation 
#                                         * delete measurement relation
# Language Version  : Perl 5
# OSs Tested        : Linux
# Command Line      : analyse_tb_configdata.pl
# Input Files       : 
# Output Files      : 
# External Programs : ttconfig, diff, ttregconfig
#                    
# Problems          :
#                    
# To Do             : 
#                     
# Comments          :
#                     
#------------------------------------------------------------------------------
use lib '/ncc/ttpro/bin/';
use strict;
use datafunctions;

# --------------- Variable declaration ------------------------------



my $TTROOT ="/ncc/ttpro";
my $newdir ="/ncc/ttpro/data/config/new/";
my $olddir ="/ncc/ttpro/data/config/old/";
my $DEBUG = "1";
my %portchange; #if has changes on port number. If no changes no entry on the hash
my %diffmatrix; #it has changes done by a Host on a tb configuration file. If no changes no entry on the hash
my $VERBOSE = 0;
my $MAXLOAD = 120000;
my $PORTNUMBERFILE = "/ncc/ttpro/data/config/portnumber.txt";

# --------------------- Sub routines --------------------------------

   

sub addmr {

    my ($tb, $target, $rate, $packetlenght, $porttarget, $porttb) = @_;

#check for m.r. exixtence 
#don't compare all values but only target and packet lenght
    my $p1  = $datamatrix{$tb};
    my @attributes = ('o-delay-t','u-delay-t','r-delay-t');
    foreach my $attribute(@attributes){
	my $p2 = $$p1{$attribute};
	foreach my $value (@$p2){
	    if($value =~ /$target\s+\d+\s+\d+\s+$packetlenght/){
            #$value = target PortNumber Rate Packetlenght
                $_=$value;
                s/^\s//;
		$value = $_;
		my @aux = split(/\s/,$value);
		if($rate > $aux[2]){
		    print "Existent relation rate $value is lower than new one $rate. Must change ! \n" if($VERBOSE);
		    print "Saving existent relation on auxmr attribute for future use. \n"if($VERBOSE);
		    my $time = time;
		    my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c ADD auxmr $tb "$target $packetlenght $aux[2] $attribute $time"`;
		    print "Removing existent relation from $attribute on $tb\n" if($VERBOSE);
		    my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c DEL $attribute $tb "$value"`;
		    if($attribute eq "o-delay-t"){
			print "Removing existent relation from o-delay-t on $target\n" if($VERBOSE);
			my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c DEL o-delay-t $target "$tb $porttarget $aux[2] $aux[3]"`;
		    }elsif($attribute eq "u-delay-t"){
			print "Removing existent relation from u-delay-t on $target\n" if($VERBOSE);
			my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c DEL r-delay-t $target "$tb $porttarget $aux[2] $aux[3]"`;
		    }elsif($attribute eq "r-delay-t"){
			print "Removing existent relation from r-delay-t on $target\n" if($VERBOSE);
			my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c DEL u-delay-t $target "$tb $porttarget $aux[2] $aux[3]"`;
		    }else{
			print "ERROR: false condition on addmr !!!";
			return 1;
		    }
		    return 0;
		}else{
		    print "Existent relation rate $aux[2] on $value has higher rate than new one $rate. No reason to change ! \n" if($VERBOSE);
		    return 1;
		}
	    }
	}
    }
    return 0;


}#addmr ends here !!!


sub delmr {

    my $attribute = "";
    my @mr = ();
    my $flag = 0;
    my ($tb, $target, $rate, $packetlenght, $porttarget, $porttb) = @_;
    my @auxmr = `$TTROOT/bin/readttreg.pl $tb auxmr`;
    foreach my $aux (@auxmr){
	if($aux =~ /$target.*$packetlenght/){
            #M.r. exists . Must del the last entry on auxmr and restore before last one 	
	    push @mr, $aux;
	    $flag = 1;	
	}
    }
    if($flag){
        my $lasttime = 0;
        my $last = "";
	my $storeaux = "";
        #which is the last entry in @mr?
	foreach my $aux (@mr){
	    my @aux = split(/\s/,$aux);
	    if($aux[4] > $lasttime){
		$lasttime = $aux[4];
		$last = $aux;
		$attribute = $aux[3];
	    }
	}
	print "Previous relation found. Must restore it! The relation is: $last \n" if($VERBOSE);
	foreach my $aux (@auxmr){
        #fill array to store again. 
	    next if($aux =~/$last/);
	    $storeaux = $storeaux . $aux;
	}
        print "Deleting previous relation from auxmr \n" if($VERBOSE);
	my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c DEL auxmr $tb delete_all`;
	my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c ADD auxmr $tb "$storeaux"`;
	my @last = split(/\s/,$last);
	if($attribute eq "o-delay-t"){
	    print "Adding previous relation to o-delay-t on both $tb and $target\n" if($VERBOSE);
	    my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c ADD o-delay-t $tb "$target $porttarget $last[2] $packetlenght"`;
	    my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c ADD o-delay-t $target "$tb $porttarget $last[2] $packetlenght"`;
	}elsif($attribute eq "r-delay-t"){
	    print "Adding previous relation to r-delay-t on $tb and to u-delay-t on $target\n" if($VERBOSE);
	    my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c ADD r-delay-t $tb "$target $porttarget $last[2] $packetlenght"`;
	    my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c ADD u-delay-t $target "$tb $porttarget $last[2] $packetlenght"`;
	}elsif($attribute eq "u-delay-t"){
	    print "Adding previous relation to u-delay-t on $tb and to r-delay-t on $target\n" if($VERBOSE);
	    my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c ADD u-delay-t $tb "$target $porttarget $last[2] $packetlenght"`;
	    my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c ADD r-delay-t $target "$tb $porttarget $last[2] $packetlenght"`;
	}else{
	    print "ERROR: false condition in delmr! \n";
	    return 1;
	}
	return 0;
    }else{
	return 0;
    }

}#delmr ends here !!!




sub main {

    my $PortNumber = "";
    my $tb = "";
    my @tb = ();
    my @difftb = ();
    my $difftb = "";
    my $portchangetb = "";
    my @portchangetb = ();
    my $ports;

    $ports =  &readattributevalue("port-number","CURRENT");
    opendir(NEW,$newdir) || die "Cannot open $newdir \n";
    while(my $filename = readdir(NEW)){
        if(($filename=~/^\./) || ($filename=~/^\../)){
	    next;
	}
	my @aux = split(/\./,$filename);#$filename always like ttxy.relationgroup.config
	if($aux[0]){
    	    $tb = $aux[0];
        }else{
	    print "ERROR: Couldn't extract test box name from $filename \n";
        }

        if(open(FP, $newdir.$filename)){	    
	    my @newfile = ();
	    my $changeflag = 0;
	    my @auxchanges = ();
            while(<FP>){
                #read the port number from the configuration file
                push @newfile, $_;
	        if(/PortNumber/){ 
		    my @aux = split(/\s/,$_);
		    $PortNumber = $aux[1]; 	       
	        }
	    }
	    close(FP);
            #trigger port number changes on  ttxy.relationgroup.config files
	    my $port = $$ports{$tb};
	    if(!($port =~/$PortNumber/)){#PortNumber has changed(e.g. itsn't equal to the one on ttreg)
  	        $portchange{$tb} = $PortNumber;
	    }
	              			       		   
            foreach my $line (@newfile){ #added lines		
		if($line=~/^dl_add/){
                    push @auxchanges, "< ". $line; #new m.r. added 
                    $changeflag = 1; 
		}elsif($line=~/^dl_del/){
		    push @auxchanges, "> ". $line; #new m.r. added
		    $changeflag = 1;
		}else{
		    next;
		}        
             }#foreach my $line	     		
	     if($changeflag){
		 $diffmatrix{$tb} = [@auxchanges];
	     }
	     my $ret = `mv $olddir$filename /ncc/ttpro/data/config/temp/`;
	     my $ret = `mv $newdir$filename $olddir$filename`;		
	}else{
	    print "ERROR: Couldn't open $filename \n";
	}
	
    }# end of while(my $filename ...

    #trigger port number changes on ttreg(in case tt-ops changes it)
    #WARNING: tt-ops changes overwrite User changes 
    if(-e $PORTNUMBERFILE){
        my %portfile;
	open(FPPORT, "<", $PORTNUMBERFILE);
	while(<FPPORT>){
	    my @aux = split(/:/,$_);
	    $portfile{$aux[0]} = $aux[1];
	}
	close(FPPORT);
	my @aux = keys %$ports;
	foreach my $tb (@aux){
	    my $oldport = $portfile{$tb}; 
	    if($portfile{$tb}=~/\d+/){
		my $oldport = $portfile{$tb};
	    }else{
		next;
	    }
	    if($oldport != $$ports{$tb}){
	        $portchange{$tb} = $$ports{$tb};
	    }
	}
    }

    # create the file for the next run
    open(FPPORT, ">", $PORTNUMBERFILE);
    while(my ($k, $v) = each %$ports){
	print FPPORT $k.":".$v."\n";
    }
    close(FPPORT);


    
    @difftb = keys %diffmatrix;
    while(<@difftb>){
	$difftb = $difftb . $_;
    }
    @portchangetb = keys %portchange;
    while(<@portchangetb>){
	$portchangetb = $portchangetb . $_;
    }
    &filldatamatrix;
    @tb = keys %datamatrix;
    sort @tb;
    if($portchangetb){ 
    #yes, at least one tb changed its port number

    #Although Hosts can't change port number themselves, 
    #the code structure bellow can be used(with very small changes) 
    #for other products that might appear in the future
	foreach $tb(@tb){
	    next if($tb=~/tt01/); #tt01 is demo. No changes are aloud from its interface
	    print "Doing port number analyses for $tb: \n" if($VERBOSE);
	    my $p1 = $datamatrix{$tb};
            #update ttreg entry with new port number value
	    if($portchangetb =~/$tb/){
		my $newport = $portchange{$tb};
		print "Changing port number value for $tb to $newport \n" if($VERBOSE);
	        #my $ret = `/home/maximo/trabalho/projetos/nsquare/code/ttregconfig/ttregconfig.pl MOD port-number $tb $newport`;       
		my $ret = `/ncc/ttpro/bin/ttregconfig.pl MOD port-number $tb $newport`;
	    }
            my $targetmatrix = $targetmatrix{$tb};	
	    foreach my $target (@$targetmatrix){
	        if($portchangetb =~/$target/){#$tb has a target which changed port number
                    my @attributes = keys %$p1;
    		    foreach my $attribute (@attributes){
		        if($attribute =~/delay/){ #all *delay* attributes have a port number parameter(o-delay-t, u-delay-t and r-delay-t)
			    my $p2 = $$p1{$attribute};
			    foreach my $value (@$p2){
			        if($value =~/$target/){#this line has the target which changed its port number			          
	                            $_ =  $value;
				    s/^\s//;
                                    $value = $_;
				    my $oldvalue = $value;
     			            my @parameters = split(/\s/,$value);
		  		    my $newvalue = $parameters[0]. " ".$portchange{$target}." ".$parameters[2]." ".$parameters[3]." ".$parameters[4]." ".$parameters[5];
                                    #in this case MOD doesn't work, thus DEL old line and ADD new one with correct port number value
				    print "Changing mesurement relation from $tb from $oldvalue to $newvalue \n" if($VERBOSE);
                                    #my $execstr ="/home/maximo/trabalho/projetos/nsquare/code/ttregconfig/ttregconfig.pl DEL $attribute $tb '$oldvalue'";
                                    my $execstr ="/ncc/ttpro/bin/ttregconfig.pl DEL $attribute $tb '$oldvalue'";
                                    my $ret = `$execstr`;
                                    #$execstr ="/home/maximo/trabalho/projetos/nsquare/code/ttregconfig/ttregconfig.pl ADD $attribute $tb '$newvalue'";
                                    my $execstr ="/ncc/ttpro/bin/ttregconfig.pl ADD $attribute $tb '$newvalue'";
				    $ret = `$execstr`;
			        }
			    }
		        }
		    }#foreach my $attribute
	        }
	    }#foreach my $target
	 }
    }#if($portchangetb){

    #need two loops with the same @tb array to update portnumber changes on the first loop and 
    #measurement relations(with possible portnumber changes) on the second

    if($difftb){
    #yes, at least one Host changed a measurement relation on his test box
	my $hostaction;
        foreach $tb(@tb){
	    next if($tb=~/tt01/); #tt01 is demo. No changes are aloud from its interface	
	    $hostaction = 0;
	    if($difftb =~/$tb/){
            #yes, $tb has a new measurement relation set
		print "Doing analyses for $tb: \n" if($VERBOSE);
	        my $p2 = $diffmatrix{$tb};
	        foreach my $aux (@$p2){
		    my $color = "";
		    my $target ="";
		    my $packetlenght ="";
		    my $rate = "";
		    my $attributename = "";
                    my $mode = "";
                    my $burst = "";
		    my @parameters = split(/\s+/,$aux);
		    if($aux =~ /dl/){#it's a delay measurement config line
		        $color = $parameters[2];
		        $target = $parameters[3];
		        $packetlenght = $parameters[4];
		        $rate = $parameters[5];                       
                        if(!(($color) && ($target) && ($packetlenght) && ($rate) && ($$ports{$target}) && ($$ports{$tb}))){
			    print "ERROR: color: $color target: $target packetlenght: $packetlenght rate: $rate porttarget: $$ports{$target} porttb: $$ports{$tb}are not very good parameters. Please have a look! \n";
                            next;
			}
		        if($color eq "BLUE"){#this section deals only with changes done by the local user		 
		            if($aux =~/^>/){#DEL: the host deleted a line
				next if(&delmr($tb, $target, $rate, $packetlenght, $$ports{$target}, $$ports{$tb}));
				$hostaction++;
				print "Deleting measurement relations: $target $$ports{$target} $rate $packetlenght for $tb and $tb $$ports{$tb} $rate $packetlenght for $target \n" if($VERBOSE);

                                #my $ret = `/home/maximo/trabalho/projetos/nsquare/code/ttregconfig/ttregconfig.pl -c DEL u-delay-t $tb "$target $$ports{$target} $rate $packetlenght"`;
                                my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c DEL u-delay-t $tb "$target $$ports{$target} $rate $packetlenght 1 0"`;
		                #my $ret = `/home/maximo/trabalho/projetos/nsquare/code/ttregconfig/ttregconfig.pl -c DEL r-delay-t $target "$tb $$ports{$tb} $rate $packetlenght"`;
                                my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c DEL r-delay-t $target "$tb $$ports{$tb} $rate $packetlenght 1 0"`;
		            }elsif($aux =~/^</){#ADD: the host added a new line								
				next if(&addmr($tb, $target, $rate, $packetlenght, $$ports{$target},$$ports{$tb} ));
                                my $networkload = $rate * $packetlenght;
                                if($networkload > $MAXLOAD){
				    print "WARNING: a measurement relation with a rate = $rate and packet size = $packetlenght is a bit too much!!! \n tt-ops please contact the Host to verify.  \n" if($VERBOSE);
				}
				$hostaction++;
				print "Adding measurement relation: $target $$ports{$target} $rate $packetlenght for $tb and $tb $$ports{$tb} $rate $packetlenght for $target\n" if($VERBOSE);
	    	                #my $ret = `/home/maximo/trabalho/projetos/nsquare/code/ttregconfig/ttregconfig.pl -c ADD u-delay-t $tb "$target $$ports{$target} $rate $packetlenght"`;
                                my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c ADD u-delay-t $tb "$target $$ports{$target} $rate $packetlenght 1 0"`;
		                #my $ret = `/home/maximo/trabalho/projetos/nsquare/code/ttregconfig/ttregconfig.pl -c ADD r-delay-t $target "$tb $$ports{$tb} $rate $packetlenght"`;
                                my $ret = `/ncc/ttpro/bin/ttregconfig.pl -c ADD r-delay-t $target "$tb $$ports{$tb} $rate $packetlenght 1 0"`;
			    }
		      
			}
	            }
	        }#foreach my $aux  
	       }
	    }
	if($hostaction > 10){
	    print "WARNING: $tb Host played a lot with the relation group. \n  $hostaction lines were added or deleted!!! \n tt-ops please contact the Host to verify. \n"  if($VERBOSE);
	}
    }# if($difftb)


}#sub main ends here!!!





# --------------- Start Point -------------------------------------
#parse command line

 

  
my $parameter = shift(@ARGV);
if($parameter){
    if($parameter eq "-v"){
	$VERBOSE = 1;
    }
}


&main;











