#!/usr/local/bin/perl
#
# $Id: createconfigfile.pl,v 1.4 2006/11/04 14:40:48 ruben Exp $
#
# All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of the author not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

#------------------------------------------------------------------------------
# Module Header
# Filename          : createconfigfile.pl
# Purpose           : creates test boxes configuration file
# Author            : Maximo Alves
# Date              : 05 March 2002
# Description       : 
# Language Version  : Perl 5
# OSs Tested        : Linux
# Command Line      : createconfigfile.pl
# Description:
# 1- fill multidimensional array with ACTIVE tbs and their relation group
#    * generates: $datamatrix{tb} = $attributevalue{attribute_value} = ( 6000 120 100 tt85 6000 120 100 ... )
#    *            $targetmatrix{tb} = (ttxx ttxy ttzz ...)
#       
#
#
# Input Files       : 
# Output Files      : 
# External Programs : ttconfig, sed
#                    
# Problems          :
#                    
# To Do             : 
#                     
# Comments          :
#                     
#------------------------------------------------------------------------------
use lib '/ncc/ttpro/bin/';
use Carp;
use strict;
use datafunctions; 


# --------------- Variable declaration ------------------------------

my %relations; # hash contains all relations for one TB
               # used to detect duplicated lines
my $TTROOT ="/ncc/ttpro";
my $PACKETLENGTH = 100;

my $DEBUG = "0";
my $VERBOSE = "0";
my $logfile = "> /ncc/ttpro/tmp/createconfigfile.log";
my $logbuffer = "";
my @activetbs = `/ncc/ttpro/bin/ttconfig -v TTLIST`;
my %tbidmap = map {my ($id,$name) = split(/\s/,$_); $name =~ s/\Q.ripe.net\E// ; ($name,$id) }  qx(/ncc/ttpro/bin/ttconfig -v LIST_OF_TESTBOXES);
my %idtbmap = map { $tbidmap{$_} => $_ } keys %tbidmap;
my $BURST = 1; #burst mode. Send 1 packet to the destination as fast as possible
my $MODE = 0; #linear mode packet distribution(not poisson)


# --------------- Start Point -------------------------------------

chomp(@activetbs);
&main;


# --------------------- Sub routines --------------------------------

sub isonline {
    #return 1 if not online
    #       0 if online

    my $target = $_[0];
    my $id = $tbidmap{$target};

    # Is this a 2NIC id ?
    if ($id > (2 ** 16))
    {
        # Use testbox where 2NIC resides to determine if it is online...
        $target = $idtbmap{$id ^ (2 ** 16)};
    }

    carp $target if (!($target =~ /^\w+$/));
    if (grep { /^$target$/ } @activetbs)
    {
	return 0;
    }
    return 1;
}

sub is_duplicated {
 #return 1 if yes
 # 0 otherwise 
    my $source = @_[0];
    my $destin = @_[1]; 
    my $attributename = @_[2];
    my $attributevalue = @_[3];
    my $key = $source . ' - ' . $destin . ' - ' . $attributename . ' - ' . $attributevalue; 

    if($relations{$key}){
	print "WARNING duplicate line : source $source, destination $destin, attribute $attributename, relation $attributevalue \n";
	return 1;
    }else{
	$relations{$key} = 1;
	return 0;
    }

}

sub useburstmode {
    my($burst, $mode);

    if(defined $_[0]){
    # burst is defined  in ttreg
	$burst = $_[0];
    }else{
	$burst = $BURST;
    }

    if(defined $_[1]){
    # mode is defined  in ttreg
	$mode = $_[1];
    }else{
	$mode = $MODE;
    }

return($mode, $burst); 
}

sub main {

    my @tb = ();
    my $tb = "";
    my $p1 = "";
    my $p2 = "";
    my $p3 = "";
    my $outbuffer = "";
    my $ports = "";

    unless( open(LOGFP, $logfile)){
	print "ERROR: could not open $logfile \n";
#	die;
    } 
    print LOGFP "*** STARTING *** \n";
    &filldatamatrix;
    $ports =  &readattributevalue("port-number","CURRENT");

    print LOGFP "* Data matrix ready \n";
    my @tbd1 = `/ncc/ttpro/bin/ttconfig -v TTLIST`;
    my @tbd3 = `/ncc/ttpro/bin/ttconfig -v IP6TTLIST`;
    my $ip4_flag = 0;
    my $ip6_flag = 0;
    my $x_flag = 0;
    chomp(@tbd1);
    chomp(@tbd3);
    my @tbd2 = @tbd1;
    my @tbd4 = @tbd3;
    @tb = keys %datamatrix;
    @tb = sort @tb;
    foreach $tb(@tb){
	next if &isonline($tb);
    	$p1 = $datamatrix{$tb};
	$p2 = $$p1{'o-delay-t'};
	$p3 = $$p1{'o6delay-t'};
	$ip4_flag = 0;
        $ip6_flag = 0;
	#$outbuffer = "#Config file for test box: $tb \n";
        $outbuffer = $outbuffer . "# \nVER  1  1.0.0 \nPERI 2 43200 3600 \n#\n";
	if(@$p2[0] eq "all_chain"){ # follow the normal procedure for config file generation	 
	    $ip4_flag = 1;
	    foreach my $tbd1(@tbd1){
		if(!($tbd1 eq $tb)){
		    $outbuffer = $outbuffer . "# From $tbd1.$domain to $tb.$domain\n";
		    $outbuffer = $outbuffer . "TRAR 3 $tbd1.$domain $tb.$domain 10 \n";
		    $outbuffer = $outbuffer . "SEND 6 $tbd1.$domain $tb.$domain 6000 100 0 120 1 0\n";
		    $outbuffer = $outbuffer . "RPCK 4 $tb.$domain $tbd1.$domain 1 6000 \n";
		}else{
		    foreach my $tbd2(@tbd2){
		        if(!($tbd2 eq $tb)){
		            $outbuffer = $outbuffer . "# From $tb.$domain to $tbd2.$domain\n";
		            $outbuffer = $outbuffer . "TRAR 3 $tb.$domain $tbd2.$domain 10 \n";
		            $outbuffer = $outbuffer . "SEND 6 $tb.$domain $tbd2.$domain 6000 100 0 120 1 0\n";
		            $outbuffer = $outbuffer . "RPCK 4 $tbd2.$domain $tb.$domain 1 6000 \n";
		        }
	            }
	        }
	    }
	}
	if(@$p3[0] && @$p3[0] eq "all_chain"){
	    $ip6_flag = 1;
            foreach my $tbd1(@tbd3){
                if(!($tbd1 eq $tb)){
                    $outbuffer = $outbuffer . "# From $tbd1.$domain to $tb.$domain [IPv6]\n";
                    $outbuffer = $outbuffer . "6TR6 3 $tbd1.$domain $tb.$domain 10 \n";
                    $outbuffer = $outbuffer . "6SND 6 $tbd1.$domain $tb.$domain 6000 100 0 120 1 0\n";
                    $outbuffer = $outbuffer . "6RCV 4 $tb.$domain $tbd1.$domain 1 6000 \n";
                }else{
                    foreach my $tbd2(@tbd4){
                        if(!($tbd2 eq $tb)){

                            $outbuffer = $outbuffer . "# From $tb.$domain to $tbd2.$domain\n";
                            $outbuffer = $outbuffer . "6TR6 3 $tb.$domain $tbd2.$domain 10 \n";
                            $outbuffer = $outbuffer . "6SND 6 $tb.$domain $tbd2.$domain 6000 100 0 120 1 0\n";
                            $outbuffer = $outbuffer . "6RCV 4 $tbd2.$domain $tb.$domain 1 6000 \n";
                        }
                    }
                }
            }
        }
	
	if(!$ip4_flag || !$ip6_flag) { # do relation groups procedure
	    my $targets = ""; 
	    my @attributes = ();
            my @auxmatrix1 = ();          
            my @auxmatrix2 = ();       
	    $targets = $targetmatrix{$tb};
	    %relations = {};

#           push @auxmatrix1, @$targets;
            #now filter out all OFFLINE tbs
            foreach my $testbox ( @$targets ){
		if(defined($testbox) && $testbox && !(&isonline($testbox))){
		    push @auxmatrix1, $testbox;		
		}
	    }

	    push  @auxmatrix1, $tb;
 
	    @auxmatrix1 =  sort  @auxmatrix1;
            @auxmatrix2 = @auxmatrix1;
	    @attributes = keys %$p1;
	    sort @attributes;
	    foreach my $target (@auxmatrix1){
                if(defined($target) && $target){#if $target is not " " 		   		   
                    next if &isonline($target);
		    if($target eq $tb){
			foreach my $peer (@auxmatrix2){
			    if(($peer eq $tb) || (!($peer))){
				next;
			    }
			    next if &isonline($peer);
			    if (!$ip4_flag) {
				    foreach my $attribute (@attributes){
			                $p2 = $$p1{$attribute};
	                                foreach my $value (@$p2){
	       		                    if($value =~ /^\s*${peer}\b/){#target is present  in this line					   
					        my @value = split(/\s/,$value);
			                	if(($attribute eq "o-delay-t" ||$attribute eq "u-delay-t" || $attribute eq "r-delay-t" )){
						        next if(&is_duplicated($tb, $peer, $attribute, $value));
							if (!$x_flag) { $outbuffer = $outbuffer . "# From $tb.$domain to $peer.$domain\n"; $x_flag = 1;}
							$outbuffer = $outbuffer . "SEND 6 $tb.$domain $peer.$domain $value[2] $value[4] 0 $value[3] $value[5] $value[6] \n";
 $outbuffer = $outbuffer . "RPCK 4 $peer.$domain $tb.$domain 1 $value[2] \n";
			                	}elsif(($attribute eq "o-tracer-t") || ($attribute eq "u-tracer-t") || ($attribute eq "r-tracer-t")){
						        next if(&is_duplicated($tb, $peer, $attribute, $value));
							if (!$x_flag) { $outbuffer = $outbuffer . "# From $tb.$domain to $peer.$domain\n"; $x_flag = 1;}
	                                    		$outbuffer = $outbuffer .  "TRAR 3 $tb.$domain $peer.$domain $value[2] \n";
			                    	}else{
 				            		# print "ERROR: $tb config file \n";
							# -> with IPv6 this is not necessarily an error condition
			                    	} 
				     	    }
			            	}
		             	    }#foreach my $attribute
			    }
			    $x_flag = 0; 
			    if (!$ip6_flag && @$p3[0]) {
				    foreach my $attribute (@attributes){
					$p2 = $$p1{$attribute};
					foreach my $value (@$p2){
	       		                    if($value =~ /^\s*${peer}\b/){#target is present  in this line					   
						my @value = split(/\s/,$value);
				            	if(($attribute eq "o6delay-t" ||$attribute eq "u6delay-t" || $attribute eq "r6delay-t" )){
						    next if(&is_duplicated($tb, $peer, $attribute, $value));
							if (!$x_flag) { $outbuffer = $outbuffer . "# From $tb.$domain to $peer.$domain [IPv6]\n"; $x_flag = 1;}
					                $outbuffer = $outbuffer . "6SND 6 $tb.$domain $peer.$domain $value[2] $value[4] 0 $value[3] $value[5] $value[6] \n";
					                $outbuffer = $outbuffer . "6RCV 4 $peer.$domain $tb.$domain 1 $value[2] \n";
					        }elsif(($attribute eq "o6tracer-t") || ($attribute eq "u6tracer-t") || ($attribute eq "r6tracer-t")){
						    next if(&is_duplicated($tb, $peer, $attribute, $value));
							if (!$x_flag) { $outbuffer = $outbuffer . "# From $tb.$domain to $peer.$domain [IPv6]\n"; $x_flag = 1;}
					                $outbuffer = $outbuffer .  "6TR6 3 $tb.$domain $peer.$domain $value[2] \n";
					        }else{
					                # print "ERROR: $tb config file \n";
							# -> with IPv6 this is not necessarily an error condition
            					}
        				    }
					}
				    }#foreach my $attribute
			    }#if(!$ip4_flag || !$ip6_flag)
			    $x_flag = 0;
			 }#foreach my $peer 
		    }else{
			if (!$ip4_flag) {
				foreach my $attribute (@attributes){

			            $p2 = $$p1{$attribute};
  	    	                    foreach my $value (@$p2){
		                	if($value =~ /^\s*${target}\b/){#target is present in this line					       
				    		my @value = split(/\s/,$value);
			            		if(($attribute eq "o-delay-t" ||$attribute eq "u-delay-t" || $attribute eq "r-delay-t" )){
						    next if(&is_duplicated($target, $tb, $attribute, $value));
							if (!$x_flag) {$outbuffer = $outbuffer .  "# From $target.$domain to $tb.$domain\n"; $x_flag = 1;}
  						        $outbuffer = $outbuffer . "SEND 6 $target.$domain $tb.$domain $$ports{$tb} $value[4] 0 $value[3] $value[5] $value[6] \n";
				        		$outbuffer = $outbuffer . "RPCK 4 $tb.$domain $target.$domain 1 $$ports{$tb} \n";
			            		}elsif(($attribute eq "o-tracer-t") || ($attribute eq "u-tracer-t") || ($attribute eq "r-tracer-t")){
						    next if(&is_duplicated($target, $tb, $attribute, $value));
							if (!$x_flag) {$outbuffer = $outbuffer .  "# From $target.$domain to $tb.$domain\n"; $x_flag = 1;}
	                                		$outbuffer = $outbuffer .  "TRAR 3 $target.$domain $tb.$domain $value[2] \n";
			            		}else{
				        		# print "ERROR: $tb config file \n";
							# -> with IPv6 this is not necessarily an error condition
			            		} 
				    	}
			     	    }
		         	}#foreach my $attribute
			}
			$x_flag = 0;
                        if (!$ip6_flag && @$p3[0]) {
                                foreach my $attribute (@attributes){

                                    $p2 = $$p1{$attribute};
                                    foreach my $value (@$p2){
                                        if($value =~ /^\s*${target}\b/){#target is present in this line
                                                my @value = split(/\s/,$value);
						if(($attribute eq "o6delay-t" ||$attribute eq "u6delay-t" || $attribute eq "r6delay-t" )){
						    next if(&is_duplicated($target, $tb, $attribute, $value));
							if (!$x_flag) {$outbuffer = $outbuffer .  "# From $target.$domain to $tb.$domain [IPv6]\n"; $x_flag = 1;}
						    $outbuffer = $outbuffer . "6SND 6 $target.$domain $tb.$domain $$ports{$tb} $value[4] 0 $value[3] $value[5] $value[6] \n";
							$outbuffer = $outbuffer . "6RCV 4 $tb.$domain $target.$domain 1 $$ports{$tb} \n";
					        }elsif(($attribute eq "o6tracer-t") || ($attribute eq "u6tracer-t") || ($attribute eq "r6tracer-t")){
						    next if(&is_duplicated($target, $tb, $attribute, $value));
							if (!$x_flag) {$outbuffer = $outbuffer .  "# From $target.$domain to $tb.$domain [IPv6]\n"; $x_flag = 1;}
					                $outbuffer = $outbuffer .  "6TR6 3 $target.$domain $tb.$domain $value[2] \n";
					        }else{
					                # print "ERROR: $tb config file \n";
							# -> with IPv6 this is not necessarily an error condition
					        }
					}
				    }
				}#foreach my $attribute
			}
			$x_flag = 0;
		    }
	       }#if($target)
	    }#foreach my $target
	}

	my $path = ">".$TTROOT . "/files/" . $tb . "_specific/usr/home/ttraffic/config/configfile";

	unless(open(FP,$path)){
	    print "ERROR: could not open $path \n";
        } else{

	    print FP $outbuffer;
	    close(FP);
	    print "$tb config file created \n" if($VERBOSE);
	}
	print $outbuffer if($DEBUG);
	$outbuffer = "";
	print LOGFP "* $path finished! \n";
	print " ******************************************************************* \n" if($DEBUG);
    }  # foreach $tb(@tb){


    my $aux1 = `sed s/tt04/jordaan/ /ncc/ttpro/files/tt04_specific/usr/home/ttraffic/config/configfile`;    
    open(FP,">/ncc/ttpro/files/tt04_specific/usr/home/ttraffic/config/configfile");
    print FP $aux1;
    close(FP);
    my $aux1 = `sed s/tt02/osdorp/ /ncc/ttpro/files/tt02_specific/usr/home/ttraffic/config/configfile`;
    open(FP,">/ncc/ttpro/files/tt02_specific/usr/home/ttraffic/config/configfile");
    print FP $aux1;
    close(FP);
    close(LOGFP);	
}#main ends here !!!





























