#!/usr/local/bin/perl
#
# All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of the author not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

#------------------------------------------------------------------------------
# Module Header
# Filename	  		: createrelationgroupconfigfile.pl
# Purpose	   		: creates test boxes relation group configuration file
# Author			: Maximo Alves
# Date		  		: 26 April 2002
# Description	   : 
# Language Version  : Perl 5
# OSs Tested		: Linux
# Command Line	  	: createrelationconfigfile.pl
# Input Files	   : 
# Output Files	  : 
# External Programs : ttconfig
#			
# Problems	  :
#			
# To Do		 : 
#			 
# Comments	  :
#			 
#------------------------------------------------------------------------------
use lib '/ncc/ttpro/bin/';
use strict;
use datafunctions; 

# --------------- Variable declaration ------------------------------


my $TTROOT ="/ncc/ttpro";
my $StandardPortNumber = 6000;
my $StandardPacketLenght = 100;
my $MaxPacketLenght = 1500;
my $MinPacketLenght = 60;
my $StandardRate = 120;
my $MaxRate = 1800;
my $DEBUG = "0";
my $VERBOSE = "0";
# --------------------- Sub routines --------------------------------

sub readattributevalue {
	my $attributename = $_[0];
	my $status = $_[1];
	my %retmatrix;

	my @aux = `/ncc/ttpro/bin/ttconfig -v ALL $status | grep  $attributename`;
	foreach my $aux (@aux){
	my @parameters = split(/\s/,$aux);
	$retmatrix{$parameters[0]} = $parameters[2];
	}

	return {%retmatrix}
}#readvalue ends here!!!



sub filldatamatrix {
	my %attributevalues;
	my @aux = ();
	my @targetgroup = ();
	my $entry  = "";
	my @buffer = `/ncc/ttpro/bin/ttconfig -v TARGETS`;
	my $i = 0;
	my $attribute = "";

	for($i=0; $i<=scalar(@buffer);$i++){
		if($buffer[$i]=~/start/){
			@aux = split(/\s/,$buffer[$i]);
			$entry = $aux[1];
		}elsif($buffer[$i]=~/end/){
			$datamatrix{$entry} = {%attributevalues};
			$targetmatrix{$entry} = [@targetgroup];	
			@targetgroup = ();
			%attributevalues = [];
			$entry = "";
		}else{
		   # $datamatrix($entry) = $buffer[$i];
			@aux = split(/\s+/,$buffer[$i]);
			$attribute =  $aux[0];
			shift @aux;

			# C++ style exception handling. try to parse a line, it will die if it is broken
			eval {
				$attributevalues{$attribute} = &parseattributevalue($attribute,\@aux,\@targetgroup);
			};
			if ($@)
			{
				# We can't continue here, but show context to the operator
				# old files should be left in place, no harm should overcome to the grid
				die "$0: While processing $entry: $@\n";
			}
		}
	}

} #filldatamatrix ends here !!!


sub parseattributevalue {

    my $i               = 0;
    my $j               = 0;
    my $auxstr          = "";
    my $attribute       = "";
    my $key             = "";
    my @attributevalues = ();
    my $p               = "";
    my $targetgroup     = "";
    my $flag            = 1;

    $targetgroup = $_[2];
    $p           = $_[1];
    $attribute   = $_[0];

    foreach my $val (@$p) {
        if ( $val =~ /all_chain/ ) {
            push( @attributevalues, "all_chain" );
            return [@attributevalues];
        }
    }
    foreach $key ( keys %attributenames ) {
        if ( $attribute =~ /$key/ ) {
            while ($#$p >= 0 )
            {
				#this logic assumes that a target is always the first line item
				# and the length is stored in $attributenames{$key}
				my($target,@targetparams) = splice(@$p,0,$attributenames{$key});

				die "Wrong number of elements for $key! check ttreg! \"$target @targetparams\" != $attributenames{$key};\n" if (scalar (@targetparams) + 1) != $attributenames{$key};

				unless ( grep { /^${target}$/ } @$targetgroup ) {
					push(@$targetgroup,$target);
				}
				push(@attributevalues,"$target @targetparams");
            }
        }
    }

    return [@attributevalues];
}    #parseattributevalue ends here !!!



sub main {

   my @tb = ();
   my $tb = "";
   my $p1 = "";
   my $p2 = "";
   my $outbuffer = "";
   my $online = "";
   my $status = "";
   my $ports;

   &filldatamatrix;

   my @online = `/ncc/ttpro/bin/ttconfig -v TTLIST`;
   foreach my $aux (@online){
	   $online = $online . " " . $aux;
   }  
   @tb = keys %datamatrix;
   @tb = sort @tb;
   $ports =  &readattributevalue("port-number","ACTIVE");
   foreach $tb(@tb){
	   $outbuffer = $outbuffer ."PortNumber $$ports{$tb} \n"; 
	   $outbuffer = $outbuffer ."StandardPacketLenght $StandardPacketLenght \n";
	   $outbuffer = $outbuffer ."MaxPacketLenght $MaxPacketLenght \n";
	   $outbuffer = $outbuffer ."StandardRate $StandardRate \n";
	   $outbuffer = $outbuffer ."MaxRate $MaxRate \n";
	   $outbuffer = $outbuffer ."MinPacketLenght $MinPacketLenght \n";
	   my @attributes = (); 
	   $p1 = "";
	   $p1 = $datamatrix{$tb};	   
	   @attributes = keys %$p1;
	   foreach my $attribute (@attributes){
	   $p2 = "";
	   $p2 = $$p1{$attribute};
	   foreach my $value (@$p2){
		   my @value = split(/\s/,$value);
		   if($online =~ /$value[1]/){
		   $status = "ON";
		   }else{
		   $status = "OFF";
		   }
		   #if you want add more attributes do it here!!!
		   if($attribute eq "o-delay-t"){
		   $outbuffer = $outbuffer . "dl BLACK  $value[1]  $value[4] $value[3] $status\n";
		   }elsif($attribute eq "u-delay-t"){
		   $outbuffer = $outbuffer . "dl BLUE  $value[1]  $value[4] $value[3] $status\n";
		   }elsif($attribute eq "r-delay-t"){
		   $outbuffer = $outbuffer .  "dl PURPLE  $value[1]  $value[4] $value[3] $status\n";
		   }					 

		   if($attribute eq "o6delay-t"){
		   $outbuffer = $outbuffer . "dl IP6BLACK  $value[1]  $value[4] $value[3] $status\n";
		   }elsif($attribute eq "u6delay-t"){
		   $outbuffer = $outbuffer . "dl IP6BLUE  $value[1]  $value[4] $value[3] $status\n";
		   }elsif($attribute eq "r6delay-t"){
		   $outbuffer = $outbuffer .  "dl IP6PURPLE  $value[1]  $value[4] $value[3] $status\n";
		   }
	   }
	   }

	my $path = ">".$TTROOT . "/files/" . $tb ."_specific/usr/home/ttraffic/config/relationgroup.config";

	unless(open(FP,$path)){
		print "ERROR: could not open $path \n";
	} else{

		print FP $outbuffer;
		close(FP);
		print "relationgroup.config for $tb  stored \n" if($VERBOSE);
	}
	print $outbuffer if($DEBUG);
	$outbuffer = "";

	}
} #Main ends here !!!

# --------------- Start Point -------------------------------------

&main;



















