# Copyright (c) 1993, 1994, 1995, 1996, 1997  The TERENA Association
#
# All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of the author not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

# $Revision: 1.3 $ %D%
#------------------------------------------------------------------------------
# Module Header
# Filename          : datafunctions.pm
# Purpose           : Provide read functions for internal registry database
# Author            : Maximo
# Date              : 7 May 2002
# Description       :
# Language Version  : Perl 
# OSs Tested        : Linux
# Command Line      : -
# Input Files       : 
# Output Files      : -
# External Programs :
# Problems          :
# To Do             : 
# Comments          : 
#------------------------------------------------------------------------------


package datafunctions;

use strict;
use vars qw(@ISA @EXPORT %attributenames %datamatrix %targetmatrix $domain);
use Exporter;

@ISA = ('Exporter');
@EXPORT = qw(%attributenames %datamatrix %targetmatrix $domain &filldatamatrix &readattributevalue);


#
# exported variables
#
%attributenames = ('o-delay-t',6,'u-delay-t',6,'r-delay-t',6,'o-tracer-t',2,'u-tracer-t',2,'r-tracer-t',2,'o6delay-t',6,'u6delay-t',6,'r6delay-t',6,'o6tracer-t',2,'u6tracer-t',2,'r6tracer-t',2);
%datamatrix; # hash with all ACTIVE tbs and their relation groups
                # keys: tbs
                # values: %attributevalues hash
%targetmatrix; # hash with all tbs and all their targets
                  # keys: tbs
                  # values: arrays of targets

$domain = "ripe.net";



#
# exported functions
#

sub readattributevalue {
    my $attributename = $_[0];
    my $status = $_[1];
    my %retmatrix;

    my @aux = `/ncc/ttpro/bin/ttconfig -v ALL $status | grep  $attributename`;
    foreach my $aux (@aux){
	my @parameters = split(/\s/,$aux);
	$retmatrix{$parameters[0]} = $parameters[2];
    }

    return {%retmatrix}
}#readvalue ends here!!!

sub filldatamatrix {
    use Data::Dumper;
    my %attributevalues; # hash with attribute values(o-delay-t, r-delay-t, etc) for one tb 
                         # keys are attribute values
                         # values are arrays like :( tt84 6000 120 100 0 1 tt85 6000 120 100 0 1 ... )
    my @aux = ();
    my $targetgroup = "";
    my @targetgroup;
    my $entry  = "";
    my @buffer = `/ncc/ttpro/bin/ttconfig -v TARGETS`;
    #each @buffer element has a ttconfig line, like:
    #start tt01
    #o-delay-t tt87 6000 120 100 0 1 tt85 6000 120 100 0 1...
    #r-delay-t tt80 6000 120 110 0 1 tt03 6000 60 101 0 1...
    #o-tracer-t tt87 10 tt85 10 ...
    #end tt01
    #start tt02
    # ...
    my $i = 0;
    my $attribute = "";

    for($i=0; $i<=scalar(@buffer);$i++){
	if($buffer[$i]=~/start/){
	    @aux = split(/\s/,$buffer[$i]);
	    $entry = $aux[1];
	}elsif($buffer[$i]=~/end/){
	    $datamatrix{$entry} = {%attributevalues};
	    @targetgroup = split(/\s/,$targetgroup);
	    $targetmatrix{$entry} = [@targetgroup];	
	    $targetgroup = "";
	    @targetgroup = ();
	    %attributevalues = [];
	    $entry = "";
	}else{
	   # $datamatrix($entry) = $buffer[$i];
	    @aux = split(/\s/,$buffer[$i]);
	    $attribute =  $aux[0];
	    shift @aux;
            $attributevalues{$attribute} = &parseattributevalue($attribute,\@aux,\$targetgroup);
	}
    }

} #filldatamatrix ends here !!!



sub parseattributevalue {

  my $i = 0;
  my $j = 0;
  my $auxstr = "";
  my $attribute = "";
  my $key = "";
  my @attributevalues = ();
  my $p = "";
  my $targetgroup = "";
  my $flag = 1;
  $targetgroup = $_[2];
  $p =  $_[1];
  $attribute = $_[0]; 
  foreach my $val(@$p){
      if($val =~ /all_chain/){
	  push(@attributevalues,"all_chain");
	  return [@attributevalues];
      }
  }
  foreach $key (keys %attributenames){
      if($attribute =~/$key/){
	  while(@$p){ #this logic assumes that a target is always the first line item
	      $auxstr =  $auxstr ." ". shift @$p;
	      if($flag){			    			  
		  unless("$$targetgroup " =~ /$auxstr /){
			  $$targetgroup =  $$targetgroup .  $auxstr;
		      }
		  		
		  $flag = 0;
	      }
	      $i++;
	      if($i == $attributenames{$key}){
		  $i = 0;
		  $attributevalues[$j] = $auxstr;
		  $auxstr = "";
		  $j++;
		  $flag = 1;
	      }
	  }
       }
   }

  [@attributevalues];
} #parseattributevalue ends here !!!

1;
