#! /bin/sh
# $Id: setup_relations.sh,v 1.5 2009/02/18 13:48:58 ruben Exp $
EXEC=
VERBOSE=
BOX=
CLEAR=
IGNORELIST=`cat /ncc/ttpro/config/relations.exclude`
DO_V4=
DO_V6=
DEF_O_DELAY_T="6000 120 100 1 0"
DEF_O_TRACER_T="10"
DEF_O6DELAY_T="6000 120 100 1 0"
DEF_O6TRACER_T="10"

PATH="/sbin:/bin:/usr/sbin:/usr/bin:/usr/local/sbin:/usr/local/bin:/home/ttraffic/bin:/ncc/ttpro/bin"

export PATH

usage()
{

                cat <<EOT
$(basename $0) [options] tbid
        -n          dry run, do not execute
        -v          be verbose
		-c			clear mode (delete all relations for a box)
        -h          this message
        -4          Only do v4 relations
        -6          Only do v6 relations
        tbid        reverse ordering, oldest first
EOT
}


lock()
{
        /usr/local/bin/lockfile -l 2700 -r 2 /ncc/ttpro/tmp/-update.lock || (echo "Either another $0 or make update busy" && exit 1)
}

unlock()
{
        rm -f /ncc/ttpro/tmp/-update.lock
}

while [ $# -gt 0 ]; do
	case $1 in
	'-h')
		usage
		exit 1
		;;
	'-c')
		CLEAR=yes
		;;
	'-n')
		EXEC=echo
		;;
	'-v')
		VERBOSE=echo
		;;
	'-4')
		if [ -n "$DO_V6" ]; then
			echo "$(basename $0): ABORT: already selected -6"
			usage
			exit 1
		fi
		DO_V4=yes
		;;
	'-6')
		if [ -n "$DO_V4" ]; then
			echo "$(basename $0): ABORT: already selected -4"
			usage
			exit 1
		fi
		DO_V6=yes
		;;
	*)
		if [ -n "${BOX}" ]; then
			echo "$(basename $0): ABORT: Already specified a box ($BOX)"
			usage
			exit 1
		fi
		BOX=${1}
		;;
	esac
	shift
done

if [ -z $BOX ]; then
	echo "$(basename $0): ABORT: No box to operate on"
	usage
	exit 1
fi

lock

if [ -z "$DO_V6" ]; then
	if [ -n "`/ncc/ttpro/bin/readttreg.pl "$BOX" ipaddr`" ]; then

		if [ -z "${CLEAR}" -a -n "`/ncc/ttpro/bin/readttreg.pl "$BOX" o-delay-t`" ]; then
			echo "ERROR: Already found configured 'o-delay-t' for $BOX, aborting"
			unlock && exit 1
		elif [ -z "${CLEAR}" -a -n "`/ncc/ttpro/bin/readttreg.pl "$BOX" o-tracer-t`" ]; then
			echo "ERROR: Already found configured 'o-tracer-t' for $BOX, aborting"
			unlock && exit 1
		else
			if [ -n "$CLEAR" ]; then
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o-delay-t CHAIN "$BOX ${DEF_O_DELAY_T}"
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o-tracer-t CHAIN "$BOX ${DEF_O_TRACER_T}"
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o-delay-t $BOX "$BOX ${DEF_O_DELAY_T}"
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o-tracer-t $BOX "$BOX ${DEF_O_TRACER_T}"

				/ncc/ttpro/bin/readttreg.pl "$BOX" o-delay-t | while read cfg; do
					${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o-delay-t $BOX "$cfg"
				done

				/ncc/ttpro/bin/readttreg.pl "$BOX" o-tracer-t | while read cfg; do
					${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o-tracer-t $BOX "$cfg"
				done
			else
				${EXEC} /ncc/ttpro/bin/makefullmesh.pl -v4 -t $BOX
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl ADD o-delay-t CHAIN "$BOX ${DEF_O_DELAY_T}"
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o-delay-t $BOX "$BOX ${DEF_O_DELAY_T}"
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl ADD o-tracer-t CHAIN "$BOX ${DEF_O_TRACER_T}"
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o-tracer-t $BOX "$BOX ${DEF_O_TRACER_T}"

				for excluded_box in $IGNORELIST; do
					${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o-delay-t $BOX "$excluded_box ${DEF_O_DELAY_T}"
					${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o-tracer-t $BOX "$excluded_box ${DEF_O_TRACER_T}"
					${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o-delay-t $excluded_box "$BOX ${DEF_O_DELAY_T}"
					${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o-tracer-t $excluded_box "$BOX ${DEF_O_TRACER_T}"
				done
			fi
		fi

	else
		echo "ERROR: No IPv4 address found in ttreg for $BOX, ignoring"
	fi
fi

if [ -z "$DO_V4" ]; then
	if [ -n "`/ncc/ttpro/bin/readttreg.pl "$BOX" ipaddr6`" ]; then
		if [ -z "${CLEAR}" -a -n "`/ncc/ttpro/bin/readttreg.pl "$BOX" o6delay-t`" ]; then
			echo "ERROR: Already found configured 'o6delay-t' for $BOX, aborting"
			unlock && exit 1
		elif [ -z "${CLEAR}" -a -n "`/ncc/ttpro/bin/readttreg.pl "$BOX" o6tracer-t`" ]; then
			echo "ERROR: Already found configured 'o6tracer-t' for $BOX, aborting"
			unlock && exit 1
		else
			if [ -n "$CLEAR" ]; then
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o6delay-t IP6TTLIST "$BOX ${DEF_O6DELAY_T}"
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o6delay-t $BOX "$BOX ${DEF_O6DELAY_T}"
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o6tracer-t IP6TTLIST "$BOX ${DEF_O6TRACER_T}"
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o6tracer-t $BOX "$BOX ${DEF_O6TRACER_T}"

				/ncc/ttpro/bin/readttreg.pl "$BOX" o6delay-t | while read cfg; do
					${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o6delay-t $BOX "$cfg"
				done

				/ncc/ttpro/bin/readttreg.pl "$BOX" o6tracer-t | while read cfg; do
					${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o6tracer-t $BOX "$cfg"
				done
			else
				${EXEC} /ncc/ttpro/bin/makefullmesh.pl -v6 -t $BOX 
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl ADD o6delay-t IP6TTLIST "$BOX ${DEF_O6DELAY_T}"
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o6delay-t $BOX "$BOX ${DEF_O6DELAY_T}"
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl ADD o6tracer-t IP6TTLIST "$BOX ${DEF_O6TRACER_T}"
				${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o6tracer-t $BOX "$BOX ${DEF_O6TRACER_T}"

				for excluded_box in $IGNORELIST; do
					${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o6delay-t $BOX "$excluded_box ${DEF_O6DELAY_T}"
					${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o6tracer-t $BOX "$excluded_box ${DEF_O6TRACER_T}"
					${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o6delay-t $excluded_box "$BOX ${DEF_O6DELAY_T}"
					${EXEC} /ncc/ttpro/bin/ttregconfig.pl DEL o6tracer-t $excluded_box "$BOX ${DEF_O6TRACER_T}"
				done
			fi
		fi
	else
		echo "WARNING: No IPv6 address found in ttreg for $BOX, ignoring"
	fi
fi

unlock
