#!/usr/local/bin/perl
#
# $Id: ttregconfig.pl,v 1.4 2006/11/04 14:40:49 ruben Exp $
#
#
# All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of the author not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

#------------------------------------------------------------------------------
# Module Header
# Filename          : ttregconfig
# Purpose           : edit ttreg database
# Author            : Maximo Alves
# Date              : 10 February 2002
# Description       : 
# Language Version  : Perl 5
# OSs Tested        : Linux
# Command Line      : described on &Print_Usage()

# Input Files       : filename.txt(optional)
# Output Files      : 
# External Programs : perl modules regread & regwrite.pm,
#                    
# Problems          :
#                    
# To Do             : 
#                     
# Comments          :
#                     
#------------------------------------------------------------------------------

use lib "/ncc/ttpro/ttreg";
use strict;
use regread;
use regwrite;
 umask(0);
# --------------- Variable declaration ------------------------------

my $command = "";
my $attribute_name = "";
my @targettable = ();
my @attributevalue = ();
my $DEBUG = 1;
my $VERBOSE = 0;
my $ttopschanges = 0;
my $changesbuffer = "";
my $TTCONFIG = "/ncc/ttpro/bin/ttconfig -v ";
my %attributenames = ('o-delay-t',4,'o6delay-t',4,'u-delay-t',4,'u6delay-t',4,'r-delay-t',4,'r6delay-t',4,'o-tracer-t',2,'o6tracer-t',2,'u-tracer-t',2,'u6tracer-t',2,'r-tracer-t',2,'r6tracer-t',2);
# --------------- Start Point -------------------------------------

&main;


# --------------------- Sub routines --------------------------------



sub Parse_Command_Line() {

    my $parameter = "";
    my $paranumber = 0;

  
    $parameter = shift(@ARGV);
    if($parameter){
	if($parameter eq "-c"){
	    $ttopschanges = ">>/ncc/ttpro/data/config/ttops.changes.txt";
	    $parameter = shift(@ARGV);
	}elsif($parameter eq "-v"){
	    $VERBOSE = 1;
	    $parameter = shift(@ARGV);
	}elsif(!(($parameter eq "ADD") || ($parameter eq "DEL") || ($parameter eq "MOD"))){
	    print "Wrong sintaxe: Expected command not found on command line \n";
	    &Print_Usage();
	}	
    }
 
    $command = $parameter;
    $paranumber = scalar(@ARGV);
    if(($command eq "ADD") && ($paranumber < 3)){
	&Print_Usage();
    }elsif(($command eq "DEL") && ($paranumber < 2)){
	&Print_Usage();
    }elsif (($command eq "MOD") && ($paranumber < 3)){
	&Print_Usage();
    }
    $attribute_name = $ARGV[0];
    @targettable = Compose_Targettable($ARGV[1]);
    @attributevalue = Obtain_Attributevalue($ARGV[2]);
    print "** goal:  $command\n" if($VERBOSE);
    print "** attribute_name: $attribute_name\n" if($VERBOSE);
    print "** targettable: @targettable\n" if($VERBOSE);
    print "** attributevalue:  @attributevalue\n" if($VERBOSE);
}# Parse_Command_Line ends here !!!

sub Print_Usage() {

    print "Command Line Sintaxe: ttregconfig [-c] [-v] command attribute_name target attribute_value \n";
    print "[-v] -> verbose mode \n\n";
    print "[-c] -> write ttreg changes on /ncc/ttpro/data/config/ttops.changes.txt \n";
    print "command ->  ADD: add new value(s) to attribute_name \n";
    print "            DEL: delete value(s) from  attribute_name \n";
    print "            MOD: modify value(s) in  attribute_name \n\n";
    print "attribute_name -> attribute name as defined in the table structure \n\n";
    print "target -> CHAIN: command  affect all OFF, SETUP, WATCH and ON  TB's \n";
    print "          TTLIST: command affect all WATCH and ON  TB's \n";
    print "          filename.txt: command affect all text boxes which names are listed on filename.txt \n";
    print "          ttxy: command affect only text box ttxy \n\n";
    print "attribute_value -> new value to be used by the ADD|MOD|DEL commands \n\n";
    print "Examples: \n";
    print "ttregconfig ADD monitor CHAIN 193.0.0.0/22 \n";
    print "ttregconfig DEL dnsserv tt23 193.23.45.101 \n";
    print "ttregconfig MOD geoloc tt04 52.2237 +/- 0.0000,  4.5327 +/- 0.0000 \n";

    exit;
}# Print_Usage ends here !!!

sub Compose_Targettable {
my @aux = ();
my $FP = "";
my $str = "";
$_ =  $_[0];

    if(/CHAIN/){
	print "address all OFF, SETUP, WATCH and ON  TB's \n" if($DEBUG);
	$str = $TTCONFIG."CHAIN";
	@aux = `$str`;
    }elsif(/TTLIST/){
	print "address all WATCH and ON TB's \n" if($DEBUG);
	if (/IP6TTLIST/) { $str = $TTCONFIG."IP6TTLIST CURRENT"; }
		else { $str = $TTCONFIG."TTLIST"; }
	@aux = `$str`;
#	while(<@aux>){
	#    @aux =  (@aux, $_);
#	}
    }elsif(/$\.txt/){
	print "address TB's listed in $_[0] \n" if($DEBUG);
        unless(open(FP, $_[0])){
	    print "ERROR: Could not open $_[0] \n";
	    exit;
	}
	while(<FP>){
	    if(/tt../){
                chop($_);
		@aux =  (@aux, $_);
	    }elsif(/\n/){
		last;
	    }
	    else{
		print "ERROR: $_[0] should contain:\n ttxy \n ttzw \n etc... \n";
		exit;
	    }
	}
    }elsif(/tt../){
	print "address $_[0] \n" if($DEBUG);
	@aux = $_[0];
    }elsif(/^Group./){
	print "address all TB's from $_[0] \n" if($DEBUG);

    }else {
	&Print_Usage;
    }
return @aux;
}# Compose_Targettable ends here !!!

sub Obtain_Attributevalue {
    my $i=0;
    my @aux = ();

    for($i = 2; $i <= $#ARGV; $i++){
	@aux = (@aux, $ARGV[$i]."\n");
    }
    return @aux;
}# Obtain_Targetvalue ends here !!!

sub Add(){
    my $ret = "";
    my @auxattributevalue = ();
    my $targettable = "";
    my $arg = "";
    my $p = "";
    my $changevalue = $_[0];
    my $auxattributevalue = "";
    my $auxstr = "";

    print "----- Adding new values ----- \n" if($VERBOSE);


    foreach $targettable (@targettable){
	if($targettable =~/\n$/){
	    chop($targettable);
	}
	print "--- Adding values for $targettable ---\n" if($VERBOSE);
	my($arg) = findreg($targettable);
        $p = readreg($arg); 
	@auxattributevalue = (@attributevalue, $p->{$attribute_name});
	foreach $auxstr (@auxattributevalue){
	    $auxattributevalue = $auxattributevalue . $auxstr;
	}
	$ret = changereg($arg,2,$changevalue,$attribute_name,$auxattributevalue);
	print "changereg return: $ret \n" if($VERBOSE);
	if($ret){
	    print "ERROR: Fail to write $attribute_name = $auxattributevalue on $targettable \n";
        }else{
	    foreach my $aux (@attributevalue){
		$changesbuffer = $changesbuffer . "ADD : $targettable : $attribute_name : $aux" if($ttopschanges);
	    }
        } 
	print "\n--- End deleting values for $targettable ---\n" if($VERBOSE);
	$auxattributevalue = "";
    }

}#Add ends here !!!

sub Del(){
    my $ret = "";
    my @auxattributevalue = ();
    my $targettable = "";
    my $arg = "";
    my $p = "";
    my $auxattributevalue = "";
    my $changevalue = $_[0];
    my @item = ();
    my $i=0;
    my $j=0;
    my $attributevalue = "";
    my $delflag = 0;

    print "----- Deleting  values ----- \n" if($VERBOSE);
    foreach $targettable (@targettable){
	if($targettable =~/\n$/){
	    chop($targettable);
	}
	print "--- Deleting values for $targettable ---\n" if($VERBOSE);
	my($arg) = findreg($targettable);
        $attributevalue = @attributevalue[0];
	if(scalar(@attributevalue) < 1){ #delete non indexable attribute value
	    print "-- simple attribute -- \n" if($VERBOSE);
	    $ret = changereg($arg,2,$changevalue,$attribute_name,$auxattributevalue);	
	}else{ #delete on composed attribute value	 
	    print "-- composed attribute -- \n" if($VERBOSE);
	    $p = readreg($arg);
	    @auxattributevalue = split(/\n/,$p->{$attribute_name});	
	    for($i = 0; $i <= scalar(@auxattributevalue); $i++){	    
		@item = split(/\s/,$auxattributevalue[$i]);              
		my @parameters = split(/\s/,$attributevalue);
		my $num = $attributenames{$attribute_name};
		my $counter = 0;
		for($j = 0; $j <= $num-1; $j++){
#		    if($parameters[$j]=~/$item[$j]/){
		    if($parameters[$j] eq $item[$j]){
			$counter++;			
		    }
		}
		if($counter == $num){    		
                    print "\n delete record: $auxattributevalue[$i] \n" if($VERBOSE);
		    $delflag = 1;
		}else{
		    $auxattributevalue = $auxattributevalue .$auxattributevalue[$i] ."\n";
		    print "+" if($VERBOSE);
		}		
	    }	
	    if($attributevalue=~/delete_all/) {
		$delflag = 1;
		$auxattributevalue = "";
	    }
	    if($delflag){
	        $ret = changereg($arg,2,$changevalue,$attribute_name,$auxattributevalue);	
	        $changesbuffer = $changesbuffer . "DEL : $targettable : $attribute_name : $attributevalue" if($ttopschanges);
    	    }else{
		print "WARNING: No match for DEL $targettable : $attribute_name : $attributevalue \n";
	    } 
	    $auxattributevalue = "";
	    $delflag = 0;
	}
	print "changereg return: $ret \n" if($VERBOSE);	
	if($ret){
	    print "ERROR: Fail to write $attribute_name = $auxattributevalue on $targettable \n";
	}
	print "\n--- End deleting values for $targettable ---\n" if($VERBOSE);
  }
}#Del ends here !!!

sub Mod(){
    my $ret = "";
    my @auxattributevalue = ();
    my $targettable = "";
    my $arg = "";
    my $p = "";
    my $auxattributevalue = "";
    my $changevalue = $_[0];
    my @item = ();
    my @auxitem = ();
    my $attributevalue = "";
    my $auxstr = "";

    print "----- Modifying  values ----- \n" if($VERBOSE);

    foreach $targettable (@targettable){
	if($targettable =~/\n$/){
	    chop($targettable);
	}
	print "--- Modifying values for $targettable ---\n" if($VERBOSE);
	my($arg) = findreg($targettable);	
	$attributevalue = @attributevalue[0];
	@item = split(/\s/,$attributevalue);
	if(!($attribute_name=~/-t$/)){#all indexable att. value must end with -t.
            #non indexable attribute value
	    print ".. simple attribute .. \n" if($VERBOSE);
	    $ret = changereg($arg,2,$changevalue,$attribute_name,$attributevalue);
	}else{ #indexable attribute value
	    print "-- composed attribute -- \n" if($VERBOSE);
	    $p = readreg($arg);
	    @auxattributevalue =  split(/\n/,$p->{$attribute_name});		
	    foreach $auxstr (@auxattributevalue){ 	    
                @auxitem = split(/\s/,$auxstr);
	        if($auxitem[0] =~ /$item[0]/){#that's what we want change 
		    $auxattributevalue = $auxattributevalue .  $attributevalue  . "\n";
		    print "\nChanging to:  $attributevalue\n" if($VERBOSE);
		}else{
		    $auxattributevalue = $auxattributevalue .  $auxstr          . "\n";
		    print "+" if($VERBOSE);
		}
	    }	
	    print "\n" if($VERBOSE);	
	    $auxattributevalue = "";	      
	    $ret = changereg($arg,2,$changevalue,$attribute_name,$auxattributevalue);
	    $changesbuffer = $changesbuffer ."MOD : $targettable : $attribute_name : $attributevalue" if($ttopschanges);
	}
	print "changereg return: $ret \n" if($VERBOSE);
	if($ret){
	    print "ERROR: Fail to write $attribute_name = $auxattributevalue on $targettable \n";
	}	   	
	print "\n --- End modifying values for $targettable ---\n" if($VERBOSE);
    }
}#Mod ends here !!!

sub main() {

my $changevalue = sub {

#$_[0] is the hash pointer
#$_[1] is the attribute name
#$_[2] is the new attribute value

  my ($regp) = $_[0];
  my @keys = @tags;
  my $flag = 1;

  print "-- Changevalue -- \n" if($VERBOSE);
  while(@keys){
      my $aux =   pop(@keys);
      my $value = $regp->{$aux};
      if($aux eq $_[1]){
        $regp->{$aux} = $_[2];
        print " \n $aux = $_[2] \n" if($VERBOSE);
	$flag = 0;
      }else {
        $regp->{$aux} = $value;
        print "."  if($VERBOSE);
      }   
  } 
  print "\n"  if($VERBOSE);
  print " WARNING Attribute name: $_[1] not found on table structure \n" if($flag);
  return  $regp;

};

  &Parse_Command_Line; 
  $_ = $command;
  if(/ADD/){
      &Add($changevalue);
  }elsif(/DEL/){
      &Del($changevalue);
  }elsif(/MOD/){
      &Mod($changevalue);
  }else{
      &Print_Usage;
  }
  if($ttopschanges){
      unless(open(CHANGESFP,$ttopschanges)){
          print "ERROR: Could not open $ttopschanges \n";
	  die;	  
       }
      print CHANGESFP $changesbuffer;
      close(CHANGESFP);
  }
}#main ends here!!!


































