# $Id: Makefile.inc,v 1.4 2007/02/09 13:49:11 ruben Exp $

TTROOT		?= /ncc/ttpro
DESTDIR?=	${TTROOT}

TODAY!=	date +%Y%m%d
VERSION?=	${TODAY}

# Default for RELENG_2
SHLIB_MAJOR	?=	2

INCLUDEDIR	?=	/include

LOCALBASE	?= /usr/local
ROOTSYS		?= ${LOCALBASE}

BACKUPSUFFIX?=old

ROOTCFLAGS	!= env ROOTSYS=${LOCALBASE} ${LOCALBASE}/bin/root-config --cflags
ROOTLIBS	!= env ROOTSYS=${LOCALBASE} ${LOCALBASE}/bin/root-config --libs
ROOTGLIBS	!= env ROOTSYS=${LOCALBASE} ${LOCALBASE}/bin/root-config --glibs
ROOTVERSION != env ROOTSYS=${LOCALBASE} root-config --version | sed -e 's/\//_/'

MYSQLCFLAGS	!= ${LOCALBASE}/bin/mysql_config --cflags
MYSQLLDFLAGS	!= ${LOCALBASE}/bin/mysql_config --libs

GLIB20CFLAGS	!= ${LOCALBASE}/bin/pkg-config --cflags glib-2.0
GLIB20LDFLAGS	!= ${LOCALBASE}/bin/pkg-config --libs glib-2.0

UNAME_S		!= uname -s

BINDIR		?= /bin/${UNAME_S}/root-$(ROOTVERSION)
LIBDIR		?= /lib/${UNAME_S}/root-$(ROOTVERSION)
SHLIBDIR	?= ${LIBDIR}
SCRIPTSDIR	?= /bin

NO_OBJ=		YES
BINOWN?=	ttraffic
BINGRP?=	test-traffic



CTAGS?=	ctags

# Level of compiler strictness, 0 to 6. 
# see /usr/share/mk/bsd.sys.mk 
# Don't enable -Werr by default (yet)
# Don't enable on pre FreeBSD4


WARNS?=5

.if defined(DEBUG)
CFLAGS+=	-g -DDEBUG
.endif

NO_WERROR?=	YES


.if defined(PROG) || defined(PROG_CXX) || defined(LIB) || defined(SCRIPTS)

# Link C/C++ binaries to the ttmwrapper script
.if defined(PROG_CXX)
SYMLINKS+= ttmwrapper ${SCRIPTSDIR}/${PROG_CXX}
.endif

.if defined(PROG)
SYMLINKS+= ttmwrapper ${SCRIPTSDIR}/${PROG}
.endif


# Create installation directory if it doesn't exist yet

beforeinstall:
	@test -d ${DESTDIR}${BINDIR} || ${ECHO} "===> Creating ${DESTDIR}${BINDIR}" && mkdir -p ${DESTDIR}${BINDIR} 
.if defined(BACKUPSUFFIX)
	@-test -f ${DESTDIR}${BINDIR}/${PROG} && ${ECHO} "===> backing up previous version to ${DESTDIR}${BINDIR}/${PROG}.${BACKUPSUFFIX}" && \
		ln -f ${DESTDIR}${BINDIR}/${PROG} ${DESTDIR}${BINDIR}/${PROG}.${BACKUPSUFFIX}
.endif
.if defined(LIB)
	@test -d ${DESTDIR}${SHLIBDIR} || ${ECHO} "===> Creating ${DESTDIR}${SHLIBDIR}" && mkdir -p ${DESTDIR}${SHLIBDIR} 
.if defined(BACKUPSUFFIX)
	@-test -f ${DESTDIR}${SHLIBDIR}/lib${LIB}.a && ${ECHO} "===> backing up previous version to ${DESTDIR}${SHLIBDIR}/lib${LIB}.a.${BACKUPSUFFIX}" && \
		ln -f ${DESTDIR}${SHLIBDIR}/lib${LIB}.a ${DESTDIR}${SHLIBDIR}/lib${LIB}.a.${BACKUPSUFFIX}
	@-test -f ${DESTDIR}${SHLIBDIR}/${SHLIB_NAME} && ${ECHO} "===> backing up previous version to ${DESTDIR}${SHLIBDIR}/${SHLIB_NAME}.${BACKUPSUFFIX}" && \
		ln -f ${DESTDIR}${SHLIBDIR}/${SHLIB_NAME} ${DESTDIR}${SHLIBDIR}/${SHLIB_NAME}.${BACKUPSUFFIX}
.endif
.endif
.if defined(SCRIPTS)
	@test -d ${DESTDIR}${SCRIPTSDIR} || ${ECHO} "===> Creating ${DESTDIR}${SCRIPTSDIR}" && mkdir -p ${DESTDIR}${SCRIPTSDIR} 
.if defined(BACKUPSUFFIX)
.for __script in ${SCRIPTS}
	@-test -f ${DESTDIR}${SCRIPTSDIR}/${__script} && ${ECHO} "===> backing up previous version to ${DESTDIR}${SCRIPTSDIR}/${__script}.${BACKUPSUFFIX}" &&  \
		ln -f ${DESTDIR}${SCRIPTSDIR}/${__script} ${DESTDIR}${SCRIPTSDIR}/${__script}.${BACKUPSUFFIX}
.endfor
.endif
.endif

.endif

release:
	@${ECHO} "bumping version to ${TODAY}"
	@${ECHO} "VERSION?=${TODAY}" > ${.CURDIR}/version.mk
	@${ECHO} "Please run \"cvs tag -R REL_${TODAY} .\" after \"cvs commit\""

tag: ${SRCS} ${SCRIPTS}
	@${ECHO} "Tagging release to REL_${VERSION}"
	@cvs tag -FR REL_${VERSION} . ${.ALLSRC}

commit: ${SRCS} ${SCRIPTS}
	@${ECHO} "commiting"
	@cvs commit . ${.ALLSRC}
