/*
Copyright (c) 2000                      RIPE NCC


All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of the author not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

/*
-------------------------------------------------------------------------------
Module Header
Filename          : NhopPlot.h
Author            : Maximo Alves
Date              : 23-AGO-2002
Description       : TTM NhopPlot class implementation
Language Version  : C++
OSs Tested        : 

-------------------------------------------------------------------------------
*/

#include <unistd.h>
#include <stdlib.h>
#include <iostream.h>
#include <fstream>
#include "TH1.h"
#include "TFile.h"
#include "TCanvas.h"
#include "TStyle.h"

#include "NhopPlot.h"

static char const rcsid[] = "$Id: NhopPlot.C,v 1.7 2003/06/20 12:42:41 maximo Exp $";                                                                                       
static char const *rcsid_p = rcsid; 


Int_t NhopPlot::fill(Int_t value){

    Double_t aux = (value - 0.0001);
    if(_serialize){
      Int_t ret = summaryhist->Fill(aux);  
    }

    Int_t mybin = hist1->Fill(aux); 
  
    return(mybin);

}

void NhopPlot:: draw(char *filename){


  if(_plothistogram){
    char command[300];
    memset(command,'\0',300);
    sprintf(command, "/ncc/ttpro/bin/ps2gif -b 41 28 574 750.75 -r -s 1.2 %s", filename);
    hist1->SetStats(kFALSE);
    hist1->SetFillColor(4);
    hist1->SetMarkerColor(2);
    hist1->SetXTitle("NHops");
    hist1->SetYTitle("Occurrences");
    hist1->Draw();
    canvas->Print(filename, "ps");

    if (system(command) != 0) {
      cerr << "ERROR converting to gif: " <<
	command << endl;
    } else {
      // remove PS file, don't clutter disk space
      unlink(filename);
    }


  }

  _mean = hist1->GetMean();
  _rms  = hist1->GetRMS();
  //  _max  = hist1->GetMaximumBin(); returns bin with maximal number of points
  //_max = hist1->GetMaximum(); 
  _max = hist1->GetMaximumStored();
  _min  = hist1->GetMinimumBin();

  delete hist1;
}

Int_t NhopPlot:: conv(char *filename){


  return(0);
}

  

void NhopPlot::drawSummary(char *filename){

  if(_plothistogram){
    char command[300];
    memset(command,'\0',300);
    sprintf(command, "ps2gif -b 41 28 574 750.75 -r -s 1.2 %s", filename);
    summaryhist->SetStats(kFALSE);
    summaryhist->SetFillColor(4);
    summaryhist->SetMarkerColor(2);
    summaryhist->SetXTitle("NHops");
    summaryhist->SetYTitle("Occurrences");
    summaryhist->Draw();
    canvas->Print(filename, "ps");
    if (system(command) != 0) {
      cerr << "ERROR converting to gif: " <<
	command << endl;
    } else {
      // remove PS file, don't clutter disk space
      unlink(filename);
      
    }

  }

  _mean = summaryhist->GetMean();
  _rms  = summaryhist->GetRMS();
  _max  = summaryhist->GetMaximumBin();
  _min  = summaryhist->GetMinimumBin();

  delete summaryhist;
  tempfile->Close();
}

Int_t NhopPlot::saveSummary(){
    if(!(_summaryfileflag)){
        tempfile = new TFile(_temprootfile,"UPDATE");
    }
    Int_t resp = summaryhist->Write("summaryhist", TObject::kOverwrite);
    tempfile->Close(); 
    return(resp);
}

NhopPlot::NhopPlot(char *name, char *title, Int_t nbins, Float_t xlow, Float_t xup, bool plothistogram){
//constructor without serialization
    _name  = new char[strlen(name)+1];
    _name  = strcpy(_name,name);
    _title = new char[strlen(title)+1];
    _title = strcpy(_title,title); 
    _nbins = nbins;
    _xlow = xlow;
    _xup = xup;
    _plothistogram = plothistogram;
    hist1 = new TH1F(_name, _title, _nbins, _xlow, _xup);
    canvas = new TCanvas();  
    setSerialize(false);
}


NhopPlot::NhopPlot(char *name, char *title, Int_t nbins, Float_t xlow, Float_t xup, char *temprootfile, bool plothistogram){
//constructor with serialization
    _name  = new char[strlen(name)+1];
    _name  = strcpy(_name,name);
    _title = new char[strlen(title)+1];
    _title = strcpy(_title,title); 
    _nbins = nbins;
    _xlow = xlow;
    _xup = xup;
    _plothistogram = plothistogram;
    _temprootfile = new char[strlen(temprootfile+1)];
    _temprootfile = strcpy(_temprootfile, temprootfile);
    canvas = new TCanvas();
    //canvas = new TCanvas("canvas", "Plots", 0, 0, 1270, 990);
    hist1 = new TH1F(_name, _title, _nbins, _xlow, _xup); 
    hist1->SetFillColor(2);
 
    //test whether temprootfile has data or is new
    ifstream tempf(temprootfile);
    if(! tempf){  
        //file does not exist yet
        //must create a new summary histogram 
        summaryhist = new TH1F("summary", "Summary plot", _nbins, _xlow, _xup);
        setSummaryfileflag(false);
    }else{       
        //file does exist 
        //must read existent summary histogram from hd to memory
        tempfile = new TFile(temprootfile,"UPDATE");
        setSummaryfileflag(true);
        summaryhist =  (TH1F*) tempfile->Get("summaryhist");      
        tempf.close(); 
    }    
  
    setSerialize(true);

}


NhopPlot::~NhopPlot(){
    delete canvas;

}





