/*
Copyright (c) 2000                      RIPE NCC

$Id: NhopPlot.h,v 1.2 2006/11/04 14:40:49 ruben Exp $

All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of the author not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

/*
-------------------------------------------------------------------------------
Module Header
Filename          : NhopPlot.h
Author            : Maximo Alves
Date              : 23-AGO-2002
Description       : TTM NhopPlot class definition
Language Version  : C++
OSs Tested        : 

The NhopPlot class:

- fill ttxx histograms
- draw ttxx histograms
- convert histogram to gif
- fill in vector with Tracestat class
- draw summary histograms


-------------------------------------------------------------------------------
*/

#include <TH1.h>
#include "TCanvas.h"

class NhopPlot {
    private:
       TH1F *hist1;
       TH1F *summaryhist; 
       TCanvas *canvas;
       TFile *tempfile;

       bool _serialize;
       bool _summaryfileflag; //true it has been open
                              //false it wasn't opened yet
       bool _plothistogram;

       char *_name;
       char *_title;
       Int_t _nbins;
       Float_t _xlow;
       Float_t _xup;
       char *_temprootfile;
       Stat_t _mean;
       Stat_t _rms;
       Double_t _max;
       Double_t _min;

    public:      
        Int_t fill(Int_t value);
        void draw(char *filename);
        Int_t conv(char *filename);
        void drawSummary(char *filename);
        Int_t saveSummary();
        void setName(char *name){strcpy(_name,name);}
        void setTitle(char *title){strcpy(_title,title);}
        void setNbins(Int_t nbins){_nbins = nbins;}
        void setXlow(Int_t xlow){_xlow = xlow;}
        void setXUp(Int_t xup){_xup = xup;}
        void setSerialize(bool serialize){_serialize = serialize;}
        void setSummaryfileflag(bool summaryfileflag){_summaryfileflag = summaryfileflag;}
        Stat_t getMean(){return(_mean);}
        Stat_t getRMS() { return(_rms);}
        Double_t getMax(){return(_max);}
        Double_t getMin(){ return(_min);}
        NhopPlot( char *name, char *title, Int_t nbins, Float_t xlow, Float_t xup, bool plothistogram);
        NhopPlot( char *name, char *title, Int_t nbins, Float_t xlow, Float_t xup, char *temprootfile, bool plothistogram);
        ~NhopPlot();
};

 
