/*
Copyright (c) 2000                      RIPE NCC


All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of the author not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

/*
-------------------------------------------------------------------------------
Module Header
Filename          : main.C
Author            : Maximo Alves
Date              : 08-Oct-2002
Revised for Linux : 
Revised 	  : 
Description       : Main program for creating AS traceroute statistics
Language Version  : C++
OSs Tested        : Debian Linux 2.2
Command line 
		    
-------------------------------------------------------------------------------
*/



#include "TROOT.h"
#include "TApplication.h"
#include "TCanvas.h"
#include "TChain.h"
#include "TTree.h"
#include "TFile.h"
#include "TBranch.h"
#include "TH1.h"
#include "TH2.h"
#include "TError.h"
#include "Delay.h"
#include <string.h>
#include <stdlib.h>
#include <fstream>
#include <stdio.h>
#include <iostream.h>

#include "NhopPlot.h"
#include "RouteStat.h"

static char const rcsid[] = "$Id: astracestat.C,v 1.24 2004/06/15 11:20:43 ruben Exp $";                                                                                       
static char const *rcsid_p = rcsid; 

// GLOBAL VARIABLE DECLARATION 

bool verbose = false;

bool plothistogram = true; // if false don't generate any plot
                           // if true generate plots normallt
                           // option used for backward calculation of RouteStat data
char *summaryfilename; //name for output root file(e.g. file with RouteStat data)
char *plottitle; //name used for any plot title
char *plotdir; //name used for the output plot file name 
char *filename; //name used for input text file listing number of AS hops for each TB 
char *temprootfilename; //name for temporary file which stores data for summary plot generation

Int_t maximal; //holds maximal bin value of a test box histogram
Int_t bigofall; //holds maximal bin value of all histograms

NhopPlot *myNhopPlot;
NhopPlot *sumNhopPlot;
RouteStat *routestat;
TFile *hfile;
TTree *tree;

// FUNCTION PROTOTYPES

void usage();

int main (int argc, char *argv[]) {


  //initialise strings

char *str1 = new char[2];
memset(str1, '\0', 2);
strcat(str1, "@");

char *str2 = new char[2];
memset(str2, '\0', 2);
strcat(str2, " ");

char *str3 = new char[2];
memset(str3, '\0', 2);
strcat(str3, ":");

  // Let ROOT be silent
  gErrorIgnoreLevel=1;

  //parse command line
  for (int i=1; i < argc; i++){
    char *pchar = argv[i];
    switch ( pchar[0] ){
    case '-':{
      switch ( pchar[1] ){
      case 'v':
	verbose = true;
	break;
      case 'd':
	gErrorIgnoreLevel=0;
	verbose = true;
	break;
      case 'H':
	//i++;
	plothistogram = false;
	break;
      case 'f':
	i++;
	filename = new char[strlen(argv[i])+1];
        memset(filename, '\0', strlen(filename)); 
	filename = strcpy(filename, argv[i]);                        
	break;
      case 's':
	i++;
	summaryfilename =  new char[strlen(argv[i])+1];
	memset(summaryfilename, '\0', strlen(summaryfilename));
	summaryfilename = strcpy(summaryfilename, argv[i]);
	break;
      case 'c':
	i++;
	plottitle = new char[strlen(argv[i])+1];
	memset(plottitle, '\0', strlen(plottitle));
	plottitle = strcpy(plottitle, argv[i]);                        
	break;
      case 'b':
	i++;
	temprootfilename = new char[strlen(argv[i])+1];
	memset(temprootfilename, '\0', strlen(temprootfilename));
	temprootfilename = strcpy(temprootfilename, argv[i]);                        
	break;
      case 'e':
	i++;
	plotdir = new char[strlen(argv[i])+1];
	memset(plotdir, '\0', strlen(plotdir));
	plotdir = strcpy(plotdir, argv[i]);                        
	break;
      case 'h':
	usage();
	return(0);
      }
    }
  
    }//switch ( pchar[0]
  }//for
 

  //ROOT initialisation
  TROOT simple ("TTM", "Plots");
  TApplication app("tracestats", NULL, NULL, NULL, 0);
  gROOT->SetBatch();

  //open file
  ifstream in; 
 

  char *timestamp = new char[500];
  memset(timestamp, '\0', 500);

  in.open(filename, ofstream::in);
  if(in.is_open()) //Then opening the file was succesful
    {
      bool savedflag = false;
      if(verbose)  cout << "\nThe specified file was opened succesfully: " << filename <<endl;
      char *line;
      line = new char[1000]; //with a TB number of around 50 it's more than enough
      memset(line, '\0', 1000);
 
      //1st line format:
      //@timestamp
      //format or the remaining lines
      //tt03.ripe.net: 1 1 1 5 4 5 3 3 4 4 4 5 4 6 4 5 5 4 5 5 4 3 5 5 7 4 3 5 6 
      //test box name: numberofAShops  numberofAShops  numberofAShops ...
      bool previousseq = false;
      while (true){       
	Int_t linesize;

	char *tbname = new char[300];
	char *asnum = new char[50];
	memset(tbname, '\0', 300);
	memset(asnum, '\0', 50);

	//get line
	in.getline(line,1000);
	if(in.eof())break;
	if(verbose) cout << "line to process: " << line << endl;
	if(!(strcspn(line, str1) == strlen(line))){
          //start a day sequence
          if(previousseq){
	    //existing previous sequence
	    if(verbose) cout << "existing previous sequence " << endl;
            //must plot summary histogram 
	    char *auxname = new char[1500];
	    memset(auxname, '\0', 1500);
	    strcat(auxname, plotdir);
	    strcat(auxname, "as.summary.nhop");      
	    myNhopPlot = new NhopPlot("IPhopplotsummary", plottitle , 30, -0.5, 29.5, temprootfilename, plothistogram);
	    myNhopPlot->drawSummary(auxname);

            // and get routestat info from it
	    routestat->setId(0);//summary id = 0
	    routestat->setMean( myNhopPlot->getMean());
	    routestat->setRms(myNhopPlot->getRMS());
	    routestat->setTimestamp( atoi(timestamp));
	    routestat->setMax(bigofall);
	    routestat->setMin( myNhopPlot->getMin());
	    int retval = tree->Fill ();
	    if(verbose)	cout <<  "Summary Fill returns: " << retval << endl;
	    //if(verbose) cout << "Dir: " << hfile->pwd() << endl;
	    //and delete it
	    delete myNhopPlot;
            bigofall = 0;
	    //then must write and close previous root file
	    hfile->Write ();
	    hfile->Close ();
	    delete tree;
	    delete hfile;
            delete auxname;
	    previousseq = false;
            savedflag = true;
	  }//if(previousseq){

          //create output root file name
          //it's stored on /ncc/ttpro/data/root/yyyy/mm/dd/routestat.as.root
	  char *auxpath = new char[1500];
	  memset(auxpath, '\0', 1500);
	  int pos = strcspn(line, str2);
   	  int pos2 = strcspn(line, str3);
	  strncat(auxpath, &line[pos+1], pos2-2);
	  strncat(timestamp, &line[pos2 + 1] , strlen(line)-2);
          strcat(auxpath, summaryfilename);
	  if(verbose) cout << "Root file: " << auxpath << endl;
       	  hfile = new TFile (auxpath , "RECREATE");
	  delete auxpath;
	  if(verbose) cout << "created TFile file " << endl;

	  previousseq = true;

          //create root tree and tie it to RouteStat
	  tree  = new TTree ("nhop", "Example");
	  if(verbose) cout << "created TTree file " << endl;
	  routestat = new RouteStat();
	  tree->Branch("routestat", "RouteStat", &routestat, 64000, 1);
	  if(verbose) cout << "set branch " << endl; 
         continue;
	}
	//extract tb name from line
	char *auxtitle = new char[1500];
	memset(auxtitle, '\0', 1500);
	linesize = strlen(line);        
	int pos = strcspn(line, str3);     
	strncpy(tbname, line, pos);
	strcat(auxtitle, plottitle);
        strcat(auxtitle, tbname);
        int id = atoi(tbname);
        //start fill histogram loop
	myNhopPlot = new NhopPlot("AShopplot", auxtitle , 30, -0.5, 29.5, temprootfilename,  plothistogram);

        delete auxtitle;
        if(verbose) cout << "Starting to fill histogram: ";
        for (int i=pos; i <= linesize ; i++){  
	  if(line[i] == ' '){
	    int as = atoi(asnum);
	    if(!(as == 0)){
	      //fill Hist
              if(as > maximal){
		maximal = as;
	      }         
	      if(as > bigofall){
		bigofall = as;
	      }
              Int_t ret = myNhopPlot->fill(as);
	      if((ret < 0) || (ret > 30)){
		//		cout << "WARNING: Out of range Fill. Ret value:  " << ret << "  TB: " << id << " Fill value: " << as << endl;
	      }
	      if(verbose)  cout << as ;	
       	      as = 0;
	    }
	    memset(asnum, '\0', 50);
	    continue;
	  }else{
	    strncat(asnum, &line[i], 1);
	  }
	} // for (int i=pos;
        if(verbose)  cout << endl ;


	Int_t resp = myNhopPlot->saveSummary();
	if(resp == 0){
	  cout << "WARNING: 0 bytes were written by saveSummary.";
	  cout << " Processing for:  "<< id  << endl;
	}

   	char *auxname = new char[1500];
	memset(auxname, '\0', 1500);
        strcat(auxname, plotdir);
        strcat(auxname, "as.nhop.");
        strcat(auxname, tbname);   

	myNhopPlot->draw(auxname);
	memset(auxname, '\0', 1500);
	routestat->setId(id);
	routestat->setMean( myNhopPlot->getMean());
	routestat->setRms(myNhopPlot->getRMS());
	routestat->setTimestamp( atoi(timestamp));
	routestat->setMax(maximal);
	routestat->setMin( myNhopPlot->getMin());
	int retval = tree->Fill ();
	if(verbose)	cout <<  "TB hist Fill returns: " << retval << endl;
	maximal = 0;
	memset(tbname, '\0', 300);
	delete myNhopPlot;
        delete auxname;
        delete tbname;
        delete asnum;
      }// while (true){
      in.close(); //Close the file 
      if(!savedflag){
	
	//used in the case of only One day processing

        //must plot summary histogram 
	char *auxname = new char[1500];
	memset(auxname, '\0', 1500);
	strcat(auxname, plotdir);
	strcat(auxname, "as.summary.nhop");      

	if(verbose) cout << "Saving root file: " << auxname <<endl; 
        myNhopPlot = new NhopPlot("IPhopplotsummary", plottitle , 40, -0.5, 29.5, temprootfilename, plothistogram);    
        myNhopPlot->drawSummary(auxname);

	// and get routestat info from it
	routestat->setId(0);//summary id = 0
	routestat->setMean( myNhopPlot->getMean());
	routestat->setRms(myNhopPlot->getRMS());
	routestat->setTimestamp( atoi(timestamp));
	routestat->setMax(bigofall);
	routestat->setMin( myNhopPlot->getMin());
	int retval = tree->Fill ();
	if(verbose)	cout <<  "Summary Fill returns: " << retval << endl;

	//and delete it
       	delete myNhopPlot;
        delete auxname;

		hfile->Write ();
	//	hfile->Close ();
	//	delete hfile;
	delete tree;
      }      
 
    }else{

      cout << "ERROR: Couldnt open the specified file: " << filename << endl;
      return(1);
    }

  delete str1;
  delete str2;
  delete str3;

  return(0);
}


void usage(){
  cout <<    "*** Usage *** "  << endl;
  cout <<    "This program creates AS level histograms(or only root files) for all TBs listed in the file given with -f argument" << endl;
  cout <<    "There're 2 distinct usage types as listed above: " << endl;
  cout <<    "-v: Verbose " << endl;
  cout <<    "-d: Debugging output" << endl;
  cout <<    "-h: print out this " << endl << endl; 
 
  cout <<    "1 - Test Boxes histogram generation: "<< endl;
  cout <<    "Command Line:  "<< endl;
  cout <<    "astracestat -c 'AS plots for ' -f /ncc/ttpro/tmp/plotasnhops.out  -s routestat.as.root -e /ncc/www/publish/pub/ripencc/mem-services/ttm/Plots/nhop/  -b /ncc/ttpro/tmp/as.temp.root [-v]   "<< endl;
  cout <<    "-c: histogram title" << endl;
  cout <<    "-f: input file listing asquery results"<< endl;
  cout <<    "-s: output root file " << endl;
  cout <<    "-e output dir for generated histograms" << endl;
  cout <<    "-b: temporary root file " << endl << endl;


  cout <<    "2 - No gif file generation(only root file):" << endl;
  cout <<    "Command Line:  "<< endl;
  cout <<    "astracestat -H -c 'notitle' -f /ncc/ttpro/tmp/plotasnhops.out  -s routestat.as.root -e /ncc/www/publish/pub/ripencc/mem-services/ttm/Plots/nhop/  -b /ncc/ttpro/tmp/as.temp.root  [-v]  "<< endl;
  cout <<    "-H: set backward  calculation" << endl;

}






    
    













