
/*
Copyright (c) 2000                      RIPE NCC


All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of the author not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

/*
-------------------------------------------------------------------------------
Module Header
Filename          : main.C
Author            : Maximo Alves
Date              : 22-AGO-2002
Revised for Linux : 
Revised 	  : 
Description       : Main program for creating IP traceroute statistics
Language Version  : C++
OSs Tested        : Debian Linux 2.2
		    
-------------------------------------------------------------------------------
*/



#include "TROOT.h"
#include "TApplication.h"
#include "TCanvas.h"
#include "TChain.h"
#include "TTree.h"
#include "TFile.h"
#include "TBranch.h"
#include "TH1.h"
#include "TH2.h"
#include "TError.h"
#include "Delay.h"
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <fstream>
#include <ostream>
#include <iostream>

#include "RootFileBuffer.h"
#include "NhopPlot.h"
#include "RouteStat.h"

using namespace std;

static char const rcsid[] = "$Id: iptracestat.C,v 1.15 2007/01/15 15:02:21 ruben Exp $";                                                                                       
static char const *rcsid_p = rcsid; 

// GLOBAL VARIABLE DECLARATION 

bool verbose = false;
bool summary = false; // if false don't plot summary plot 
                      // if true plot ONLY summary plot
bool plothistogram = true; // if false don't generate any plot
                           // if true generate plots normallt
                           // option used for backward calculation of RouteStat data
Int_t timestamp;
char *filename; //name for input root file(e.g. file with delay data)
char *summaryfilename; //name for output root file(e.g. file with RouteStat data)
char *summaryfilenametxt = "/var/tmp/routestat.ip.root.txt"; //name of temporary file with RouteStat data
char *temprootfilename; //name for temporary file which stores data for summary plot generation
                        //if you want to run two instances of this program at the same time
                        //you MUST choose different names
char *temprouteidfile;  //name for output routeid filename
char *plottitle; //name used for any plot title
char *plotname; //name used for the output plot file name 
char ids[8];
Int_t id; //test bos ID
Int_t const TBNUMBER = 1000;
Double_t mystruct1[TBNUMBER][4]; // targertid - packetit - nhops
bool loopflag = true; //if true must store element on structure1
                      //if false must check its timestamp
Int_t recordposition = 0; // next free structure position to store
Int_t iposition = 0; // stores storage postion when one element has a better timestamp
Int_t str2recposition = 0;
Int_t targetId = 0;
Double_t arrivalTime = 0;
Double_t packetId = 0;
Int_t routeId = 0;
Int_t nhops = 0;     
Double_t stpacketId = 0; //stored on struct1
Int_t stnhops = 0;    //stored on struct1
Double_t newtimediff = 0;
Double_t oldtimediff = 0;
char routeidline[1000]; //taking into account 30 as maximal number of rope and 10 chars as routeid size 
                         //plus more a huge safe margin
char routestatline[300];

Int_t maximal = 0; // holds maximal nhop for the histogram

struct Route{
    Int_t id;
    Stat_t mean;
    Stat_t rms;
    Int_t timestamp;
    Int_t max;
    Double_t min;  
};

NhopPlot *myNhopPlot;
RouteStat *routestat;
RootFileBuffer *myFileBuffer; 
std::ofstream out;   //Fileobject used in writing a file
std::ifstream in;    //Fileobject used in reading from a file
Route route;

// FUNCTION PROTOTYPES

void usage();
void writeStruct();
void readStruct();

int main (int argc, char *argv[]) {


  //initialise structs

  memset(mystruct1,'\0',TBNUMBER*4);
  memset(routeidline, '\0', 1000);
  memset(ids,'\0',8);   
  memset(routestatline, '\0', 300);

  // Let ROOT be silent
  gErrorIgnoreLevel=1;

  //parse command line
  for (int i=1; i < argc; i++){
    char *pchar = argv[i];
    switch ( pchar[0] ){
    case '-':{
      switch ( pchar[1] ){
      case 'v':
	verbose = true;
	break;
      case 'd':
	gErrorIgnoreLevel=0;
	break;
      case 't':
	i++;
	timestamp = atoi(argv[i]);
	break;
      case 'f':
	i++;
	filename = new char[strlen(argv[i])+1];
	filename = strcpy(filename, argv[i]);                        
	break;
      case 'H':
	//i++;
	plothistogram = false;
	break;
      case 'o':
	i++;
	summaryfilename =  new char[strlen(argv[i])+1];
	summaryfilename = strcpy(summaryfilename, argv[i]);
	//	summaryfilenametxt =  new char[strlen(argv[i])+5];
	//	summaryfilenametxt = strcpy(summaryfilenametxt, argv[i]);
	//	strcat(summaryfilenametxt,".txt");
	break;
      case 'S':
	i++;
	summary = true;
	break;
      case 'b':
	i++;
	temprootfilename = new char[strlen(argv[i])+1];
	temprootfilename = strcpy(temprootfilename, argv[i]);                        
	break;
 
      case 'c':
	i++;
	plottitle = new char[strlen(argv[i])+1];
	plottitle = strcpy(plottitle, argv[i]);                        
	break;

      case 's':
	i++;
	id = atoi(argv[i]);
	strcpy(ids, argv[i]);
	strcat(routeidline, ids);
	strcat(routeidline, ": ");
	break;    
      case 'e':
	i++;
	plotname = new char[strlen(argv[i])+1];
	plotname = strcpy(plotname, argv[i]);                        
	break;
      case 'h':
	usage();
	return(0);
      }
    }
 
    }//switch ( pchar[0]
  }//for

  //ROOT initialisation
  TROOT simple ("TTM", "Plots");
  TApplication app("tracestats", NULL, NULL, NULL, 0);
  gROOT->SetBatch();
  if(summary){
    //after drawing all plots draw the summary plot
    myNhopPlot = new NhopPlot("IPhopplotsummary", plottitle , 30, -0.5, 29.5, temprootfilename, plothistogram);    
    myNhopPlot->drawSummary(plotname); 


  
    // write to summaryfilename.txt
    id = 0; //summary id = 0
    writeStruct();
        
    //read summaryfilename.txt and 
    //create routestat tree branch
    readStruct();
    //    delete myNhopPlot;      
  }else{
    //before the summary must draw plots for all TBs     
       
    //open root file
    myFileBuffer = new RootFileBuffer();
    Int_t ret = myFileBuffer->openFile(filename);
    if(ret == 0){
      //it has fail to open file
      return(1);
    }
    // create a Delay object for retrieving the  data
    Delay *delay = new Delay();
    myFileBuffer->setBranchAddress("delay", &delay);
        
    //loop through the data and find out for each target
    //the record with the closest timestamp 
    Stat_t nTotalEvents = myFileBuffer->getEntries();              
    for (Int_t eventNo=0; eventNo < nTotalEvents; eventNo++) {
                        
      // read next delay into memory
      if ( myFileBuffer->getEvent(eventNo) > 0) {

	packetId = delay->GetArrivalTime();                             
        if(packetId == -1){
	  //-1.0 if undefined 
	  continue;
	}
	targetId = delay->GetTargetId();
	// is targetId in struct1
	for (int i=0; i < TBNUMBER; i++){
	  if (targetId == mystruct1[i][0]){
	    //yes it's
	    loopflag = false;
	    stpacketId = mystruct1[i][1]; 
	    stnhops = mystruct1[i][2];//assignment to `Int_t' from `double' here is harmless
	    iposition = i;
	    break;
	  }                                
	}

 
	nhops =  delay->GetNhops();             
	routeId = delay->GetRouteId();
	if (loopflag){
	  //new target must record on array
	  mystruct1[recordposition][0] = delay->GetTargetId();
	  mystruct1[recordposition][1] = delay->GetArrivalTime();
	  mystruct1[recordposition][2] = nhops;
	  mystruct1[recordposition][3] = routeId;      
	  recordposition++;
	  loopflag = false;
	}else{
	  //existent target, must check whether its packetIp(time) is closer to
	  //timestamp than what is stored
	  Double_t aux1 = (packetId - timestamp);
	  Double_t aux2 = (stpacketId - timestamp);
	  newtimediff = fabs(aux1);
	  oldtimediff = fabs(aux2);
	  if(newtimediff < oldtimediff){
	    //Yep, packetId has a time value which is closer to timestamp 
	    //so that's the one to keep on struct1
	    mystruct1[iposition][1] = packetId;
	    mystruct1[iposition][2] = nhops;
	    mystruct1[iposition][3] = routeId;
	    iposition++;
	  }            
	}                                    
    
      }// if ( (T.GetEvent(eventNo)
      loopflag = true;
    }// for (int i=0; i

    delete myFileBuffer; 
    myNhopPlot = new NhopPlot("IPhopplot", plottitle , 30, -0.5, 29.5, temprootfilename, plothistogram);
    
    for (int j=0; j <= recordposition; j++){
      if(mystruct1[j][3] == 0){
	continue;
      }
      if(verbose){
	cout << "Recordposition: " << j <<endl;                 
	cout << "targetId: " << mystruct1[j][0] << endl;
	cout << "packetId: " << mystruct1[j][1] << endl;
	cout << "Nhops: " << mystruct1[j][2] << endl;
	cout << "routeId: " << mystruct1[j][3] << endl;
      }
      if(mystruct1[j][2] > maximal){
	maximal  = mystruct1[j][2]; //stores the maximal bin value,  assignment to `Int_t' from `double' here is harmless
      }
      Int_t ret = myNhopPlot->fill(mystruct1[j][2]);
      if((ret < 0) || (ret > 30)){
	//	cout << "WARNING: Out of range Fill. Ret value:  " << ret << "  TB: " << id << " Fill value: " << mystruct1[j][2] << endl;
      }

    }
    Int_t resp = myNhopPlot->saveSummary();
    if(resp == 0){
      cout << "WARNING: 0 bytes were written by saveSummary.";
      cout << " Processing for:  "<< id  << endl;
    }
    
    myNhopPlot->draw(plotname); 
    
 
    // write to summaryfilename.txt
    writeStruct();

  }//else(summary)
    
  //Destroy used classes
  delete myNhopPlot;

     
  exit(0);
}


void usage(){
  cout <<    "*** Usage *** "  << endl;
  cout <<    "This program creates nhop IP level histograms and routestat.ip.root files   " << endl;
  cout <<    "Each program call will generate the outputs mentioned above for the specified test box(or a " << endl;
  cout <<    "summary of all test boxes)" << endl;
  cout <<    "There're 3 distinct usage types as listed above: " << endl;
  cout <<    "-v: Verbose " << endl;
  cout <<    "-d: Debugging output" << endl;
  cout <<    "-h: print out this " << endl; 
 
  cout <<    "1 - One Test Box histogram generation: "<< endl;
  cout <<    "Command Line:  "<< endl;
  cout <<    "iptracestat  -d XX  -c 'IP plots for XX' -b /ncc/ttpro/tmp/ip.temp.root -t 1041415810 -f /ncc/ttpro/data/root/2002/12/01/ttXX.ripe.net.20021201.root -s ./routestat.ip.root -e ./ip.nhop.XX  [-v]"<< endl;
  cout <<    "-s: Test Box id " << endl;
  cout <<    "-c: histogram title" << endl;
  cout <<    "-b: temporary root file " << endl;
  cout <<    "-t: unix timestamp defining time point for calculation"	<< endl;
  cout <<    "-f: input root file "<< endl;
  cout <<    "-o: output root file " << endl;
  cout <<    "-e output gif file name for generated histogram" << endl  << endl;

  cout <<    "2 - Summary plot generation: "<< endl;
  cout <<    "Command Line:  "<< endl;
  cout <<    "iptracestat -c 'IP Summary for 20030108' -b /ncc/ttpro/tmp/ip.temp.root -t 1042020215 -s /ncc/ttpro/data/root/2003/01/08/routestat.ip.root -e /ncc/www/publish/pub/ripencc/mem-services/ttm/Plots/nhop/ip.summary.nhop -S [-v]"<< endl;
  cout <<    "-S: set summary histogram  generation " << endl  << endl; 

  cout <<    "3 - No gif file generation(only root file):" << endl;
  cout <<    "Command Line:  "<< endl;
  cout <<    "iptracestat -H -c notitle -d XX -b /ncc/ttpro/tmp/ip.temp.root -t 1020255010 -f /ncc/ttpro/data/root/2002/05/01/ttXX.ripe.net.20020501.root -s /ncc/ttpro/data/root/2002/05/01/routestat.ip.root -e XX.ip.nhop [-v]"<< endl;
  cout <<    "-H: set backward  calculation" << endl;

}




void writeStruct()
{
    //write values to Routestat structure
    route.id = id;
    route.mean = myNhopPlot->getMean();
    route.rms = myNhopPlot->getRMS();
    route.timestamp = timestamp;
    
    if(summary){
      Route routeaux;
      //in order to get the maximal bin value must scan through all maximal values from 
      // all processed test boxes 
      maximal = 0;
      in.open(summaryfilenametxt,ofstream::binary | ofstream::in);
      if(in.is_open()){    
	  while (true){
         	in.read((char*)&routeaux,sizeof(Route)); //read information from a file to Route
		if(in.eof())break;
		if(routeaux.max > maximal){
		  maximal = routeaux.max;
		}
	  }
          in.close(); //Close the file 
	  route.max = maximal;
      }else{
	cout << "\n\nERROR: Couldn't open the specified file: " << summaryfilenametxt << endl;
      }
    }else{
      //store the maximal value for the TB being processed
      route.max =  maximal;
    }
    route.min = myNhopPlot->getMin();    

    //Create a binary file with specified filename
    out.open(summaryfilenametxt, ofstream::binary | ofstream::out | ofstream::app );
    if(out.is_open()){ //Then we can write into the file
        out.write((char*)&route,sizeof(Route));
        out.close();
    }else{//error
        cout << "\nERROR: The struct was not written in file: " << summaryfilenametxt << endl;
    }

}                     

void readStruct()
{

  routestat = new RouteStat();  
  TFile *hfile;
  hfile = new TFile (summaryfilename , "RECREATE");
  TTree *tree  = new TTree ("nhop", "Example");
  tree->Branch("routestat", "RouteStat", &routestat, 64000, 1);
  int counter = 0;        


  in.open(summaryfilenametxt,ofstream::binary | ofstream::in);
  if(in.is_open()) //Then opening the file was succesful

    {
      cout << "\nThe specified file was opened succesfully: " << filename;
      while (true){
       
	in.read((char*)&route,sizeof(Route)); //read information from a file to Route
	if(in.eof())break;
	cout << "\n" << route.id << " " << route.mean << "  " << route.rms << " " << route.timestamp << " " << route.max << " " << route.min << endl ;
	routestat->setId(route.id);
	routestat->setMean(route.mean);
	routestat->setRms(route.rms);
	routestat->setTimestamp(route.timestamp);
	routestat->setMax(route.max);
	routestat->setMin(route.min);
	tree->Fill ();
	counter++;
      }
      in.close(); //Close the file 


      hfile->Write ();
      hfile->Close ();
      cout << "Output ROOT file has: " << counter << " entries" << endl;

    }
  else{

    cout << "\n\nERROR: Couldn't open the specified file: " << summaryfilenametxt << endl;
  }


}                     





    
    






