/*
Copyright (c) 2000                      RIPE NCC


All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of the author not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

/*
-------------------------------------------------------------------------------
Module Header
Filename          : main.C
Author            : Maximo Alves
Date              : 22-AGO-2002
Revised for Linux : 
Revised 	  : 
Description       : Main program for creating x time plots:
                    mean x time
                    rms  x time
                    max  x time
                    min  x time

Language Version  : C++
OSs Tested        : Debian Linux 2.2
Command line 
 
-------------------------------------------------------------------------------
*/



#include "TROOT.h"
#include "TApplication.h"
#include "TCanvas.h"
#include "TChain.h"
#include "TTree.h"
#include "TFile.h"
#include "TBranch.h"
#include "TH1.h"
#include "TH2.h"
#include "TError.h"
#include <TPaveLabel.h>
#include "Delay.h"
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <fstream>
#include <iostream>
#include <cstring>
#include <string>
#include "RouteStat.h"

using namespace std;

static char const rcsid[] = "$Id: xtimeplot.C,v 1.7 2007/01/15 15:03:26 ruben Exp $";                                                                                       
static char const *rcsid_p = rcsid; 

// GLOBAL VARIABLE DECLARATION 

bool verbose = false;
Int_t starttime;
Int_t endtime;
Int_t id;
string filename, 
 type,
 plottitle,
 plotdir,
 plotfile,
 command,
 strid;

// FUNCTION PROTOTYPES

void usage();


int 
main(int argc, char *argv[])
{
	//Let ROOT be silent
	gErrorIgnoreLevel = 1;

	//parse command line
	for (int i = 1; i < argc; i++) {
		char           *pchar = argv[i];
		switch (pchar[0]) {
		case '-':{
				switch (pchar[1]) {
				case 'v':
					verbose = true;
					break;
				case 't':
					i++;
					type = argv[i];
					break;
				case 'f':
					i++;
					filename = argv[i];
					break;
				case 'i':
					i++;
					strid = argv[i];
					id = atoi(argv[i]);
					break;
				case 'a':
					i++;
					starttime = atoi(argv[i]);
					break;
				case 'e':
					i++;
					endtime = atoi(argv[i]);
					break;
				case 'b':
					i++;
					plottitle = argv[i];
					break;
				case 'o':
					i++;
					plotdir = argv[i];
					if (plotdir[plotdir.length()] != '/')
						plotdir += '/';
					break;
				case 'd':
					gErrorIgnoreLevel = 0;
					break;
				case 'h':
					usage();
					return (0);
				}
			}

		} //switch (pchar[0]
	} //for

	plotfile = plotdir + type + ".6monthtrend." + strid;

	//ROOT initialisation
	
	TROOT simple("TTM", "Plots");
	TApplication	app("tracestats", NULL, NULL, NULL, 0);
	gROOT->SetBatch();

	TChain			T("nhop");

	//chain root files
	ifstream in;

	in.open(filename.c_str(), ofstream::in);
	if (in.is_open())
	{
		char           *rootfile;

		//Then opening the file was succesful
		if (verbose)
			cout << "The specified file was opened succesfully: " << filename << endl;
		rootfile = new char[500];
		memset(rootfile, '\0', 500);
		while (true) {

			in.getline(rootfile, 256);
			if (in.eof())
				break;
			if (verbose)
				cout << "Root file to process: " << rootfile << endl;
			T.Add(rootfile);

		}
		in.close();
		//Close the file

	} else {
		cerr << "ERROR: Couldn't open the specified file: " << filename << "(" << strerror(errno) << ")" << endl;
	}

	TCanvas        *canvas = new TCanvas("canvas", "Plots", 0, 0, 1270, 990);
	TPad           *pad1 = new TPad("pad1", "This is pad 1", 0.01, 0.48, 0.49, 0.94);
	TPad           *pad2 = new TPad("pad2", "This is pad 2", 0.51, 0.48, 0.99, 0.94);
	TPad           *pad3 = new TPad("pad3", "This is pad 3", 0.01, 0.01, 0.49, 0.47);
	TPad           *pad4 = new TPad("pad4", "This is pad 4", 0.51, 0.01, 0.99, 0.47);

	pad1->Draw();
	pad2->Draw();
	pad3->Draw();
	pad4->Draw();

	TPaveLabel     *pl = new TPaveLabel(0.274, 0.956, 0.748, 0.998, plottitle.c_str(), "br");
	pl->SetTextSize(0.45);
	pl->Draw();

	//create a RouteStat object for storage of per event data
	RouteStat * route = new RouteStat();
	T.SetBranchAddress("routestat", &route);
	time_t		endtime_root = endtime - 788918400;
	time_t		starttime_root = starttime - 788918400;

	Int_t		binnum = (endtime_root - starttime_root) / 86400;

	TH1F           *meanhist = new TH1F("xtime1", "Mean plot", binnum, starttime_root, endtime_root);
	TH1F           *ratiohist = new TH1F("xtime2", "ratio mean/stdev plot", binnum, starttime_root, endtime_root);
	TH1F           *maxhist = new TH1F("xtime3", "Max plot", binnum, starttime_root, endtime_root);
	TH1F           *stddevhist = new TH1F("xtime4", "Standard Dev. plot", binnum, starttime_root, endtime_root);

	Int_t		nTotalEvents = T.GetEntries();
	//initialization to ` Int_t ' from `Double_t' here is harmless
	Int_t aux;
	for (Int_t eventNo = 0; eventNo < nTotalEvents; eventNo++) {

		if (T.GetEvent(eventNo) > 0) {
			Int_t		mid = route->getId();
			if(verbose)
				cout << " " << mid;
			if (mid == id) {
				Double_t	mmean = route->getMean();
				Double_t	mrms = route->getRms();
				if (mmean <= 0) {
					continue;
				}
				if (mrms <= 0) {
					continue;
				}
				Int_t		max = route->getMax();
				Double_t	stddev = mmean / mrms;
				Int_t		mtimestamp = route->getTimestamp();
				aux = (mtimestamp - 788918400);
				meanhist->Fill(aux, mmean);
				maxhist->Fill(aux, max);
				stddevhist->Fill(aux, stddev);
				aux = aux / 86400;
				if(verbose)
					cout << "TB id: " << mid << " - Mean: " << mmean << " - RMS: " << mrms << " - Max: " << max << " - Stddev: " << stddev << " - Timestamp: " << aux << endl;
			}
		}
	}

	int		ndiv = 0;
	ndiv = -506;
	pad1->cd();
	pad1->SetGridx();
	pad1->SetGridy();
	meanhist->SetFillColor(7);
	meanhist->SetStats(kFALSE);
	meanhist->SetLineColor(1);
	meanhist->SetXTitle("Time");
	meanhist->SetYTitle("Mean");
	meanhist->SetNdivisions(ndiv, "x");
	meanhist->GetXaxis()->SetTimeDisplay(1);
	meanhist->GetXaxis()->SetTimeFormat("%b %d");
	meanhist->Draw();


	pad2->cd();
	pad2->SetGridx();
	pad2->SetGridy();
	stddevhist->SetFillColor(7);
	stddevhist->SetStats(kFALSE);
	stddevhist->SetLineColor(1);
	stddevhist->SetXTitle("Time");
	stddevhist->SetYTitle("Standard Dev");
	stddevhist->SetNdivisions(ndiv, "x");
	stddevhist->GetXaxis()->SetTimeDisplay(1);
	stddevhist->GetXaxis()->SetTimeFormat("%b %d");
	stddevhist->Draw();

	pad3->cd();
	pad3->SetGridx();
	pad3->SetGridy();
	maxhist->SetFillColor(7);
	maxhist->SetStats(kFALSE);
	maxhist->SetLineColor(1);
	maxhist->SetXTitle("Time");
	maxhist->SetYTitle("Max");
	maxhist->SetNdivisions(ndiv, "x");
	maxhist->GetXaxis()->SetTimeDisplay(1);
	maxhist->GetXaxis()->SetTimeFormat("%b %d");
	maxhist->Draw();


	ratiohist->Divide(meanhist, stddevhist, 1, 1);
	pad4->cd();
	pad4->SetGridx();
	pad4->SetGridy();
	ratiohist->SetFillColor(7);
	ratiohist->SetStats(kFALSE);
	ratiohist->SetLineColor(1);
	ratiohist->SetXTitle("Time");
	ratiohist->SetYTitle("Ration mean/std dev");
	ratiohist->SetNdivisions(ndiv, "x");
	ratiohist->GetXaxis()->SetTimeDisplay(1);
	ratiohist->GetXaxis()->SetTimeFormat("%b %d");
	ratiohist->Draw();

	canvas->Print(plotfile.c_str(), "ps");

	if (verbose)
		cout << "Generated postscript in " << plotfile << ", converting to gif" << endl;

	command = "/ncc/ttpro/bin/ps2gif -b 41 28 574 750.75 -r -s 1.2 " + plotfile;
	if (system(command.c_str()) != 0) {
		cerr << "ERROR converting to gif: " << command << endl;
	} else {
		//remove PS file, don 't clutter disk space
		unlink(plotfile.c_str());
	}

	//clean up

	delete canvas;
	delete		stddevhist;
	delete		maxhist;
	delete		ratiohist;
	delete		meanhist;

	return (0);
}


void usage(){
  cout <<    "*** Usage *** "  << endl;
  cout <<    "This program reads routestat root files and after processing them generates trend plots"  << endl;
  cout <<    "Command line: " << endl;
  cout <<    " xtimeplot -f /ncc/ttpro/tmp/filelist.nhop -i XX  -a 1025481600 -e 1041292800 -t ip -o /ncc/www/publish/pub/ripencc/mem-services/ttm/Plots/nhop/ -b 'xtime plots for tt01' [-v]"<< endl;
  cout <<    "-v: Verbose " << endl;
  cout <<    "-d: Debugging output " << endl;
  cout <<    "-h: print out this " << endl; 
  cout <<    "-f: name of file listing routestat.*.root files to process"	<< endl;
  cout <<    "-i: test box id used to create histogram "<< endl;
  cout <<    "-a: start time in unix time" << endl;
  cout <<    "-e: end time in unix time" << endl;
  cout <<    "-t: type ip or as" << endl;
  cout <<    "-o: directory where to store histogram " << endl;
  cout <<    "-b: text for histogram label" << endl;
}
