/*
Copyright (c) 2003		      RIPE NCC


All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of the author not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

/*
-------------------------------------------------------------------------------
Module Header
Filename	: TTMDelayHistograms.h
Author		: Rene Wilhelm
Date		: 17-JAN-2003
Description	: TTM Histograms Class defintion
Language Version: C++
OSs Tested	: Solaris 2.6, Linux gcc2.95

The TTMDelayHistograms class groups all histogram data and funtions needed to
create one TTM plots for a given source and target box. The TTMDelayData
class will create as many instances of this object as are required by
the configured source and target boxes.

The interface to the outside is provided by two methods:

  Fill() - and fills ROOT histograms
  Plot() - create plot(s) from the previously filled histograms. 
	
$Id: TTMDelayHistograms.h,v 1.3 2007/08/28 16:39:54 franzsch Exp $
-------------------------------------------------------------------------------
*/

#ifndef TTMDelayHistograms_included
#define TTMDelayHistograms_included


#include <ctime>  
#include <TH1.h>
#include <TH2.h>
#include <TPad.h>
#include <TStyle.h>

#include "TestBox.h"
#include "Percentiles.h"
#include "RoutingInfo.h"
#include "TTMOptions.h"
#include "TTMHistograms.h"
#include "utils.h"

using namespace std;	// XXX what is the Best Current Practice ?


class TTMDelayHistograms : public TTMHistograms {

private:

	// pointers to histogram objects.

	TH2F	*Delay2D;
	TH1F	*Delay1D;
	TH1F	*PacketsSent;
	TH1F	*ClocksOK;
	TH1F	*BadSourceClock;
	TH1F	*BadTargetClock;
	TH1F	*BadClocks;
	TH2F	*RoutesNHops;
	TH1F	*PacketsLost;


	TH1F	*Arrived;
	TH1F	*ArrivedInvalid;

	Percentiles *LossPercentile;	  
	Percentiles *DelayPercentile;	 
	RoutingInfo *RoutesInfo;	

	// parameters for histograms

   string   outputOption; // Indicates what parts of the plots to show.
	double	mindelay;	// lower delays will go to underflow bin
	double	maxdelay;	// higher delays will go to overflow bin
	short	logscale;	// whether or not to use logscale for delays
	short	allpackets;	// also include packets with bad clocks in plots
	int	binsX;		// number of bins on X-axis
	int	binsY;		// number of bins on Y-axis

	// private methods

	void ClearHistos();
	void CreateHistos();
	void CreateXMLSummary(const std::string &sourcename, const std::string &targetname);
	void DeleteHistos();
	void PlotHistos(const string &sourcename, const string &targetname);

public:
	TTMDelayHistograms(TestBox* source, int targetnum, TTMOptions* opt);
	virtual ~TTMDelayHistograms();
	virtual void Fill(Delay* packet);
	virtual void Plot();
};
#endif

//	void processDelay(Delay* delay);     UNUSED?
