/*
Copyright (c) 2000                      RIPE NCC


All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of the author not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

/*
-------------------------------------------------------------------------------
Module Header
Filename          : TTMHistograms.C
Author            : Rene Wilhelm
Date              : 13-OCT-2000
Description       : Implementation of TTM TTMHistograms class
Language Version  : C++
OSs Tested        : Solaris 2.6
External Programs : ps2gif (which in turn needs ghostscript and imaging tools)
$Id: TTMHistograms.C,v 1.10 2007/08/28 16:39:54 franzsch Exp $
-------------------------------------------------------------------------------
*/

#define PATH "/ncc/ttpro/data/root"   // path to TTM ROOT data storage
#define TYPE 'H'		      // hierarchical (year/month/day) storage

#include <cstdlib>
#include <ctime>
#include <iostream>
#include <TCanvas.h>
#include <TLine.h>
#include <TPaveLabel.h>
#include <TPostScript.h>
#include <TText.h>

#include "Delay.h"
#include "TTMHistograms.h"
#include "utils.h"

RCSID("$Id: TTMHistograms.C,v 1.10 2007/08/28 16:39:54 franzsch Exp $");

/*
-------------------------------------------------------------------------------
Subroutine Header
Description	  : Add new datapoint to Histograms
Input		  : pointer to Delay object with result from TTM measurement
Side Effects	  : various ROOT histograms updated
Language Version  : C++
OSs Tested        : Linux
-------------------------------------------------------------------------------
*/


/*
-------------------------------------------------------------------------------
Subroutine Header
Description	  : Constructor for TTMHistogram objects
Input		  :
-------------------------------------------------------------------------------
*/

TTMHistograms::TTMHistograms(TestBox *src, int tgtnum, TTMOptions* opt) {


	// default construtor
	// initialize everything to zero or NULL

	source = src;
	targetNum  = tgtnum;

	verbose    = opt->Verbose();
	endtime    = opt->EndTime();
	starttime  = endtime - opt->TimePeriod();

	cstarttime1=new char[20];
	cstarttime2=new char[8];
	cendtime1=new char[20];
	cendtime2=new char[8];

	ipVersion  = opt->IPversion();
	outputDir  = opt->OutputPath();
	tmpDir     = opt->TmpDir();
	plotFormat = opt->PlotFormat();


	tm *timestruct = gmtime(&endtime);
	strftime(cendtime1, 20, "%Y-%m-%d %H:%M", timestruct);
	strftime(cendtime2, 8, "%b %d", timestruct);

	timestruct = gmtime(&starttime);
	strftime(cstarttime1, 20, "%Y-%m-%d %H:%M", timestruct);
	strftime(cstarttime2, 8, "%b %d", timestruct);
}

/*
-------------------------------------------------------------------------------
Subroutine Header
Description	  : Destructor for TTMHistogram objects
		    delete all dynamically created data
-------------------------------------------------------------------------------
*/

TTMHistograms::~TTMHistograms() {

	if (verbose && (source != NULL)) {
		cout << "destructing TTMHistograms object " <<
		    source->GetShortName() << " targetNum: " <<
		    targetNum << endl;
	}

	delete[] cstarttime2;
	delete[] cendtime1;
	delete[] cendtime2;

}

/*
-------------------------------------------------------------------------------
Subroutine Header
Description	  : 

//  arguments:     *pad        -> pointer to current pad
//                 *ptr_text   -> pointer to string
//                 x           -> x-coordinate NTC
//                 y           -> y-coordinate NTC
//                 align       -> alignment (see ROOT TattText class)
//		   font	       -> font id   (see ROOT TattText class)
//  purpose:       writes 'ptr_text' on 'pad' as TText on
//                 coordinates 'x' and 'y' with alignment
//                 'align'. Use TLatex-class for a better
//                 correspondence between screen and print-out
//                 in newer versions of ROOT (above 2.21/08)
-------------------------------------------------------------------------------
*/

void TTMHistograms::write_stats(TPad* pad, const char* ptr_text, const Float_t x,
		const Float_t y, const Short_t align, Font_t font)
{
  pad->cd();
  TText *xlabel = new TText(x, y, ptr_text);
  xlabel -> SetTextFont(font);
  xlabel -> SetTextColor(1);
  xlabel -> SetTextSize(0.10);
  xlabel -> SetTextAlign(align);
  xlabel->Draw();
  return;
}

