/*
Copyright (c) 2001		      RIPE NCC


All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of the author not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

/*
-------------------------------------------------------------------------------
Module Header
Filename	  : TTMOptions.h
Author	    : Rene Wilhelm
Date	      : 18-10-2001
Revised	   : 10-12-2002, add IP version option (v4, v6)
Description       : TTM Options class definition
Language Version  : C++
OSs Tested	: Solaris 2.6, Solaris 8, Debian Linux 2.2

A TTMOptions object stores the various command line options of TTM programs
Encapsulating this information in one object allows for more flexible
code and eliminates the need for globally scoped parameters.

$Id: TTMOptions.h,v 1.7 2007/08/28 16:39:54 franzsch Exp $
-------------------------------------------------------------------------------
*/


#ifndef TTMOPTIONS_H
#define TTMOPTIONS_H


#include <string>

#include <cmath>
#include <cstdlib>
#include <cstring>
#include <ctime>

#include <unistd.h>

#include "utils.h"

using namespace std;	// XXX what is the Best Current Practice ?

#define TTCONFIG_COMMAND	"/ncc/ttpro/bin/ttconfig"
#define TTCONFIG_COMMAND_ARGS   "-v active"
#define TTCONFIG_COMMAND_ARGS6  "-v active6"

class TTMOptions {

private:
	uint		sourceId, targetId;
	time_t		endTime, timePeriod;
	string		inputPath, 
			outputPath,
         outputOption,
			progName,
			configCmd;		// command to fetch TB config
	string tmpDir;    // tmp directory for .ps (.eps) files
	float   	minDelay, maxDelay;
	float   	maxIPDV;
	bool		debug, checkdata, verbose, logScale, allPackets, dateInFilename, enableErrorPlot, savePercentiles, createHTML,
				customMaxIPDV,customMinDelay,customMaxDelay;
	period		sumPeriod;
	plotformat	plotFormat;
	char		storageType;		// storage type of ROOT data
						// 'H' = yyyyy/mm/dd hierarchy

	int   		ipVersion;		// which data to use IPv4, IPv6

	const char* optionString;

	void usage();
	void invaliddate();
	void decodePeriod (string datestring, time_t& end, time_t& period);
	void parseISO8601 (string datestring, time_t& isotime, char& flag);

public:
	int  IPversion()    {return ipVersion;}
	uint SourceId()     {return sourceId;}
	uint TargetId()     {return targetId;}
	const string &InputPath()   {return inputPath;}
	const string &OutputPath()  {return outputPath;}
	const string &ProgramName() {return progName;}
	const string &ConfigCommand()   {return configCmd;}
	const string &TmpDir()  {return tmpDir;}
   const string &OutputOption() {return outputOption;}
	float &MinDelay()    {return minDelay;}
	float &MaxDelay()    {return maxDelay;}
	float &MaxIPDV()    {return maxIPDV;}
	time_t &EndTime()    {return endTime;}
	time_t &TimePeriod() {return timePeriod;}
	char &StorageType()  {return storageType;}
	period SummaryPeriod()  {return sumPeriod;}
	plotformat PlotFormat() {return plotFormat;}

	bool &Verbose()      {return verbose;}
	bool &CheckData()      {return checkdata;}
	bool &Debug()      {return debug;}
	bool &Logscale()    {return logScale;}
	bool &AllPackets()  {return allPackets;}
	bool &DateInFilename() {return dateInFilename;}
	bool &EnableErrorPlot(){return enableErrorPlot;}
	bool &SavePercentiles(){return savePercentiles;}
	bool &CreateHTML(){return createHTML;}
	bool &CustomMaxIPDV(){return customMaxIPDV;}
	bool &CustomMaxDelay(){return customMaxDelay;}
	bool &CustomMinDelay(){return customMinDelay;}
	

	TTMOptions(int argc, char** argv, const char* optstring);
	~TTMOptions();
};

#endif
