/*
Copyright (c) 2001		     RIPE NCC


All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of the author not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

/*
-------------------------------------------------------------------------------
Module Header
Filename	  : TestBox.C
Author	    	  : Rene Wilhelm
Date	      	  : 13-AUG-2001
Description       : TestBox class implemenation
Language Version  : C++
OSs Tested        : Solaris 2.6, Debian Linux 2.2

$Id: TestBox.C,v 1.5 2006/11/13 15:21:28 ruben Exp $
-------------------------------------------------------------------------------
*/

#include "TestBox.h"

#include <cstdlib>
#include <cstring>

#ifdef DEBUG 
#include <iostream>
#endif

#include "utils.h"

RCSID("$Id: TestBox.C,v 1.5 2006/11/13 15:21:28 ruben Exp $");


/*
-------------------------------------------------------------------------------
Subroutine Header
Purpose	   	:     TestBox constructor 
Side Effects    :     initialize private data members
Comments	:
-------------------------------------------------------------------------------
*/

TestBox::TestBox() {

	id = 0 ;
}

/*
-------------------------------------------------------------------------------
Subroutine Header
Purpose	   	  :   Initializing constructor
Input 		  :   character string 
-------------------------------------------------------------------------------
*/

TestBox::TestBox(int num, const string &label) {

	id = num;
	SetName(label);
}

/*
-------------------------------------------------------------------------------
Subroutine Header
Purpose	  	:   TestBox destructor
Side Effects    :   deletes ROOT TChain + dynamically created datapath string
Comments	:
-------------------------------------------------------------------------------
*/



/*
-------------------------------------------------------------------------------
Subroutine Header
Purpose	   	  :   Set name of this box
Input 		  :   character string 
-------------------------------------------------------------------------------
*/

void TestBox::SetName(const string &label) {

	name = label;

	// determine short, non-qualified hostname for use in
	// autogenerated filenames.  For uniform presentation,
	// XXX two exceptions are hardcoded, in all other cases shortname
	// is just the first part of the fully qualified domain name

	if (name == "osdorp.ripe.net") {
		shortname = "tt02";
	}
	else if (name == "jordaan.ripe.net" ) {
		shortname = "tt04";
	}
	else {
		// shortname is first part of full hostname
		string::size_type pos;
		if ((pos = name.find('.')) != string::npos) {
			shortname = name.substr(0,pos);
		}
		else {
			// no '.' in hostname ?
			shortname = name;
		}
	}
}


/*
-------------------------------------------------------------------------------
Subroutine Header
Purpose	   	  :   Add a box to the list of targets for test-traffic
Input parameters  :   target - pointer to TestBox object
Comments	  :   No checks for duplicates, we assume caller knows
		      what he is doing ...
-------------------------------------------------------------------------------
*/

void TestBox::AddTarget(TestBox *target) {
	targetList[target->GetId()] = target;
}
	
/*
-------------------------------------------------------------------------------
Subroutine Header
Purpose	   	  :   Get list of targets for this box
Output 		  :   return value is pointer to targetList 
		      number of targets is returned via reference
-------------------------------------------------------------------------------
*/

map<uint,TestBox*>& TestBox::GetTargets(void) {
	return(targetList);
}

/*
-------------------------------------------------------------------------------
Subroutine Header
Purpose	   	  :   Find box with specified targetId in targetList
Output 		  :   return value is pointer to relevant TestBox object
		      (or NULL if not found)
-------------------------------------------------------------------------------
*/

TestBox *TestBox::FindTargetById(uint targetid) {

	return (targetList.find(targetid) != targetList.end()) ? targetList[targetid] : NULL;
}

/*
-------------------------------------------------------------------------------
Subroutine Header
Purpose	   	  :   Set list of targets to be processed for this box 
Note		  :   This list can be a subset of the list of boxes 
		      which the box was sending traffic to. 
-------------------------------------------------------------------------------
*/

void TestBox::SetTargets(const map<uint,TestBox*>& targets) {

	targetList = targets;
}
