/*
Copyright (c) 2001		      RIPE NCC


All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of the author not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

/*
-------------------------------------------------------------------------------
Module Header
Filename	  : summarize.C
Author	    : Rene Wilhelm
Date	      : 14-NOV-2001
Description       : Main program for creating TTM delay summary pages
Language Version  : C++
OSs Tested	: Solaris 2.6, Solaris 8, Debian Linux 2.2

Command line      :
    required args : -p <timeperiod>  iso8601 format time period; sets time
				     period and last day for summaries
    optional args : -o <outputdir>   directory tree where to store HTML pages
		    -w		     Create summary pages from week files
		    -v		     verbose

$Id: summarize.C,v 1.8 2006/11/13 15:21:28 ruben Exp $
-------------------------------------------------------------------------------
*/

#define NUMINTERVALS 4	 // difficult to autodetect from data

// XXX needed for solaris
using namespace std;

#include <ctime>


#include <TROOT.h>
#include <TApplication.h>
#include <TError.h>


#include "utils.h"
#include "TestBoxConfig.h"
#include "TTMOptions.h"
#include "TTMSummaryChain.h"

RCSID("$Id: summarize.C,v 1.8 2006/11/13 15:21:28 ruben Exp $");

int main (int argc, char** argv) {


// fetch options and set defaults

	TTMOptions options(argc, argv, "46vdwo:R:p:");

	// Ignore ROOT messages
	gErrorIgnoreLevel=1;

	if (options.Debug())
		gErrorIgnoreLevel=0;

// Initialize ROOT system

	TROOT simple ("TTM", "Summaries");
	gROOT->SetBatch();		   // no graphics windows

	TApplication theApp("delayStats", NULL, NULL, NULL, 0);

// now we can start ...

	
	TestBoxConfig config(options);

	TTMSummaryChain summary;
	summary.SetConfig(options, config, NUMINTERVALS);

	summary.Chain();
	summary.OutputHTML(0.4);
	summary.OutputXML();

	return(0);
}
