/*
Copyright (c) 2000                      RIPE NCC


All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of the author not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

/*
-------------------------------------------------------------------------------
Module Header
Filename          : utils.h
Author            : Rene Wilhelm
Date              : 17-OCT-2000
Description       : misscellaneous declarations and types
Language Version  : C++
OSs Tested        : Solaris 2.6, Solaris 8, Debian Linux 2.2
$Id: utils.h,v 1.5 2006/11/13 15:21:28 ruben Exp $
-------------------------------------------------------------------------------
*/

#ifndef utils_included
#define utils_included 

#include <algorithm>
#include <functional>


//
//  Common constant definitions
//

#define SECONDSPERDAY 86400
#define ROOT_TIMEZERO	788918400

// Commandline to convert ps files to gif
//
// BoundingBox parameters are taken from a manual run of the
// bbfig tool; will be fine as long as ROOT PostScript prolog
// does not change.
//
// rotate to get proper orientation
// scale up a bit to make text more legible
//

#define PS2GIF "ps2gif -b 41 28 574 750.75 -r -s 1.2"
#define PS2PDF "ROOTps2pdf"


//
//  External globals
//

extern char* progname;
extern int   verbose;

// 
// Simple type definitions
// 

enum plotformat {
	csv,		// comma seperated fields  (Ascii)
	gif,		// bitmap image
	ps,		// PostScript tuned for paper   (90 degree rotated)
	eps,		// PostScript tuned for viewing
	pdf		// PostScript turned into pdf for viewing
};

enum period {
	day,
	week
};

/* Helper macros for accessing the key/value pair of a STL map iterator */

#define PAIR_KEY(a)	((*(a)).first)
#define PAIR_VAL(a) ((*(a)).second)

/* Helper function object to delete vectors elements with a for_each loop */
template<class T> struct delete_pointer: public unary_function<T, void> 
{
	void operator() (T x) { if (x) delete x; }
};

/* 
 * http://root.cern.ch/root/html/TAttText.html#TAttText:description
 *
 *    font  : Text font code = 10*fontnumber + precision
 *             Font numbers must be between 1 and 14
 *             precision = 0 fast hardware fonts (steps in the size)
 *             precision = 1 scalable and rotatable hardware fonts (see below)
 *             precision = 2 scalable and rotatable hardware fonts
 *             precision = 3 scalable and rotatable hardware fonts. Text size
 *                           is given in pixels.
 *
 *        1 : times-medium-i-normal      "Times New Roman"      1           4
 *        2 : times-bold-r-normal        "Times New Roman"      0           7
 *        3 : times-bold-i-normal        "Times New Roman"      1           7
 *        4 : helvetica-medium-r-normal  "Arial"                0           4
 *        5 : helvetica-medium-o-normal  "Arial"                1           4
 *        6 : helvetica-bold-r-normal    "Arial"                0           7
 *        7 : helvetica-bold-o-normal    "Arial"                1           7
 *        8 : courier-medium-r-normal    "Courier New"          0           4
 *        9 : courier-medium-o-normal    "Courier New"          1           4
 *       10 : courier-bold-r-normal      "Courier New"          0           7
 *       11 : courier-bold-o-normal      "Courier New"          1           7
 *       12 : symbol-medium-r-normal     "Symbol"               0           6
 *       13 : times-medium-r-normal      "Times New Roman"      0           4
 */

#define PRECISION_MODE (2)
#define TATTTEXT_HELVETICA_MEDIUM_R_NORMAL	(40 + PRECISION_MODE )
#define TATTTEXT_HELVETICA_BOLD_R_NORMAL	(60 + PRECISION_MODE )

#endif

/**
 * Prevent RcsID from being optimized away
 * \param[in] x - Just use "$Id: utils.h,v 1.5 2006/11/13 15:21:28 ruben Exp $" here
 * \remark 
 * GCC is getting smarter, so this might not work in the future
 *
 */
#define RCSID(x) namespace {                                \
    inline const char* rcsid_f(const char *p) { return p; } \
    const char *rcsid=rcsid_f(x);                           \
  }
