#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include "packet.h"
#include "merge_packets.h"
#include "testbox.h"

#define MAXWORDS 200

static char const rcsid[] = "$Id: packet.c,v 1.13 2007/02/06 12:15:07 ruben Exp $";

struct packet *list_of_packets[MAXPACKETS];
struct packet *list_of_packets6[MAXPACKETS];

int number_of_packets;
int number_of_packets6;

int read_existing_packets (const char* directory, const char * hostname, int date, int format, int debug) {

   /* Read an existing file with packets */
   /* format: BINARY or ASCII */
   /* Date:   YYYYMMDD */
   /* File:   directory/PCKB[6].hostname.date */


   FILE *fp;
   FILE *fp6=NULL;

   char line[MAXWORDS];
   char *point = &line[0];
   char **ap;
   char *arg[MAXWORDS];
   char *temp;
   char *temp6;

   struct packet *newpacket;

   int i, readpipe, readpipe6;

   temp = (char *) malloc ( strlen(directory) + 6 + strlen(hostname) + 10);
   temp6 = (char *) malloc ( strlen(directory) + 6 + strlen(hostname) + 11);
   temp = strcpy (temp, directory);
   temp6 = strcpy (temp6, directory);

   if (format == ASCII) {
      temp = strcat ( temp, "/PCKA.");
   } else {
      temp = strcat ( temp, "/PCKB.");
      temp6 = strcat ( temp6, "/PCKB6.");
   }
   temp = strcat ( temp, hostname );
   i    = sprintf (line, ".%d", date);
   temp = strcat ( temp, line); 

   temp6 = strcat ( temp6, hostname );
   temp6 = strcat ( temp6, line);

   if (debug == TRUE) {
      printf ("[IPv4] Trying to read file %s\n", temp);
      printf ("[IPv6] Trying to read file %s\n", temp6); 
   }
 
   if (format == ASCII) {
      fp = openfile (temp, "r", &readpipe) ;
   } else {

       /* make sure PCKB is unzipped; feof will fail for pipes, and
          we need to write a new, uncompressed PCKB anyhow */

       char *gunzip;
       gunzip = (char *) malloc (strlen(temp) + 32);

       sprintf(gunzip, "gunzip %s >/dev/null 2>&1",temp);
       setproctitle("Decompressing %s", temp);
       system(gunzip);
       fp = openfile (temp, "rb", &readpipe);

       sprintf(gunzip, "gunzip %s >/dev/null 2>&1",temp6);
       setproctitle("Decompressing %s", temp6);
       system(gunzip);
       fp6 = openfile (temp6, "rb", &readpipe6);

       free(gunzip);
   }
   if (fp == NULL) {
      if (debug == TRUE) {
          printf ("[IPv4] No merged file exists for this day/host\n");
      }
   }
   if (fp6 == NULL) {
      if (debug == TRUE) {
          printf ("[IPv6] No merged file exists for this day/host\n");
      }
   }

   if (debug == TRUE) {
	if (fp) {
	        printf ("Opened file %s", temp);
	        if (readpipe) {
	                printf(" with gzcat");
	        }
	        printf("\n");
	}
        if (fp6) {
                printf ("Opened file %s", temp6);
                if (readpipe6) {
                        printf(" with gzcat");
                }
                printf("\n");
        }
   }
   free (temp);
   free (temp6);


   number_of_packets = 0;
   number_of_packets6 = 0;

   if (format == ASCII ) {
      while ( (point = fgets(line, MAXWORDS, fp)) != NULL ) {
#if defined(USE_STRTOK)            /* POSIX compliant */
         ap = arg;
         *ap = strtok (point, " \t");
         while (*ap != NULL) {
             if (**ap != '\0') {
                ++ap;
             }
             *ap = strtok (NULL, " \t");
         }
#else    		              /*  BSD specific */
         for (ap = arg ; (*ap = strsep (&point, " \t")) != NULL ; ) {
            if (**ap != '\0') {
               ++ap;
            }
         }
#endif
         newpacket = (struct packet *) malloc (sizeof(struct packet));
         newpacket->packet_id     = atoi (arg[2]);
         newpacket->source_id     = test_box_id (arg[3]);
         newpacket->source_port   = atoi (arg[5]);
         newpacket->target_id     = test_box_id (arg[6]);
         newpacket->target_port   = atoi (arg[7]);
         newpacket->packet_size   = atoi (arg[8]);
         newpacket->arrival_time  = atof (arg[9]);
         newpacket->delay         = atof (arg[10]);
         newpacket->source_clock  = strtoul (arg[11], NULL, 16); 
         newpacket->target_clock  = strtoul (arg[12], NULL, 16);
         if (atoi(arg[1]) > 12) {
            newpacket->nhops      = atoi (arg[12]);
            newpacket->routeid    = atoi (arg[13]);
            newpacket->source_ntp = atof (arg[14]);
            newpacket->target_ntp = atof (arg[15]);
         } else {
            newpacket->nhops      = -1;
            newpacket->routeid    = -1;
            newpacket->source_ntp = -1.0; 
            newpacket->target_ntp = -1.0;
         }
         if (number_of_packets < MAXPACKETS) {
            list_of_packets[number_of_packets] = newpacket;
            number_of_packets++;
	 }
	 else {
            fprintf (stderr, "ABORT: Found more than %d packets on this file\n", MAXPACKETS);
	    exit(1);
         }
      }
   } else {
      if (fp) {
          setproctitle("Reading IPv4 PCKB data for %s from %d", hostname, date);
	      do {
	         if (number_of_packets < MAXPACKETS) {
	            list_of_packets[number_of_packets] = (struct packet *) malloc (sizeof(struct packet));
	            i = fread ( list_of_packets[number_of_packets], sizeof (struct packet), 1, fp); 
	            number_of_packets++;
		 }
		 else {
	            fprintf (stderr, "ABORT: Found more than %d IPv4 packets on this file\n", MAXPACKETS);
		    exit(1);
	         }
	      } while (feof(fp) == 0) ;
	      number_of_packets--;
      }
      if (fp6) {
          setproctitle("Reading IPv6 PCKB data for %s from %d", hostname, date);
              do {
                 if (number_of_packets6 < MAXPACKETS) {
                    list_of_packets6[number_of_packets6] = (struct packet *) malloc (sizeof(struct packet));
                    i = fread ( list_of_packets6[number_of_packets6], sizeof (struct packet), 1, fp6);
                    number_of_packets6++;
                 }
                 else {
                    fprintf (stderr, "ABORT: Found more than %d IPv6 packets on this file\n", MAXPACKETS);
                    exit(1);
                 }
              } while (feof(fp6) == 0) ;
              number_of_packets6--;
      }
   }
 
   if (debug == TRUE) { printf ("Read %d IPv4 packets, %d IPv6 packets, started sorting\n", number_of_packets, number_of_packets6);}

   if (fp != NULL) {
	   if (readpipe) {
		pclose(fp);   /* pclose does a wait(3) to prevent zombies */
	   }
	   else {
   		fclose (fp);
	   }
   }

   if (fp6 != NULL) {
	   if (readpipe6) {
        	pclose(fp6);   /* pclose does a wait(3) to prevent zombies */
	   }
	   else {
        	fclose (fp6);
	   }
   }

   setproctitle("Sorting existing data for %s from %d",hostname,date);
   sort_packets(debug);
   setproctitle(NULL);

   if (debug==TRUE) { printf ("Sorted packets\n");}

   return number_of_packets;
}

FILE* open_packet_file (int format, const char *directory, const char *hostname, int date, int ip6flag) {
/*    ================
      Open binary file for packets: PCKB.hostname.date 
*/

   FILE *fp;
   char *temp;
   char  line[MAXWORDS];
   int   i;

if (!ip6flag) {

   temp = (char *) malloc ( strlen(directory) + 6 + strlen(hostname) + 10);
   temp = strcpy (temp, directory);
   if (format == BINARY) {
      temp = strcat ( temp, "/PCKB.");
   } else {
      printf ("Format not supported\n");
      return NULL;
   }
   temp = strcat ( temp, hostname );
   i    = sprintf (line, ".%d", date);
   temp = strcat ( temp, line); 

   if ( (fp = fopen (temp, "wb")) == NULL ) {
      printf ("Cannot open file %s\n", temp);
   }
} else {

   temp = (char *) malloc ( strlen(directory) + 6 + strlen(hostname) + 11);
   temp = strcpy (temp, directory);
   if (format == BINARY) {
      temp = strcat ( temp, "/PCKB6.");
   } else {
      printf ("Format not supported\n");
      return NULL;
   }
   temp = strcat ( temp, hostname );
   i    = sprintf (line, ".%d", date);
   temp = strcat ( temp, line);

   if ( (fp = fopen (temp, "wb")) == NULL ) {
      printf ("Cannot open file %s\n", temp);
   }
}
   free (temp);

   return fp;
}



int write_packet_data (FILE* fp, struct packet *thispacket) {
/*  =================
    Write one packet to file *fp, binary format 
*/
   int written;

   written = fwrite ( thispacket, sizeof (struct packet), 1, fp); 

   return TRUE;
}

int close_new_packet_file (FILE *fp) {
/*  =====================
    Close file with packets 
*/

   fclose (fp);
   return TRUE;
}

void print_packet (struct packet * thispacket) {
/*   ============
     Print packet data on standard output 
*/
   printf ("Packet_id      %d\n", thispacket->packet_id);
   printf ("Source_id      %d\n", thispacket->source_id);
   printf ("Source_port    %d\n", thispacket->source_port);
   printf ("Target_id      %d\n", thispacket->target_id);
   printf ("Target_port    %d\n", thispacket->target_port);
   printf ("Packet_size    %d\n", thispacket->packet_size);
   printf ("Arrival_time   %f\n", thispacket->arrival_time);
   printf ("Delay          %f\n", thispacket->delay);
   printf ("Source_clock   %X\n", thispacket->source_clock);
   printf ("Target_clock   %X\n", thispacket->target_clock);
   printf ("Nhops          %d\n", thispacket->nhops);
   printf ("Route_id       %d\n", thispacket->routeid);
   printf ("Source_NTP     %f\n", thispacket->source_ntp);
   printf ("Target_NTP     %f\n", thispacket->target_ntp);
   printf ("\n");

   return;
}

/*
 * Subroutine header
 *
 * Purpose:       open data file
 * Description:   opens a data file in specified mode
 *                if filename to be read ends in .gz, read via gzcat
 *                if filename to be read does not exist, check if .gz exists
 *                return NULL in case of error
 */

FILE *openfile (const char *filename, const char *mode, int *gzip) {

	char *temp;
	FILE *fp;
	int found;

	found = FALSE;
	*gzip = FALSE;

	if (strchr(mode, 'r') == NULL) {
		/* open for writing */
		fp = fopen (filename, mode);
	}
	else {
		/* open for reading */
		if (strcmp(filename + strlen(filename) - 3, ".gz") == 0 ) {
		        *gzip = TRUE;
			temp = (char *) malloc (strlen(filename) + 7);
        		temp = strcpy (temp, "gzcat "); 
			temp = strcat (temp, filename);
			fp = popen(temp, mode);
			free(temp);
		}
		else if ((fp = fopen(filename, mode)) == NULL) {
			/* open failed, check if .gz exists */

			temp = (char *) malloc (strlen(filename) + 10);
        		temp = strcpy (temp, filename); 
			temp = strcat (temp, ".gz");

			if ((fp = fopen (temp, "r")) != NULL) {
	         		/* yes it does! */
				fclose(fp);
		        	*gzip = TRUE;
				temp = strcpy (temp, "gzcat "); 
				temp = strcat (temp, filename);
				temp = strcat (temp, ".gz");
				fp = popen(temp, mode);
			}
			free(temp);
		}
	}
	return (fp);
}
