#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include "packet.h"
#include "merge_packets.h"
#include "testbox.h"

#define MAXWORDS  200
#define MAXERRORS 250

static char const rcsid[] = "$Id: read_new_packet.c,v 1.15 2007/02/06 12:15:07 ruben Exp $";

int read_new_packets (const char * directory, const char * hostname, int date, int debug) {
/*  ================
    Read sent packets from "directory/SNDP.hostname.date", merge file with 
    existing data structure.
*/

   char  *filename, *filename6, *temp;
   int    readpipe, readpipe6;
   int    i, trgid, srcid;
   char   line[MAXWORDS];
   char  *point = &line[0];
   char **ap;
   char  *arg[MAXWORDS];
   char  *arg6[MAXWORDS];
   int    argc;
   int    argc6;
   int    errorcount, errorcount6;
   FILE  *fp, *fp6;
   struct packet *newpacket;
   int    packet_idx, org_number_of_packets, org_number_of_packets6;

   unsigned int packetid;

   newpacket = (struct packet *) malloc (sizeof(struct packet));

   /* size of filename:
      filename := <directory>/SNDP.<hostname>.yyyyddmm
      add one more for terminating \0
   */

   filename = (char *)  malloc ( strlen(directory) + 6 + strlen(hostname) + 10);
   sprintf(filename, "%s/SNDP.%s.%d", directory, hostname, date);

   filename6 = (char *)  malloc ( strlen(directory) + 6 + strlen(hostname) + 11);
   sprintf(filename6, "%s/SNDP6.%s.%d", directory, hostname, date);

   errorcount=0;
   errorcount6=0;
   readpipe = FALSE;
   readpipe6 = FALSE;

   fp = openfile(filename, "r", &readpipe);
   if (fp == NULL) {
         printf ("Cannot open SNDP file %s\n", filename);
	 //return FALSE;
   }
   fp6 = openfile(filename6, "r", &readpipe6);
   if (fp6 == NULL) {
         if (debug == TRUE) printf ("Cannot open SNDP6 file %s\n", filename6);
   }

   if (debug == TRUE) {
	if (fp) {
	        printf ("Opened file %s", filename);
	        if (readpipe) {
	                printf(" with gzcat");
	        }
	        printf("\n");
	}
        if (fp6) {
                printf ("Opened file %s", filename6);
                if (readpipe6) {
                        printf(" with gzcat");
                }
                printf("\n");
        }
   }



   srcid = test_box_id (hostname) ;

   org_number_of_packets = number_of_packets;
   org_number_of_packets6 = number_of_packets6;
 
   setproctitle("Reading IPv4 SNDP data for %s from %d", hostname, date);
   while ((fp) && ((point = fgets(line, MAXWORDS, fp)) != NULL) ) {


     if (line[0] == '\0') {
        /* no data or null characters, silently skip */
        continue;
     }

     temp=point+strlen(point);	/* chop newline, remove trailing spaces */
     while (((*temp == '\n') || (*temp == ' ') || (*temp == '\0'))
            && temp > point) {
	*temp = '\0';
	temp--;
     }
	
	
     argc = 0;
#if defined(USE_STRTOK)            /* POSIX compliant */
     ap = arg;
     *ap = strtok (point, " \t");
     while (*ap != NULL) {
         if (**ap != '\0') {
            ++ap;
	    ++argc;
         }
         *ap = strtok (NULL, " \t");
     }
#else                              /*  BSD specific */
      for (ap = arg ; (*ap = strsep (&point, " \t")) != NULL ; ) {
         if (**ap != '\0') {
            ++ap;
	    ++argc;
         }
      }
#endif

     /* Sanity check: line should start with SNDP and have at least 11 tokens */
      if ( (argc < 11) || (strncmp(arg[0], "SNDP", 4) != 0)) {
	errorcount++;
        if (errorcount < MAXERRORS) {
           fprintf(stderr, "merge_packets: SNDP.%s.%d bad input, line skipped:\n", hostname, date);
           for (i = 0; i < argc; i++) {
                fprintf (stderr, "%s ", arg[i]);
           }
           fprintf(stderr, "\n");
        }
        else if (errorcount == MAXERRORS) {
           fprintf(stderr, "SNDP.%s.%d: more than %d errors detected\n", hostname, date, MAXERRORS);
           fprintf(stderr, "  suppressing further error reports for this file\n"
);
	}
        continue;
      }

      packetid = atoi(arg[10]);
      trgid = test_box_id (arg[4]);

      packet_idx = search_packet(packetid, srcid, trgid, org_number_of_packets);

      if (packet_idx == -1) {
       /* Not found, Create a new packet */

         if (number_of_packets < MAXPACKETS) {

	         /* Create a new packet */
         	newpacket = (struct packet *) malloc (sizeof(struct packet));
         	newpacket->packet_id     = packetid;
         	newpacket->source_id     = srcid;
         	newpacket->target_id     = trgid;
         	newpacket->source_port   =  0;
         	newpacket->target_port   =  0; 
         	newpacket->packet_size   =  atoi (arg[8]);
         	newpacket->arrival_time  = -1.0;
         	newpacket->delay         = -1.0;
         	newpacket->source_clock  = -1;
         	newpacket->target_clock  = -1;
         	newpacket->nhops         = -1;
         	newpacket->routeid       = -1;
         	newpacket->source_ntp    = -1.0;
         	newpacket->target_ntp    = -1.0;
         	list_of_packets[number_of_packets] = newpacket;
         	number_of_packets++;
         }
         else {
            fprintf (stderr, "ABORT: Limit of %d packets exceeded on %s\n", MAXPACKETS, filename);
            exit(1);
         }
      }
   }
   setproctitle("Reading IPv6 SNDP data for %s from %d", hostname, date);

   while ( (fp6) && ((point = fgets(line, MAXWORDS, fp6)) != NULL) ) {

     temp=point+strlen(point);  /* chop newline, remove trailing spaces */
     while (((*temp == '\n') || (*temp == ' ') || (*temp == '\0'))
            && temp > point) {
        *temp = '\0';
        temp--;
     }


     argc6 = 0;
#if defined(USE_STRTOK)            /* POSIX compliant */
     ap = arg6;
     *ap = strtok (point, " \t");
     while (*ap != NULL) {
         if (**ap != '\0') {
            ++ap;
            ++argc6;
         }
         *ap = strtok (NULL, " \t");
     }
#else                              /*  BSD specific */
      for (ap = arg6 ; (*ap = strsep (&point, " \t")) != NULL ; ) {
         if (**ap != '\0') {
            ++ap;
            ++argc6;
         }
      }
#endif

     /* Sanity check: line should start with SNDP6 and have at least 11 tokens */
      if ( (argc6 < 11) || (strncmp(arg6[0], "SNDP6", 5) != 0)) {
        errorcount6++;
        if (errorcount6 < MAXERRORS) {
           fprintf(stderr, "merge_packets: SNDP6.%s.%d bad input, line skipped:\n", hostname, date);
           for (i = 0; i < argc6; i++) {
                fprintf (stderr, "%s ", arg6[i]);
           }
           fprintf(stderr, "\n");
        }
        else if (errorcount6 == MAXERRORS) {   
           fprintf(stderr, "SNDP6.%s.%d: more than %d errors detected\n", hostname, date, MAXERRORS);
           fprintf(stderr, "  suppressing further error reports for this file\n"
);
        }
        continue;
      }

      packetid = atoi(arg6[10]);
      trgid = test_box_id (arg6[4]);
          
      packet_idx = search_packet6(packetid, srcid, trgid, org_number_of_packets6);
      
      if (packet_idx == -1) {
       /* Not found, Create a new packet */

         if (number_of_packets6 < MAXPACKETS) {

                 /* Create a new packet */
                newpacket = (struct packet *) malloc (sizeof(struct packet));
                newpacket->packet_id     = packetid;
                newpacket->source_id     = srcid;
                newpacket->target_id     = trgid;
                newpacket->source_port   =  0;
                newpacket->target_port   =  0; 
                newpacket->packet_size   =  atoi (arg6[8]);
                newpacket->arrival_time  = -1.0;
                newpacket->delay         = -1.0;
                newpacket->source_clock  = -1;
                newpacket->target_clock  = -1;
                newpacket->nhops         = -1;
                newpacket->routeid       = -1;
                newpacket->source_ntp    = -1.0;
                newpacket->target_ntp    = -1.0;
                list_of_packets6[number_of_packets6] = newpacket;
                number_of_packets6++;
         }
         else {
            fprintf (stderr, "ABORT: Limit of %d packets exceeded on %s\n", MAXPACKETS, filename6);
            exit(1);
         }
      }
   }

   if (debug == TRUE) {
       printf ("Read finished, now %d packets IPv4, %d packets IPv6, started sorting\n", number_of_packets, number_of_packets6);
   }

   if (fp != NULL) {
	   if (readpipe == TRUE) {
	      pclose(fp);
	   }
	   else {
	      fclose (fp);
	   }
   }

   if (fp6 != NULL) {
	   if (readpipe6 == TRUE) {
	      pclose(fp6);
	   }
	   else {
	      fclose (fp6);
   	   }
   }


   /* Check if packets are sorted in a logical order */
   /* This is an extremely bad sorting algorithm if it wasn't for the fact
      that the packets are already sorted... */

   setproctitle("Sorting new data for %s from %d",hostname,date);
   sort_packets(debug);
   setproctitle(NULL);

   if (debug == TRUE) { printf ("Sorted packets\n"); }
   
   free (filename);
   free (filename6);

   return TRUE;
}

