#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "packet.h"
#include "merge_packets.h"
#include "testbox.h"

#define MAXWORDS  200
#define MAXERRORS 251

static char const rcsid[] = "$Id: read_received.c,v 1.14 2007/02/06 12:15:07 ruben Exp $";

int read_received_packets (char *filename, int debug) {
/*  =====================
    Read RCDP[6] file with packet data
*/

   FILE  *fp;
   char   line[MAXWORDS];
   char  *point = &line[0];
   char **ap;
   char  *arg[MAXWORDS];
   int    npar=10;
   int    argc;
   int    readpipe;
   int    errorcount;

   int    srcid, trgid, found, packetid;
   unsigned int msgid;
   int  low;
   int ip6flag = FALSE;

   errorcount=0;

   if (strstr(filename, "RCDP6")) ip6flag = TRUE;

   fp = openfile(filename, "r", &readpipe);

   if (fp == NULL) {
	if (ip6flag) printf ("Cannot open RCDP6 file %s\n", filename);
	else printf ("Cannot open RCDP file %s\n", filename);
        return FALSE;
   }

   if (debug == TRUE) {
	printf ("Opened file %s", filename);
	if (readpipe) {
		printf(" with gzcat");
	}
	printf("\n");
   }


   while  ( (point = fgets(line, MAXWORDS, fp)) != NULL ) {

     argc = 0;

     if (line[0] == '\0') {
	/* no data or null characters, silently skip */
	continue;
     }

#if defined(USE_STRTOK)            /* POSIX compliant */
     ap = arg;
     *ap = strtok (point, " \t");
     while (*ap != NULL) {
         if (**ap != '\0') {
            ++ap;
	    ++argc;
         }
         *ap = strtok (NULL, " \t");
     }
#else                              /*  BSD specific */
      for (ap = arg ; (*ap = strsep (&point, " \t")) != NULL ; ) {
         if (**ap != '\0') {
            ++ap;
	    ++argc;
         }
      }
#endif
     if (argc > 2) {
     	npar  = atoi (arg[1]);
     }

     /* Sanity check: line should start with RCDP[6] and have at least 12 tokens */
      if ( (strncmp(arg[0], "RCDP", 4) != 0) || (argc != (npar + 2)) ) {
	errorcount++;
	if (errorcount < MAXERRORS) {
	   fprintf(stderr, "merge_packets: %s: unexpected/incomplete input:\n", filename);
	   for (low = 0; low < argc; low++) {
		fprintf (stderr, "%s ", arg[low]);
	   }
       	   fprintf(stderr, "*line skipped*\n");
	}
	else if (errorcount == MAXERRORS) {
	   fprintf(stderr, "%s: more than %d errors detected\n", filename, MAXERRORS);
	   fprintf(stderr, "  suppressing further error reports for this file\n");
	}

        continue;
      }

      msgid = atoi (arg[11]);
      found = FALSE;

if (!ip6flag) {      

      srcid = test_box_ip_id (arg[3], AF_INET);
      trgid = test_box_ip_id (arg[5], AF_INET);

      packetid = search_packet(msgid, srcid, trgid, number_of_packets);

      if (packetid >= number_of_packets) {
         fprintf(stderr, "[IPv4] index %d out of range; only %d packets in list\n", 
              packetid, number_of_packets);
      }

      if (packetid != -1) {
         /* Check if packet already merged */
         if (list_of_packets[packetid] -> arrival_time < 0.0) {
            /* Packet not yet merged */
            list_of_packets[packetid]->source_port  = atoi (arg[4]) ;
            list_of_packets[packetid]->target_port  = atoi (arg[6]) ;
            list_of_packets[packetid]->arrival_time = atof (arg[7]) ;
            list_of_packets[packetid]->delay        = atof (arg[8]) * 1000.0 ;
            list_of_packets[packetid]->source_clock = strtoul (arg[ 9], NULL, 16);
            list_of_packets[packetid]->target_clock = strtoul (arg[10], NULL, 16);
            list_of_packets[packetid]->nhops        = -1;
            list_of_packets[packetid]->routeid      = -1;
	    if (npar == 10) {
            	list_of_packets[packetid]->source_ntp = -1.0;
                list_of_packets[packetid]->target_ntp = -1.0;
	    }
	    else if (npar == 12) {
		/* format used starting February 2000 */
            	list_of_packets[packetid]->source_ntp = atof(arg[12]) * 1000.0;
                list_of_packets[packetid]->target_ntp = atof(arg[13]) * 1000.0;
	    }
	    else if (npar == 14) {
		/* format used December 1999 - February 2000 */
		/* arg[12] = source NTP maxerr, arg[14] = target NTP maxerr */
            	list_of_packets[packetid]->source_ntp = atof(arg[13]) * 1000.0;
                list_of_packets[packetid]->target_ntp = atof(arg[15]) * 1000.0;
	    }

         }
      } 
} else {

      srcid = test_box_ip_id (arg[3], AF_INET6);
      trgid = test_box_ip_id (arg[5], AF_INET6);

      packetid = search_packet6(msgid, srcid, trgid, number_of_packets6);

      if (packetid >= number_of_packets6) {
         fprintf(stderr, "[IPv6] index %d out of range; only %d packets in list\n",
              packetid, number_of_packets);
      }

      if (packetid != -1) {
         /* Check if packet already merged */
         if (list_of_packets6[packetid] -> arrival_time < 0.0) {
            /* Packet not yet merged */
            list_of_packets6[packetid]->source_port  = atoi (arg[4]) ;
            list_of_packets6[packetid]->target_port  = atoi (arg[6]) ;
            list_of_packets6[packetid]->arrival_time = atof (arg[7]) ;
            list_of_packets6[packetid]->delay        = atof (arg[8]) * 1000.0 ;
            list_of_packets6[packetid]->source_clock = strtoul (arg[ 9], NULL, 16);
            list_of_packets6[packetid]->target_clock = strtoul (arg[10], NULL, 16);
            list_of_packets6[packetid]->nhops        = -1;
            list_of_packets6[packetid]->routeid      = -1;
            if (npar == 10) {
                list_of_packets6[packetid]->source_ntp = -1.0;
                list_of_packets6[packetid]->target_ntp = -1.0;
            }
            else if (npar == 12) {
                /* format used starting February 2000 */
                list_of_packets6[packetid]->source_ntp = atof(arg[12]) * 1000.0;
                list_of_packets6[packetid]->target_ntp = atof(arg[13]) * 1000.0;
            }
            else if (npar == 14) {
                /* format used December 1999 - February 2000 */
                /* arg[12] = source NTP maxerr, arg[14] = target NTP maxerr */
                list_of_packets6[packetid]->source_ntp = atof(arg[13]) * 1000.0;
                list_of_packets6[packetid]->target_ntp = atof(arg[15]) * 1000.0;
            }

         }
      } 
   }
}
   /* Close file descriptor */
   if (readpipe) {
      pclose (fp);   /* pclose does a wait(3) to prevent zombies */
   }
   else {
      fclose (fp);   
   }

   return TRUE;
}
