#include <stdio.h>
#include <strings.h>
#include <stdlib.h>
#include <assert.h>
#include <sys/time.h>

#include "packet.h"
#include "merge_packets.h"

static char const rcsid[] = "$Id: sort_search.c,v 1.5 2007/02/06 12:12:16 ruben Exp $";

/* Sort the list_of_packets array */

int compare_packet(const void *a, const void *b)
{
	const struct packet * const *p_a = (const struct packet * const *)a,
						* const *p_b = (const struct packet * const *)b;
	return (*p_a)->packet_id < (*p_b)->packet_id ? -1 : ((*p_a)->packet_id > (*p_b)->packet_id ? 1 : (*p_a)->target_id - (*p_b)->target_id);
}

void sort_packets (int debug) {

	struct timeval tp;

	if (debug)
		gettimeofday(&tp,NULL);

	if (number_of_packets) {
		struct packet **base=list_of_packets;
		if (mergesort(base, number_of_packets, sizeof(struct packet*), compare_packet)) {
			perror("sort_packets for IPv4");
			abort();
		}
		assert(base == list_of_packets);
	}
	if (number_of_packets6) {
		if (mergesort(list_of_packets6, number_of_packets6, sizeof(struct packet*), compare_packet)) {
			perror("sort_packets for IPv6");
			abort();
		}
	}

	if (debug) {
		struct timeval tp2,res;
		gettimeofday(&tp2,NULL);
		timersub(&tp2,&tp,&res);
		printf("Sorting for %u elements took %ld.%.6ld\n", number_of_packets + number_of_packets6, res.tv_sec,res.tv_usec);
	}
}


/* Search packet in first numpackets of array:
   return index if found, -1 if not found */

int search_packet(unsigned int pcktid, int srcid, int trgid, int numpackets) {

	int low, high, med, found;
	int p_index=-1;

	low   = 0;
	high  = numpackets - 1;
	found = FALSE;

	if ((high > 0) && (srcid != list_of_packets[0]->source_id)) {

		/* all packets in list are from same source; if first
		   one doesn't match, no point in searching entire list */

		return(-1);
	}

	while ((low != high) && (high > 0)) {
		med   = (low + high) / 2;

		if (list_of_packets[med]->packet_id == pcktid) {
			if (list_of_packets[med]->target_id == trgid) {
				found = TRUE;
				p_index = med;
				low = high;
				break;
			}

			if ((list_of_packets[med]->target_id > trgid)) {
				med--;
				found = FALSE;
				while (med >= low  &&  list_of_packets[med]->packet_id == pcktid) {
					if (list_of_packets[med]->target_id == trgid) {
						found = TRUE;
						p_index = med;
						low = high;
						break;
					}
					med--;
				}
				low = high;
				break;
			}
			/* the case where trgid > target_id
			   will be considered below */


		}
		if (list_of_packets[med+1]->packet_id == pcktid) {
			med++;
			if (list_of_packets[med]->target_id == trgid) {
				found = TRUE;
				p_index = med;
				low = high;
				break;
			}

			if (list_of_packets[med]->target_id < trgid) {

				/* the case where target_id > trgid
				   has already been considered above */

				med++;
				found = FALSE;
				while (med <= high  &&  list_of_packets[med]->packet_id == pcktid) {
					if (list_of_packets[med]->target_id == trgid) {
						found = TRUE;
						p_index = med;
						break;
					}
					med++;
				}
				low = high;
			}
			else {
				/* not found */
				low = high;
			}

		}
		else if ((high-low) == 1) {
			/* not found */
			low = high;
		}
		else if (list_of_packets[med]->packet_id > pcktid) {
			/* search in lower half */
			high = med;
		}
		else {
			/* search in upper half */
			low  = med;
		}
	}

	if (found == TRUE) {
		return(p_index);
	} 
	else {
		return(-1);
	}

}

int search_packet6(unsigned int pcktid, int srcid, int trgid, int numpackets) {

        int low, high, med, found;
        int p_index=-1;

        low   = 0;
        high  = numpackets - 1;
        found = FALSE;

        if ((high > 0) && (srcid != list_of_packets6[0]->source_id)) {

                /* all packets in list are from same source; if first
                   one doesn't match, no point in searching entire list */

                return(-1);
        }

        while ((low != high) && (high > 0)) {
                med   = (low + high) / 2;

                if (list_of_packets6[med]->packet_id == pcktid) {
                        if (list_of_packets6[med]->target_id == trgid) {
                                found = TRUE;
                                p_index = med;
                                low = high;
                                break;
                        }

                        if ((list_of_packets6[med]->target_id > trgid)) {
                                med--;
                                found = FALSE;
                                while (med >= low  &&  list_of_packets6[med]->packet_id == pcktid) {
                                        if (list_of_packets6[med]->target_id == trgid) {
                                                found = TRUE;
                                                p_index = med;
                                                low = high;
                                                break;
                                        }
                                        med--;
                                }
                                low = high;
                                break;
                        }
                        /* the case where trgid > target_id
                           will be considered below */


                }
                if (list_of_packets6[med+1]->packet_id == pcktid) {
                        med++;
                        if (list_of_packets6[med]->target_id == trgid) {
                                found = TRUE;
                                p_index = med;
                                low = high;
                                break;
                        }

                        if (list_of_packets6[med]->target_id < trgid) {

                                /* the case where target_id > trgid
                                   has already been considered above */

                                med++;
                                found = FALSE;
                                while (med <= high  &&  list_of_packets6[med]->packet_id == pcktid) {
                                        if (list_of_packets6[med]->target_id == trgid) {
                                                found = TRUE;
                                                p_index = med;
                                                break;
                                        }
                                        med++;
                                }
                                low = high;
                        }
                        else {
                                /* not found */
                                low = high;
                        }

                }
                else if ((high-low) == 1) {
                        /* not found */
                        low = high;
                }
                else if (list_of_packets6[med]->packet_id > pcktid) {
                        /* search in lower half */
                        high = med;
                }
                else {
                        /* search in upper half */
                        low  = med;
                }
        }

        if (found == TRUE) {
                return(p_index);
        } 
        else {
                return(-1);
        }

}
