#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "testbox.h"
/* #include "rv.h" */

#define MAXWORDS 100

static char const rcsid[] = "$Id: testbox.c,v 1.11 2007/02/06 11:57:44 ruben Exp $";

int read_test_box_config (int format, const char* tbfile, int debug) {
/*  ====================
    Read test-box data from file tbfile, either (format=) ASCII or BINARY format,
    stores data into the testbox structure.
*/

   char    line[MAXWORDS];
   char   *point = &line[0];
   char  **ap;
   char   *arg[MAXWORDS];
   int     i;
   FILE   *fp;

   number_of_boxes = 0;
   memset(test_box_list,0,sizeof(test_box_list));

   /* Open file */
   if (format == ASCII) {
      fp = fopen (tbfile, "r");
   } else if (format == BINARY) {
      fp = fopen (tbfile, "rb");
   } else {
      printf ("Unknown format\n");
      return FALSE;
   }

   /* Check if file is actually opened */
   if (fp == NULL) {
      printf ("RTBC: Cannot open list of testboxes %s\n", tbfile);
      return FALSE;
   }

   /* Read data from file */
   if (format == ASCII) {
      while ( (point=fgets(line, MAXWORDS, fp)) != NULL ) {
         /* # sign indicates comment */
         if (line[0] == '#') { continue; }
         /* Split line into pieces */

#if defined(USE_STRTOK)            /* POSIX compliant */
         ap = arg;
         *ap = strtok (point, " \t");
         while (*ap != NULL) {
             if (**ap != '\0') {
                ++ap;
             }
             *ap = strtok (NULL, " \t");
         }
#else                              /*  BSD specific */
         for (ap = arg ; (*ap=strsep(&point, " \t")) != NULL ; ) {
            if (**ap != '\0') {
               ++ap;
            }
         }
#endif

         test_box_list[number_of_boxes].id = atoi(arg[0]);
         strcpy (test_box_list[number_of_boxes].name, arg[1]);
	 if (strstr(arg[2], ".")) {
		strncpy (test_box_list[number_of_boxes].ip_addr, arg[2], strlen(arg[2])-1);
		test_box_list[number_of_boxes].family = AF_INET;
	 } else if (strstr(arg[2], ":")) {
		strncpy (test_box_list[number_of_boxes].ip6_addr, arg[2], strlen(arg[2])-1);
		test_box_list[number_of_boxes].family = AF_INET6;
	 }
	 
	 if ( ++number_of_boxes == MAXTESTBOXES ) {
		fprintf(stderr, "ERROR: Max number of testboxes exceeded increase MAXTESTBOXES in testbox.h\n");
		exit(1);
	 }
      }
   } else if (format == BINARY) {
/*      i = 0;
      while ( feof(fp) == 0) {
         nread = fread ( &test_box_list[i].id, sizeof(int), 1, fp);
         if (feof(fp) != 0) { break; }
         nread = fread ( &j, sizeof(int), 1, fp);
         nread = fread ( &test_box_list[i].name[0], sizeof(char), j, fp);
         nread = fread ( &test_box_list[i].ipno, sizeof (unsigned int), 1, fp);
         i++;
         number_of_boxes++;
      } */ 
   }
   fclose (fp);

   if (debug == TRUE) {
      /* List test-boxes */
      for (i=0 ; i<number_of_boxes ; i++) {
          if (strstr(test_box_list[i].ip_addr,".")) printf ("%d\t%s\t%s\n", test_box_list[i].id,
              test_box_list[i].name, test_box_list[i].ip_addr);
          if (strstr(test_box_list[i].ip6_addr,":")) printf ("%d\t%s\t%s\n", test_box_list[i].id,
              test_box_list[i].name, test_box_list[i].ip6_addr);
      }
   }
   
   return TRUE;
}

int test_box_id (const char * test_box_name) {
/*  ===========
    Returns ID for a testbox "test_box_name"
*/

   int i;
   for (i=0 ; i<number_of_boxes; i++) {
      if (strcmp(test_box_list[i].name, test_box_name) == 0) {
         return test_box_list[i].id;
      }
   }
   printf ("Host %s Unknown in testbox config\n", test_box_name);
   return -1;
}

int test_box_ip_id (char * test_box_ip, int family) {
/*  ==============
    Returns ID for a test-box with IPv4 or IPv6 address
*/

   int i;

   if (family == AF_INET) {
        for (i=0; i<number_of_boxes; i++) {
                if (strcmp(test_box_list[i].ip_addr,test_box_ip) == 0)
                        return test_box_list[i].id;
      }
   }
   else if (family == AF_INET6) {
        for (i=0; i<number_of_boxes; i++) {
                if (strcmp(test_box_list[i].ip6_addr,test_box_ip) == 0)
                        return test_box_list[i].id;
      }
   }
   else printf("oha!\n");

   printf ("Warning : Unknown testbox %s\n", test_box_ip);
   return -1;
}

