#!/usr/local/bin/perl -w
use strict;
use POSIX;

#
# $Id: pathmtu.pl,v 1.2 2006/11/04 12:44:37 ruben Exp $
#
# Copyright (c) 2000  RIPE NCC
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of the author not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
#------------------------------------------------------------------------------
# Module Header
# Filename          :   pathmtu.pl
# Purpose           :   Automated generation of TTM webpages in html
# Authors           :   Michael Swoboda <michael@ripe.net>, RIPE NCC
# Date              :   27-AUG-2003
# Description       :   Component of TTwwwpages for Test Traffic project
# Usage:		pathmtu.pl
#
#------------------------------------------------------------------------------

$DATADIR="/ncc/ttpro/data/pmtu";
$OUTPUTDIR="/ncc/ttpro/pmtu/html";

my $progname = $0;

$i = 0;
$j = 0;
$z = 0;
$currentdate = strftime("%Y%m%d",localtime());
$year = strftime("%Y",localtime());
$month = strftime("%B",localtime());
$day = strftime("%d",localtime());
$currentday = $day; 

$tbNo = `/ncc/ttpro/bin/ttconfig -v IP6TTLIST | wc -l`; 

opendir( DIR, "$DATADIR/" || die ("Cannot open directory"));
@testboxes = grep { /^PMTU\.tt\d+\.ripe\.net\.${currentdate}$/ } readdir(DIR);
closedir(DIR);

open( OUT, ">$OUTPUTDIR/index.html" ) || die "Cannot open file for write\n";
print (OUT "<!--#set var=\"title\" value=\"Path MTU detecion\"-->");
print (OUT "<!--#include virtual=\"/ssi/ttmhead1.html\"-->");
print (OUT "<P>");
print (OUT "<CENTER><H1>Path MTU Detection</H1><H3>Collected on the $currentday. $month $year</H3>");
print (OUT "<TABLE BORDER=1 CELLPADDING=5 NOSAVE>");

$dtbNo = $tbNo + 2;
print(OUT "<TR><TH COLSPAN = \"$dtbNo\">Destination Testbox</TH><TR>");

print(OUT "<TR>");
print(OUT "<TD>&nbsp;</TD>");
print(OUT "<TD>&nbsp;</TD>");


#find file with recent date

@sortTestboxes =  sort(@testboxes);
foreach $x (@sortTestboxes)
{
	@tb2 = split(/\./, $x);
	#date of the file 
	$filedate = @tb2[4];
	$dtb2 = @tb2[1];
	print(OUT "<TH>$dtb2</TH>");

	push(@tbarray,$dtb2);
}

print(OUT "</TR>");
$stbNo = $tbNo + 1;
print(OUT "<TR><TH ROWSPAN = \"$stbNo\">Source Testbox</TH>");

foreach $l (@sortTestboxes)
{
	@tb = split(/\./, $l);
	$filedate = @tb[4];
		
	@tb = split(/\./, $l);
	$stb = @tb[1];
	$stbfile = $l;
	
	$filename = "$DATADIR/$stbfile";
	open( INFO, $filename) || die "Cannot open file\n";
	@DTBLIST = <INFO>;
	close(INFO);
	
	#print source boxes
	print(OUT "<TH>$stb</TH>");

	$i = 0;

	#returns the latest hour in the file
	foreach $ww (@DTBLIST)
	{
		$w = $ww;
		chop $w;
		$lastdumpHH = calcHours($w);
	}

	foreach $gg (@DTBLIST)
	{
		$g=$gg;
		chop $g;
		if (((checkerr($g)) == 1) && ($i <= $tbNo-1)) 
		{
			if ($i == $j)
		        {
				print(OUT "<TD BGCOLOR=\"999999\">&nbsp;</TD>");
			}
			else
			{
				print(OUT "<TD BGCOLOR=\"red\">ERR</TD>");
			}
			$i++;
		}
		else
		{
			$fileavailable = 0;
			$hours = calcHours($g);
			$sourcetb = linesource($g);
			$desttb = linedest($g);
			
			foreach (@tbarray)
			{
				if ($_ eq $desttb) 
				{
					$fileavailable = 1;
				}
			}

			if(($hours == $lastdumpHH) && ($stb eq $sourcetb) && ($fileavailable == 1))
			{
			
				@line = split(' ', $g);
				$dtb1 = @line[2];
				@part = split(/\./, $dtb1);
				$dtb = @part[0];
				$mtu = @line[3];
		
				#if the source and destination tb is the same, print a grey cell
				#if ($i == $j)
				
				if ($sourcetb eq $desttb)
				{
					print(OUT "<TD BGCOLOR=\"999999\">&nbsp;</TD>");
				}
				else
				{	
					if ($mtu < 1500)
					{	
						$color = "FFFF66";
					}
					else
					{
						$color = "33CC33";
					}
					print(OUT "<TD BGCOLOR=\"$color\">$mtu</TD>");
				}
				$i++;
			}
		}
	}
	$j++; 
		
	print(OUT "</TR>\n");
}


print (OUT "</TABLE></CENTER");

print(OUT "<BR><BR><BR><BR><BR>");
print (OUT "<!--#include virtual=\"/ssi/ttmfoot1.html\"-->");

close (OUT);

sub calcHours
{
	my $g = shift;
	#print "$g ";
	
	if((checkerr($w)) != 1)
	{
	
		@line = split(' ', $g);
		$timestamp = @line[0];
	
		$linedate = scalar gmtime($timestamp);

		@date = split(' ',$linedate);
		$time = @date[3];
	
		@HH = split(/:/,$time);
		$timeHH = @HH[0];
	}
	return ($timeHH);							
}

sub linedest
{
	my $g = shift;
	@line = split(' ', $g);
	$tbname = @line[2];
	@tbnumber = split(/\./, $tbname);
	$tbno = $tbnumber[0];
	return ($tbno);
}

sub linesource
{
	my $g = shift;
	@line = split(' ', $g);
	$tbname = @line[1];
	@tbnumber = split(/\./, $tbname);
	$tbno = $tbnumber[0];
	return ($tbno);
}
						
sub checkerr
{

	$err = 0;
	my $g = shift;
	@line = split(' ', $g);
	$error = @line[1];
	if ($error eq "ERROR")
	{
		$err = 1;
	}
	
	return ($err);
						
}

