#!/usr/local/bin/perl -Wall
#
# $Id: tunneltrace.cgi,v 1.2 2006/11/04 14:40:50 ruben Exp $
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of the author not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
#------------------------------------------------------------------------------
# Module Header
# Filename          :   tunneltrace.pl
# Purpose           :   Automated generation of TTM webpages in html
# Authors           :   Michael Swoboda <michael@ripe.net>, RIPE NCC
# Date              :   24-SEP-2003
# Description       :   Component of TTwwwpages for Test Traffic project
# Usage:                tunneltrace.pl
#------------------------------------------------------------------------------

use strict;
use Data::Dumper;
use CGI;
use CGI::Carp;
my $cgi = new CGI;

my $TTROOT="/ncc/ttpro/data/pmtu";
my $OUTPUTDIR="$TTROOT/html";

my $year;
my $month;
my $day;
my @currentBoxes;

#switch for 0 == all boxes, 1 == only complete native boxes, 2 == only complete tunneled boxes, 3 == selected boxes 
my $boxSwitch = 0;

#switch for 0 == defaultDate, 1 == selectedDate
my $dateSwitch = 0;

#switch for 0 == defaultTime, 1 == selectedTime
my $timeSwitch = 0;

my @directory = openDir();

my $dirPointer = \@directory;
my $hour = 0;

#my $year = $cgi->param('year');
#my $month = $cgi->param('month');
#my $day = $cgi->param('day');
#my $hour = $cgi->param('hour');

if ($timeSwitch == 0)
{
	$hour = getDefaultHour();
}
else
{
	$hour = getSelectedHour();
}

my $date = "";

if ($dateSwitch == 0)
{
        $date = getDefaultDate(@directory);
}
else
{
        $date = getSelectedDate();
}

@currentBoxes = getBoxes($date, @directory);

if ($boxSwitch == 3)
        {
                @currentBoxes = modifyBoxes(@currentBoxes);
        }


my $table;

$table = fillTableArray($date, $hour, $dirPointer, @currentBoxes);

calculateTimeFormat($date, \$year, \$month, \$day);

createHtmlTable($table, \$year, \$month, \$day, $hour, @currentBoxes);



########################################################################
# subroutines
########################################################################

#open directory and read it into array, sort array 
#return value: @sortDir
sub openDir
{
	opendir (DIR, "$TTROOT/data/" || die ("Cannot open directory"));
	my @dir = readdir(DIR);
	closedir(DIR);

        my @sortDir = sort(@dir);
	return (@sortDir);

}

#get the selected date for the desired query
#return value: @Date
sub getSelectedDate
{
	#format Date: year, month, day
	my $date = "20030929";
	return ($date);
}

#get the selected hour for the desired query
#return value: $hour
sub getSelectedHour
{
        my $hour = "09";

        return ($hour);
}

#get the default hour
#return value: $hour
sub getDefaultHour
{
        my $hour = `date +%H -u`;
	chop($hour);
	
	#if there is in hour x no file(beacause it's created every 15min after every hour) the hour is decreased by one
	
	#if the first digit is a zero it has to be added again
	if ((substr($hour,0,1)) eq "0")
	{
		$hour--;
		$hour = "0$hour";
	}
	else
	{
		$hour--;
	}
	return ($hour);
	
}

#get the default date of the directory, which is the last one in the sorted directory array
#parameter(s): @directory
#return value: $fileDate
sub getDefaultDate
{
	my @dir = @_;
	my $fileDate;
	
	#last day in the directory;
	#my $i;
	#foreach $i (@dir)
	#{
	#        unless (($i eq ".") || ($i eq "..") || ($i !~ "PMTU"))
	#
        #	{
	#		my @splitedLines = split(/\./, $i);
	#	        $fileDate = $splitedLines[4];
	#	}
	#}
	
	#today;
	$fileDate = `date  -u +%Y%m%d`;
	chop($fileDate);
	#/print Dumper $fileDate;

	
	return ($fileDate);
}

#get the current boxes from the directory
#parameter(s): $date, @directory
#return value: @boxes
sub getBoxes
{
	my $date = shift;
	my @dir = @_;
	my @boxes;
	my $i;
	foreach $i (@dir)
        {
                unless (($i eq ".") || ($i eq "..") || ($i !~ "PMTU"))
                {
			my @splitedLines = split(/\./, $i);
			my $box = $splitedLines[1];
			if (($splitedLines[4]) eq $date)
			{
				push(@boxes, $box);
			}
                }
        }
	return (@boxes);
}


#create a two dimensional array, and fills it with the mtu values
#parameter(s): $date, $time, $dirPointer, @boxes
#return value: $table
sub fillTableArray
{

        my $date = shift;
	my $hour = shift;
	my $dirPointer = shift;
	my @boxes = @_;
	my $x;
	my $y;
	my $i;
	my $table;
	my $noBoxes = @boxes;
	my $boxNo = "";
	my @boxData;
	my $count = 0;
		
	for ($y=0;$y<$noBoxes;$y++)
	{
		#create two-dimensional matrix $y==Yaxis, $x==Xaxis
		#fill the left side with sourceboxes;
		$table->[$y+1]->[0] = $boxes[$y];
	}

	for ($x=0;$x<$noBoxes;$x++)
        {
		#fill the upper side with destionation boxes
                $table->[0]->[$x+1] = $boxes[$x];
        }
	
	my @mtuTemp;
	#gets every line of a file and puts it into an array
	foreach $i (@boxes)
	{
		$boxNo = $i;
		
		#gets the content of the file from one testbox	
		@boxData = getBoxData($date, $boxNo);
		my $line;
		
		#reads from a file every the mtu size and push it into an array 
		foreach $line (@boxData)
		{
			chop $line;
			my $lineHH = calcHours($line);
			my $sourceBox = getSourceBox($line);
			my $destBox = getDestBox($line);
			
			my $fileAvailable = 0;
			#checks if there is also a file for the data of the specific testbox
			foreach (@boxes)
                        {
	                        if ($_ eq $destBox)
                        	{
                                	$fileAvailable = 1;
                        	}
                        }
			
			#reads the mtu size, but only when:
			#the hours of the entry are the same as the desired hours
			#the sourcebox is not the destination testbox
			#when also a file is available for this entry
			if (($lineHH eq $hour) && ($boxNo ne $destBox) && ($fileAvailable == 1))
			{
				
				my $mtuSize;
				if (checkErr($line) == 1)
				{
					$mtuSize = "ERR"	
				}

				else
				{
					my @lineArray = split(' ', $line);
        	                        $mtuSize = $lineArray[3];
				}

				push (@mtuTemp, $mtuSize);
				my $numberMtu = @mtuTemp;

			}
			
		}

	}
	my $xx;
	my $yy;
	my $mtuCount = 0;

	#fills the two dimensional array with the mtu values
	for ($xx=1;$xx<($noBoxes+1);$xx++)
        {
                for($yy=1;$yy<($noBoxes+1);$yy++)
                {
			if ($xx != $yy)
			{
				$table->[$yy]->[$xx] = $mtuTemp[$mtuCount];
				$mtuCount++;
			}

			#if the source and destination testbox is the same; write 0
			else 
			{	
				$table->[$yy]->[$xx] = 0;
			}
                }

        }
	$table->[0]->[0] = 0;

 	

	#print Dumper $table;
	return ($table);
}


#modifys the boxes; which box is choosen or not
#parameter(s): @boxes
#return value: @boxes
sub modifyBoxes
{
	my @boxes = @_;
	my $noBoxes = @boxes;
	for (my $x=0; $x<=$noBoxes ;$x++)
	{
		my $temp = rand(10);
		if ($x == 5)
		{
			splice(@boxes, $x, 1);
		}

	}

	return (@boxes);


}


#get the lines of a testboxfile
#parameter(s): $date, $boxNo
#return value: @boxData
sub getBoxData
{
	my $date = shift;
	my $boxNo = shift;
	my @boxData;
	my $filename = "PMTU.$boxNo.ripe.net.$date";
	
	my $pathName = "$TTROOT/data/$filename";
        
	open (TBFILE, $pathName) || die "Cannot open file\n";
        @boxData = <TBFILE>;
        close (TBFILE);
	
	return @boxData;
}

#checks one line of a file, if there was an ERROR message caused by the shell script
#parameter(s): $g
#return value: $err
sub checkErr
{
	
	my $err = 0;
        my $g = shift;
	my $error;
        my @line = split(' ', $g);
        $error = $line[1];
        if ($error eq "ERROR")
        {
                $err = 1;
        }

        return ($err);

}

#gets a line and calculate the hours of a unix timestamp
#parameter(s): $g
#return value: $timeHH
sub calcHours
{
        my $g = shift;
	my @line;
	my $timestamp;
	my $line;
	my @date;
	my $time;
	my $timeHH;
	my $linedate;
	my @HH;
		
        if((checkErr($g)) != 1)
        {

                @line = split(' ', $g);
                $timestamp = $line[0];

                $linedate = scalar gmtime($timestamp);

                @date = split(' ',$linedate);
                $time = $date[3];

                @HH = split(/:/,$time);
                $timeHH = $HH[0];
        }
        return ($timeHH);
}

#gets a line and filters the destination box
#parameter(s): $g
#return value: $tbno
sub getDestBox
{
        my $g = shift;
        my @line = split(' ', $g);
        my $tbname = $line[2];
        my @tbnumber = split(/\./, $tbname);
        my $tbno = $tbnumber[0];
        return ($tbno);
}

#gets a line and filters the destination box
#parameter(s): $g
#return value: $tbno
sub getSourceBox
{
        my $g = shift;
        my @line = split(' ', $g);
        my $tbname = $line[1];
        my @tbnumber = split(/\./, $tbname);
        my $tbno = $tbnumber[0];
        return ($tbno);
}

#calulate day, month(written) and year
#parameter(s): $date, \$year, \$month, \$day
#return value: -
sub calculateTimeFormat
{
	my $date = shift;
	my $year = shift;
	my $month = shift;
	my $day = shift;

	${$year} = substr($date,0,4);
	${$month} = substr($date,4,2);
	${$day} = substr($date,6,2);
	${$year} = substr($date,0,4);
}


#prints Html table
#parameter(s): $table, \$year, \$month, \$day, $hour
#return value: -
sub createHtmlTable
{
	my $table = shift;
	my $year = shift;
        my $month = shift;
	my $day = shift;
	my $hour = shift;
	my @currentBoxes = @_;
	my $noBoxes = @currentBoxes;
	my $now = `date | cut -f5 -d" "`;
	my $colspanFields = $noBoxes + 2;
	my $rowspanFields = $noBoxes + 1;
	
	#conversion from UTC time to localtime
	#$hour = $hour + 2;	

	#open( OUT, ">$OUTPUTDIR/test.index.html" ) || die "Cannot open file for write\n";
	print "Content-Type: text/html\n\n";
	
	print ("<!--#set var=\"title\" value=\"Tunneltrace with PMTU\"-->");
	print (OUT "<!--#include virtual=\"/ssi/ttmhead1.html\"-->");
	print (OUT "<p>");
	print (OUT "<center>");
        print (OUT "<h1>Tunneltrace with PMTU</h1>");
        print (OUT "<h3>Collected on the ${$day}.${$month}.${$year} at $hour o'clock(UTC).</h3>");
	print (OUT "HTML file created at: $now<BR><BR>");
	print (OUT "<a href=\"faq.html\" target=\"_blank\">FAQ<a/>");
	print (OUT "<table border=1 cellpadding=5 nosave>");
	print (OUT "<tr><td>&nbsp;</td><td>&nbsp;</td>");
	print (OUT "<th colspan = \"$colspanFields\">Destination Testbox</th></tr>");
	print (OUT "<th rowspan = \"$rowspanFields\">Source Testbox</th>");
	

	for ( my $l=0;$l<=$noBoxes;$l++)
	{	
		my $temp = $table->[0][$l];
		if ($temp eq "0")
		{	
			print (OUT "<td bgcolor=\"999999\">&nbsp;</td>");
		}
		else
		{
			print (OUT "<td>$temp</td>");
		}
	}
	print (OUT "</tr>");
	


	for (my $m=1;$m<=$noBoxes;$m++)
        {
		print (OUT "<tr>");		
		for (my $n=0;$n<=$noBoxes;$n++)
		{
			my $temp = $table->[$n][$m];
			my $color;
			if (($temp ne 1500) && ($n != 0))
                        {

				$color = "FFFF66";
                        }
                        elsif (($temp eq 1500) && ($n != 0))  
                        {
				$color = "33CC33";
                        }
			else
			{
				$color = "white";
			}
			
			if ($temp eq "0")
			{
				print (OUT "<td bgcolor=\"999999\">&nbsp;</td>");
			}
			else
			{
				print (OUT "<td bgcolor=\"$color\">$temp</td>");			
			}
		}
		print (OUT "</tr>");
        }

	print (OUT "</table>");
	print (OUT "<br><br>");
	print (OUT "<form>");
	print (OUT "<table border =\"0\">");
	print (OUT "<tr><th>Year</th><th>Month</th><th>Day</th><th>Hour</th></tr>");
	print (OUT "<tr>");  
	print (OUT "<td");
	print (OUT "<select name=\"year\" size=\"1\">");
        print (OUT "<option>2003</option>");
	print (OUT "<option>2004</option>");
	print (OUT "<option>2005</option>");
	print (OUT "<option>2006</option>");
	print (OUT "<option>2007</option>");
	print (OUT "<option>2008</option>");
	print (OUT "<option>2009</option>");
	print (OUT "<option>2010</option>");
	print (OUT "</select>");
	print (OUT "</td>");
        
	print (OUT "<td");
	print (OUT "<select name=\"month\" size=\"1\">");
        for(my $i=1;$i<=12;$i++)
        {
		print (OUT "<option>$i</option>");
        }
	print (OUT "</select>");
        print (OUT "</td>");

	print (OUT "<td>");
        print (OUT "<select name=\"day\" size=\"1\">");
        for(my $j=1;$j<=31;$j++)
        {
		print (OUT "<option>$j</option>");
        }
	print (OUT "</select>");
	print (OUT "</td>");

	print (OUT "<td>");
        print (OUT "<select name=\"hour\" size=\"1\">");
        for(my $k=1;$k<=24;$k++)
        {
                print (OUT "<option>$k</option>");
        }
        print (OUT "</select>");
        print (OUT "</td>");

	print (OUT "</tr>");       
	print (OUT "</table>");

 
	print (OUT "<table border=\"0\">");
	print (OUT "<tr>");
	print (OUT "<th colspan=\"20\">Select all testboxes:");
        print (OUT "<input type=\"checkbox\" name=\"selectAll\" value=\"selectAll\">");
        print (OUT "</th>");
	print (OUT "</tr>");
        
	print (OUT "<tr>");
        print (OUT "<th colspan=\"20\">Select all native testboxes:");
        print (OUT "<input type=\"checkbox\" name=\"selectNative\" value=\"selectNative\">");
        print (OUT "</th>");
        print (OUT "</tr>");

	print (OUT "<tr>");
        print (OUT "<th colspan=\"20\">Select all tunneled testboxes:");
        print (OUT "<input type=\"checkbox\" name=\"selectTunneled\" value=\"selectTunnneled\">");
        print (OUT "</th>");
        print (OUT "</tr>");

	print (OUT "<tr>");
        print (OUT "<th colspan=\"20\">&nbsp;");
        print (OUT "</th>");
        print (OUT "</tr>");

	print (OUT "<tr>");
        print (OUT "<th colspan=\"20\">Choose desired testboxes:</th>");
	print (OUT "</tr>");
	print (OUT "<tr>");
        print (OUT "<td>");
    

	for(my $x=0;$x<=($noBoxes-1);$x++)
	{
		print (OUT "<input type=\"checkbox\" name=\"tb\" value=\"$currentBoxes[$x]\" checked>$currentBoxes[$x]&nbsp;");
		
		for (my $five=4; $five<=100; $five = $five+5)
		{
			if ($x==$five)
			{
				print (OUT "<td/></tr><tr><td>"); 
			}
		}
	}
        print (OUT "</td>");
        print (OUT "</tr>");
        print (OUT "</table>");
        print (OUT "<br><br>");
        print (OUT "<input type=\"submit\" value=\"Start query\">");
        print (OUT "</form>");

}

