#include "TROOT.h"
#include "TTree.h"
#include "TChain.h"
#include "TFile.h"
#include "TBranch.h"

#include "Delay.h"


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <errno.h>
#include <sys/types.h>
#include <unistd.h>

#define CONFIGFILE "/ncc/ttpro/config/LIST_OF_TESTBOXES"
#define ROOTDATADIR  "/ncc/ttpro/root_data"
#define OUTPUTDIR  "/ncc/ttpro/spool"
#define MAXCHARS   2048

char *progname;
int errno;

// forward declaration
Int_t name2id (char* name, char* configfile);

// Usage
void usage () {
   fprintf(stderr, "Usage: %s <date (yyyymmdd)>\n", progname);
   exit(1);
}
	
//--------------
// MAIN PROGRAM 
//--------------
main (int argc, char **argv) {
 
	extern char *optarg;
	extern int optind;

	int ch, oflag, verbose;

	int day, month, year;
	char *datestring, *hostname, *filename;
	char *suffix, pathname[MAXCHARS];

	struct dirent *direntry;
	Int_t host_id, eventNo;

	TROOT simple ("Simple", "Example");

	Int_t split = 1;
	Int_t bsize = 64000;
	Delay *delay = 0;

	// For the input file.
	FILE  *fp;
	DIR   *dirp;
	TFile *hfile;
	int    eventsfound, files, accept;

	oflag = 0;
	verbose = 0;
	progname=argv[0];

// for the time being: no option flags! 

	while ((ch = getopt(argc, argv, "v")) != -1)
	   switch(ch) {
	   case 'o':
	 if (oflag == 0) {
	         oflag = 1;    
	    // non-default output directory
	      }
	      else {
	         usage();
	      }
	 break;
	   case 'v':
	         verbose = 1;    
	    break;
	   default:
	      usage();
	}
	argc -= optind;
	argv += optind;

	
	// only two arguments may be left to process now: hostname and date!

	if (argc != 1) {
	   usage();
	}

	// get ID of specified hostname

//	hostname = argv[0];
//	host_id = name2id(hostname, CONFIGFILE);
	
	// decode date

//	datestring=argv[1];
	datestring=argv[0];
	if (strlen(datestring) != 8 ||
	    sscanf(datestring, "%4d%2d%2d", &year, &month, &day) != 3) {
	      fprintf (stderr,"%s: ERROR decoding datestring %s\n", progname, argv[1]);
	 usage();
	}


	// open directory with per host ROOT files (info on data _send_ by host)

	sprintf(pathname, "%s/%4.4d/%2.2d/%2.2d", ROOTDATADIR, year, month, day);
	if ((dirp = opendir(pathname)) == NULL) {
	fprintf(stderr, "%s: opening directory %s: %s\n", progname,
		pathname, strerror(errno));
	}
	
	// open output file, create new Tree on this file

	sprintf(pathname, "%s/%s.%s.root", OUTPUTDIR, "datagrid", datestring);
	hfile = new TFile (pathname, "RECREATE");

	TTree *tree  = new TTree ("tree", "Example");
	delay = new Delay();
	tree->Branch ("delay", "Delay", &delay, bsize, split);


	// chain all .root files in directory 

	TChain chain("tree");
	while ((direntry = readdir(dirp)) != NULL) {
		filename=direntry->d_name;
		if (((suffix=strrchr(filename, '.')) != NULL) && 
		    (strcmp(suffix, ".root") == 0)) {
			sprintf(pathname, "%s/%4.4d/%2.2d/%2.2d/%s",
			    ROOTDATADIR, year, month, day, filename);
			chain.Add(pathname);
		}
	}
	closedir(dirp);

	chain.SetBranchAddress("delay", &delay);
	Int_t nTotalEvents = chain.GetEntries();
	if (verbose == 1) {
		printf("Total #entries in %s chain: %d\n", datestring, nTotalEvents);
	}

        eventsfound=0;
	for (eventNo=0; eventNo < nTotalEvents; eventNo++) {

	        // read next delay into memory 

		if ( (chain.GetEvent(eventNo) < 0) ) {
			// problem reading this event ?
            		continue;
        	}

		// if source or target matches: add to output Tree

		uint src = delay->GetSourceId();
		uint dst = delay->GetTargetId();
		if (((src == 13) || (src == 8) || (src == 28)  ||
		     (src == 31) || (src == 53) || (src == 54) ||
		     (src == 57) || (src == 58) || (src == 59) ||
		     (src == 63))	&&
		    ((dst == 13) || (dst == 8) || (dst == 28)  ||
		     (dst == 31) || (dst == 53) || (dst == 54) ||
		     (dst == 57) || (dst == 58) || (dst == 59) ||
		     (dst == 63))) {
			eventsfound++;
			tree->Fill ();
		}
	}
	if (eventsfound == 0) {
		fprintf(stderr, "%s: no events found for testbox id %d !?\n",
			progname, host_id);
	}
	else if (verbose == 1) {
		printf("testbox id %d: %d events found for %s\n", host_id,
			eventsfound, datestring);
	}

	hfile->Write ();
	hfile->Close ();

	return 0;
} 


Int_t name2id (char* name, char* configfile) {

// find host name in configfile, return the ID 

	char    line[MAXCHARS];
	char   *point = &line[0];
	char  **ap;
	char   *token[MAXCHARS];
	FILE   *fp;
	Int_t  id;

	fp = fopen (configfile, "r");

	/* Check if file is actually opened */
	if (fp == NULL) {
		fprintf (stderr, "%s: Cannot open %s?\n",progname, configfile);
		exit(1);
	}

	/* Read data from file */
	while ( (point=fgets(line, MAXCHARS, fp)) != NULL ) {
		/* # sign indicates comment */
		if (line[0] == '#') { continue; }
		/* Split line into pieces */

#if defined(USE_STRTOK)            /* POSIX compliant */
		ap = token;
		*ap = strtok (point, " \t");
		while (*ap != NULL) {
			if (**ap != '\0') {
				++ap;
			}
			*ap = strtok (NULL, " \t");
		}
#else                              /*  BSD specific */
		for (ap = token ; (*ap=strsep(&point, " \t")) != NULL ; ) {
			if (**ap != '\0') {
				++ap;
			}
		}
#endif

		if (strcmp(name, token[1]) == 0 ) {
		// found it !
			id = atoi(token[0]);
			return(id);
		}
	}
	fclose (fp);
	fprintf(stderr, "%s: %s not found in %s\n", progname, name, configfile);
	exit(1);

}

static char const   rcsid[] = "$Id: root4grid.cxx,v 1.2 2006/11/04 14:40:50 ruben Exp $";
static char const  *rcsid_p = rcsid;    // Prevent g++ from optimizing out rcsid
