#------------------------------------------------------------------------------
#
#$Id: bill.pl,v 1.2 2006/11/04 14:40:50 ruben Exp $
#
# Module Header
# Filename          : bill
# Purpose           : check billing data, late payments etc.
# Author            : Roderik/Daniel
# Date              : 17 November 1997 (& earlier)
# Description       :
# Language Version  : Perl (Tested with v5.00401)
# OSs Tested        : BSD/OS 2.1
# Command Line      : reg -x bill
# Input Files       : $REGDIR/* (usually /ncc/registries/*)
# Output Files      :  -
# External Programs : perl module regread.pm
# Problems          :
# To Do             :
# Comments          : This is a 'reg x-script' which can be called with
#                     'reg -x <scriptname>. However, Why not just 'use'
#                     the needed modules in here too? Now it can also be 
#                     called directly if we want...
#                     (Note: 'use strict' is necessary here because the strict
#                     directive apparently doesn't survive a 'do <filename>'.
# The following line will be read by 'reg -x':
# reg-x-description: check billing data, late payments etc.
#------------------------------------------------------------------------------

# checks that have been dropped (can be found in old reg source):
# - checking confirm against commit, commit against bill-categ
# - SMALL & QUARTERLY


use strict;
use regread;
use Time::Local;

my ($regp);    # pointer to registry data
my (%seenbill);       # invoice #s seen 
my (@f);      # registry names
my ($f);
my ($i);      # iterator
my ($type);
my ($tmp,$lvl);
my ($inv,$amt,$due,$bill,$pmt,$comments);
my (@confirm);
my (@commit);
my (%errtxt);
my (%sortval);        # Ro:added - Number that helps sorting output
# is number of days of earliest overdue invoice, 2 when there are only
# other errors, 1 when there is only a 'bill-remind'.
# when normal service, 1000 is added

@f = regfiles(0);
        
while ($f = shift(@f)) {
    my (%inv); # should all be initialised to empty at each round
    my (%pmt);
    my (%due);
    $sortval{$f} = 0;

    $regp = readreg($f);

    # init some values
    ($tmp) = $regp->{'confirm'} =~ /\n([^\n]+)$/;
    @confirm = split(/\s+/, $tmp);
    ($tmp) = $regp->{'commit'} =~ /\n([^\n]+)$/;
    @commit = split(/\s+/, $tmp);

    # commit faxed but not sent?
    if ($commit[0]) {
	if ($commit[2] eq 'sent') {
	    $i = &days($commit[0], &thisdate);
	    $errtxt{$f} .= "  commit due $i days\n" if $i > 7;
	}
    }

    ($lvl)=servicelevel($regp);

    # normal service but no invoice?
    $errtxt{$f} .= "  no invoice\n" if $lvl eq 'NORMAL' && !$regp->{'bill-parent'} && !$regp->{'invoice'};

    # no commitment but not closed?
    $errtxt{$f} .= "  no commitment (confirm $confirm[0] $confirm[1])\n" if $confirm[0] && !$commit[0] && $regp->{'type'} ne 'CLOSED' && !$regp->{'bill-parent'};

    # read invoices (into %inv), check for errors & double bills
    foreach $i (split(/\n/, $regp->{'invoice'})) {
	($tmp,$inv,$comments) = split(/\s+/,$i);
	if ($inv =~ /^\@(\d{6})=(\d+)(%\d+)?#(U\d{5,6})$/) {
	    $due = $1;
	    $amt = $2;
	    $bill = $4;
	} else {
	    $errtxt{$f} .= "  ERROR invoice: $i\n";
	}

	$errtxt{$f} .= "  double invoice $bill, also seen in $seenbill{$bill}\n" if $seenbill{$bill};
	$seenbill{$bill} = $f;
	if ($comments !~ /^REVOKED/) {
	    $inv{$bill} = $amt;
	    $due{$bill} = $due;
	}
    }

    # read payments (into %pmt), check for errors & double payments
    foreach $i (split(/\n/, $regp->{'payment'})) {
	($tmp,$pmt) = split(/\s+/,$i);
	if ($pmt =~ /^=(\d+)(%\d+)?#(U\d{5,6})$/) {
	    $amt = $1;
	    $bill = $3;
	} else {
	    $errtxt{$f} .= "  ERROR payment: $i\n";
	}
	if ($pmt{$bill}) {
	    $errtxt{$f} .= "  Double payment for $bill\n"; 
	} else {
	    $pmt{$bill} = $amt;
	}
    }

    # payment but no invoice?
    foreach $i (sort(keys(%pmt))) {
	$errtxt{$f} .= "  Payment but no invoice for $i\n" if !$inv{$i};
    }

    # payment short or overdue?
    foreach $i (sort(keys(%inv))) {
	if ($pmt{$i}) {
	    $tmp = $inv{$i} - $pmt{$i};
	    if ($tmp > 30) {
		$errtxt{$f} .= "  payment $i short $tmp\n";
	    }
	} else {
	    $tmp = &days($due{$i}, &thisdate);
	    if ($tmp > 10) {
		$errtxt{$f} .= "  payment overdue $tmp days, $i/$inv{$i} ECU, due $due{$i}\n";
		$sortval{$f} = $tmp if $sortval{$f} < $tmp;
	    }
	}
    }

    # do billing-reminders & sort-values
    $errtxt{$f}='' if $f=~/^(eu.zz|zz.example)$/;
    $sortval{$f} = 2 if $errtxt{$f} ne '' && $sortval{$f} == 0;

    if ($regp->{'bill-remind'}) {
	$regp->{'bill-remind'} =~ s/\n/\n bill-remind: /g;
	$regp->{'bill-remind'} =~ s/\f/\n              /g;
	$errtxt{$f} .= " bill-remind: $regp->{'bill-remind'}\n";
	$sortval{$f} = 1 if $sortval{$f} == 0;
    }
    if ($sortval{$f} > 0 && $regp->{'reg-remind'}) {
	$regp->{'reg-remind'} =~ s/\n/\n reg-remind: /g;
	$regp->{'reg-remind'} =~ s/\f/\n             /g;
	$errtxt{$f} .= " reg-remind: $regp->{'reg-remind'}\n";
    }
    $sortval{$f} += 1000 if $sortval{$f} > 0 && $lvl eq 'NORMAL';
    $sortval{$f} += 2000 if $sortval{$f} > 0 && $regp->{'type'} ne 'PROVIDER' && $regp->{'type'} ne 'ENTERPRISE';
} # end of 'while' loop for each registry


print "* 'no normal service' registries who are some months late with payment,\n  should be closed in cooperation with hostmaster\n  (as soon as there are procedures for this)\n";
print "* 'no normal service' registries who have no commitment sent in,\n  should be closed by the 'registry setup person'. Forward this mail.\n\n";
print "CLOSED/PENDING REGISTRIES:\n\n";    
foreach $f (sort decreasing keys %sortval) {
    print "\n\nNORMAL SERVICE:\n\n" if $sortval{$f}<2000 && $tmp>2000;
    print "\n\nNO NORMAL SERVICE:\n\n" if $sortval{$f}<1000 && $tmp>1000;
    print "$f:\n$errtxt{$f}" if $errtxt{$f};
    $tmp=$sortval{$f};
}

sub decreasing {
    return -($sortval{$a} <=> $sortval{$b})
}

sub thisdate {
	my ($sec, $min, $hour, $day, $month, $year) = localtime(time);
	$month++;
	$hour = sprintf("%02d", $hour);
	$min = sprintf("%02d", $min);
	$month = sprintf("%02d", $month);
	$day = sprintf("%02d", $day);
	return ("$year$month$day");
}

sub days {	# compute number of days between dates
    my ($from, $to) = @_;

    if ($from =~ /0000$/ || $to =~ /0000$/) {
	return(0);
    }

    $from = timelocal(0, 			#sec
		      0, 			#min
		      0, 			#hour
		      substr($from,4, 2), 	#mday
		      substr($from,2, 2)-1, 	#month
		      substr($from,0, 2), 	#year
		      );

    $to = timelocal(0, 			#sec
		    0, 			#min
		    0, 			#hour
		    substr($to,4, 2), 	#mday
		    substr($to,2, 2)-1,	#month
		    substr($to,0, 2), 	#year
		     );

    return(int(($to-$from)/3600/24));
}
