#------------------------------------------------------------------------------
#
#$Id: check.pl,v 1.2 2006/11/04 14:40:50 ruben Exp $
#
# Module Header
# Filename          : check
# Purpose           : General reg consistency check
# Author            : Roderik/Daniel
# Date              : 17 November 1997 (& earlier)
# Description       :
# Language Version  : Perl (Tested with v5.00401)
# OSs Tested        : BSD/OS 2.1
# Command Line      : reg -x check
# Input Files       : $REGDIR/* (usually /ncc/registries/*)
# Output Files      : -
# External Programs : perl module regread.pm
# Problems          :
# To Do             :
# Comments          : This is a 'reg x-script' which can be called with
#                     'reg -x <scriptname>. However, Why not just 'use'
#                     the needed modules in here too? Now it can also be 
#                     called directly if we want...
#                     (Note: 'use strict' is necessary here because the strict
#                     directive apparently doesn't survive a 'do <filename>'.
# The following line will be read by 'reg -x':
# reg-x-description: general consistency check
#------------------------------------------------------------------------------

use strict;
use regread;

my ($PUBDIR) = 'PUBDIR-CONF'; # public FTP data

my ($regp);	# pointer to registry data
my (@f);	# registry names
my (@p);	# registry names in PUBDIR
my ($f);
my ($i);	# iterator
my ($type);
my (@tmp);
my ($tmp);

@f = &regfiles(0);

# read files in PUBDIR and check extra files
opendir(PUBDIR, $PUBDIR) or die "opendir $PUBDIR:$!\n";
@p = sort(readdir(PUBDIR));
closedir(PUBDIR);
while ($f = shift(@p)) {
    next if $f =~ /^\./;
    next if $f =~ /^README$/;
    if (!grep(/^$f$/, @f)) {
	print "extra file \"$f\" in $PUBDIR\n";
    }
}

while ($f = shift(@f)) {
    $regp = readreg($f);
    
    next if $regp->{'type'} eq 'PENDING';
    next if $regp->{'type'} eq 'CLOSED';
    
    # general errors
    if ($regp->{'error'}) {
	foreach $i (split("\n", $regp->{'error'})) {
	    print "$f - $i\n";
	}
    }

    # address lines
    $tmp = $regp->{'address'};
    $tmp =~ s/\f/\n/g;
    @tmp = split(/\n/,$tmp);
    print "$f - address longer than 7 lines\n" if @tmp > 7;
    $tmp = $regp->{'bill-addr'};
    $tmp =~ s/\f/\n/g;
    @tmp = split(/\n/,$tmp);
    print "$f - billing address longer than 7 lines\n" if @tmp > 7;
    
    # agreement
    ($tmp) = split(/\s+/,$regp->{'confirm'},2);
    print "$f - no agreement (entry established $tmp)\n" if $tmp && !$regp->{'commit'}; 
    
    print "$f - ENTERPRISE cannot have lst-provs entry\n" if $regp->{'type'} eq 'ENTERPRISE' && $regp->{'lst-provs'};

    if ($regp->{'type'} =~ /^(PROVIDER|ENTERPRISE|FYI)$/ or $regp->{'bill-hack'} eq 'NORMAL') {
	# check public entry
	if (! -f "$PUBDIR/$f") {
		print "$f - public entry does not exist\n";
		return(1);
	}
	@p = stat("$PUBDIR/$f"); # $p[2] = mode
	if ($p[2] & 0444 != 0444) {
	    print "$f - public entry is not readable for everyone\n";
	}
    }

    # check for missing/superfluous attributes
    if ($regp->{'type'} eq 'PROVIDER' or $regp->{'type'} eq 'ENTERPRISE') {
	print "$f - lst-localir missing\n" unless $regp->{'lst-localir'};
    } else {
	print "$f - no PROVIDER/ENTERPRISE, should not have reg-remind\n" if $regp->{'reg-remind'};
	print "$f - no PROVIDER/ENTERPRISE, should not have bill-remind\n" if $regp->{'bill-remind'};
	print "$f - no PROVIDER/ENTERPRISE, should not have hm-remind\n" if $regp->{'hm-remind'};
	print "$f - no PROVIDER/ENTERPRISE, should not have bill-hack\n" if $regp->{'bill-hack'};
    }
}
