#------------------------------------------------------------------------------
# Module Header
# Filename          : contrib
# Purpose           : Gives overview of registries' payment this year
# Author            : Roderik/Daniel
# Date              : 14 November 1997 (& earlier)
# Description       :
# Language Version  : Perl (Tested with v5.00401)
# OSs Tested        : BSD/OS 2.1
# Command Line      : reg -x contrib [year]
# Input Files       : $REGDIR/* (usually /ncc/registries/*)
# Output Files      : -
# External Programs : perl module regread.pm
# Problems          :
# To Do             :
# Comments          : This is a 'reg x-script' which can be called with
#                     'reg -x <scriptname>. However, Why not just 'use'
#                     the needed modules in here too? Now it can also be 
#                     called directly if we want...
#                     (Note: 'use strict' is necessary here because the strict
#                     directive apparently doesn't survive a 'do <filename>'.
# The following line will be read by 'reg -x':
# reg-x-description: overview of registries' payments
#------------------------------------------------------------------------------

use strict;
use regread;

my ($contyear) = shift @ARGV;
die "contrib: too many arguments\n" if @ARGV;

my (@f);	# registry names
my ($f);
my ($regp);	# pointer to hash with registry data
my ($amt);
my ($vat);
my ($bill);
my ($due);
my ($i);	# iterator
my ($sent);
my ($type);
my ($budget);   # budget over this year
my (%sum);	# sums
#	local(@legend);

if ($contyear) {
    die "contrib: year should be >1900\n" if $contyear<1900;
    exit;
} else {
    $contyear=&thisyear;
}

@f = regfiles(0);
	
printf "%-12.12s %7s %8s %8s %8s %8s\n\n",
    'Registry',
    'ServLvl',
    'Commitd.',
    'Invoiced',
    'Paid',
    'Open';

while ($f = shift(@f)) {
    my ($level);
    my ($commit);
    my ($inv);
    my ($vatb);
    my ($vatp);
    my ($paid);
    my ($open);
    my ($comments);
    my (@commit);
    my ($tmp);
    my ($tmp2);

    $regp = readreg($f);
    $type = $regp->{'type'};
    next if $type ne 'PROVIDER' && $type ne 'ENTERPRISE';

    $i = sprintf("%-12.12s  - %s\n", $regp->{'regid'}, $regp->{'org'});
#		push(@legend, $i);

    if ($regp->{'bill-signup'} =~ /^(\d{2})\s+(\d+)/) {
	if ($1 eq $contyear) {
	    $sum{'signup'} += $2;
	}
    }


    @commit = split(/\s+/, $regp->{'commit'});
    if ($commit[0]) {
	$commit = &getmoney($contyear, $commit[1]);
	if ($contyear eq substr($commit[0], 0, 2)) {
	    $i = substr($commit[0],2,2);
	    if ($i>3 && $i<7) {
		$commit *= 0.75;
	    } elsif ($i>6 && $i<10) {
		$commit *= 0.5;
	    } elsif ($i>9) {
		$commit *= 0.25;
	    }
	}
    }

    foreach $i (split(/\n/, $regp->{'invoice'})) {
	($sent,$tmp,$comments) = split(/\s+/,$i,3);
	if ($tmp =~ /^\@(\d{6})=(\d+)(%(\d+))?\#(U\d{5,6})$/) {
	    $due = $1;
	    $amt = $2;
	    $vat = $4 ? $4 : 0;
	    $bill = $5;
	} else {
	    print "$f - ERROR invoice: $i\n";
	}
		    
	if (substr($bill, 1, 2) eq $contyear && $comments !~ /^REVOKED/) {
	    $inv += $amt-$vat;
	    $vatb += $vat;
	}
    }
			
    foreach $i (split(/\n/, $regp->{'payment'})) {
	($tmp2,$tmp,$comments) = split(/\s+/,$i,3);
	if ($tmp =~ /^=(\d+)(%(\d+))?\#(U\d{5,6})$/) {
	    $amt = $1;
	    $vat = $3 ? $3 : 0;
	    $bill = $4;
	} else {
	    print "$f - ERROR payment: $i\n";
	}

	if (substr($bill, 1, 2) eq $contyear) {
	    $paid += $amt-$vat;
	    $vatp += $vat;
	}
    }

    unless ($inv) {
	if ($regp->{'bill-parent'}) {
	    $inv = "via $regp->{'bill-parent'}";
	}
    }
			
    $open = $inv-$paid;
    ($level,) = servicelevel($regp);
    if ($open) {
	if ($level eq 'NORMAL') {
	    $sum{'normal'} += $open;
	} else {
	    $sum{'none'} += $open;
	}
    } else {
	$open = "";
    }

    $sum{'commit'} += int($commit);
    $sum{'inv'} += int($inv);
    $sum{'vatb'} += int($vatb);
    $sum{'vatp'} += int($vatp);
    $sum{'paid'} += int($paid);

    printf "%-12.12s %7s %8s %8s %8s %8s\n",
    $f,
    $level,
    $commit,
    $inv,
    $paid,
    $open;
}
print "\n";
printf "%-12.12s %7s %8s %8s %8s\n",
    '',
    '',
    '--------',
    '--------',
    '--------';
printf "%-12.12s %7s %8s %8s %8s\n",
    'TOTAL',
    '',
    $sum{'commit'},
    $sum{'inv'},
    $sum{'paid'};
printf "%-12.12s %7s %8s %8s %8s\n",
    '',
    '',
    '========',
    '========',
    '========';
print "\n";

$budget = $money{"$contyear NCCBUDGET"};
printf "%-12.12s %7d %7d%% %7d%% %7d%%\n\n\n",
    '%of budget',
    $budget,
    $sum{'commit'}*100/$budget,
    $sum{'inv'}*100/$budget,
    $sum{'paid'}*100/$budget;

print "Signup fees in invoiced amounts for '$contyear: $sum{'signup'}\n\n";

print "Total VAT invoiced (not included above): $sum{'vatb'}\n";
print "Total VAT received (not included above): $sum{'vatp'}\n\n";

print "Total unpaid invoices for NORMAL registries: $sum{'normal'} \n";
print "Total unpaid invoices for 'NONE' registries: $sum{'none'} \n";

#	print "\fLegend\n------\n\n";
#	foreach $i (@legend) }
#		print $i;
#       }

sub getmoney {	# not really, but close
    my ($year, $categ) = @_;

    if ($categ =~ /SUPERNATIONAL:(\d+)/) {
	return(($money{"$year LARGE"})*$1);
    } else {
	return($money{"$year $categ"});
    }
}

sub thisyear {
    my ($sec, $min, $hour, $day, $month, $year) = localtime(time);
    return($year);
}
