#------------------------------------------------------------------------------
# Module Header
# Filename          : publ
# Purpose           : (re)publish registry files on FTP site
# Author            : Roderik/Daniel
# Date              : 17 November 1997 (& earlier)
# Description       :
# Language Version  : Perl (Tested with v5.00401)
# OSs Tested        : BSD/OS 2.1
# Command Line      : reg -x publ [-v]
# Input Files       : $REGDIR/* (usually /ncc/registries/*)
# Output Files      : $PUBDIR/* 
# External Programs : perl modules regread.pm, Getopt/Std.pm
# Problems          :
# To Do             :
# Comments          : This is a 'reg x-script' which can be called with
#                     'reg -x <scriptname>. However, Why not just 'use'
#                     the needed modules in here too? Now it can also be 
#                     called directly if we want...
#                     (Note: 'use strict' is necessary here because the strict
#                     directive apparently doesn't survive a 'do <filename>'.
# The following line will be read by 'reg -x':
# reg-x-description: (re)publish registry files on FTP site
#------------------------------------------------------------------------------

use strict;
use regread;
use Getopt::Std;

my ($PUBDIR) = 'PUBDIR-CONF'; # public FTP data

my ($regp);	# pointer to registry data
my (@f);	# registry names
my (@p);	# registry names
my ($f);
my ($p);
my ($i);	# iterator
my ($type);

die "publ: Wrong argument specified (only -v is possible)\n" if !getopts('v');

die "cannot write $PUBDIR\n" if !-w $PUBDIR;

print "removing expired registry files ...\n" if $Getopt::Std::opt_v;
opendir(PUBDIR, $PUBDIR) or die "opendir $PUBDIR:$!\n";
@p = sort(readdir(PUBDIR));
closedir(PUBDIR);
foreach $p (@p) {
    next if $p =~ /^\./;
    next if $p =~ /^README$/;
    next if -f "$REGDIR/$p";
    print STDERR "$p no longer exists, removing from $PUBDIR\n";
    unlink("$PUBDIR/$p") or warn "unlink $PUBDIR/$p: $!\n";
}

print "publishing registries ...\n" if $Getopt::Std::opt_v;
@f = &regfiles(0);
foreach $f (@f) {
    print "$f\n" if $Getopt::Std::opt_v;
    $regp = readreg($f);
    print "$f should not have been published (but was)!" if $regp->{'type'} !~ /^(PROVIDER|ENTERPRISE|FYI)$/ && $regp->{'bill-hack'} ne 'NORMAL' && -f "$PUBDIR/$f";
    &regwrite::writepub($f, $regp); # this will unlink and write new one
}
print "publish done.\n" if $Getopt::Std::opt_v;
