#------------------------------------------------------------------------------
#
#$Id: remind.pl,v 1.2 2006/11/04 14:40:50 ruben Exp $
#
# Module Header
# Filename          : reminder
# Purpose           : lists all reminders (of a certain category)
# Author            : Roderik/Daniel
# Date              : 17 November 1997 (& earlier)
# Description       : This basically reads registry files and writes them
#                     back again, which will take care of changes to
#                     attribute ordering for all registries.
# Language Version  : Perl (Tested with v5.00401)
# OSs Tested        : BSD/OS 2.1
# Command Line      : reg -x remind xx (where xx is reg, hm, bill, aud)
# Input Files       : $REGDIR/* (usually /ncc/registries/*)
# Output Files      :  -
# External Programs : perl module regreaad
# Problems          :
# To Do             :
# Comments          : This is a 'reg x-script' which can be called with
#                     'reg -x <scriptname>. However, Why not just 'use'
#                     the needed modules in here too? Now it can also be 
#                     called directly if we want...
#                     (Note: 'use strict' is necessary here because the strict
#                     directive apparently doesn't survive a 'do <filename>'.
# The following line will be read by 'reg -x':
# reg-x-description: lists all reminders (of a certain category)
#------------------------------------------------------------------------------

use strict;
use regread;
use Getopt::Std;
use Time::Local;

die "remind: Wrong flag specified (only -v is possible)\n" if !getopts('v');
die "Usage: reg -x remind [-v] <xx> , where xx == reg/hm/bill/aud\n" if !$ARGV[0];

my ($categ) = $ARGV[0];	# reminder category

my (@f);	# registries
my ($f);	# current registry
my ($remday,$remtxt,$tmp);
my ($regp);     # pointer to registry data

@f = &regfiles(0);
while ($f = shift(@f)) {
    $regp = readreg($f);
    foreach $tmp (split(/\n/,$regp->{"$categ-remind"})) {
	($remday,$remtxt) = ($tmp =~ /^(9\d{5})\s+(.*)$/);
	$remtxt =~ s/\f/\n             /g;
	if (&days($remday, &thisdate) > 0) {
	    printf "%-11s: %s %s\n",$f,$remday,$remtxt;
	} elsif ($Getopt::Std::opt_v) {
	    printf "%-11s-%s %s\n",$f,"Due at $remday:",$remtxt;
	}
    }
}

sub thisdate {
	my ($sec, $min, $hour, $day, $month, $year) = localtime(time);
	$month++;
	$hour = sprintf("%02d", $hour);
	$min = sprintf("%02d", $min);
	$month = sprintf("%02d", $month);
	$day = sprintf("%02d", $day);
	return ("$year$month$day");
}

sub days {	# compute number of days between dates
    my ($from, $to) = @_;

    if ($from =~ /0000$/ || $to =~ /0000$/) {
	return(0);
    }

    $from = timelocal(0, 			#sec
		      0, 			#min
		      0, 			#hour
		      substr($from,4, 2), 	#mday
		      substr($from,2, 2)-1, 	#month
		      substr($from,0, 2), 	#year
		      );

    $to = timelocal(0, 			#sec
		    0, 			#min
		    0, 			#hour
		    substr($to,4, 2), 	#mday
		    substr($to,2, 2)-1,	#month
		    substr($to,0, 2), 	#year
		     );

    return(int(($to-$from)/3600/24));
}
