#!/usr/local/bin/perl5
#
# Stripped down version of Andrew's map_index.cgi to only show 
# box's locations. To be linked from gpsinfo page
#
# Rene Wilhelm - 20020421

use CGI qw/:standard/;   # Load CGI module

#---------------------------------------------------------------
#  INPUTS TO THE SCRIPT
#---------------------------------------------------------------

my %vars;                        # the values of all input variables
my @inputs = (param, url_param); # all input variables, both URL and POST
my $varlist = "sel src dst box srcvar dstvar suffix redirect .submit";

# sel:      which to select first (src, dst)
# src:      the source testbox, if one is selected
# dst:      the destination testbox, if one is selected
# box:      the most recently selected testbox (will be set as src or dst later)

# srcvar:   redirection: what variable to send the src as, to the redirection script
# dstvar:   redirection: what variable to send the dst as, to the redirection script
# suffix:   redirection: any suffix (such as .ripe.net) to include in the values sent
# redirect: redirection: where to redirect this script, once src and dst are set

# any other variables are for the redirection script, and should simply be passed along

foreach $var (@inputs) {
    $vars{$var} = param($var)     if (!$vars{$var});
    $vars{$var} = url_param($var) if (!$vars{$var});
}

# set defaults
$vars{'sel'}    = 'src' if (!$vars{'sel'});
$vars{'srcvar'} = 'src' if (!$vars{'srcvar'} && $vars{'redirect'});
$vars{'dstvar'} = 'dst' if (!$vars{'dstvar'} && $vars{'redirect'});

#---------------------------------------------------------------
# Functions for assembling query strings
#---------------------------------------------------------------

# create query strings (for links)
sub querystring {
    my $string = "";
    my @inputs = @_;
    my $prefix = "";

    foreach $var (@inputs) {
	if ($var eq '&') {
	    $prefix = $var;
	} else {
	    $string .= "&$var=$vars{$var}" if $vars{$var};
	}
    }
    $string =~ s/^&// if (!$prefix);  # delete leading ampersand
    return $string;
}

#---------------------------------------------------------------
#  Get global folder location defaults, and GPS information
#---------------------------------------------------------------

do "maps.locations";

my @TTLIST;
my %TTvals;
my %ttInfo;
my $status = "";

my @GPSINFO = `$CGIgps`;
chomp @GPSINFO;
foreach $f (@GPSINFO) {

    # is this a status line?
    if ($f =~ /^status:\t(\w+)$/) {
	$status = $1;
	push @TTLIST, $status;
	$TTvals{$status} = "---- $status ----";

    # is this a testbox line?
    } else {
	my ($name, $lat, $long, $place) = split ('\t', $f);
	$ttInfo{$name}{'place'}  = $place;
	$ttInfo{$name}{'status'} = $status;
	$ttInfo{$name}{'lat'} = $lat;
	$ttInfo{$name}{'long'} = $long;
	push @TTLIST, $name;
	$TTvals{$name} = "$name: $place";
    }
}

if (defined $vars{'box'}) {
	$name=$vars{'box'};
	print redirect("http://www.mapquest.com/maps/map.adp?latlongtype=decimal&latitude=$ttInfo{$name}{'lat'}&longitude=$ttInfo{$name}{'long'}");
	exit(0);
	print header;
	print "<html><body>";
	print "<A HREF=\"";
	print "http://www.mapquest.com/maps/map.adp?latlongtype=decimal&latitude=$ttInfo{$name}{'lat'}&longitude=$ttInfo{$name}{'long'}";
	print "\">$name</A></body>/</html>";
	exit(0);
}

#----------------------------------------------------------------
#  Otherwise, create page showing global map
#----------------------------------------------------------------

    print header,
	"<html><head>\n",
	title("Test Traffic Satellites & GPS information"), "\n",
	"<meta http-equiv=\"Content-Type\" content=\"text/html\; charset=iso-8859-1\">\n",
	`cat $SSIpre/$SSIdir/ttmhead1.inc`, "\n",
	h1('Test Traffic Measurements'), "\n",
	h2('Testbox locations'), "\n",
	"<P>When moving mouse over the map testbox name and location will ",
	"appear below (javascript required)</P>",
	"<P>Selecting a box will redirect you to the ",
	"<A HREF=\"http://www.mapquest.com\">mapquest</A> server ",
	"where you can explore the location in more detail</P>\n";

#----------------------------------------------------------------
#  Handle error messages generated above
#----------------------------------------------------------------

if ($error_msg) {
    print 
	"<script language=javascript>\n",
	"alert ('$error_msg');\n",
	"</script>\n";
}

#----------------------------------------------------------------
#  Output map (executed from maps/maps.include)
#----------------------------------------------------------------

do "$CGIpre/$CGIdir/map_import.pl";
print &import_maps(url(-relative=>1) . "?" . "src=src", "box", "");

#----------------------------------------------------------------
#  End HTML output
#----------------------------------------------------------------

print `cat $SSIpre/SSIdir/ttmfoot1.inc`;
