#!/usr/local/bin/perl5
#
# $Id: map_import.pl,v 1.2 2006/11/04 14:49:59 ruben Exp $
#
# Copyright (c) 2002  RIPE NCC
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of the author not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
#------------------------------------------------------------------------------
# Module Header
# Filename          :   map_gen.pl
# Purpose           :   Generation of interactive testbox maps with GMT tools.
# Author	    :	Andrew Marcus <andrew@ripe.net>, RIPE NCC intern
#                   :                 <awm22@cornell.edu>
# Date	            :   05-MAR-2002
# Description       :   For TTM project website, easy navigation
# Input             :   maps.include, user input
# Output            :   return parsed maps.include
# Language Version  :   perl5
# External Programs :   map_gen.pl, maps.config
#------------------------------------------------------------------------------

sub import_maps {

    #--------------------------------------------------------------------------
    # global variables:
    #--------------------------------------------------------------------------

    do "maps.locations";   # get global folder location defaults

    #--------------------------------------------------------------------------
    #inputs:
    #--------------------------------------------------------------------------
    my $url = $_[0];      # what url to redirect the selection to
    my $var = $_[1];      # which variable name to assign the selection to
    my $redirect = $_[2]; # which url to redirect the cgi script to

    my $htmlstring = `cat $CGIpre/$CGIdir/maps.include`;

    #--------------------------------------------------------------------------
    # check $url for syntax
    #--------------------------------------------------------------------------
    
    $url =~ /(\S+)\?(\S+)/;   # separate url into address?query
    my $address = $1;
    my $query = $2;

    # put url into old_style format ("&" between variables) 
    $query =~ s/;/&/g;

    # format: url?var1=val1&var2=val2&...
    if    ( $query =~ /\w+=\S*(&\w+=\S*)*(&)?$/ ) {   
	# make sure it ends without an "&"
	$query =~ s/&$//;
    }
    # format: url or url?
    elsif ( $query eq "" ) {
    }
    # invalid query
    else {
	return "<h2>map_import.pl ERROR: specified query string $query invalid!</h2>";
    }
    
    #--------------------------------------------------------------------------
    # parse the html for all instances of #link#?ttxx (links)
    #--------------------------------------------------------------------------

    my $redir = "";
    if ($redirect) {
	$redir = "&redirect=$redirect";
    }
    
    $htmlstring =~ s{\#link\#\?(\w+)}
                    {$address\?$query&$var=${1}${redir}}g;

    #--------------------------------------------------------------------------
    # parse the html for all instances of #maps#?map_name (alternate map)
    #--------------------------------------------------------------------------

    $htmlstring =~ s{\#map\#\?(\w+)}
                    {$altMAP\?map=$1&$query&var=$var&url=${address}${redir}}g;

    #--------------------------------------------------------------------------
    # parse the html for all instances of #cluster#?map_name (alternate map)
    #--------------------------------------------------------------------------

    $htmlstring =~ s{\#cluster\#\?(\w+)}
                    {$altCLUSTER\?map=$1&$query&var=$var&url=${address}${redir}}g;


    return $htmlstring;
}

1     # value to return to the "require" calling function











