#!/usr/local/bin/perl5

use CGI qw/:standard/;   # Load CGI module

#---------------------------------------------------------------
#  INPUTS TO THE SCRIPT
#---------------------------------------------------------------

my %vars;                        # the values of all input variables
my @inputs = (param, url_param); # all input variables, both URL and POST
my $varlist = "sel src dst box srcvar dstvar suffix redirect .submit";

# sel:      which to select first (src, dst)
# src:      the source testbox, if one is selected
# dst:      the destination testbox, if one is selected
# box:      the most recently selected testbox (will be set as src or dst later)

# srcvar:   redirection: what variable to send the src as, to the redirection script
# dstvar:   redirection: what variable to send the dst as, to the redirection script
# suffix:   redirection: any suffix (such as .ripe.net) to include in the values sent
# redirect: redirection: where to redirect this script, once src and dst are set

# any other variables are for the redirection script, and should simply be passed along

foreach $var (@inputs) {
    $vars{$var} = param($var)     if (!$vars{$var});
    $vars{$var} = url_param($var) if (!$vars{$var});
}

foreach $var (@inputs) {
    unless ($varlist =~ /\b$var\b/) {   # if the variable is unexpected, pass it to the redirect script
	$vars{'redirect'} .= "&" . &querystring($var) if ($vars{'redirect'});
    }
}

# set defaults
$vars{'sel'}    = 'src' if (!$vars{'sel'});
$vars{'srcvar'} = 'src' if (!$vars{'srcvar'} && $vars{'redirect'});
$vars{'dstvar'} = 'dst' if (!$vars{'dstvar'} && $vars{'redirect'});

#---------------------------------------------------------------
# Functions for assembling query strings
#---------------------------------------------------------------

# create query strings (for links)
sub querystring {
    my $string = "";
    my @inputs = @_;
    my $prefix = "";

    foreach $var (@inputs) {
	if ($var eq '&') {
	    $prefix = $var;
	} else {
	    $string .= "&$var=$vars{$var}" if $vars{$var};
	}
    }
    $string =~ s/^&// if (!$prefix);  # delete leading ampersand
    return $string;
}

#---------------------------------------------------------------
#  Get global folder location defaults, and GPS information
#---------------------------------------------------------------

do "maps.locations";

my @TTLIST;
my %TTvals;
my %ttInfo;
my $status = "";

my @GPSINFO = `$CGIgps`;
chomp @GPSINFO;
foreach $f (@GPSINFO) {

    # is this a status line?
    if ($f =~ /^status:\t(\w+)$/) {
	$status = $1;
	push @TTLIST, $status;
	$TTvals{$status} = "---- $status ----";

    # is this a testbox line?
    } else {
	my ($name, $lat, $long, $place) = split ('\t', $f);
	$ttInfo{$name}{'place'}  = $place;
	$ttInfo{$name}{'status'} = $status;
	push @TTLIST, $name;
	$TTvals{$name} = "$name: $place";
    }
}

#---------------------------------------------------------------
#  Guarantee different boxes as source and destination
#---------------------------------------------------------------

if ( ($vars{'src'} && $vars{'dst'}) && ($vars{'src'} eq $vars{'dst'}) ) {
    $error_msg = "Source and destination are the same!";
    $vars{'dst'} = "";

    if ($vars{'sel'} eq 'src') {
	$vars{'dst'} = "";
    } else {
	$vars{'src'} = "";
    }
}
if ( (($vars{'src'} && $vars{'box'}) && ($vars{'src'} eq $vars{'box'})) ||
     (($vars{'dst'} && $vars{'box'}) && ($vars{'dst'} eq $vars{'box'})) ) {
    $error_msg = "Source and destination are the same!";
    $vars{'box'} = "";
}

#---------------------------------------------------------------
#  Deal with input testbox (from map selection tool)
#---------------------------------------------------------------

if ($vars{'box'}) {
    if (! exists $ttInfo{$vars{'box'}}) {  # whoops, the testbox doesn't exist! (not among these statuses, anyway)
	                                   # perhaps user selected a status instead of a testbox
	$error_msg = "Error: testbox not found, try again"; 

    }elsif ($vars{'src'} && $vars{'dst'}) {        # should never happen
	$error_msg = "Error: already have source and destination!";

    } elsif ($vars{'src'}) {
	$vars{'dst'} = $vars{'box'};

    } elsif ($vars{'dst'}) {
	$vars{'src'} = $vars{'box'};

    } elsif ($vars{'sel'} eq 'dst') {
	$vars{'dst'} = $vars{'box'};
    } else {
	$vars{'src'} = $vars{'box'};
    }

    $vars{'box'} = "";
}

#---------------------------------------------------------------
#  If source and destination are set, bring up the results page
#---------------------------------------------------------------

if ($vars{'src'} && $vars{'dst'}) {
    if ($vars{'redirect'}) {

	$vars{'redirect'} =~ /(\S+)\?(\S+)/;   # separate url into address?query
	my $address = $1;
	my $query   = $2;

	$address = $vars{'redirect'} if (!$address);   # no '?' in string

	# put url into old_style format ("&" between variables) 
	$query =~ s/;/&/g;

	# format: url?var1=val1&var2=val2&...
	if ( $query =~ /\w+=\S*(&\w+=\S*)*(&)?$/ ) {   
	    # make sure it ends with an "&"
	    $query =~ s/([^&])$/$1&/;
	}
	# format: url or url?
	elsif ( $query eq "" ) {
	}
	# invalid url
	else {
	    print redirect("$PLOTdir/$vars{'src'}/$vars{'src'}.$vars{'dst'}.html?error");
	    return;
	}
	my $redirect = $address . "?" . $query . 
	               $vars{'srcvar'} . "=" . $vars{'src'} . $vars{'suffix'} . "&" .
		       $vars{'dstvar'} . "=" . $vars{'dst'} . $vars{'suffix'};
	print redirect($redirect);
	return;

    } else {
	$vars{'ipv'} = 4 if (!defined $vars{'ipv'});
	$vars{'base'} = $vars{'src'} if (!defined $vars{'base'});
	print redirect("plots.cgi?ipv=$vars{'ipv'}&base=$vars{'base'}&src=$vars{'src'}&dst=$vars{'dst'}&url=" . url(relative=>1))

#	if (-e "$PLOTpre/$PLOTdir/$vars{'src'}/$vars{'src'}.$vars{'dst'}.html") {
#	    print redirect("$PLOTdir/$vars{'src'}/$vars{'src'}.$vars{'dst'}.html");
#	    return;
#	} else {
#	    $error_msg  = "Sorry, no data exists between these two testboxes.  ";
#	    $error_msg .= "Try another " . (($vars{'sel'} eq 'src') ? "destination" : "source") . " testbox.";
#	    if ($vars{'sel'} eq 'src') {
#		$vars{'dst'} = ""; 
#	    } else {
#		$vars{'src'} = "";
#	    }
#	}
    }
}

#----------------------------------------------------------------
#  Otherwise, create page, and ask for missing value {src, dst}
#----------------------------------------------------------------

{  # just to make following variables local

    my $sel =  $vars{'sel'};
    my $les = ($sel eq 'src' ? 'dst' : 'src');
    my $box = ($vars{'src'} ? $vars{'src'} : $vars{'dst'});

    my $url = url(-relative=>1);
    my $redirect_query = &querystring  ('&','srcvar','dstvar','suffix','redirect');
    
    my $reset_URL      = "$url?sel=src"           . $redirect_query;
    my $reset_same_URL = "$url?sel=$sel"          . $redirect_query;
    my $reset_swap_URL = "$url?sel=$les"          . $redirect_query;
    my $swap_URL       = "$url?sel=$les&box=$box" . $redirect_query;
    
    my %sel_names = ('src'=>'source', 'dst'=>'destination');
    
    print 
	header,
	"<html><head>\n",
	title("Test Traffic Satellites & GPS information"), "\n",
	"<meta http-equiv=\"Content-Type\" content=\"text/html\; charset=iso-8859-1\">\n",
	`cat $SSIpre/$SSIdir/ttmhead1.inc`, "\n",
	h1('Test Traffic Measurement Results'), "\n";

    if ($box) {    # if one testbox has already been selected
	print
	    "<table><tr align='left' valign='bottom'><td>\n",
	    h2( ucfirst($sel_names{$sel}) . ":",
	       "<a href='$PLOTdir/$box/index.html'>$box</a> ",
	       "<font size='-1'>$ttInfo{$box}{'place'}</font> ",
	       "Status: <strong class=", lc $ttInfo{$box}{'status'}, ">$ttInfo{$box}{'status'}</strong>"), "\n",

	    "</td><td>&nbsp;&nbsp;</td><td>\n",
	    start_form(-action=>$swap_URL),
	    submit(-value=>"Make this the $sel_names{$les}"),
	    end_form, "\n",
	    "</td><td>\n",
	    start_form(-action=>$reset_URL),
	    submit(-value=>"Reset"), 
	    end_form, "\n",
	    "</td></tr></table>\n", hr, "\n",

	    "<table><tr align='left' valign='bottom'><td>\n",
	    h2( ucfirst($sel_names{$les}) . ":",
	       "Please select a testbox from the menu:"), "\n",
	    "</td></tr></table>\n",
    	
    } else {
	print 
	    "<table><tr align='left' valign='bottom'><td>\n",
	    h2( ucfirst($sel_names{$sel}) . ":",
	       "Please select a testbox from the menu:"), "\n",
	    "</td><td>&nbsp;&nbsp;</td><td>\n",
	    start_form(-action=>$reset_swap_URL),
	    submit(-value=>"Start with a $sel_names{$les}"),
	    end_form, "\n",
	    "</td></tr></table>\n",
    }
}
#----------------------------------------------------------------
#  Handle error messages generated above
#----------------------------------------------------------------

if ($error_msg) {
    print 
	"<script language=javascript>\n",
	"alert ('$error_msg');\n",
	"</script>\n";
}

#----------------------------------------------------------------
#  Output testbox form
#----------------------------------------------------------------

my $next_URL = url(-relative=>1) . "?" . &querystring('sel','src','dst','srcvar','dstvar','suffix','redirect');

print start_form(-name=>"ttbox", -action=>$next_URL), "\n";

if ($vars{'src'} || $vars{'dst'}) 
{
    my (%other_box_vals, @other_boxes) = ();

    foreach $f (@TTLIST) {
	if ($vars{'src'} ne $f && $vars{'dst'} ne $f) {
	    $other_box_vals{$f} = $TTvals{$f};
	    push (@other_boxes, ($f));
	}
    }
    print 
	popup_menu(-name=>'box',
		   -values=>\@other_boxes,
		   -labels=>\%other_box_vals,
		   -onChange=>'javascript:document.ttbox.submit();'),"\n";
} 
else 
{
    print 
	popup_menu(-name=>'box',
		   -values=>\@TTLIST,
		   -labels=>\%TTvals,
		   -onChange=>'javascript:document.ttbox.submit();'),"\n";
}
print 
    submit(-value=>'Submit'),
    end_form, "\n";    

print
    h2("Or by dragging your mouse over the map:"),"\n";

#----------------------------------------------------------------
#  Output map (executed from maps/maps.include)
#----------------------------------------------------------------

do "$CGIpre/$CGIdir/map_import.pl";
print &import_maps(url(-relative=>1) . "?" . &querystring('sel','src','dst','srcvar','dstvar','suffix'), 
		   "box", $vars{'redirect'});

#----------------------------------------------------------------
#  End HTML output
#----------------------------------------------------------------

print `cat $SSIpre/SSIdir/ttmfoot1.inc`;
    











