#!/usr/local/bin/perl5
#
# $Id: plots.cgi,v 1.9 2006/11/04 14:49:59 ruben Exp $
#
#
# This is a wrapper for the ttm plot pages, to provide them
# dynamically with an interface which returns to the map_index
# tool, to allow further selection.
# This tool eliminates the need to have the links hardcoded into
# the plot pages individually.
#
#---------------------------------------------------------------

use strict;
use CGI qw/:standard/;   # Load CGI module

#---------------------------------------------------------------
#  INPUTS TO THE SCRIPT
#---------------------------------------------------------------

my $src      = param('src');             # source testbox
my $dst      = param('dst');             # destination testbox
my $url      = param('url');             # url of the script which invoked this
my $type     = param('type') || "delay"; # type of plots

my $ipv      = param('ipv') || "4";
my $prefix   = "";

if ($ipv == 6) {
	$prefix = "IPv6." ;
}

#---------------------------------------------------------------
#   Other variable declarations
#---------------------------------------------------------------

my $body;		# HTML code with <IMG> of the ttm plots
my $title;		# HTML <head> + start of <body>
my $linkcode1;		# HTML code with hyperlinks
my $linkcode2;		# HTML code with hyperlinks
my $podtype;		# type of plot for hyperlink to plots-on-demand
my $plottitle;		# used in title <h1> of page
my $ssihead;		# SSI header
my $ssifoot;		# SSI footer

#----------------------------------------------------------------
# Read parameters and determine box locations
#----------------------------------------------------------------

our $WWWpre;		# prefix to get html files on the webserver
our $SSIpre;		# prefix to get ssi include files on the webserver
our $SSIdir;		# directory to get ssi include files on the webserver
our $PLOTdir;		# directory on web server holding plots
our $CGIgps;		# pathname of GPSINFO file on web server

do 'maps.locations';	# retrieve above parameters from configfile

#----------------------------------------------------------------

my %TTvals;

my @GPSINFO = `$CGIgps`;
chomp @GPSINFO;
foreach my $f (@GPSINFO) {

    # is this a status line?
    if (! ($f =~ /^status:\t(\w+)$/)) {
	# not at status line, but testbox info
        my ($name, $lat, $long, $place) = split ('\t', $f);
        $TTvals{$name} = "$name - $place";
	$TTvals{$name} =~ s/^tt/TT/;
    }
}

#----------------------------------------------------------------

if ($type eq "delay") {
	$type = "";
	$podtype = "delay";
	$plottitle = "Delay Plots";
}
elsif (($type eq "jitter") || ($type eq "ipdv") || ($type eq "ipdv.")) {
	$type = "ipdv.";
	$podtype = "jitter";
	$plottitle = "Delay Variation Plots";
}

if ($ipv == 6) {
	$plottitle = "IPv6 $plottitle";
}
#----------------------------------------------------------------
#  Determine plot file names
#----------------------------------------------------------------

my $gifdir   = "$PLOTdir";
my $daygif   = "$gifdir/day/${prefix}$type$src.$dst.gif";
my $weekgif  = "$gifdir/week/${prefix}$type$src.$dst.gif";
my $monthgif = "$gifdir/month/${prefix}$type$src.$dst.gif";
my $trendgif = "$gifdir/trends/${prefix}trends.$type$src.$dst.gif";

#----------------------------------------------------------------
#  Compute links to traceroutes
#----------------------------------------------------------------

# Set defaults

my ($hour, $day, $mon, $year);
my ($startdate_24hr, $startdate_7d, $startdate_30d, $enddate);  # dates
my ($traceurl_24hr, $traceurl_7d, $traceurl_30d);		# URLs


# yesterday's traceroutes should be there by 0800 GMT

my $endtime = time();

($a, $a, $hour, $day, $mon, $year, $a, $a, $a) = gmtime($endtime);

if ($hour < 6) {
        # last data not processed yet, push back one day
	$endtime -= 86400;
	($a, $a, $hour, $day, $mon, $year, $a, $a, $a) = gmtime($endtime);
}

$year += 1900;
$enddate = $year . substr("0".++$mon, length($mon )-1, 2) .
                  substr("0".$day,length($day)-1, 2);


($a, $a, $hour, $day, $mon, $year, $a, $a, $a) = gmtime($endtime - 86400);
$year += 1900;
$startdate_24hr = $year .  substr("0".++$mon, length($mon )-1, 2) .
                  substr("0".$day,length($day)-1, 2);

($a, $a, $hour, $day, $mon, $year, $a, $a, $a) = gmtime($endtime - 7 * 86400);
$year += 1900;
$startdate_7d = $year .  substr("0".++$mon, length($mon )-1, 2) .
                substr("0".$day,length($day)-1, 2);

($a, $a, $hour, $day, $mon, $year, $a, $a, $a) = gmtime($endtime - 30 * 86400);
$year += 1900;
$startdate_30d = $year .  substr("0".++$mon, length($mon )-1, 2) .
                substr("0".$day,length($day)-1, 2);


$traceurl_24hr = "/cgi-bin/ttm/routing?SRC=$src&DST=$dst&ipv=$ipv&" .
		"date_start=$startdate_24hr&date_end=$enddate";

$traceurl_7d = "/cgi-bin/ttm/routing?SRC=$src&DST=$dst&ipv=$ipv&" .
		"date_start=$startdate_7d&date_end=$enddate";

$traceurl_30d = "/cgi-bin/ttm/routing?SRC=$src&DST=$dst&ipv=$ipv&" .
		"date_start=$startdate_30d&date_end=$enddate";




#------------------------------------------------------------------
#  Assemble HTML output
#------------------------------------------------------------------

my $ssihead  = header() .           # HTTP header
              "<html><head>\n" .
	       title("RIPE NCC: Test Traffic Results $src to $dst") . "\n" .
	       "<meta http-equiv='Content-Type' content='text/html\;".
               " charset=iso-8859-1'>\n";

#$ssihead .=   `cat $SSIpre/$SSIdir/ttmhead1.inc`;

$ssihead .= "<link REL='STYLESHEET' HREF='/templates/nccstyle1.css'
		TYPE='text/css'>
	      <style TYPE='text/css'>
	      <!-- A, ADDRESS, B, BODY, P, CENTER, TH, DD, DL, DT, H1, H2,
               H3, LI, UL, H4, H5, OL, TABLE ,TD { font-family: Arial,
	       sans-serif, Helvetica; } -->
	      </style> <LINK REL='SHORTTCUT ICON'
		HREF='http://www.ripe.net/favicon.ico'>";

$ssihead .=  "<script language=\"JavaScript1.2\" src=\"/ripencc/mem-services/ttm/ttm.js\" type=\"text/javascript\"></script>
		</head>
	      <body text='#003399' bgcolor='#FFFFFF' link='#990099'
	       vlink='#003399' alink='#CC0000' onload='checkLicense();' >
	      <a NAME='top of page'></a>";


my $ssifoot  = `cat $SSIpre/$SSIdir/ttmfoot1.inc`;

# output HTML header and start of page

$title = "<H1><img align='middle' src='/ripencc/pics/ncclogo-small.gif'
          alt='ncc logo' border=0 height=63> &nbsp;&nbsp;&nbsp;";
$title .= "Test Traffic &nbsp;$plottitle</H1>";
$title .= "<div align='center'><H3>$TTvals{$src}<BR>to<BR>$TTvals{$dst}</H3>";

print $ssihead, "\n", $title, "\n";

if ( -f "$WWWpre/$daygif" || -f "$WWWpre/$weekgif" ) {
	$linkcode1 = "[ <a href=\"$url?sel=src&ipv=$ipv&src=$dst&dst=$src\">Swap Source and Destination</a> ] &nbsp; ";
	$linkcode1 .= "[ <a href=\"$url?sel=dst&ipv=$ipv&dst=$dst\">Select a new Source</a> ] &nbsp; ";
	$linkcode1 .= "[ <a href=\"$url?sel=src&ipv=$ipv&src=$src\">Select a new Destination</a> ] &nbsp; ";
	$linkcode1 .= "[ <a href=\"$url\">Reset</a> ]\n";

	$linkcode2 = "[ <a href=\"/cgi-bin/ttm/plots-on-demand?SRC=${src}&DST=${dst}&plottype=$podtype&ipv=$ipv&SUBMIT=false\">Change time interval, min/max delay</a> ]";


	$linkcode2 .= " &nbsp; [ <a href=../General/tt_matrix.html#about>help about the plots</a> ]";

#	$body  = "<hr>";
	$body .= "<TABLE WIDTH=800 BORDER=0><TR ALIGN=CENTER>";
	$body .= "<TD WIDTH=300>Last 24hrs</TD>";
	$body .= "<TD WIDTH=200 COLSPAN=2><em>Click Image to Enlarge</em></TD>";
	$body .= "<TD WIDTH=300>Last 7 days</TD>\n";
	$body .= "</TR><TR ALIGN=CENTER><TD ALIGN=CENTER COLSPAN=2>";
	$body .= "<A HREF=\"#day\"><img width=\"300\" ";
	$body .= "SRC=\"$daygif\"></A></TD>";
	$body .= "<TD COLSPAN=2><A HREF=\"#week\"><img width=\"300\" ";
	$body .= "SRC=\"$weekgif\"></A></TD></TR>\n";
	if ($type eq "") {
		$body .= "<TR></TR><TR ALIGN=CENTER><TD>Last 30 days</TD>";
		$body .= "<TD WIDTH=100>&nbsp;</TD><TD WIDTH=100>&nbsp;</TD>";
		$body .= "<TD>Last 6 months</TD></TR>";
		$body .= "<TR ALIGN=CENTER><TD COLSPAN=2><A HREF=\"#month\">";
		$body .= "<img width=\"300\" SRC=\"$monthgif\"></A></TD>";
		$body .= "<TD COLSPAN=2><A HREF=\"#trends\">";
		$body .= "<img width=\"300\" SRC=\"$trendgif\"></A></TD></TR>";
	}
	$body .= "\n</TABLE>";

	$body .= "<hr>$linkcode1";
	$body .= "<hr><h2><a NAME=\"day\">&nbsp;</a>$src to $dst - Last 24 hours</h2>";
        $body .= "<strong>[&nbsp;<a href=\"$traceurl_24hr\">";
	$body .= "collected traceroutes</a>&nbsp;] $linkcode2</strong><p>\n";
	$body .= "<img SRC=\"$daygif\">\n";
	$body .= "<p><hr><h2><a NAME=\"week\">&nbsp;</a>$src to $dst - Last 7 days</h2>";
        $body .= "<strong>[&nbsp;<a href=\"$traceurl_7d\">";
	$body .= "collected traceroutes</a>&nbsp;] $linkcode2</strong><p>\n";
	$body .= "<img SRC=\"$weekgif\">\n";

	if ($type eq "") {
		$body .= "<p><hr><a NAME=\"month\">&nbsp;</a><h2>$src to $dst - Last 30 days</h2>";
	        $body .= "<strong>[&nbsp;<a href=\"$traceurl_30d\">";
		$body .= "collected traceroutes</a>&nbsp;]$linkcode2</strong><p>\n";
		$body .= "<img SRC=\"$monthgif\">\n";
		$body .= "<p><hr><h2><a NAME=\"trends\">&nbsp;</a>$src to $dst - 6 months trend</h2>";
		$body .= "<img SRC=\"$trendgif\">\n";
	}

	print $body, "<hr></div>\n", $ssifoot;

}
else {
	print "<p>Sorry, no data exists between these two boxes.</p>\n",
	   "</div>\n", $ssifoot;
}

exit (0);
