///////////////////////////////////////////////////////////
//
// $Id: maps.functions.js,v 1.2 2006/11/04 14:49:59 ruben Exp $
//  These functions are used by map_index.cgi and it's 
//  helper scripts.  They provide the necessary Javascript
//  coding to enable layers to pop up correctly in several
//  different supported browser types:
//    * Microsoft Internet Explorer 4+
//    * Netscape Navigator 4+
//    * Netscape 6+ and other W3C-compliant browsers
//
//  For older browsers with Javascript but without support
//  for layers, there are also functions to provide an
//  alternative click-interface, which opens windows
//  containing the requested content, loading the final
//  result back into the starting window. 
//
//  See README for more details about cross-platform support
///////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////
// BROWSER TYPE: which DOM does it support?

var ns4 = document.layers;
var ie4 = document.all;
var ns6 = document.getElementById && !ie4 && !ns4;

// alert ((ns4?" ns4":"") + (ie4?" ie4":"") + (ns6?" ns6":""));

// Comments:
// Modern W3C settings will be used only where both other DOMs fail,
// otherwise, back-compatibility can cause conflicts, causing the map
// layer code not to work correctly.

// Internet Explorer 5+ supports ie4 DOM and most of the ns6 (W3C) DOM,
// however, the latter is still not completely correctly implemented,
// so therefore the ie4 DOM settings will be used for this browser type.


///////////////////////////////////////////////////////////
// FUNCTION TO DISPLAY TESTBOX AND MAP INFORMATION

function display( ttbox ) {
    document.displayform.displaybox.value = ttbox;
}

///////////////////////////////////////////////////////////
// NO-LAYERS FUNCTIONS: display maps in windows instead

// if the browser does not support layers, we want it to open a new window with the zoomed-in map
function open_window( link, name, mywidth, myheight ) {
    if (! window.focus) return true;   // browser does not support this method, open in same window
    var mywindow = window.open(link, name, "width="+mywidth+",height="+myheight+",scrollbars=yes");
    mywindow.focus();
    return false;  // the link will not be followed on this page
}
    
// return to the basepage with this link
function open_basepage(mylink){
    // if browser does not support it, stay in same window
    if (! (window.focus && window.opener)) return true;

    // if this is the basepage already, stay in this window
    if (window.name == "base_map") return true;

    // otherwise, return to basepage, closing intermediate windows
    // and load specified link into that window
    var mywindow = window;
    while (mywindow.opener && mywindow.opener.name != "base_map") {
	myopener = mywindow.opener;
	mywindow.close();
	mywindow = myopener;
    }
    mywindow.opener.focus();
    mywindow.opener.location.href=mylink;
    mywindow.close();
    return false;  // the link will not be followed on this page, since it is on a different page
}

///////////////////////////////////////////////////////////
// LAYERS FUNCTIONS: show and hide layers

// display a layer containing a map (or cluster)
function map_display( x ) {

    // if we're in a different window, we know layers aren't supported
    if (window.name != "base_map") return true;

    if (ns4) {
        // Netscape 4: need to identify nesting of layer
        var layer_ref = "document.layers['"+x+"']";
	while (parent[x] && parent[x] != '') {
	    x = parent[x];
	    layer_ref = "document.layers['"+x+"']." + layer_ref;
	}
	layer_ref += ".visibility = 'show'";
	eval (layer_ref);
    } else
    if (ie4) {
	// Internet Explorer 4+
	document.all[x].style.visibility = 'visible';
    } else
    if (ns6) {
        // Netscape 6 and other W3C-compliant browsers
        var map_layer = document.getElementById( x );
        map_layer.style.visibility = 'visible';
    }
    return true;
}

// hide a layer containing a map (or cluster)
function map_hide( x ) {

    // if we're in a different window, we know layers aren't supported
    if (window.name != "base_map") return true;

    if (ns4) {
        // Netscape 4: need to identify nesting of layer
  	var layer_ref = "document.layers['" + x + "']";
	while (parent[x] && parent[x] != '') {
	    x = parent[x];
	    layer_ref = "document.layers['" + x + "']." + layer_ref;
	}
	layer_ref += ".visibility = 'hide'";
	eval (layer_ref);
    } else
    if (ie4) {
        // Internet Explorer 4
	document.all[x].style.visibility = 'hidden';
    } else
    if (ns6) {
        // Netscape 6 and other W3C-compliant browsers
        var map_layer = document.getElementById( x );
        map_layer.style.visibility = 'hidden';
    }
    return true;
}

// recursively hide each of layers above this one
function map_hide_children ( x, from ) {
    // base case, recursion escape clause:
    if (!children[x]) return;

    // only Netscape 6 is allowed to close layers from an image tag
    // all other browsers call this from an imagemap area
    if (from == 'img' && !ns6) {
	return false;
    }

    for (var i in children[x]) {
 	var child = children[x][i];
	map_hide (child);
	map_hide_children (child, from);
    }
    return true;
}











