#! /usr/local/bin/perl -wT
# vi:ts=4:sw=4:
# $Id: hotlist.cgi,v 1.3 2004/02/20 15:28:27 ruben Exp $

use 5.006;
use strict;
use lib qw(/home/ttraffic/lib/perl5.6.0);
use CGI 2.86 qw/-newstyle_urls -nosticky :standard :imagemap/;
use CGI::Carp qw(fatalsToBrowser);
use RIPE::TTM qw(:all -nohotlist);

# TODO
#
# - Summary support
# - POD support
# - IPv6 Support

$ENV{'PATH'} 		= "/bin/:/usr/bin:/ncc/ttpro/bin:/home/ttraffic/bin";

sub create_hotlist();

my ($header,$footer, $subtitle);

my $gfxformat = "gif";

my (
	$cgi,
	$view,
	$ipv,
	@hotlist,
	$period,
	$expandnode,
	$self_url,
	$referrer,
	);

$ipv=4;

# Make usable through SSI and standalone operation
$cgi = exists($ENV{'REDIRECT_QUERY_STRING'}) ?
		new CGI($ENV{'REDIRECT_QUERY_STRING'}) :
		new CGI;

$self_url = exists($ENV{'REDIRECT_SCRIPT_URL'}) ? 
				$ENV{'REDIRECT_SCRIPT_URL'} :
				$cgi->url(-relative=>1);

$referrer = exists($ENV{'HTTP_REFERER'}) ? 
				$ENV{'HTTP_REFERER'} :
				$self_url;


# Dump the page

if (defined($cgi->param('ipv')) && $cgi->param('ipv') =~ /^[46]$/)
{
	$ipv = $cgi->param('ipv'); 
}
else
{
	$cgi->param('ipv',$ipv);
	print redirect $cgi->url(-absolute=>1, -query=>1);
	exit(0);
}

$subtitle = exists $ENV{'TTMCHAIN'} ? "Hotlist Overview for $ENV{'TTMCHAIN'}"
										: "Hotlist Overview";
$subtitle = $ipv == 6 ? "IPv6 $subtitle" 
							: "IPv4 $subtitle";

($header,$footer) = get_headers("Test Traffic Measurement","$subtitle");

# Retrieve previous cookie
if ($cgi->cookie("hotlist$ipv"))
{
	@hotlist = $cgi->cookie("hotlist$ipv");
}

# Create cookie
if ((defined($cgi->param('hotlist')) && $cgi->param('hotlist')) || (defined($cgi->param('clear')) ))
{
	my $hotlist_cookie = $cgi->cookie(
		-name => "hotlist$ipv",
		-expires => defined($cgi->param('clear')) ? "now" : "+6M",
		-value => [ $cgi->param('hotlist') ]
	);

	# Cleanup

	$cgi->delete('submit') if defined($cgi->param('submit'));
	$cgi->delete('clear') if defined($cgi->param('clear'));
	$cgi->delete('hotlist') if defined($cgi->param('hotlist'));

	print redirect( -uri => $cgi->url(-absolute=>1, -query=>1), -cookie => $hotlist_cookie);
	exit(0);
}
elsif (defined($cgi->param('referrer')))
{
	print redirect( -uri => $cgi->param('referrer'));
}
else
{
	$cgi->param( -name => 'hotlist', -value => \@hotlist);
}

print header, $header,	
		create_hotlist(),
	$footer;

sub create_hotlist()
{
	my (@included_testboxes) = map { s/\..*$//; $_ } $ipv == 6 ? @RIPE::TTM::testboxes6 : @RIPE::TTM::testboxes4;
    return start_form(-method=> "GET",
                            -name=>"theform",
                            -action=>$cgi->url(-relative=>1),
                            ),
		hidden( -name => 'ipv', -value=> $ipv),
		hidden( -name => 'referrer', -value=> $referrer),
		table({ -width=>"100%" }, Tr(td(
			p("Switch protocol: ", map  { $ipv == $_ ? u("[IPv$_]") : "[" . a({ -href => "$self_url?ipv=$_" }, "IPv$_" ) . "]" } (4,6)         ),
			$#hotlist > 0 ? p("You have created a hotlist and you might want to bookmark this ", a({ -href => $cgi->url(-query => 1, ) }, "link") ,br
							  "To erase the hotlist, click ", a({ -href => "$self_url?ipv=$ipv;clear" }, "here"), "..." ) 
							: p({ -class => "error" }, "Your hotlist is empty, please create one if you like") ,
				), td(

					CGI::div( { -style=> "font-size: x-small;" },
						table( 
							Tr(td({ -align => "right" }, a({ -href => "${RIPE::TTM::overview_url}" }, "View plot overview..."))),
							Tr(td({ -align => "right" }, a({ -href => "${RIPE::TTM::pod_url}" }, "Go to Plots on Demand..."))),
							Tr(td({ -align => "right" }, a({ -href => "${RIPE::TTM::routing_url}" }, "Query Routing Vector Database..."))),
							Tr(td({ -align => "right" }, a({ -href => "${RIPE::TTM::summary_url}" }, "View summaries..."))),
						)
					),
				))),
			CGI::div({-style => 'font-size: 60%; white-space: nowrap;'},
				checkbox_group( -name => "hotlist",
								-values => [grep { /^@{[join('|',@included_testboxes)]}$/ } sort tboxsort keys %RIPE::TTM::chainmap],
								-labels => { map { $_ => "$_: $RIPE::TTM::gpsinfo{$_}{'place'}" } keys %RIPE::TTM::gpsinfo } ,
								-defaults => \@hotlist,
								-columns => 2,
								-attributes => { map { $RIPE::TTM::gpsinfo{$_}{status} ne 'ACTIVE' && $_ => { 'class' => 'inactive' }  } keys %RIPE::TTM::gpsinfo },
							),
			),
			defaults,
			reset,
			submit(-name=>"submit", -value=> $referrer ne $self_url ? "Create hotlist and go back" : "Create hotlist" ),
		end_form;
}

