#! /usr/local/bin/perl -wT
# $Id: plotindex.cgi,v 1.1 2004/02/20 15:28:27 ruben Exp $
# vi:sw=4:ts=4:

use lib qw(/home/ttraffic/lib/perl5.6.0);
use strict;
use CGI 2.86 qw/-newstyle_urls -nosticky :standard :imagemap/;
use CGI::Carp qw(fatalsToBrowser);
use RIPE::TTM qw(:all);

# TODO
#
# use summaries to point to interesting graphs
# replace gpsinfo code
# make the following uri params uniform under routing, pod and summary:
#      ipv, hotlist, period
# recent visit list to be stored in cookie


sub show_miniplots($$$@);

my (
	$cgi,
	$view,
	$period,
	$expandnode,
	$self_url,
	@included_testboxes,
	$ipv,
	$ipv_prefix,
	@recentlist,
	$recentlist_cookie,
	@boxes,
	);

my ($header,$footer, $subtitle);

my $gfxformat = "gif";

# XXX Add your views and descriptions here, not in the code. the code will adapt

my %views	= (
				"Delay"		=> "",
				"Jitter"	=> "ipdv",
				"Trends"	=> "trends",
			);
my @views_order = ("Delay", "Jitter", "Trends");

my %periods	= (
				"Day"	=> "day",
				"Week"	=> "week",
				"Month" => "month",
				"6 Month Trend"	=> "trends",
			);
my %view_period	= (
				""		=> [qw(Day Week Month)],
				"ipdv."		=> [qw(Day Week)],
				"trends."	=> [("6 Month Trend")],
			);

# XXX the _order arrays are because of keys %ahash returns random ordered data

MAIN: {

	# Make usable through SSI and standalone operation
	$cgi = exists($ENV{'REDIRECT_QUERY_STRING'}) ?
			new CGI($ENV{'REDIRECT_QUERY_STRING'}) :
			new CGI;

	$self_url = exists($ENV{'REDIRECT_SCRIPT_URL'}) ? 
					$ENV{'REDIRECT_SCRIPT_URL'} :
					$cgi->url(-relative=>1);

	$view="";
	$period="day";
	$expandnode="";
	$ipv=4;
	$recentlist_cookie = "";

	if ($cgi->param())
	{
		$view = $cgi->param('view') . "." if defined($cgi->param('view')) && $cgi->param('view') =~ /^(@{[join('|',values %views)]})$/;
		$expandnode = $cgi->param('node') if defined($cgi->param('node')) && $cgi->param('node') =~ /^(@{[join('|',keys %RIPE::TTM::chainmap)]})$/;

		$view = "" if $view eq ".";

		if (defined($cgi->param('period')) && $cgi->param('period') =~ /^(@{[join('|', (map {$periods{$_} } @{$view_period{$view}}))]})$/)
		{
			$period = $cgi->param('period');
		}
		else
		{
			$period = $periods{$view_period{$view}->[0]};
		}

		$ipv = $cgi->param('ipv') if defined($cgi->param('ipv')) && $cgi->param('ipv') =~ /^[46]$/;

		# Die if parameters aren't sane...

		if ($cgi->param('submit'))
		{
			$cgi->delete('submit');
			print redirect $cgi->url(-absolute=>1, -query=>1);
			exit(0);
		}

		if ($cgi->param('create') && $cgi->param('node'))
		{
			$cgi->delete('create');
			print redirect $cgi->url(-absolute=>1, -query=>1);
			exit(0);
		}
	}
	else
	{
		$cgi->param('ipv',$ipv);
		print redirect $cgi->url(-absolute=>1, -query=>1);
		exit(0);
	}

	# Handle exceptions here:
	# 

	$view = 'trends.' if $period eq 'trends';

	$ipv_prefix = ($ipv == 6) ? "IPv6." 
								: "";

	# Remove ipv4/ipv6 only boxes if the other protocol family is specified.

	@included_testboxes = map { s/\..*$//; $_ } $ipv == 6 ? @RIPE::TTM::testboxes6 : @RIPE::TTM::testboxes4;

	foreach my $ttid (keys %RIPE::TTM::chainmap)
	{
		if (grep { /^${ttid}$/ } @included_testboxes)
		{
			$RIPE::TTM::chainmap{$ttid} = [ grep { /^@{[join('|',@included_testboxes)]}$/ } @{$RIPE::TTM::chainmap{$ttid}} ];
		}
		else
		{
			delete $RIPE::TTM::chainmap{$ttid};
		}
	}

	# Dump the page

	# Create header
	
	$subtitle = exists $ENV{'TTMCHAIN'} ? "Plot Overview for $ENV{'TTMCHAIN'}"
										: "Plot Overview";
	$subtitle = $ipv == 6 ? "IPv6 $subtitle" 
							: "IPv4 $subtitle";
	$subtitle .= $expandnode ? " box $expandnode" 
								: "";

	$subtitle = "Hotlist $subtitle" if $#RIPE::TTM::hotlist > 0;

	($header,$footer) = get_headers("Test Traffic Measurement","$subtitle");

	# Manage cookies

	@recentlist = $cgi->cookie("recentlist$ipv");
	if ($expandnode && (!grep {/^$expandnode$/} @recentlist))
	{
		unshift(@recentlist, $expandnode);
		$recentlist_cookie = $cgi->cookie(
								-name => "recentlist$ipv",
								-expires => "+6M",
								-value => [ @recentlist[0 .. ($#recentlist < 5 ? $#recentlist : 5) ] ]
		);
	}

	@boxes = sort tboxsort keys %RIPE::TTM::chainmap;

	if ($expandnode)
	{
		if ($#RIPE::TTM::hotlist > 0 )
		{ 
			die "$expandnode not present... Please disable hotlist temporarily and try again.\n" unless exists $RIPE::TTM::chainmap{$expandnode};
		}
		else
		{
			die "No relations present for $expandnode ... Either no measurements for this protocol or not in this chain.\n" unless exists $RIPE::TTM::chainmap{$expandnode};
		}
	}

	print header(-cookie => [$recentlist_cookie], -expires=>'+1d' ), 
		$header,
		$#recentlist > -1 ? (
						start_form(-method=> "GET",
									-name=>"theform",
									-action=>$cgi->url(-relative=>1),
									),
						popup_menu(-name=> "node",
									-values=> \@recentlist,
									-labels => { map { $_ => "$_: $RIPE::TTM::gpsinfo{$_}{'place'}" } keys %RIPE::TTM::gpsinfo },
									-onChange => "this.form.submit();",
									),
						submit(-name=>"submit", -value=>"Go to recent visited"),
						end_form
							)
							: "",
		$#RIPE::TTM::hotlist > 0 ? 
				p(a({ -href => $cgi->url(-absolute=>1, -query=>1) . ";nohotlist"  }, "turn off"), " your hotlist temporarily or ", 
				a({ -href => "${RIPE::TTM::hotlist_url}?ipv=$ipv" }, "edit"), " the one that is currently active") 
			: defined($cgi->param('nohotlist')) ? 
				p(a({ -href => $cgi->url(-relative=>1) . "?" . cleanup_query_string("nohotlist", undef, $cgi) }, "turn on"), " temporarily disabled hotlist...") 
				: p(a({ -href => "${RIPE::TTM::hotlist_url}?ipv=$ipv" }, "create a hotlist"), " to discard plots you are not interested in..."),

		table(
			Tr( td( {-colspan => 2}, 
				table({ -width => "100%" }, Tr(
					td({ -align => "center" },"[Plot&nbsp;type:&nbsp;", (map {
						$view =~ /^$views{$_}\.?$/ ? u(b($_)) : a({ -href=> "$self_url?" . cleanup_query_string("view", $views{$_}, $cgi)  }, $_)	
						} @views_order), "]"
					),
					td({ -align => "center" },"[Periods:&nbsp;", (map {
						$periods{$_} eq $period ? u(b($_)) : a({ -href=> "$self_url?" . cleanup_query_string("period",$periods{$_},$cgi) }, $_)	
						} @{ $view_period{$view}}), "]"
					),
					td( { -align => "center" }, "[Switch protocol: " . join('/',map  { $ipv == $_ ? u("IPv$_") : a({ -href => "$self_url?" . cleanup_query_string("ipv", $_, $cgi) }, "IPv$_" ) } (4,6)) . "]"),
				)),
			)),
			Tr(td( {-colspan => 2},hr)),
			Tr(
			td({-width => "10%", -valign=>"top"},
				CGI::div( { -id => "plotindexsidebar" } ,
					table( {-cellspacing=>0, -cellpadding=>0},
						map {
							my ($node) = $_;
							my (@foldout) = ();
							my $hrefhash = { -href=> "$self_url?" . cleanup_query_string("node",$node eq $expandnode ? undef: $_,$cgi), onmouseover=>"this.T_STICKY=true;this.T_DELAY=1000;return escape('<b>$_</b>: @{[ exists($RIPE::TTM::gpsinfo{$_}) ? $RIPE::TTM::gpsinfo{$_}{'longdesc'} : \"RESERVED\" ]}')" };
							my $expandgfx = sprintf("${RIPE::TTM::images_url}/node/ftv2%s%snode.gif", 
															$node eq $expandnode ? "m" : "p",
															$node eq $boxes[$#boxes] ? "last" : "");

							@foldout = sort tboxsort @{$RIPE::TTM::chainmap{$_}}  if $node eq $expandnode;

							@foldout = map {
								Tr(
									td({ -class=> "closed" }, img( { -src => "${RIPE::TTM::images_url}/node/ftv2vertline.gif", -alt => "|", -border => 0 } )),
									td({ -class=> "menuexpanded" }, img( { -src => $_ eq $foldout[$#foldout] ? "${RIPE::TTM::images_url}/node/ftv2lastnode.gif" : "/images/node/ftv2node.gif", -alt => "`", -border => 0 } )),
									td({ -class=> "menuclosed" }, 
										a({-href => "$self_url?" . cleanup_query_string("node",$node,$cgi) . "#$_" , onmouseover=>"this.T_STICKY=true;this.T_DELAY=1000;return escape('<b>$_</b>: @{[ exists($RIPE::TTM::gpsinfo{$_}) ? $RIPE::TTM::gpsinfo{$_}{'longdesc'} : \"RESERVED\" ]}')"}, "&nbsp;$_"))
								), 
							} @foldout if $node eq $expandnode;

							Tr( {-class=> $node eq $expandnode ? "menuexpanded" : "menuclosed" },
								td(a( $hrefhash, img( { -width => "16", -height => "22", -src => $expandgfx, -alt => "+", -border => 0 }))),
								td({-colspan => 2},  a( $hrefhash, "&nbsp;$_"))
							), @foldout
						} @boxes 
					)
				)
			),
			td( { -valign=>"top"}, CGI::div({ -style => "overflow:auto; height: 28em"},
				$expandnode ? 
					show_miniplots($view,$period,$expandnode, sort tboxsort @{$RIPE::TTM::chainmap{$expandnode}}) :
					"",
			)),
		)), $footer;
}

sub show_miniplots($$$@)
{
	my ($view,$period,$source,@dest) = @_;
	my ($URLsummary,$URLpod,$URLrouting, $URLshowplots_in, $URLshowplots_out);

	# Build up src->dst and dst->src matrices

	if ($source)
	{
		$URLsummary = new CGI("ipv=$ipv;period=$period;boxname=$expandnode");
		$URLrouting = new CGI("SRC=$source");
		$URLpod = new CGI("SRC=$source");

		return CGI::div({align => "center" , -style=> "font-size: smaller;" },
					h2(b("$source:")," @{[ exists($RIPE::TTM::gpsinfo{$source}) ? $RIPE::TTM::gpsinfo{$source}{'longdesc'} : \"RESERVED\" ]}"),
					table(
							Tr(
								td({-align=>"right"}, "[", 
									join("/", map  { $ipv == $_ ? u("IPv$_") : a({ -href => "$self_url?" . cleanup_query_string("ipv", $_, $cgi) }, "IPv$_" )  } (4,6) ),
								"]"),
								td({-align=>"right"}, "[",
									$period =~ /^(day|week)$/ ? (a({ -href => "$RIPE::TTM::summary_url?" . $URLsummary->query_string() },"Summaries for this period"))
																				: ("No summaries available for this period"),
								"]"),
								td({-align=>"right"}, "[", a({ -href=>"${RIPE::TTM::pod_url}?" . $URLpod->query_string()  }, "Go to plots on demand for this box"), "]"),
								td({-align=>"right"}, "[", a({ -href=>"${RIPE::TTM::routing_url}?" . $URLrouting->query_string() }, "Go to the routing vectors for this box"), "]")
							),
						)
					), 
		table({cellpadding=>"4", cellspacing=>"0", width=>"90%"}, 
			Tr(
				th({ -class => "datatable" }, h2("Target")),
				th({ -class => "datatable" }, h2("INcoming traffic")),
				th({ -class => "datatable" }, h2("OUTgoing traffic"))
			), 
			(map { 
				my ($inactiveclass) =  (!(exists($RIPE::TTM::gpsinfo{$_}) && $RIPE::TTM::gpsinfo{$_}{'status'} eq "ACTIVE" &&  $RIPE::TTM::gpsinfo{$source}{'status'} eq "ACTIVE")) ? "inactive" : "active";
				Tr( { -class=> $inactiveclass },
					td({ -valign=>"top"}, CGI::div( {-class => $inactiveclass }, a({name=>$_,onmouseover=> &maketboxtooltip($_,$dest[0])}, b(span({ style => "font-size: xx-large"},$_))), $inactiveclass eq "inactive" ? p({ -style => "font-size: xx-small;" }, ($RIPE::TTM::gpsinfo{$source}{'longdesc'} =~ /^ACTIVE/ ? "destination" : "source") . " box is $inactiveclass",br,"showing last known plots") : "&nbsp;" )), 
					td({ -align=>"center"}, a({ href=>"${RIPE::TTM::detail_url}?ipv=$ipv;src=$_;dst=${source}#${period}${view}"}, img( { -width => 226, -height => 167, border => 0, src => "${RIPE::TTM::base_url}/${period}/mini/${ipv_prefix}${view}$_.${source}.gif", alt => "Mini ${period} IPv$ipv plot for $_.${source}" }))),
					td({ -align=>"center"}, a({ href=>"${RIPE::TTM::detail_url}?ipv=$ipv;src=${source};dst=$_#${period}${view}"}, img( { -width => 226, -height => 167, border => 0, src => "${RIPE::TTM::base_url}/${period}/mini/${ipv_prefix}${view}${source}.$_.gif", alt => "Mini ${period} IPv$ipv plot for $_.${source}"}))),
				), Tr(td( { -colspan=>3 }, hr )) 
				} @dest)
		);
	}
	return i("Please select one of the following sources in the left hand column to see its relations")
}

sub maketboxtooltip($$)
{
	my $boxname = $_[0];
	my $topboxname = $_[1];
	return maketooltip( 
		(b($boxname) . ": @{[ exists($RIPE::TTM::gpsinfo{$boxname}) ? $RIPE::TTM::gpsinfo{$boxname}{'longdesc'} : \"RESERVED\" ]}"),
		(a( { -href=>"/summaries/?boxname=$boxname"}, "Day summaries")),
		(a( { -href=>"/summaries/?boxname=$boxname;period=week"}, "Week summaries")),
		(a( { -href => "#$topboxname" }, "top")),
	);
}

# vi:ts=4:sw=4:
