#!/usr/local/bin/perl  -wT
# $Id: summary.cgi,v 1.6 2004/07/07 10:33:11 ruben Exp $
# vi:ts=4:sw=4:

# Copyright (c) 2002                            RIPE NCC
#
# All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of the author not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

#------------------------------------------------------------------------------
# Module Header

# TODO: revisit 'relative diff' computation, also color 0 -> 100 changes
# TODO: seperate marking for delays and Loss ?

#
# Data structure:

# testbox hash

# box {$shortname} {'incoming'}{'short2.5per'}

# Sort on:
#		minimum, median, maximum, loss  [in|out]
#		absolute change, relative change, value 
#		descending / ascending  (absolute values)

use 5.006;
use lib qw(/home/ttraffic/lib/perl5.6.0);
use strict;
use XML::DOM;
use CGI 2.86 qw/-newstyle_urls -nosticky :standard :imagemap/;
use CGI::Carp qw(fatalsToBrowser);
use NDBM_File;
use Fcntl; 
use RIPE::TTM qw(:all);

sub cellcolor($$$$);
sub cellemphasis($$$$);
sub decodeXML($$$$$);
sub insertsort($$$$$);
sub outputCSV($);
sub outputForm($);
sub outputHTML($);
sub sortvalue($$$$);
sub createplotlink($$);
sub createxmlname();

my $progname = $ENV{'SCRIPT_NAME'} || "dummy";

my $self_url;
my $cgi;

my %box;				# hash with testbox summary data
my %param;				# hash with additional parameters
my @boxlist = ();			# sorted list of testbox names
my $lightgray = "#f8f8f8";
my $red="#ff2222";			# color definitions
my $green="#22ff22";

my @red = (
	"#ffdddd",
	"#ffcccc",
	"#ffbbbb",
	"#ffaaaa",
	"#ff9999",
	"#ff8888",
	"#ff7777",
	"#ff6666",
	"#ff5555",
	"#ff4444",
	"#ff3333",
	"#ff2222",
);

my @green = (
	"#ddffdd",
	"#ccffcc",
	"#bbffbb",
	"#aaffaa",
	"#99ff99",
	"#88ff88",
	"#77ff77",
	"#66ff66",
	"#55ff55",
	"#44ff44",
	"#33ff33",
	"#22ff22",
);

my %default;

my @classes = ("incoming", "outgoing");
my @fieldlist = ("perc2.5" , "Median", "perc97.5", "Loss");

#### Main routine

MAIN: {

my ($name, $class, $field, $value);
my ($sortclass, $sortfield, $sortorder);
my $threshold;				#  < 0  sort on absolute diffs
					#  = 0  sort on value
					#  > 0  sort on relative diffs

$default{'file'}      = "summary.xml";
$default{'format'}    = "html";
$default{'sortfield'} = "name";
$default{'sortkey'}   = "value";
$default{'sortorder'} = "ascending";
$default{'threshold'} = "40";
$default{'unit'}      = "percent";
$default{'boxname'}   = "tt01";
$default{'ipv'}       = "4";
$default{'period'}    = "day";

# get parameters;

# Make usable through SSI and standalone operation
$cgi = exists($ENV{'REDIRECT_QUERY_STRING'}) ?
		new CGI($ENV{'REDIRECT_QUERY_STRING'}) :
		new CGI;

$self_url = exists($ENV{'REDIRECT_SCRIPT_URL'}) ? 
				$ENV{'REDIRECT_SCRIPT_URL'} :
				$cgi->url(-relative=>1);

$param{'file'}      = $cgi->param('file')      || $default{'file'};
$param{'format'}    = $cgi->param('format')    || $default{'format'};
$param{'sortorder'} = $cgi->param('sortorder') || $default{'sortorder'};
$param{'sortfield'} = $cgi->param('sortfield') || $default{'sortfield'};
$param{'sortkey'}   = $cgi->param('sortkey')   || $default{'sortkey'};
$param{'unit'}      = $cgi->param('unit')      || $default{'unit'};
$param{'ipv'}       = $cgi->param('ipv')       || $default{'ipv'};
$param{'period'}       = $cgi->param('period')       || $default{'period'};

# Now that we know about our chain, lets change $default{'boxname'} to something sensible
# XXX RVS: Yes "beautyfull oneliner once again, but it gets me there 

if ($RIPE::TTM::use_chain_auth)
{
	$default{'boxname'} = ((sort tboxsort (grep {/(@{[join('|',$param{'ipv'} == 6 ? @RIPE::TTM::testboxes6 : @RIPE::TTM::testboxes4 )]})/} keys %RIPE::TTM::chainmap))[0]);
}
else
{
	$default{'boxname'} = ((sort tboxsort ($param{'ipv'} == 6 ? @RIPE::TTM::testboxes6 : @RIPE::TTM::testboxes4 ))[0]);
}
$param{'boxname'}   = $cgi->param('boxname')   || $default{'boxname'};

# can't use above constructs for 'threshold' field because '0' is valid value
if (! defined $cgi->param('threshold')) {
	$param{'threshold'} = $default{'threshold'};
}
else {
	$param{'threshold'} = $cgi->param('threshold');
}


# sanity check: no special chars in filename, only words
# (i.e. alphanumeric + '_') are allowed, optionally seperated by '.' 

# True untaint here, see perlsec(1)
($param{'file'}) = $param{'file'} =~ /^((\w\.*){1,})$/;

if (! defined($param{'file'})) {
	die ("$param{'file'} - Invalid characters in filename\n");
}

my $prefix="";
if ($param{'ipv'} == 6) {
	$prefix = "IPv6.";
}

my $filename = createxmlname();

if ((! -f $filename) ||  (! -r $filename) ){
	die("cannot open file $filename\n");
}

# sanity check: threshold is a (optionally floating point) number
if (! ($param{'threshold'} =~ /^\s*\d+\.*\d*\s*$/)) {
	die ("$param{'threshold'} - Not a number\n");
}

if ($param{'unit'} eq "percent") {
	# absolute change in value 
	$threshold = $param{'threshold'} / 100;
}
elsif ($param{'unit'} eq "miliseconds") {
	$threshold = -$param{'threshold'};
}
else {
	die("$param{'unit'} - unknown unit for change field\n");
}


my $change;
if (defined($param{'$sortkey'}) && $param{'$sortkey'} eq "absolute change") {
	$change = -1;
}
elsif (defined($param{'$sortkey'}) && $param{'sortkey'} eq "relative change") {
	$change = +1;
}
else {
	# sort on the shorterm value
	$change=0;
}

# FIXME
if ($param{'format'} eq "xml") {

	# Someone tries to come in the SSI way... we cannot force redirect so lets print a link instead..
	if (_is_included())
	{
		print header, p(a({ -href=> $progname . "?" . $cgi->query_string()}, "Use this link to view the raw XML" ));
	}
	else
	{
		# Surpress non chainmembers by skipping printing
		my $do_output=1;

		# no point in decoding, give it to the user as-is

		open(XML, $filename) || die ("$filename: cannot open\n");
		print "Content-type: text/xml\n\n";
		while (defined( my $input = <XML>)) {

			if($RIPE::TTM::use_chain_auth)
			{
				$do_output = 0 if ($input =~ /^<target\s+name="([^"]+)">/ && !exists $RIPE::TTM::chainmap{$1});
			}

			# XXX Should do it the XML::DOM way
			print $input if $do_output;

			$do_output = 1 if ( $do_output == 0 && $input =~ /^<\/target>/ );
		}
	}
	exit(0);
}

# decode XML and output in requested format

decodeXML($filename, $param{'sortfield'}, $change, $threshold, $param{'sortorder'});

if ($param{'format'} eq "html") {
	outputHTML($threshold);
}
elsif ($param{'format'} eq "ascii") {
	if (_is_included())
	{
		print header, p(a({ -href=> $progname . "?" . $cgi->query_string()}, "Use this link to view the raw ascii" ));
	}
	else
	{
		outputCSV($threshold);
	}
}
else {
	die ("$param{'format'}: Unknown output format\n");
}

exit(0);

}

#---------------------------------------------------------------------
# Subroutine header
#  
#  decode XML file, fill globals %box and @boxlist
#  (the ordered list of names, starting point for HTML output)
#---------------------------------------------------------------------

sub decodeXML($$$$$) {

	my ($filename, $sortfield, $change, $threshold, $sortorder) = @_;
	my ($nodes, $node, $att, $n);

	my $parser = new XML::DOM::Parser;
	my $doc = $parser->parsefile ("$filename");

	#
	# only one source in an XML file
	#

	$nodes = $doc->getElementsByTagName ("source");
	$node  = $nodes->item(0);	      
	$att = $node->getAttributeNode("name");
	$param{'source'} = $att->getValue;

	#
	# fetch other parameters characterizing this summary
	#
	
	$nodes = $doc->getElementsByTagName ("param");
	$node  = $nodes->item(0);	      
	
	$att = $node->getAttributeNode("date");
	$param{'date'} = $att->getValue;
	
	$att = $node->getAttributeNode("period");
	$param{'period'} = $att->getValue;
	
	$att = $node->getAttributeNode("shortterm");
	$param{'shortterm'} = $att->getValue;
	
	$att = $node->getAttributeNode("longterm");
	$param{'longterm'} = $att->getValue;
	
	
	$nodes = $doc->getElementsByTagName ("target");
	$n = $nodes->getLength;
	
	for (my $i = 0; $i < $n; $i++)
	{
		my $node = $nodes->item ($i);
	    	$att = $node->getAttributeNode("name");
		my $name = $att->getValue;

		if ($RIPE::TTM::use_chain_auth)
		{
			next unless exists($RIPE::TTM::chainmap{$name});
		}

		foreach my $class (@classes) {
	    		my $clist = $node->getElementsByTagName ($class);
			foreach my $field (@fieldlist) {
	    			my $flist = ($clist->item(0))->getElementsByTagName($field);
    				$att = ($flist->item(0))->getAttributeNode("short");
				my $value = $att->getValue;

				if ($field eq 'Loss') {
					# convert fraction to percentage
					if ($value != -1) {
						$value = $value * 100;
						$value = sprintf("%.1f", $value);
					}
				}

				$box{$name}{$class}{$field}{'short'} = $value;

    				$att = ($flist->item(0))->getAttributeNode("long");
				$value = $att->getValue;

				if ($field eq 'Loss') {
					# convert fraction to percentage
					if ($value != -1) {
						$value = $value * 100;
						$value = sprintf("%.1f", $value);
					}
				}

				$box{$name}{$class}{$field}{'long'} = $value;
			}
		}
		# insertion sort
		if ($sortfield ne "name") {
			# can't do numerical comparison on strings
			insertsort  ($name, $sortfield, $change, $threshold, $sortorder);
		}
		else {
			@boxlist = (@boxlist , $name);
		}
	}

	if ($sortfield eq "name") {
		if ($sortorder eq "descending") {
			@boxlist = reverse sort tboxsort keys %box;
		}
		else {
			@boxlist = sort tboxsort keys %box;
		}
	}

	# Avoid memory leaks - cleanup circular references
	# for garbage collection
	$doc->dispose;

}

#---------------------------------------------------------------------
# Subroutine header
#  
#  insert new record in @boxlist, such that list remains sorte
#  (insertion sort algorithm) The sortvalue() subroutine is
#  expected to return a unique value for a given class,field
#
#---------------------------------------------------------------------
#


sub insertsort($$$$$) {

	my ($name, $sortfield, $change, $threshold, $sortorder) = @_;
	
	my $boxno = 0;
	my $value = sortvalue($name, $sortfield, $change, $threshold);

	BOX: while ($boxno <= $#boxlist) {
		my $temp = sortvalue($boxlist[$boxno], $sortfield, $change, $threshold);	
		if ((($sortorder eq "ascending")  && ($temp > $value)) ||
		    (($sortorder eq "descending") && ($temp < $value)))   {
			last BOX;
		}

		$boxno++;
	}

	#
	# found the right spot, now insert it
	#

	for (my $i=$#boxlist; $i >= $boxno; $i--) {
		$boxlist[$i+1] = $boxlist[$i];
	}
	
	$boxlist[$boxno]  = $name;
}

#---------------------------------------------------------------------
# Subroutine header
#  
# Purpose           : determine value to be used in sorting
# Input             : box name, sortfield, sortkey, threshold, global %box
# Output            : value which can be used in insertion sort
# Comments          : the sortkey tells what to do with the sortfield:
#		      return the value, relative change or absolute change
#		      When sorting on 'marked cells', the threshold
#		      determines if a cell is 'red', 'green' or neutral
#
#---------------------------------------------------------------------

sub sortvalue($$$$) {
	my ($name, $sortfield, $sortkey, $threshold) = @_;

	my $sortclass;

	if ($sortfield eq "name") {
		return($name);
	}
	
	if ($sortfield ne "marked cells") {
		($sortclass, $sortfield) = split(/ /, $sortfield);

	        if ($sortkey == 0) {
			# return recent 'shortterm' value
			return($box{$name}{$sortclass}{$sortfield}{'short'});
		}
		elsif ($sortkey > 0) {
	        	if (($box{$name}{$sortclass}{$sortfield}{'short'} == -1) ||
	                    ($box{$name}{$sortclass}{$sortfield}{'long'} <= 0)) {
				return (-1);
			}

			# relative change between short and long

			my $diff = $box{$name}{$sortclass}{$sortfield}{'short'}
				- $box{$name}{$sortclass}{$sortfield}{'long'};
			return ($diff/$box{$name}{$sortclass}{$sortfield}{'long'});
		}
		else {
			# $change < 0  , absolute change
			my $diff = $box{$name}{$sortclass}{$sortfield}{'short'}
				- $box{$name}{$sortclass}{$sortfield}{'long'};
			return ($diff);
		}
	}
	else {
		# sort on marked cells, derive one number from 8 cell colors
		# weigth of a cell is higher if it is a parameter less
		# likely to change (2.5 percentile has highest weight)
		# also to get better grouping, incoming has slightly
		# higher weight than outgoing
		
		my $value = 0;
		my $weight = 1.1;
		foreach my $class (@classes) {
			$weight += $#fieldlist;
			foreach my $field (@fieldlist) {
				my $col = cellcolor($name, $class, $field, $threshold); 
				if (grep { /^\Q${col}\E$/ } ($green, @green) ) {
					$value += $weight;
				}
				if (grep { /^\Q${col}\E$/ } ($red, @red) ) {
					$value += 2 * $weight;
				}
				$weight--;
			}
			$weight += 0.1;
		}
		return($value);
	}
}
		
#---------------------------------------------------------------------
# Subroutine header
#
# Purpose           : create HTML output 
# Comments          : 
# Input             : threshold for coloring
#		      globals $cgi (CGI object) and %param hash
# Output            : HTML page
# 
#---------------------------------------------------------------------

sub outputHTML($) {

	my ($threshold) = @_;

	my $period = $param{'period'};

	# URL for cross reference to other box'es statistics

	my $url="${self_url}?sortfield=$param{'sortfield'}&ipv=$param{'ipv'}&" .
		 "sortkey=$param{'sortkey'}&sortorder=$param{'sortorder'}&" .
		 "threshold=$param{'threshold'}&unit=$param{'unit'}&period=$param{'period'}";

	$url =~ s/ /+/g;
	$url .= "&nohotlist" if (defined($cgi->param('nohotlist')));	

	my $boxname = $param{'boxname'};

	my $prefix = "";
	if ($param{'ipv'} == 6) {
		$prefix = "IPv6 ";
	}

	print header,print_header("${prefix}TTM $param{'period'} summaries for $param{'source'}" ,
		     (!$RIPE::TTM::use_chain_auth || exists($RIPE::TTM::chainmap{$param{'boxname'}})) ? "$RIPE::TTM::gpsinfo{$boxname}{'org'}, $RIPE::TTM::gpsinfo{$boxname}{'city'}, $RIPE::TTM::gpsinfo{$boxname}{'country'}" .
		     " on $param{'date'}" : "$param{'boxname'} is not in chain $ENV{'TTMCHAIN'}" );

	print p("Go to ", (
			map  { $param{'ipv'} == $_ ? u("[IPv$_]") : "[" . a({ -href => "$self_url?ipv=$_" }, "IPv$_" ) . "]" } (4,6)
			), " $param{'period'} summaries");


	print <<"EOT";
		<p>
		The table below displays the TTM delay &amp; loss parameters for 
		traffic received from and send to the listed test-boxes by 
		$param{'source'}. 
		Each cell lists both the result for the last $period as well 
		as (in parentheses) the median value of this parameter for 
		the preceding 10 ${period}s. If the two differ by more than 
		$param{'threshold'} $param{'unit'} the background of the cell 
		will be colored in a gradient: 
		<font color="$red">red</font> if the parameter 
		increased, <font color="$green">green</font> if it decreased.
		</p>
EOT

	print p("The gradients are defined as followed:"),
				table({-border => 0, -cellpadding => 0, -cellspacing => 0}, 
					Tr( td(i("Slighty above&nbsp;")),
						(map { 
							td({ -bgcolor => "$_" }, "&nbsp;")  
						} @red),
						td(i("&nbsp;A good deal"))
					  ),"\n",
					Tr( td(i("threshold&nbsp;")),
						(map { 
							td({ -bgcolor => "$_" }, "&nbsp;")  
						} @green),
						td(i("&nbsp;above threshold"))
					  ),"\n",
					);
	

	print "<p>The table is sorted on ";

	my $sortfield = $param{'sortfield'};

	if (($sortfield ne "marked cells") && ($sortfield ne "name")) {
		if ($param{'sortkey'} ne "value") {
			print "$param{'sortkey'} in ";
		}
	}

	print <<"EOT";
		<em>$sortfield</em> in <em>$param{'sortorder'}</em> order.
		These settings can be changed through the form following.
		The hyperlinks take you to the pages with 
		the delay plots for the last 24 hours 7 days and 30 days, 
		thus providing a more detailed look at the data. 
		</p>

EOT

	print outputForm($threshold);

	# my $basehref = "http://www.ttm.ripe.net/plots/showplots.cgi";
	

	print STDOUT (($RIPE::TTM::use_chain_auth && !exists($RIPE::TTM::chainmap{$param{'boxname'}})) ? "$param{'boxname'} is not in chain $ENV{'TTMCHAIN'}" : table( Tr(td( table( { -rules => "rows", -style => "border: 5px solid ; font-size: smaller;", -cellspacing=>"0",  -cellpadding => "5"},
			Tr(th({ -class => "datatable" }, "&nbsp;"),
				th({ -class => "datatable", -colspan=>4}, $classes[0]),
				th({ -class => "datatable" }, "&nbsp;"),
				th({ -class => "datatable", -colspan=>4}, $classes[1]),
			),"\n",
			Tr(th({ -class => "datatable", -style => "border-right: solid;", -align =>"left"}, "Box name, organisation &amp; location"),
				th({ -class => "datatable" }, "2.5 perc."),
				th({ -class => "datatable" }, "median"),
				th({ -class => "datatable" }, "97.5 perc."),
				th({ -class => "datatable", -style => "border-right: solid;" }, "% Lost"),
				th({ -class => "datatable" }, "&nbsp;"),
				th({ -class => "datatable" }, "2.5 perc."),
				th({ -class => "datatable" }, "median"),
				th({ -class => "datatable" }, "97.5 perc."),
				th({ -class => "datatable", -style => "border-right: solid;" }, "% Lost"),
			),"\n",
			# XXX Tr(td({-colspan=>10},hr)),
			( map {
				my $name = $_;

				Tr( { -align => "center"}, 
					td( { -align => "left", width => "25%", -style => "border-right: solid;"},
							a( {-name => "$name" }),
							a( {-href => "${url}&boxname=$name" }, "$name:" ),
							("$RIPE::TTM::gpsinfo{$name}{'org'}", br, "$RIPE::TTM::gpsinfo{$name}{'city'}, $RIPE::TTM::gpsinfo{$name}{'country'}")
						),
					(map {
						my $class = $_;
						my $seperator = ($class eq "incoming") ? td("&nbsp;") : "";
						(map {

						# determine CELL COLOR
						my $field = $_;
						my $cellcolor = cellcolor($name, $class, $field, $threshold);
						my $cellemphasis = cellemphasis($name, $class, $field, $threshold);
						my $value = $box{$name}{$class}{$field}{'short'};

						if ($value == -1) {	
							$value = "-";	# -1 == no data
						} 
					
						td( { $cellcolor ne "0" ? (-bgcolor => $cellcolor) : () , $field eq "Loss" ? (-style => "border-right: solid;") : () },
							table(Tr(td( { -align => "center", -style=> $cellemphasis },
							a( {-target => "_blank", -href => ($class eq "incoming") ? createplotlink(${name},${boxname}) : createplotlink(${boxname},${name}) } ,
#										"${basehref}?&src=${name};dst=${boxname}#$param{'period'}" : 
#										"${basehref}?&src=${boxname};dst=${name}#$param{'period'}" }, 
								$value))), Tr(td( { -align => "center", -style=> $cellemphasis },
							($box{$name}{$class}{$field}{'long'} == -1 ? "&nbsp;" : "($box{$name}{$class}{$field}{'long'})")
							)))
						)
					} @fieldlist),
					$seperator		# seperator columns
				} @classes)), "\n",
				# XXX Tr(td({-colspan=>10},hr))
			} @boxlist )))),"\n",),
	print_footer);
}


#---------------------------------------------------------------------
# Subroutine header
#
# Purpose           : Determine a cells color, depending on the threshold
#		      and the difference between short and long term
#		      cell value. Red signifies increase in delay,
#		      green indicates decrease in delay.
# Input             : - name, class & field of current cell to retrieve
#		        value from global %box hash
#		      - threshold at which to color: 
#				0 = don't color at all
#			       >0 = color according to relative change
#			       <0 = color according to absolute change
# Output            : a HTML color attribute  (#rrggbb) or 0 (don't color)
#---------------------------------------------------------------------


sub cellcolor($$$$) {
	my ($name, $class, $field, $threshold) = @_;

	my ($diff, $severity);

        if (($threshold == 0) ||
	    ($box{$name}{$class}{$field}{'short'} == -1) ||
	    ($box{$name}{$class}{$field}{'long'} <= 0)) {
		# no coloring
		return( $box{$name}{$class}{$field}{'short'} == (-1) ? $lightgray : (0));
	}

	if ($threshold > 0) {
		# relative change between short and long

		$diff = ($box{$name}{$class}{$field}{'short'}
				- $box{$name}{$class}{$field}{'long'});
		$diff = $diff / abs($box{$name}{$class}{$field}{'long'});
		
		# XXX 12 is size of @red/@green
		$severity = abs(int ($diff * 12));

	}
	else {
		# $threshold < 0  , absolute change
		$diff = $box{$name}{$class}{$field}{'short'}
				- $box{$name}{$class}{$field}{'long'};
		$threshold = abs($threshold);

		# XXX 12 is size of @red/@green
		$severity = abs(int (($diff / abs($box{$name}{$class}{$field}{'long'})) * 12));
	}

	# XXX use this to construct "hotlist of the day"
	# warn "$param{'boxname'} -> $name: $severity" if $severity > 0;

	if ($diff > $threshold) {
		# new > old
		return $severity > $#red ? ("$red") : ($red[$severity]);
	}
	elsif ($diff < -$threshold) {
		return $severity > $#green ? ("$green") : ($green[$severity]);
	}
	else {
		# within 'normal' range
		# no coloring
		return(0);
	}
}
sub cellemphasis($$$$) {
	my ($name, $class, $field, $threshold) = @_;
	my ($style) = "font-weight: inherit";
	my ($diff);

	if ($field eq "perc97.5")
	{
		if ($threshold > 0) {
			# relative change between short and long

			# XXX We really want to see congestion here, so I'm multiplying the
			# threshold factor by two to exclude "noise" every measurement
			# relation has. 

			$threshold *= 2;

			$diff = ($box{$name}{$class}{'perc97.5'}{'short'}
					- $box{$name}{$class}{'Median'}{'short'});
			$diff = $diff / abs($box{$name}{$class}{'Median'}{'short'});
		}
		else {
			# $threshold < 0  , absolute change
			$diff = $box{$name}{$class}{'perc97.5'}{'short'}
					- $box{$name}{$class}{'Median'}{'short'};
		}
		if ($diff > $threshold) {
			$style = "font-weight: bold";
		}
	}
	if ($box{$name}{$class}{$field}{'short'} < 0  && $box{$name}{$class}{$field}{'short'} != -1
		|| $box{$name}{$class}{$field}{'long'} < 0 &&  $box{$name}{$class}{$field}{'long'} != -1 )
	{
			$style = "font-weight: bold; text-decoration: blink";
	}
	return $style;
}

#---------------------------------------------------------------------
# Subroutine header
#
# Purpose           : print the HTML query form 
# Comments          : 
# Input             : global %param hash
# Output            : HTML code
# 
#---------------------------------------------------------------------

sub outputForm($) {

	my $value;
	my @boxes_available;
	my @options = (
	"name", "marked cells", map {
		my $class = $_;
		map {
			"$class $_"
		} @fieldlist
	} @classes);

	my $URLoverview = new CGI("ipv=$param{'ipv'};period=$param{'period'};node=$param{'boxname'}");
	my $URLrouting = new CGI("SRC=$param{'boxname'}");
	my $URLpod = new CGI("SRC=$param{'boxname'}");

	
	if ($RIPE::TTM::use_chain_auth)
	{
		@boxes_available = (grep {/(@{[join('|',$param{'ipv'} == 6 ? @RIPE::TTM::testboxes6 : @RIPE::TTM::testboxes4 )]})/} keys %RIPE::TTM::chainmap);
	}
	else
	{
		@boxes_available = ($param{'ipv'} == 6 ? @RIPE::TTM::testboxes6 : @RIPE::TTM::testboxes4 );
	}

	@boxes_available = keys %{{ map { $_ => 1 } grep { /^(@{[ join('|',keys %RIPE::TTM::gpsinfo) ]})$/ } @boxes_available }};

	return hr,
	start_form(-method=> "GET",
				-action=> $progname ),
	table({ -border => 0},
		Tr(
		td("Testbox:"), 
		td( { -colspan => 3, -align =>"right" }, 
			popup_menu(-name=> "boxname",
						-values=> [(sort tboxsort @boxes_available ) ], 
						-labels => { map { $_ => "$_: $RIPE::TTM::gpsinfo{$_}{'place'}" } @boxes_available },
						-default => $cgi->param('boxname'),
						)
			)
		),
		Tr(
			td("Select period"),
			td(
				popup_menu(-name=> "period",
							-values=> [ qw(day week) ],
							-default => $param{'period'} ),
				)
		),
		Tr(
			td("Sort summary table on"),
			td(
				popup_menu(-name=> "sortfield",
							-values=> [ @options],
							-default => $param{'sortfield'} ),
				popup_menu(-name=> "sortkey",
							-values=> [ "value", "relative change", "absolute change" ],
							-default => $param{'sortkey'} ),
			),
			td(
				popup_menu(-name=> "sortorder",
							-values=> [ "ascending", "descending" ],
							-default => $param{'sortorder'} )
			)
		),"\n",
		Tr(
			td("Threshold for coloring differences"),
			td({ -align=>"left" },
				textfield(-name => "threshold",
							-default=> sprintf("%5.1f", $param{'threshold'})),
				popup_menu(-name=> "unit",
							-values=> [ "percent", "miliseconds" ],
							-default => $param{'unit'} ),
			),
			td("")
		),"\n",
		Tr(
			td("Output format:"),
			td(
			radio_group(-name => "format",
						-values => [ "html" , "xml", "ascii" ],
						-default => $param{'format'}),
			hidden( -name => "boxname", -value => $param{'boxname'}),
			hidden( -name => "file", -value => $param{'file'} ),
			hidden( -name => "period", -value => $param{'period'}),
			submit( -name => "SUBMIT", -value => "Submit"),

			)
		),"\n",
		Tr(
			td("Switch protocol:"),
			td(
				map  { $param{'ipv'} == $_ ? u("[IPv$_]") : "[" . a({ -href => "$self_url?" . cleanup_query_string("ipv", $_, $cgi) }, "IPv$_" ) . "]" } (4,6)
			)
		),"\n",

	), end_form, 
	(
		$#RIPE::TTM::hotlist > 0 ? 
				p(a({ -href => $cgi->url(-absolute=>1, -query=>1) . ";nohotlist"  }, "turn off"), " your hotlist temporarily or ", 
				a({ -href => "${RIPE::TTM::hotlist_url}?ipv=$param{'ipv'}" }, "edit"), " the one that is currently active") 
			: defined($cgi->param('nohotlist')) ? 
				p(a({ -href => $cgi->url(-relative=>1) . "?" . cleanup_query_string("nohotlist", undef, $cgi) }, "turn on"), " temporarily disabled hotlist...") 
				: p(a({ -href => "${RIPE::TTM::hotlist_url}?ipv=$param{'ipv'}" }, "create a hotlist"), " to discard plots you are not interested in..."),
	),
	table(
		Tr(
			td({-align=>"center"}, "[", a({ -href=>"${RIPE::TTM::overview_url}?" . $URLoverview->query_string()  }, "Go to the delay plot $param{'period'} overview for this box"), "]"),
			td({-align=>"center"}, "[", a({ -href=>"${RIPE::TTM::pod_url}?" . $URLpod->query_string()  }, "Go to plots on demand for this box"), "]"),
			td({-align=>"center"}, "[", a({ -href=>"${RIPE::TTM::routing_url}?" . $URLrouting->query_string() }, "Go to the routing vectors for this box"), "]")
			)
		),
	hr;
}

#---------------------------------------------------------------------
# Subroutine header
#
# Purpose           : create CSV Ascii output 
# Comments          : 
# Input             : globals %param hash and sorted @boxlist
# Output            : HTML page with CSV table in plain text 
# 
#---------------------------------------------------------------------

sub outputCSV($) {

	
	print "Content-type: text/plain\n\n";
	print "TTM summaries for $param{'source'}\n\n";

	foreach my $name (@boxlist) {

		print "$name";
		foreach my $class (@classes) {
			foreach my $field (@fieldlist) {

				my $value;
				$value = $box{$name}{$class}{$field}{'short'};
				print ",$value";

				$value = $box{$name}{$class}{$field}{'long'};
				print ",$value";
			}
		}
		print "\n";
	}
}

sub createplotlink($$)
{
	use Sys::Hostname;

	my ($src,$dst) = @_;
	my $basehref;

	SWITCH: for (hostname()) {
		/^tipu$/	&& do {

			$basehref = "http://www.ttm.ripe.net/plots/showplots.cgi?ipv=$param{'ipv'};src=${src};dst=${dst}#$param{'period'}";

				last SWITCH;
			};

		/^(mango|peach)\.ripe\.net$/	&& do {

			$basehref = "http://www.ripe.net/ttm/Plots/plots.cgi?ipv=$param{'ipv'}&url=map_index.cgi&base=$param{'boxname'}&src=${src}&dst=${dst}";

				last SWITCH;
			};
		die "running on \"$_\" is not supported";
	}
	return $basehref;

}

sub createxmlname()
{
	use Sys::Hostname;

	my $xmlloc;
	my $prefix="";
	my $period="";
	if ($param{'ipv'} == 6) {
		$prefix = "IPv6.";
	}

	$period = "week" if ($param{'period'} eq "week");

	SWITCH: for (hostname()) {
		/^tipu$/	&& do {

			$xmlloc = "$param{'period'}/$param{'boxname'}-$prefix$param{'file'}";

				last SWITCH;
			};

		/^(mango|peach)\.ripe\.net$/	&& do {

			$xmlloc = "$param{'boxname'}/${prefix}${period}$param{'file'}";

				last SWITCH;
			};
		die "running on \"$_\" is not supported";
	}
	return $xmlloc;
}
