#!/usr/local/bin/perl
#
# $Id: gpsinfo.pl,v 1.5 2004/10/08 13:15:56 ttraffic Exp $
#
# Copyright (c) 2000  RIPE NCC
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of the author not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
#------------------------------------------------------------------------------
# Module Header
# Filename          :   gpsinfo.pl
# Purpose           :   Automated generation of TTM webpages in html
# Authors           :   Fotis Georgatos <fotis@ripe.net>, RIPE NCC
# Date              :   23-AUG-2000
# Description       :   Component of TTwwwpages for Test Traffic project
# Input             :   derived from "ttconfig" script  
# Output            :   html code in stdout
# Language Version  :   perl
#------------------------------------------------------------------------------

$TTROOT = "/ncc/ttpro";

my($GPSINFO) = "$TTROOT/bin/ttconfig -v GPSINFO |";
my($READTTREG) = '/ncc/ttpro/bin/readttreg.pl'; 

my(%ttInfo)  =  &readGPSINFOLines(); # build hash structure

sub tboxsort();

$coords="";

HTML_header();
HTML_table_start();
foreach $f (sort tboxsort keys %ttInfo )
  {
  $latDeg=$ttInfo{$f}{'lat'};
  $longDeg=$ttInfo{$f}{'long'};
  $coords="$coords$latDeg,$longDeg;";
  HTML_table_row();
  }
HTML_table_finish();
HTML_footer();

##########################################################################

sub HTML_header {
print <<EOF;
<!--#set var="title" value="Test Traffic Satellites & GPS information"-->
<!--#include virtual="/ssi/ttmhead1.html"-->

<P ALIGN="CENTER">
  <H1>Test Traffic Project: GPS Satellites</H1>
</P>
<P>
  <B>For the geographical location of a testbox, click on the
coordinates or consult <A HREF="locations.cgi">our maps</A></B>.
</P>
EOF
}

sub HTML_table_start {
print <<EOF;
<TABLE BORDER=1 NOSAVE>
  <TR>
    <TD><B>TTbox</B></TD>
    <TD>GPS stats</TD>
    <TD>Geographical coordinates</TD>
    <TD>Altitude</TD>
    <TD>Hosting site & Location</TD>
  </TR>
EOF
}

sub HTML_table_row {
  print "  <TR>\n";
  print "    <TD><CENTER><A HREF=\"$f\">$f</A></CENTER></TD>\n";
  print "    <TD><CENTER><A TARGET=\"Plots\" HREF=\"$f/satellites.$f.html\">";
  print "      <IMG SRC=\"$f/mini/satellites.$f.gif\" HEIGHT=50 WIDTH=62 BORDER=1></A></CENTER></TD>\n";
  print "    <TD><CENTER><A Target=\"mapquest\" HREF=\"http://www.mapquest.com/maps/map.adp?latlongtype=decimal&latitude=$latDeg&longitude=$longDeg\">";
  print      "$latDeg : $longDeg</A></CENTER></TD>\n";
  print "    <TD><DIV ALIGN=\"RIGHT\">$ttInfo{$f}{'alt'}</DIV></TD>\n";
  print "    <TD><B>$ttInfo{$f}{'place'}</B></TD>";
  print "  </TR>\n";
}

sub HTML_table_finish {
    print <<EOF;
</TABLE>
EOF
}

sub HTML_footer {
print <<EOF;
<!--#include virtual="/ssi/ttmfoot1.html"-->
EOF
}

sub readGPSINFOLines { 
	my(%ttInfo) = @_; 
        open( GPS, $GPSINFO)  or die "can't get GPSINFO";

	while (<GPS>) {
		chomp;
		my($geoloc,$altitude);
		my(@list) = split ('\t', $_);
		$ttInfo{$list[0]}{'name'} = $list[0]; 
		$ttInfo{$list[0]}{'lat'} = $list[1]; 
		$ttInfo{$list[0]}{'long'} = $list[2]; 
		$ttInfo{$list[0]}{'place'} = $list[3]; 

		$geoloc = qx($READTTREG $list[0] geoloc);
		($altitude) = ((split(/,/, $geoloc))[2]) =~ /^\s*(-?\d+\.?\d*)/;
		$ttInfo{$list[0]}{'alt'} = defined($altitude) ? sprintf("%.0f&nbsp;m",$altitude) : "???&nbsp;m";
	}
	return (%ttInfo);
}

sub tboxsort()
{
	# Other sort options possible here which global parameter
	# Think like hopwise of geographicaly wise sorting

	($a =~ /tt(\d+)/)[0] <=> ($b =~ /tt(\d+)/)[0]
}
