#!/usr/local/bin/perl
#
# $Id: ttxx.pl,v 1.12 2004/09/29 17:42:17 wilhelm Exp $
#
# Copyright (c) 2000  RIPE NCC
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of the author not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
#------------------------------------------------------------------------------
# Module Header
# Filename          :   ttxx.pl
# Purpose           :   Automated generation of TTM webpages in html
# Authors           :   Ruben van Staveren <ruben@ripe.net>, Fotis Georgatos <fotis@ripe.net>, RIPE NCC
# Date              :   18-AUG-2000
# Description       :   Component of TTwwwpages for Test Traffic project
# Input             :   derived from "ttconfig" script  
# Output            :   $OUTPUTDIR/*/index.html
# Language Version  :   perl
#
# Modifications     :   29-May-2001, Henk
#                       HREF's to ttxx.ttyy.html, instead of ttxx.ttyy.gif
#			23-Nov-2001, Rene, add support for ipdv plots
#		    :   13-Aug-2003, Michael, add support for IPv6 and ipdv IPv6 plots
#		    :   04-Dec-2003, RVS Add support for 2NIC relations
#
# Usage:		ttxx.pl [-ipdv] [-6]
#
#------------------------------------------------------------------------------

$TTROOT="/ncc/ttpro";
$OUTPUTDIR="$TTROOT/data/plots";
#$OUTPUTDIR="/home/michael/plots";

sub tboxsort();

my $progname = $0;

$type="";
$delvar="";
$protoc="IPv4";

#IP Version of the cgi script: 4 || 6
$ipv="4";

#Delay || IPDV type of the cgi script: delay || ipdv
$var="delay";


while ($#ARGV>=0) {
        $arg = shift(@ARGV);

	if ($arg eq "-ipdv") {
		$delvar = "ipdv";		
	}

	elsif ($arg eq "-6") {
		$protoc = "IPv6";		
        }

        else {
                printf STDERR "Usage: $progname [-ipdv] [-6]\n";
                exit(1);
        }
}

if (($protoc eq "IPv4") && ($delvar eq "ipdv")) {
        $title = "Delay variation";	
	$type = "ipdv.";
	$ipv = "4";
	$var="ipdv";
}

elsif (($protoc eq "IPv6") && ($delvar ne "ipdv")) {
	$title = "Delay IPv6";
	$type = "IPv6.";
	$ipv = "6";
	$var="delay";
}

elsif (($protoc eq "IPv6") && ($delvar eq "ipdv")) {
	$title = "Delay variation IPv6";	
	$type = "IPv6.ipdv.";
	$ipv = "6";
	$var="ipdv";
}

else {
        $title = "Delay";
}

$commTTLIST = "$TTROOT/bin/ttconfig -v LIST_OF_TESTBOXES ACTIVE |";
$commTTWATCH = "$TTROOT/bin/ttconfig -v LIST_OF_TESTBOXES WATCH |";


open( INFO, $commTTLIST) || die "Cannot get list of testboxes\n";
(@TTLIST) = grep {/^tt/ } map {chomp;my $node = (split(/\s/))[1]; $node =~ s/\Q.ripe.net\E//; $node} grep { $protoc eq "IPv6" ? /:/ : !/:/ } (<INFO>);
close(INFO); 

open( INFO, $commTTWATCH) || die "Cannot get list of testboxes\n";
(@TTWATCH) = grep {/^tt/ } map {chomp;my $node = (split(/\s/))[1]; $node =~ s/\Q.ripe.net\E//; $node} grep { $protoc eq "IPv6" ? /:/ : !/:/ } (<INFO>);
close(INFO); 


%TTLIST = map { $_ => 1 } @TTLIST;
@TTLIST = sort tboxsort keys %TTLIST;

%TTWATCH = map { $_ => 1 } @TTWATCH;
@TTWATCH = keys %TTWATCH;

foreach $ff (@TTLIST)
{
  $f=$ff;

  open( OUT, ">$OUTPUTDIR/$f/${type}index.html" ) || die "Cannot open $OUTPUTDIR/$f/${type}index.html for write\n";
  print (OUT "<!--#set var=\"title\" value=\"$f: TT weekly $title plots\"-->");
  print (OUT "<!--#include virtual=\"/ssi/ttmhead1.html\"-->");
  print (OUT "<P>\n<BR CLEAR=LEFT\n<BR>\n");
  print (OUT "<CENTER> <H1>$title plots from/to $f</H1> </CENTER>\n\n");
  print (OUT "<CENTER> For an explanation of what is in these plots, click\n");
  print (OUT "<A HREF=\"http://www.ripe.net/test-traffic/General/tt_matrix.html#${type}overview\">here</CENTER>\n\n");
  print (OUT "<CENTER><TABLE BORDER=1 NOSAVE>");
  print (OUT "<TR><TH>TTbox</TH><TH>INcoming traffic</TH><TH>OUTgoing traffic</TH></TR>");

  foreach $gg (@TTLIST)
  {
    $g=$gg;

    if ( $g ne $f )
    {
      print(OUT "<TR>");
      print(OUT "<TD>$g</TD>");
      if ( grep { /$g/ } @TTWATCH ) {
	print(OUT "<TD><SMALL>Plots are temporarily unavailable</SMALL></TD>");
	print(OUT "<TD><SMALL>Plots are temporarily unavailable</SMALL></TD>");
      }
      else {
        print (OUT "<TD><A TARGET=Plots HREF=\"../plots.cgi?src=$g&dst=$f&ipv=$ipv&type=$var&url=map_index.cgi\">");
	print(OUT "<IMG SRC=\"../week/mini/$type$g.$f.gif\" HEIGHT=170 WIDTH=213 BORDER=0></A></TD>");
        print (OUT "<TD><A TARGET=Plots HREF=\"../plots.cgi?src=$f&dst=$g&ipv=$ipv&type=$var&url=map_index.cgi\">");
	print(OUT "<IMG SRC=\"../week/mini/$type$f.$g.gif\" HEIGHT=170 WIDTH=213 BORDER=0></A></TD>");
      }
      print(OUT "</TR>\n");
    }
  }

  print (OUT "</TABLE></CENTER>");
  print (OUT "<!--#include virtual=\"/ssi/ttmfoot1.html\"-->");

  close (OUT);
}

sub tboxsort()
{
	# Other sort options possible here which global parameter
	# Think like hopwise of geographicaly wise sorting

	($a =~ /tt(\d+)/)[0] <=> ($b =~ /tt(\d+)/)[0]
						||
	($a =~ /tt\d+([a-z]?)/)[0] cmp ($b =~ /tt\d+([a-z]?)/)[0]
}
