#!/usr/local/bin/perl5
#
# $Id: ttxx.ttyy.pl,v 1.6 2001/12/11 06:11:37 ttraffic Exp $
#
# Copyright (c) 2000  RIPE NCC
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of the author not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
#
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
# AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
#------------------------------------------------------------------------------
# Module Header
# Filename          :   ttxx.ttyy.pl
# Purpose           :   Automated generation of TTM webpages in html
# Authors           :   Fotis Georgatos <fotis@ripe.net>, RIPE NCC
# Date              :   18-AUG-2000
# Description       :   Component of TTwwwpages for Test Traffic project
# Input             :   derived from "ttconfig" script  
# Output            :   $OUTPUTDIR/*/ttxx.ttyy.html
#                       $OUTPUTDIR/*/ttyy.ttxx.html
# Language Version  :   perl
#
# Modifications     :   29-May-2001, Henk, Added second output file.
#			23-Nov-2001, Rene, add support for ipdv plots
#------------------------------------------------------------------------------

$TTROOT="/ncc/ttpro";
$OUTPUTDIR="$TTROOT/data/plots";

my $progname = $0;

$type="";

while ($#ARGV>=0) {
        $arg = shift(@ARGV);
	if ($arg eq "-ipdv") {
		$type = "ipdv.";
	}
	else {
		printf STDERR "Usage: $progname [-ipdv]\n";
	        exit(1);
	}
}

if ($type eq "ipdv.") {
	$title = "Delay variation"
}
else {
	$title = "Delay"
}

open( INFO, "$TTROOT/bin/ttconfig -v TTLIST |") || die "Cannot get list of testboxes\n";
@TTLIST = <INFO>;
close(INFO); 

foreach $ff (@TTLIST)
{
  $f=$ff;
  chop $f;

  foreach $gg (@TTLIST)
  {
    $g=$gg;
    chop $g;

    if ( $g ne $f )
    {
      #
      # Create $OUTPUTDIR/ttxx/ttxx.ttyy.html"
      #
      open( OUT, ">$OUTPUTDIR/$f/$type$f.$g.html" ) || die "Cannot open file for write\n";
      print (OUT "<!--#set var=\"title\" value=\"RIPE NCC: Test Traffic Results\"-->\n");
      print (OUT "<!--#include virtual=\"/ssi/ttmhead1.html\"-->\n");
      print (OUT "<P> <BR CLEAR=LEFT> <BR>");

      print (OUT "<CENTER> <H1>$title plots for $f to $g</H1> </CENTER>\n\n");
      print (OUT "<CENTER><H2><A NAME=\"day\">Last 24 hours</A></H2> <IMG   SRC=\"day.$type$f.$g.gif\"> </CENTER>\n");
      print (OUT "<CENTER><H2><A NAME=\"week\">Last  7 days</A> </H2> <IMG  SRC=\"week.$type$f.$g.gif\"> </CENTER>\n");

      if ("$type" eq "") {
	   print (OUT "<CENTER><H2><A NAME=\"month\">Last 30 days</A> </H2> <IMG SRC=\"month.$f.$g.gif\"> </CENTER>\n");
	   print (OUT "<CENTER><H2><A NAME=\"trends\">6 months trend</A></H2> <IMG SRC=\"trends.$f.$g.gif\"> </CENTER>\n");
      }
      print (OUT "<!--#include virtual=\"/ssi/ttmfoot1.html\"-->");
      close (OUT);

      #
      # Create $OUTPUTDIR/ttxx/ttyy.ttxx.html"
      #
      open( OUT, ">$OUTPUTDIR/$f/$type$g.$f.html" ) || die "Cannot open file for write\n";
      print (OUT "<!--#set var=\"title\" value=\"RIPE NCC: Test Traffic Results\"-->\n");
      print (OUT "<!--#include virtual=\"/ssi/ttmhead1.html\"-->\n");
      print (OUT "<P> <BR CLEAR=LEFT> <BR>");

      print (OUT "<CENTER> <H1>$title plots for $g to $f</H1> </CENTER>\n\n");
      print (OUT "<CENTER><H2><A NAME=\"day\">Last 24 hours</A></H2> <IMG SRC=\"day.$type$g.$f.gif\"> </CENTER>\n");
      print (OUT "<CENTER><H2><A NAME=\"week\">Last  7 days</A> </H2> <IMG SRC=\"week.$type$g.$f.gif\"> </CENTER>\n");
      if ("$type" eq "") {
	   print (OUT "<CENTER><H2><A NAME=\"month\">Last 30 days</A> </H2> <IMG SRC=\"month.$g.$f.gif\"> </CENTER>\n");
	   print (OUT "<CENTER><H2><A NAME=#trends>6 months trend</A></H2> <IMG SRC=\"trends.$g.$f.gif\"> </CENTER>\n");
      }
      print (OUT "<!--#include virtual=\"/ssi/ttmfoot1.html\"-->");
      close (OUT);
    }
  }
}
