/*
 * Copyright (c) 2002 Endace Measurement Systems Ltd, Hamilton, New Zealand.
 * All rights reserved.
 *
 * This source code is proprietary to Endace Measurement Systems and no part
 * of it may be redistributed, published or disclosed except as outlined in
 * the written contract supplied with this product.
 *
 * $Id: dag.h,v 1.1.1.1 2004/11/10 10:37:21 tom Exp $
 */

#define _DAG_H

#include <asm/types.h>
#include <linux/version.h>

#define DAG_DRIVER_VERSION "2.4.14 $Date: 2004/11/10 10:37:21 $"

#define DAG_HDR_REV "$Revision: 1.1.1.1 $"

#ifndef MIN
#define MIN(a,b)	((a) < (b) ? (a) : (b))
#endif

#define LinuxVersionCode(v, p, s) (((v)<<16)+((p)<<8)+(s))
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
#define ioremap(addr,n) vremap(addr,n)
#define iounmap(addr)   vfree(addr)
#endif
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,37)
#define test_and_set_bit(x,y) set_bit(x,y)
#endif
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,3)
#define copy_to_user memcpy_tofs
#define copy_from_user memcpy_fromfs
#define put_user_noverify put_user
#else
#define put_user_noverify __put_user
#endif
#ifdef __KERNEL__
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,2,19))
typedef struct wait_queue      *wait_queue_head_t;
#endif
#else
typedef void		       *wait_queue_head_t;
#endif

/* For 2.4.0-testn n>~4 */
#ifndef put_user_ret
#define put_user_ret(x,ptr,ret) ({ if (put_user(x,ptr)) return ret; })
#endif
#ifndef get_user_ret
#define get_user_ret(x,ptr,ret) ({ if (get_user(x,ptr)) return ret; })
#endif
#ifndef copy_to_user_ret
#define copy_to_user_ret(to,from,n,retval) ({ if (copy_to_user(to,from,n)) return retval; })
#endif
#ifndef copy_from_user_ret
#define copy_from_user_ret(to,from,n,retval) ({ if (copy_from_user(to,from,n)) return retval; })
#endif

/*
 * The 0x121b ID belongs to ATM Ltd, which is what the Dag series of cards
 * grew out of. All commercial systems today shall use the Endace ID.
 */
#define PCI_VENDOR_ID_DAG 0x121b
#define PCI_VENDOR_ID_ENDACE 0xeace
#define PCI_DEVICE_ID_DAG2_OLD 0xd210
#define PCI_DEVICE_ID_DAG2_10 0x2100
#define PCI_DEVICE_ID_DAG2_11 0x2110
#define PCI_DEVICE_ID_DAG2_11D 0x211D
#define PCI_DEVICE_ID_DAG2_11E 0x211E

#define PCI_DEVICE_ID_DAG3_00 0x3000
#define PCI_DEVICE_ID_DAG3_01 0x3010
#define PCI_DEVICE_ID_DAG3_10 0x3100
#define PCI_DEVICE_ID_DAG3_20 0x3200
#define PCI_DEVICE_ID_DAG3_30 0x3300
#define PCI_DEVICE_ID_DAG3_40 0x3400
#define PCI_DEVICE_ID_DAG3_50 0x3500
#define PCI_DEVICE_ID_DAG3_50E 0x350E
#define PCI_DEVICE_ID_DAG3_51ECM 0x351C
#define PCI_DEVICE_ID_DAG3_60 0x360D
#define PCI_DEVICE_ID_DAG3_60E 0x360E
#define PCI_DEVICE_ID_DAG3_68E 0x368E

#define PCI_DEVICE_ID_DAG3_80 0x3800

#define PCI_DEVICE_ID_DAG4_00 0x4000
#define PCI_DEVICE_ID_DAG4_10 0x4100
#define PCI_DEVICE_ID_DAG4_11 0x4110
#define PCI_DEVICE_ID_DAG4_20 0x4200
#define PCI_DEVICE_ID_DAG4_20E 0x420E
#define PCI_DEVICE_ID_DAG4_21 0x4210
#define PCI_DEVICE_ID_DAG4_22 0x4220
#define PCI_DEVICE_ID_DAG4_22E 0x422E
#define PCI_DEVICE_ID_DAG4_23 0x4230
#define PCI_DEVICE_ID_DAG4_23E 0x423E
#define PCI_DEVICE_ID_DAG4_30 0x4300
#define PCI_DEVICE_IS_DAG4_30E 0x430E

#define PCI_DEVICE_ID_DAG6_00 0x6000
#define PCI_DEVICE_ID_DAG6_10 0x6100

#define DAGN(DEVICE)    (((DEVICE)&0xf000)>>12)
#define DAG2(DEVICE)	(((DEVICE)&0xf000)==0x2000)
#define DAG3(DEVICE)    (((DEVICE) >= 0x3000) && ((DEVICE) < 0x3800))
#define DAG32(DEVICE)	(((DEVICE)&0xff00)==0x3200)
#define DAG34(DEVICE)	(((DEVICE)&0xff00)==0x3400)
#define DAG35(DEVICE)	(((DEVICE)&0xff00)==0x3500)
#define DAG35O(DEVICE)	(((DEVICE)&0xff0f)==0x3500)
#define DAG36(DEVICE)	(((DEVICE)&0xff00)==0x3600)
#define DAG38(DEVICE)   (((DEVICE)&0xff00)==0x3800)
#define DAG4(DEVICE)	(((DEVICE) >= 0x3800) && ((DEVICE) < 0x5000))
#define DAG41(DEVICE)	(((DEVICE)&0xff00)==0x4100)
#define DAG42(DEVICE)	(((DEVICE)&0xff00)==0x4200)
#define DAG43(DEVICE)	(((DEVICE)&0xff00)==0x4300)
#define DAG5(DEVICE)    (((DEVICE)&0xf000)==0x5000)
#define DAG6(DEVICE)    (((DEVICE)&0xf000)==0x6000)

#define ETHERDAG(DEVICE)  (((DEVICE)&0x000f)==0x000e)
#define ECM(DEVICE)  (((DEVICE)&0x000f)==0x000c)

/* These defines are for reading the software (xilinx image) capabiltities.
   In kernelspace, use a dag_device ptr, in userspace use a dag_inf ptr. */

#ifdef __KERNEL__
#define IsSYS(x) ((x)->info.soft_caps.system)
#define IsATM(x) ((x)->info.soft_caps.atm)
#define IsPOS(x) ((x)->info.soft_caps.pos)
#define IsETH(x) ((x)->info.soft_caps.eth)
#define IsDUCK(x) ((x)->info.soft_caps.duck)
#define IsFIFO(x) ((x)->info.soft_caps.fifo)
#define IsTX(x) ((x)->info.soft_caps.tx)
#define IsPBM(x) ((x)->info.soft_caps.pbm)
#define IsARM(x) ((x)->info.soft_caps.arm)
#define IsDUCKEMU(x) ((x)->info.soft_caps.duckemu)
#else
#define IsSYS(x) ((x)->soft_caps.system)
#define IsATM(x) ((x)->soft_caps.atm)
#define IsPOS(x) ((x)->soft_caps.pos)
#define IsETH(x) ((x)->soft_caps.eth)
#define IsDUCK(x) ((x)->soft_caps.duck)
#define IsFIFO(x) ((x)->soft_caps.fifo)
#define IsTX(x) ((x)->soft_caps.tx)
#define IsPBM(x) ((x)->soft_caps.pbm)
#define IsARM(x) ((x)->soft_caps.arm)
#define IsDUCKEMU(x) ((x)->soft_caps.duckemu)
#endif

#define RESET_REG 0x6c
#define RESET_REG_SET(x) (x | 0x40000000)

#define PCI_DB_OFFSET 0x64
#define PCI_ICS_OFFSET 0x68   /* interrupt status reg on DAG */

#define DAG_ADDR_SIZE 0x8000
#define DAG_IO_SIZE   0x10000 /*64k io window*/

#define DAG_MAX_BOARDS 10

#define MAILBOX_SET(x) ((x) | 0x80000000)
#define MAILBOX_CLEAR(x) ((x) & 0x7fffffff)
#define MAILBOX_TEST(x) ((x) & 0x80000000)

typedef struct dagioaddr
{
	unsigned int	offset, val;
} dagioaddr;

typedef struct monparams {
	unsigned int	words[16];
} monparams_t;

typedef struct dagioblk
{
	unsigned int	offset, d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11;
} dagioblk;

typedef enum reset_magic {
	DAGRESET_FULL	= 0,
	DAGRESET_REBOOT	= 1,
	DAGRESET_INIT	= 2,
	DAGRESET_DUCK	= 3,
	DAGRESET_GOODBYE= 4,
	DAGRESET_LEVEL2	= 5,
} reset_magic_t;

typedef struct array {
	char	x[256];
} array_t;

/* GPP record type defines */
#define TYPE_LEGACY       0
#define TYPE_HDLC_POS     1
#define TYPE_ETH          2
#define TYPE_ATM          3
#define TYPE_AAAL5        4

/* GPP control flags */
#define FORMAT_PADDED     0x00
#define FORMAT_VARIABLE   0x01

/* Bitfield flag values used in dagioaddr.val field for DAG_IOOFFSET ioctl */
#define OFFSET_UPDATE     0x01
#define OFFSET_BLOCK      0x02  

/* ioctl cmds */
#define DAG_IOC_MAGIC '3'

#define DAG_IOCRESET      _IOW(DAG_IOC_MAGIC,  0, reset_magic_t)
#define DAG_IOGPSTIME     _IOR(DAG_IOC_MAGIC,  1, unsigned int)
#define DAG_IOGPSTIMEWAIT _IOR(DAG_IOC_MAGIC,  2, unsigned int)
#define DAG_IOREAD        _IOR(DAG_IOC_MAGIC,  3, dagioaddr)
#define DAG_IOWRITE       _IOWR(DAG_IOC_MAGIC, 4, dagioaddr)
#define DAG_IOINF         _IOR(DAG_IOC_MAGIC,  5, dag_inf)
#define DAG_IOCXRELOAD    _IOW(DAG_IOC_MAGIC,  6, dagioaddr)
/* 7 was IODL, which is obsolete. Downloading is done from userspace */
#define DAG_IOBUFADDR     _IOWR(DAG_IOC_MAGIC,  8, unsigned int)
#define DAG_IOBUFADDRWAIT _IOWR(DAG_IOC_MAGIC,  9, unsigned int)
#define DAG_IOMREAD       _IOR(DAG_IOC_MAGIC,  10, dagioaddr)
#define DAG_IOMWRITE      _IOWR(DAG_IOC_MAGIC, 11, dagioaddr)
#define DAG_IOM12READ     _IOR(DAG_IOC_MAGIC,  12, dagioblk)
#define DAG_IOM12WRITE    _IOWR(DAG_IOC_MAGIC, 13, dagioblk)
#define DAG_IOREADY       _IO(DAG_IOC_MAGIC, 14)
#define DAG_IOCRUN        _IOW(DAG_IOC_MAGIC,  15, unsigned int)
#define DAG_IOGETSPEED    _IOR(DAG_IOC_MAGIC,  16, int)
#define DAG_IOSETSPEED    _IOW(DAG_IOC_MAGIC, 17, int)
#define DAG_IOMBURST      _IOWR(DAG_IOC_MAGIC, 18, dagioaddr)
#define DAG_IOGETDUCK     _IOR(DAG_IOC_MAGIC,  19, duck_inf)
#define DAG_IOSETDUCK     _IOW(DAG_IOC_MAGIC,  20, duck_inf)
#define DAG_IOCSTART      _IO(DAG_IOC_MAGIC,  21)
#define DAG_IOSETXLX      _IOW(DAG_IOC_MAGIC, 22, array_t)
#define DAG_IOOFFSET      _IOWR(DAG_IOC_MAGIC, 23, unsigned int)
#define DAG_IOCSTOP       _IO(DAG_IOC_MAGIC,  24)
#define DAG_IOCMONITOR	  _IOWR(DAG_IOC_MAGIC, 25, monparams_t)
#define	DAG_IOCLOCK	  _IOWR(DAG_IOC_MAGIC, 26, int)

#define DAG_IOC_MAX 24

/* Monitor Protocol */
#define Protocol	0x01000000
#define Protocol_Mask	0xff000000

/* Monitor Commands (Protocol 1) */
#define Cmd_Min                 0
#define Cmd_Version	        0
#define Cmd_Read_Word		1
#define Cmd_Write_Word		2
#define Cmd_Program_Xilinx	3
#define Cmd_Enter_Address       4
#define Cmd_Read_12_Words	5
#define Cmd_Write_12_Words	6
#define Cmd_Max			6

/* Monitor Return Codes */
#define Ret_OK			1
#define Ret_Good_Init		2

#define Ret_Bad_Init		3
#define Ret_Bad_Proto		4
#define Ret_Invalid_Cmd		5
#define	Ret_Unsupported_Cmd     6

#define Ret_Reset_Ex		7
#define Ret_UndefOp_Ex		8
#define Ret_Software_Ex		9
#define Ret_Abort_Prefetch_Ex	10
#define Ret_Abort_Data_Ex	11
#define Ret_IRQ_Ex		12
#define Ret_FIQ_Ex		13

/* ARM space memory locations */
#define Data_Block       0x3100
#define Data_Hole_Base	 0x00
#define Data_Hole_Top    0x04
#define Data_Device_Code 0x08
#define Data_Xilinx1     0x700
#define Data_Xilinx2     0x780
#define Data_Xilinx3     0x800
#define Data_Xilinx4     0x880

#define Dag3_PMC             0x20000000
#define Dag3_PMC_Config      0x04
#define Dag3_PMC_Config_OC3  0x15
#define Dag3_PMC_Config_OC12 0x1f
#define Dag3_PMC_Phy	     0x18
#define Dag3_PMC_Phy_OC12    0x1
#define Dag3_PMC_Phy_FLoop   0x4
#define Dag3_PMC_Phy_RClk    0x8

/* dag_inf.fifo values */
#define Dag3_Fifo_Unknown 0
#define Dag3_Fifo_1Mbit 1
#define Dag3_Fifo_4Mbit 2

/* Number codes for specifying a xilinx number when reloading */
/* Interestingly, we start counting with Xilinx 1, not 0 */
#define Load_Xilinx_1 0
#define Load_Xilinx_2 1
#define Load_Xilinx_3 2

/* Bitfields for DAG_IOSETDUCK, Set_Duck_Field parameter */
/* The order of definition is important for functionality */
/* be careful with changes and know what you are doing */
/* The current order is defined by the origial priority */
/* given in the drivers parse routine for IOSETDUCK */
#define Set_Duck_Config       0x0002
#define Set_Duck_Clock_Inc    0x0004
#define Set_Duck_Clock_Wrap   0x0008
#define Set_Duck_DDS_Rate     0x0010
#define Set_Duck_Load_High    0x0020
#define Set_Duck_Synth_Freq   0x0040
#define Set_Duck_Crystal_Freq 0x0080
#define Set_Duck_Sync_Rate    0x0100
#define Set_Duck_Off_Limit    0x0200
#define Set_Duck_Command      0x1000
#define Set_Duck_Reset        0x2000
#define Set_Duck_Wait_Health  0x4000
#define Set_Duck_Clear_Stat   0x8000

#define Duck_Cmd_Read_Running 0x001
#define Duck_Cmd_Read_Int     0x002
#define Duck_Cmd_Go           0x001
#define Duck_Cmd_Stop         0x002
#define Duck_Cmd_Reset        0x004
#define Duck_Cmd_Go_on_Sync   0x008
#define Duck_Cmd_Load_on_Sync 0x080
#define Duck_Cmd_Load_Immed   0x100
#define Duck_Cmd_Host_Latch   0x200
#define Duck_Cmd_Host_Pulse   0x400

#define Duck_Config_Muxin_RS422     0x01
#define Duck_Config_Muxin_Host      0x02
#define Duck_Config_Muxin_Ovrflow   0x04
#define Duck_Config_Muxin_Aux       0x08
#define Duck_Config_Muxout_RS422   0x100
#define Duck_Config_Muxout_Input   0x200
#define Duck_Config_Muxout_Host    0x400
#define Duck_Config_Muxout_Ovrflow 0x800

/* phy values for phy_fac_loop and phy_local_clk */
#define Phy_Unsupported 0
#define Phy_Off         1
#define Phy_On          2

/* Capabilities defines */
#define CAPS_INVALID		 0
#define CAPS_BASIC		 1
#define CAPS_ATM		 2
#define CAPS_POS		 3
#define CAPS_ATM_DUCK		 4
#define CAPS_POS_DUCK		 5
#define CAPS_ATM_TX		 6
#define CAPS_ETH		 7
#define CAPS_ETH_DUCK		 8
#define CAPS_ATM_DUCK_FIFO	 9
#define CAPS_POS_DUCK_FIFO	10
#define CAPS_ETH_DUCK_FIFO	11
#define CAPS_PCI_DAG4		12
#define CAPS_PCI_DAG34		13
#define CAPS_PCI_DAG35		14
#define CAPS_PCI_DAG42		15
#define CAPS_PCI_DAG35ECM	16
#define CAPS_PCI_DAG42E		17
#define CAPS_PCI_DAG35PBM	18
#define CAPS_PCI_DAG35PC	19
#define CAPS_PCI_DAG35E		20
#define CAPS_PCI_DAG60		21
#define CAPS_PCI_DAG38		22
#define CAPS_PCI_DAG43		23
#define CAPS_PCI_DAG43E		24
#define CAPS_PCI_DAG36D		25
#define CAPS_PCI_DAG36E		26
#define CAPS_PCI_DAG36G		27

#define CAPS_MAGIC      0xa6000000
#define CAPS_MAGIC_MASK 0xff000000

#define Max_Inf 4
#define Capabilities_Offset 0x98

enum dag_speed {
	SP_DEFAULT,
	ETH_10,
	DS3,
	E3,
	ETH_100,
	OC3c_ATM,
	OC3c_POS,
	OC12c_ATM,
	OC12c_POS,
	ETH_1000,
	OC48c_ATM,
	OC48c_POS
};

typedef struct scaps_bits
{
	unsigned system : 1;
	unsigned atm    : 1;
	unsigned pos    : 1;
	unsigned eth    : 1;
	unsigned duck   : 1;
	unsigned fifo   : 1;
	unsigned tx     : 1;
	unsigned pbm    : 1;
	unsigned arm    : 1;
	unsigned duckemu: 1;
        unsigned gpp    : 1;
        unsigned btx    : 1;
        unsigned code   : 1;
} scaps_bits;

typedef struct duck_inf
{
	unsigned long	Command, Config, Clock_Inc, Clock_Wrap, DDS_Rate;
	unsigned long	Crystal_Freq;
	unsigned long	Synth_Freq, Sync_Rate;
	unsigned long	long Last_Ticks;
	unsigned long	Resyncs;
	unsigned long	Bad_Diffs, Bad_Offs, Bad_Pulses;
	unsigned long	Worst_Error, Worst_Off;
	unsigned long	Off_Limit, Off_Damp;
	unsigned long	Pulses, Single_Pulses_Missing, Longest_Pulse_Missing;
	unsigned long	Health, Sickness;
	long		Error, Offset;
	long		Stat_Start, Stat_End;	/* these are really time_t's */
	unsigned long	Set_Duck_Field;
} duck_inf;

typedef struct dag_inf
{
	char		revision[80];
	unsigned char	id, fifo;
	unsigned short	device_code;
        unsigned char   pci_rev;
	unsigned int	buf_size;
	unsigned int	buf_phy_base;
	int		firmware;
	unsigned int	caps;
	int		duck_def_index;

	/* These are register offsets in PCI io space
	   Use these with DAG_IOREAD and DAG_IOWRITE */
	int		ToHM0,  ToHM1,  ToHM2,  ToHM3;
	int		ToHM4,  ToHM5,  ToHM6,  ToHM7;
	int		ToHM8,  ToHM9,  ToHM10, ToHM11;
	int		ToHM12, ToHM13, ToHM14, ToHM15;

	int		ToAM0,  ToAM1,  ToAM2,  ToAM3;
	int		ToAM4,  ToAM5,  ToAM6,  ToAM7;
	int		ToAM8,  ToAM9,  ToAM10, ToAM11;
	int		ToAM12, ToAM13, ToAM14, ToAM15;

	int		ToHDB;	/* ARM -> Host doorbell register */
	int		ToADB;	/* Host -> ARM doorbell register */
	int		Ctrl;     /* Control and Reset register */
	int		Int_Enables; /* Dag bitwise interrupt enables */
	int		Hole_Base;
	int		Hole_Length;

	int		Duck_Command;
	int		Duck_Config;
	int		Duck_High_Load;
	int		Duck_DDS_Rate;
	int		Duck_High_Read;
	int		Duck_Low_Read;
	int		Duck_Clock_Inc;
	int		Duck_Clock_Wrap;
	int		Duck_Wbit;	/* magic address bit for emulation */

	/*ARM Space Memory Map base address definitions */
	unsigned int	ram_base;
	unsigned int	pci_base; /* PLX on dag2.x, same as xilinx base on dag3.x */
	unsigned int	xilinx1_base;
	unsigned int	xilinx2_base;
	unsigned int	xilinx3_base;
	unsigned int	phy_base;
	unsigned int	eeprom_base;
        unsigned int    pbm_base;
        unsigned int    gpp_base;
        unsigned int    btx_base;

	scaps_bits	soft_caps;
} dag_inf;

/*
 * PCI Burst Manager, such as implemented in the DAG4.1
 * See the draft at
 * http://dag.cs.waikato.ac.nz/~djmiller/burstmanager.html
 */
typedef struct dagpbm {
	volatile unsigned	memaddr;	/* 0x00 */
	volatile unsigned	memsize;	/* 0x04 */
	volatile unsigned	burstthresh;	/* 0x08 */
	volatile unsigned	segsize;	/* 0x0c */
	volatile unsigned	wrsafe;		/* 0x10 */
	volatile unsigned	redzone;	/* 0x14 */
	volatile unsigned	curaddr;	/* 0x18 */
	volatile unsigned	cs;		/* 0x1c */
	volatile unsigned	capacity;	/* 0x20 */
	volatile unsigned	underover;	/* 0x24 */
	volatile unsigned	pciwptr;	/* 0x28 */
	volatile unsigned	bursttmo;	/* 0x2c */
	volatile unsigned	build;		/* 0x30 */
	volatile unsigned	maxlevel;	/* 0x34 */
	volatile unsigned	segaddr;	/* 0x38 */
} dagpbm_t;

typedef enum pbmcs {
	DAGPBM_PAUSED	= 	0x01,
	DAGPBM_AUTOWRAP	=	0x02,
	DAGPBM_FLUSH	=	0x04,
	DAGPBM_BYTESWAP	=	0x08,
	DAGPBM_SAFETY	=	0x10,
	DAGPBM_WIDEBUS	=	0x20,
	DAGPBM_SYNCL2R	=	0x40,
	DAGPBM_REQPEND	=	0x80,
	DAGPBM_DEAD	=	0x100,
} pbmctrl_t;

typedef struct daggpp {
        volatile unsigned        control;         /* 0x00 */
        volatile unsigned        reserved;        /* 0x04 */
        volatile unsigned        padword;         /* 0x08 */
        volatile unsigned        snaplen;         /* 0x0c */
} daggpp_t;

typedef struct dagbtx {
        volatile unsigned        header;          /* 0x00 */
        volatile unsigned        minlen;          /* 0x04 */
        volatile unsigned        maxlen;          /* 0x08 */
        volatile unsigned        inclen;          /* 0x0c */
        volatile unsigned        load;            /* 0x10 */
        volatile unsigned        delay;           /* 0x14 */
} dagbtx_t;

enum drvflags {
	drvf_chrdev_registered	= 0x0001,	/* reg_chrdev done	*/
	drvf_present		= 0x0002,	/* PCI card is present	*/
	drvf_mmio_mapped	= 0x0004,	/* mmio mapped		*/
	drvf_sdram_mapped	= 0x0008,	/* sdram mapped		*/
	drvf_dagmem_allocated	= 0x0010,	/* dagmem_alloc	success	*/
	drvf_buffer_mapped	= 0x0020,	/* memory hole mapped	*/
	drvf_ducktimer		= 0x0040,	/* ducktimer allocated	*/
	drvf_irq_requested	= 0x0080,	/* request_irq done	*/
	drvf_intr_enabled	= 0x0100,	/* interrupt enabled	*/
	drvf_selftest_passed	= 0x0200,	/* selftest passed	*/
	drvf_monproto_ok	= 0x0400,	/* protocol recognized	*/
	drvf_proc_registered	= 0x8000,	/* should be last	*/
};
typedef enum drvflags drvflags_t;

typedef struct dag_device
{
	drvflags_t	drvflags;
	unsigned char	irq, id, fifo;
	unsigned short	device_code;
	unsigned char	*mmio;
	unsigned char	*buf_addr;
	unsigned char	*sdram;
	unsigned int	buf_size, phy_addr, pci_io, sdramlen;
	unsigned long	gps_time;

	wait_queue_head_t dag_gps_wait;
	wait_queue_head_t dag_buf_wait;

	int		busy, gps_unread, buf_unread, buf_tx_freed;
	unsigned long	buf_mark;
	unsigned long	buf_offset;
	unsigned int	buf_offset_active;
	struct pci_dev	*pcidev;

	unsigned int	P_BA0, P_BA1, P_BA2, P_COM;
	unsigned char	P_LIN, P_LAT, P_REV;

	wait_queue_head_t dag_cmd_wait;
	wait_queue_head_t dag_except_wait;
	wait_queue_head_t dag_duck_wait;

	struct timer_list *ducktimer;
	struct timer_list *offsettimer;
	int		firmware, i_total, i_gps;
	int		i_rxbuf, i_txbuf, i_duck;
	int		i_wrap, i_safety, i_phy;
	int		i_invalid, i_unknown;
	unsigned long	cmd_sequence;
	enum dag_speed	speed;
	char		xilinx1[128];
	char		xilinx2[128];
	unsigned int	caps, int_enables, int_mask;
	dag_inf		info;
	duck_inf	duck_info;
	dagpbm_t	*pbm;
        daggpp_t        *gpp;
        dagbtx_t        *btx;
	unsigned	byteswap;
#ifdef CORAL
#define CORAL_MAX_BLKS 64
	int		num_blks;
	coral_blk_info_t cbi[CORAL_MAX_BLKS];
	coral_io_mode_t	iomode;
	int		listen, cdone, clost;
	int		dblk;
	int		ublk;
#endif
} dag_device;

/*
 * Available from both user and kernel space
 */
char	*dag_device_name(unsigned device, int flag);
char	*dag_caps_name(unsigned device, int flag);
char	*dag_soft_caps_name(scaps_bits soft_caps, int flag);
char	*dag_phyconf_name(unsigned device, int flag);
char	**dag_xrev_parse(unsigned char *xrev, unsigned xlen, char *argv[5]);
char	* dag_xrev_name(unsigned char *xrev, unsigned xlen);

#ifdef __KERNEL__
struct pci_dev	*dag_find_device(struct pci_dev *dev);
#endif
