/*
 * XXX This code needs update to support ARM COFF objects
 */
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <sys/ioctl.h>

#include <fcntl.h>
#include <string.h>

#include <dag.h>
#include <daglib.h>

int
dag_open(char *dagname, dag_inf *infp, int checkready)
{
	int             dagfd;
	int             retval;

	if((dagfd = open(dagname, O_RDWR)) < 0)
		return dagfd;

	if(infp != NULL) {
		if((retval = ioctl(dagfd, DAG_IOINF, infp)) < 0)
			return retval;

		if(strncmp(DAG_HDR_REV, infp->revision, sizeof(infp->revision))) {
			fprintf(stderr,
				"dag_open: using old tool with library %s "
				"and new driver %s or vice versa\n",
				DAG_HDR_REV, infp->revision);
			errno = ERANGE;
			return -1;
		}
	}

	if(checkready) {
		if((retval = ioctl(dagfd, DAG_IOREADY, NULL)) < 0)
			return retval;
	}

	return dagfd;
}

int
dag_dl_file(int dag_handle, int file_handle, int address, dag_inf * inf)
{
	int             retval, size = 0;
	dagioaddr       myio;
	dagioblk        myblk;

	if(dag_handle < 0)
		return -ENODEV;

	if(file_handle < 0)
		return -ENOENT;

	if(!DAG2(inf->device_code)) {
		myblk.offset = address;
		while(read(file_handle, &myblk.d0, sizeof(int) * 12)) {
			retval = ioctl(dag_handle, DAG_IOM12WRITE, &myblk);
			if(retval < 0) {
				printf("Download failed! Address 0x%08x\n", myblk.offset);
				return retval;
			}
			myblk.offset += (sizeof(int) * 12);
			size += (sizeof(int) * 12);
		}
	} else {
		myio.offset = address;
		while(read(file_handle, &myio.val, sizeof(int))) {
			retval = ioctl(dag_handle, DAG_IOMWRITE, &myio);
			if(retval < 0) {
				printf("Download failed! Address 0x%08x\n", myio.offset);
				return retval;
			}
			myio.offset += sizeof(int);
			size += sizeof(int);
		}
	}

	return size;
}

int
dag_register_dump(int dag_handle, dag_inf * inf)
{
	int             retval;
	dagioaddr       myio;

	if(dag_handle < 0)
		return -ENODEV;

	for(myio.offset = 0x3000; myio.offset < 0x3040; myio.offset += 4) {
		retval = ioctl(dag_handle, DAG_IOMREAD, &myio);
		if(retval < 0)
			return retval;
		printf("r%02d %08x  ", (myio.offset - 0x3000) / 4, myio.val);
		if(!(((myio.offset - 0x3000) / 4 + 1) % 4))
			printf("\n");
	}

	return 0;
}

/*
 * Negative returns are errors, 0 is off, 1 is on 
 */
int
dag_get_phy_facility_loopback(int dag_handle, dag_inf * inf)
{
	dagioaddr       myio;
	int             retval;

	if((inf->device_code == PCI_DEVICE_ID_DAG3_20)) {

		myio.offset = inf->phy_base + Dag3_PMC_Phy;
		if((retval = ioctl(dag_handle, DAG_IOMREAD, &myio)) < 0) {
			printf("DAG_IOMREAD ioctl failed!\n");
			return retval;
		}

		if(myio.val & Dag3_PMC_Phy_FLoop)
			return 1;
		else
			return 0;
	} else {
		return -ENOSYS;
	}
}

/*
 * Negative returns are errors, 0 is off, 1 is on 
 */
int
dag_get_phy_recovered_clock(int dag_handle, dag_inf * inf)
{
	dagioaddr       myio;
	int             retval;

	if((inf->device_code == PCI_DEVICE_ID_DAG3_20)) {

		myio.offset = inf->phy_base + Dag3_PMC_Phy;
		if((retval = ioctl(dag_handle, DAG_IOMREAD, &myio)) < 0) {
			printf("DAG_IOMREAD ioctl failed!\n");
			return retval;
		}

		if(myio.val & Dag3_PMC_Phy_RClk)
			return 1;
		else
			return 0;
	} else {
		return -ENOSYS;
	}
}

/*
 * 0 is off, 1 is on 
 */
int
dag_set_phy_facility_loopback(int dag_handle, dag_inf * inf, int set)
{
	dagioaddr       myio;
	int             retval;

	if((inf->device_code == PCI_DEVICE_ID_DAG3_20)) {

		myio.offset = inf->phy_base + Dag3_PMC_Phy;
		if((retval = ioctl(dag_handle, DAG_IOMREAD, &myio)) < 0) {
			printf("DAG_IOMREAD ioctl failed!\n");
			return retval;
		}

		if(set)
			myio.val |= Dag3_PMC_Phy_FLoop;
		else
			myio.val &= ~Dag3_PMC_Phy_FLoop;

		if((retval = ioctl(dag_handle, DAG_IOMWRITE, &myio)) < 0) {
			printf("DAG_IOMWRITE ioctl failed!\n");
			return retval;
		}

		return 0;
	} else {
		return -ENOSYS;
	}
}

/*
 * 0 is off, 1 is on 
 */
int
dag_set_phy_recovered_clock(int dag_handle, dag_inf * inf, int set)
{
	dagioaddr       myio;
	int             retval;

	if((inf->device_code == PCI_DEVICE_ID_DAG3_20)) {

		myio.offset = inf->phy_base + Dag3_PMC_Phy;
		if((retval = ioctl(dag_handle, DAG_IOMREAD, &myio)) < 0) {
			printf("DAG_IOMREAD ioctl failed!\n");
			return retval;
		}

		if(set)
			myio.val |= Dag3_PMC_Phy_RClk;
		else
			myio.val &= ~Dag3_PMC_Phy_RClk;

		if((retval = ioctl(dag_handle, DAG_IOMWRITE, &myio)) < 0) {
			printf("DAG_IOMWRITE ioctl failed!\n");
			return retval;
		}

		return 0;
	} else {
		return -ENOSYS;
	}
}

/*
 * 0 is off, 1 is on 
 */
int
dag_set_phy_pos_scramble(int dag_handle, dag_inf * inf, int set)
{
	dagioaddr       myio;
	int             retval;

	if((inf->device_code == PCI_DEVICE_ID_DAG3_20) &&
	    ((inf->caps == CAPS_POS) || (inf->caps == CAPS_POS_DUCK))) {

		myio.offset = 0xc0;
		if((retval = ioctl(dag_handle, DAG_IOREAD, &myio)) < 0) {
			printf("DAG_IOREAD ioctl failed!\n");
			return retval;
		}

		if(set)
			myio.val |= 0x01;
		else
			myio.val &= ~0x01;

		if((retval = ioctl(dag_handle, DAG_IOWRITE, &myio)) < 0) {
			printf("DAG_IOWRITE ioctl failed!\n");
			return retval;
		}

		return 0;
	} else {
		return -ENOSYS;
	}
}
