/* $Id: dmz.c,v 1.1 2001/08/01 08:24:25 marks Exp $ */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <syslog.h>

#include "dmz.h"

int flags=0;

int main(int argc, char *argv[])
{
	int err;

	if((err=checkargs(argc,argv))) {
		usage();
		exit(err);
	}

	if((err=mydaemon())) {
		exit(err);
	} 

	if((err=execute(argc,argv))) {
		exit(err);
	}

	return err;
}

int
mydaemon()
{
	int fd,err=0;

	switch(fork()) {
	case -1:
		 return -1;
	case 0:
		break;
	default:
		exit(0);
	}

	if ((err=setsid())==-1) {
		syslog(LOG_ERR,"setsid: %m.");
		return err;
	}

	if((err=chdir("/"))) {
		syslog(LOG_ERR,"chdir: %m.");
		return err;
	}

	if(!(flags&DMZ_NOCLOSE)) {

		if((fd=open("/dev/null",O_RDWR, 0))==-1) {
			syslog(LOG_ERR,"open: %m.");
			return fd;
		}

		if((err=dup2(fd, 0))) {
			syslog(LOG_ERR,"dup2 on stdin: %m.");
			return err;
		}
		if((err=dup2(fd, 1))==-1) {
			syslog(LOG_ERR,"dup2 on stdout: %m.");
			return err;
		}
		if((err=dup2(fd, 2))==-1) {
			syslog(LOG_ERR,"dup2 on stderr: %m.");
			return err;
		}

		if (fd > 2)
			if((err=close(fd))) {
				syslog(LOG_ERR,"close: %m.");
				return err;
			}
	}

	return 0;
}

int checkargs(int argc, char *argv[])
{
	if(argc<2) {
		return -argc;
	}

	if(strstr(argv[0],"dmz_nc")!=NULL) {
		syslog(LOG_ERR,"noclose");
		flags|=DMZ_NOCLOSE;
	}

	return 0;
}

int execute(int argc,char *argv[]) {
	int err=0,i,len;
	char *cmdarg[argc+1];

	for(i=0;i<(argc-1);i++) {
		len=strlen(argv[i+1]);
		if((cmdarg[i]=(char *)malloc(len+1))==NULL) {
			syslog(LOG_ERR,"malloc: %m");
			return errno;
		}
		memset(cmdarg[i],0,len+1);
		strncpy(cmdarg[i],argv[i+1],len);
	}
	cmdarg[i]=NULL; /* make sure cmdarg[] ends with a NULL */

	if((err=isexecutable(cmdarg[0]))) {
		return err;
	}

	if((err=execv(cmdarg[0],cmdarg))) {
		syslog(LOG_ERR,"execv: %m");
		return err;
	}

	return err;
}

int isexecutable(char *path)
{
	struct stat sb;
	int err=0;

	if((err=stat(path,&sb))!=0) {
		syslog(LOG_ERR,"stat: %m");
		return(err);
	}

	if(!(sb.st_mode&S_IFREG)) {
		syslog(LOG_ERR,"%s not a regular file.",path);
		return(-1);
	}

	if((err=access(path,X_OK))!=0) {
		syslog(LOG_ERR,"%m on %s",path);
		return(err);
	}

	return 0;
}

void  usage()
{
	printf("usage: dmz <program path> [program options]\n");
}
