/*
 * Copyright (c) 2002 Endace Measurement Systems Ltd, Hamilton, New Zealand.
 * All rights reserved.
 *
 * This source code is proprietary to Endace Measurement Systems and no part
 * of it may be redistributed, published or disclosed except as outlined in
 * the written contract supplied with this product.
 *
 * $Id: dagnew.h,v 1.1.1.1 2005/04/05 12:11:52 tom Exp $
 */

#ifndef DAGNEW_H
#define DAGNEW_H

# if 0
# include	<sys/ioccom.h>
# else
# include	<sys/ioctl.h>
# include	<sys/user.h>
# endif

# ifdef	__linux__
# include	<dag.h>
# endif

#ifndef __KERNEL__
#include <sys/types.h>  /* u_short and friends. */
#endif /* __KERNEL__ */

/*
 * Driver definitions
 */
typedef enum dagminor {
	DAGMINORBITS	=	4,
	DAGMINORMASK	=	(1<<DAGMINORBITS)-1,

	DAGMINOR_DAG	=	0,
	DAGMINOR_MEM	=	1,
	DAGMINOR_IOM	=	2,
	DAGMINOR_ARM	=	3,
	DAGMINOR_RAM	=	4,
	DAGMINOR_MAX	=	5,
} dagminor_t;

/*
 * Dag ATM monitor commands
 */
# define	DAGMON_MIN		0
# define	DAGMON_VERSION		0
# define	DAGMON_READ_WORD	1
# define	DAGMON_WRITE_WORD	2
# define	DAGMON_PROGRAM_XILINX	3
# define	DAGMON_ENTER_ADDRESS	4
# define	DAGMON_READ_12_WORDS	5
# define	DAGMON_WRITE_12_WORDS	6
# define	DAGMON_MAX		6

# define	DAGMON_OK		1
# define	DAGMON_GOOD_INIT	2
# define	DAGMON_BAD_INIT		3
# define	DAGMON_BAD_PROTO	4
# define	DAGMON_INVALID_CMD	5
# define	DAGMON_UNSUPPORTED_CMD	6

# define	DAGMON_RESET_EX		7
# define	DAGMON_UNDEFOP_EX	8
# define	DAGMON_SOFTWARE_EX	9
# define	DAGMON_ABORT_PRE_EX	10
# define	DAGMON_ABORT_DATA_EX	11
# define	DAGMON_IRQ_EX		12
# define	DAGMON_FIQ_EX		13

# ifndef __linux__
/*
 * Just serves as a boilerplate to pass into the _IOWR struct
 */
typedef enum reset_magic {
	DAGRESET_FULL	= 0,
	DAGRESET_REBOOT	= 1,
	DAGRESET_INIT	= 2,
	DAGRESET_DUCK	= 3,
	DAGRESET_GOODBYE= 4,
} reset_magic_t;

typedef struct monparams {
	u_int	words[16];
} monparams_t;
# endif

typedef struct soft_caps {
	u_int	arm:1;
	u_int	pbm:1;
} soft_caps_t;

# ifdef __linux__
typedef struct dag_inf	daginf_t;
# else /* FreeBSD */
typedef struct daginf {
	u_short		device_code;	/* PCI device ID */

	u_int		id;
	u_int		phy_addr;	/* PCI address of large buffer */
	u_int		buf_size;	/* its size */
	int		firmware;

	u_int		ram_base;
	u_int		pci_base;
	u_int		xilinx1_base;
	u_int		xilinx2_base;
	u_int		xilinx3_base;
	u_int		phy_base;	/* offset within appropriate address space */
	u_int		eeprom_base;


	soft_caps_t	soft_caps;	/* also Linux emulation */
} daginf_t;
# endif

# ifdef __linux__
# define	DAGIOCRESET	DAG_IOCRESET
# define	DAGIOCMONITOR	DAG_IOCMONITOR
# define	DAGIOCINFO	DAG_IOINF
# define	DAGIOCLOCK	DAG_IOCLOCK
# define	phy_addr	buf_phy_base

# else /* __FreeBSD */
# define	DAGIOCRESET	_IOW('D',  1, u_int)
# define	DAGIOCMONITOR	_IOWR('D', 2, monparams_t)
# define	DAGIOCINFO	_IOR('D',  3, daginf_t)
# define	DAGIOCLOCK	_IOWR('D', 4, int)

# define	DAG_IOCRESET	DAGIOCRESET
# define	DAG_IOCMONITOR	DAGIOCMONITOR
# define	DAG_IOINF	DAGIOCINFO
# define	DAG_IOCLOCK	DAGIOCLOCK

# define	dag_inf		daginf_t

# define	DAG32(X)	(((X)&0xff00)==0x3200)
# define	DAG35(X)	(((X)&0xff00)==0x3500)
# endif


#endif /* DAGNEW_H */
