#ifdef __KERNEL__
#  include <linux/config.h> /* retrieve the CONFIG_* macros */
#  if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
#    define MODVERSIONS /* force it on */
#  endif

#  ifdef MODVERSIONS
#    include <linux/modversions.h>
#  endif

#  include <linux/kernel.h>
#  include <linux/wait.h>
#  ifdef CORAL
#    include	<sys/types.h>
#    include	<coral_ioctl.h>
#  endif

#else
#  include <string.h>
#  include <stdio.h>
#endif

# include	<dag.h>

#ifndef	NULL
# define	NULL	((void *)0)
#endif

char *
dag_device_name(unsigned device, int flag)
{
	static char retbuf[64];

	switch(device) {
		/*
		 * DAG2 series
		 */
	  case 0x0d21: return "vintage DAG2 card, unknown type, upgrade card firmware!";
	  case 0x2100: return "Dag 2.10, 2nd prototype OC3c";
	  case 0x2110: return "Dag 2.11, OC3c ATM";
	  case 0x211D: return "Dag 2.11D, DS3 ATM";
	  case 0x211E: return "Dag 2.11E, 10/100baseT Ethernet";
		/*
		 * DAG3 series
		 */
	  case 0x3000: return "Dag 3.00, 1st prototype OC3c/OC12c";
	  case 0x3010: return "Dag 3.01, 2nd prototype OC3c/OC12c";
	  case 0x3100: return "Dag 3.10, OC3c/OC12c";
	  case 0x3200: return "Dag 3.2x";
	  case 0x3300: return "Dag 3.30, PMC OC3c/OC12c w/fifo";
	  case 0x3400: return "Dag 3.4";
	  case 0x340E: return "Dag 3.4E";
	  case 0x341E: return "Dag 3.41E";
	  case 0x3500: return "Dag 3.5";
	  case 0x351c: return "Dag 3.51ECM";
	  case 0x350E: return "Dag 3.5E";
	  case 0x360D: return "Dag 3.6 DS3";
	  case 0x360E: return "Dag 3.6E";
	  case 0x368E: return "Dag 3.6GE";
	  case 0x3800: return "Dag 3.8 Dual OC3/12";
		/*
		 * DAG4 series
		 */
	  case 0x4000: return "Dag 4.00, 1st prototype OC48c ATM";
	  case 0x4100: return "Dag 4.10, OC48c ATM/PoS";
	  case 0x4110: return "Dag 4.11, OC48c ATM/PoS";
	  case 0x4200: return "Dag 4.2, OC48c ATM/PoS";
	  case 0x420E: return "Dag 4.2GE, Gigabit Ethernet";
	  case 0x4210: return "Dag 4.21, OC48c ATM/PoS";
	  case 0x4220: return "Dag 4.22, OC48c ATM/PoS";
	  case 0x422E: return "Dag 4.22GE, Dual Gigabit Ethernet";
	  case 0x4230: return "Dag 4.23, OC48c ATM/PoS";
	  case 0x423E: return "Dag 4.23GE, Dual Gigabit Ethernet";
	  case 0x4300: return "Dag 4.3, OC48c ATM/PoS";
	  case 0x430E: return "Dag 4.3GE, Dual Gigabit Ethernet";
		/*
		 * DAG5 series
		 */
	  case 0x50EE: return "Dag 5 Ethernet Evaluation board";
		/*
		 * DAG6 series
		 */
	  case 0x6000: return "Dag 6.00, 1st prototype OC192c POS";
	  case 0x6100: return "Dag 6.10, OC192c POS/10G Ethernet";
		/*
		 * None of the above
		 */
	  default:
		if(flag) {
			(void)sprintf(retbuf, "unknown card 0x%.4x", device);
			return retbuf;
		} else {
			return NULL;
		}
	}
	/*NOTREACHED*/
}

char *
dag_caps_name(unsigned caps, int flag)
{
	static char retbuf[64];

	switch (caps & ~CAPS_MAGIC_MASK) {
	  case CAPS_INVALID:		return "invalid";
	  case CAPS_BASIC:		return "basic";
	  case CAPS_ATM:		return "ATM";
	  case CAPS_POS:		return "PoS";
	  case CAPS_ATM_DUCK:		return "ATM, DUCK";
	  case CAPS_POS_DUCK:		return "PoS, DUCK";
	  case CAPS_ATM_TX:		return "ATM, TX";
	  case CAPS_ETH:       		return "ETH";
	  case CAPS_ETH_DUCK:  		return "ETH, DUCK";
	  case CAPS_ATM_DUCK_FIFO:	return "ATM, DUCK, FIFO";
	  case CAPS_POS_DUCK_FIFO:	return "POS, DUCK, FIFO";
	  case CAPS_ETH_DUCK_FIFO:	return "ETH, DUCK, FIFO";
	  case CAPS_PCI_DAG4:		return "PCI, DAG4";
	  case CAPS_PCI_DAG34:		return "PCI, DAG3.4";
	  case CAPS_PCI_DAG35:		return "PCI, DAG3.5";
	  case CAPS_PCI_DAG35ECM:	return "PCI, DAG3.5ECM";
	  case CAPS_PCI_DAG35PBM:	return "PCI, DAG3.5PBM";
	  case CAPS_PCI_DAG35PC:	return "PCI, DAG3.5PC";
	  default:
		if(flag) {
			(void)sprintf(retbuf, "unknown capabilites 0x%.6x", caps);
			return retbuf;
		} else {
			return NULL;
		}
	}
	/*NOTREACHED*/
}

char *
dag_soft_caps_name(scaps_bits soft_caps, int flag)
{
  static char retbuf[64];
  int len=0;
  
  if (soft_caps.system)
    len += sprintf(retbuf+len, "System ");
  if (soft_caps.atm)
    len += sprintf(retbuf+len, "ATM ");
  if (soft_caps.pos)
    len += sprintf(retbuf+len, "PoS ");
  if (soft_caps.eth)
    len += sprintf(retbuf+len, "Ethernet ");
  if (soft_caps.duck)
    len += sprintf(retbuf+len, "Duck ");
  if (soft_caps.fifo)
    len += sprintf(retbuf+len, "FIFO ");
  if (soft_caps.tx)
    len += sprintf(retbuf+len, "Transmit ");
  if (soft_caps.pbm)
    len += sprintf(retbuf+len, "PBM ");
  if (soft_caps.arm)
    len += sprintf(retbuf+len, "ARM ");
  if (soft_caps.duckemu)
    len += sprintf(retbuf+len, "DuckEmu ");
  if (soft_caps.gpp)
    len += sprintf(retbuf+len, "GPP ");
  if (soft_caps.btx)
    len += sprintf(retbuf+len, "BTX ");
  if (soft_caps.code)
    len += sprintf(retbuf+len, "Code ");
  
  
  if(flag && (!len)) {
    (void)sprintf(retbuf, "unknown!");
    return retbuf;
  }
  
  return retbuf;
}

char *
dag_phyconf_name(unsigned phyconf, int flag)
{
	static char retbuf[64];

	switch(phyconf) {
	  case SP_DEFAULT:	return "default";
	  case ETH_10:		return "10BaseT";
	  case DS3:		return "ATM DS3";
	  case E3:		return "ATM E3";
	  case ETH_100:		return "100BaseT";
	  case OC3c_ATM:	return "ATM OC3c";
	  case OC3c_POS:	return "PoS OC3c";
	  case OC12c_ATM:	return "ATM OC12c";
	  case OC12c_POS:	return "PoS OC12c";
	  case ETH_1000:	return "1000BaseXX";
	  case OC48c_ATM:	return "ATM OC48c";
	  case OC48c_POS:	return "POS OC48c";
	  default:
		if(flag) {
			(void)sprintf(retbuf, "unknown %d", phyconf);
			return retbuf;
		} else {
			return NULL;
		}
	}
	/*NOTREACHED*/
}

/*
 * We put a lot of paranoia and redundancy into the code to make
 * it foolproof against spurious data.
 */
char **
dag_xrev_parse(unsigned char *xrev, unsigned xlen, char *argv[5])
{
	int	i, rlen;
	char	**ap = argv;
	char	expect[] = "abcd";

	xrev += 13;
	xlen -= 13;

	for(i = 0 ; i < 4 ; i++) {
		/*
		 * Record tag: a,b,c,d
		 */
		if(xrev[0] != expect[i])
			goto done;		/* must be a,b,c,d in that order */
		xrev++;
		xlen--;
		/*
		 * 2 Bytes record length, little endian
		 */
		rlen = (((unsigned)xrev[0]) << 8) | xrev[1];
		xrev += 2;
		xlen -= 2;
		/*
		 * Actual record
		 */
		if(rlen > xlen)
			goto done;		/* record points past buffer */
		*ap++ = xrev;
		xrev += rlen;
		xlen -= rlen;
	}
done:
	while(ap < &argv[5])
		*ap++ = NULL;
	return argv;
}

/*
 * This routine will partially destroy the contents of xrev by
 * writing '\0' markers into the buffer and letting argv point
 * into the buffer.
 */
char *
dag_xrev_name(unsigned char *xrev, unsigned xlen)
{
	static char	retbuf[128];
	char		*bp = retbuf;
	char		*argv[5], **ap;

	ap = dag_xrev_parse(xrev, xlen, argv);
	while(*ap != NULL)
		bp += sprintf(bp, "%s ", *ap++);
	if(ap != argv)
		*--bp = '\0';		/* remove trailing space */
	else
		*bp = '\0';		/* empty buffer */
	return retbuf;
}
