/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: ntpns.h,v 1.4 2005/01/26 15:28:53 tom Exp $
 *
 * This file exposes the API for refclocks and other plugins.
 */

#ifndef __NTPNS_H

#define __NTPNS_H

/* version numbers */
#define NTPNS_MAJOR	1
#define NTPNS_MINOR	1

#include <sys/queue.h>

#include <timestamp.h>

#include "../../bind-src/include/isc/eventlib.h"

extern int dopt, Dopt;

#define debug(level, ...) \
	do { if (level >= dopt) fprintf(stderr, ## __VA_ARGS__); } while (0)

void fhexdump(FILE *f, u_int len, const void *p);

#define nomem() warn("Could not allocate memory.")

typedef const char * ref_init_t(evContext ctx);
/* Declare the init functions of loadable modules */

/* -------- */
/* config.c */
/* -------- */

struct configlist;

typedef int conffunc_t(evContext ctx, FILE *f, void *priv, char **tok, int flag);
#define CFG_HELP	0x0001
#define CFG_NO		0x0002
#define CFG_SHOW	0x0004

struct configlist *ConfigCreate(void);
void ConfigHelp(FILE *f, const char *s);
void ConfigDestroy(struct configlist *);
int ConfigAdd(struct configlist *, const char *keyword, conffunc_t *func, const char *help);
void ConfigDel(struct configlist *, const char *keyword);
int ConfigCall(struct configlist *, evContext ctx, FILE *f, void *priv, char **tok, int flag);

void ConfigSource(evContext ctx, FILE *fo, FILE *fi, int fd, int flags);
#define CONFIG_PROMPT	0x0001

/* ----- */
/* log.c */
/* ----- */


struct logh;

struct logh *LogNewHandle(struct logh *, const char *name);
int LogActive(struct logh *lh);
void LogDeleteHandle(struct logh *);
void Log(struct logh *, const char *fmt, ...) __printflike(2, 3);
void LogNoNewline(struct logh *, const char *fmt, ...) __printflike(2, 3);
void LogErr(struct logh *, const char *fmt, ...) __printflike(2, 3);
void LogAssert(const char *file, const char *func, int line, const char *fmt, ...) __printflike(4, 5) __dead2;

extern struct logh *Lerror, *Lwarn, *Linfo, *Ldebug, *Lclockstats;

#define Assert(e, ...) do { if (!(e)) \
    LogAssert(__FILE__, __func__, __LINE__, __VA_ARGS__); \
    } while (0)

#define Abort(...) LogAssert(__FILE__, __func__, __LINE__, __VA_ARGS__)

int Warn(FILE *f, int error, const char *fmt, ...);
int Err(FILE *f, int error, const char *fmt, ...);
int update_string(FILE *f, char **p, const char *q);

/* --------- */
/* pll.c */
/* --------- */

extern int clock_step;		/* increments when clock is stepped */
extern int pll_locked;		/* zero if the PLL is not locked */

/* --------- */
/* pps-api.c */
/* --------- */

struct ppsdev;

int PpsCreate(struct ppsdev **, int flanks);
void PpsDestroy(struct ppsdev **);
conffunc_t PpsConfig;
int PpsPickup(struct ppsdev **p, struct ts *ts, int flanks);

/* ---------- */
/* resolver.c */
/* ---------- */
struct addrinfo;
typedef void resfunc_t(evContext ctx, void *priv, int error, struct addrinfo *ai, FILE *f);
int resolver(evContext ctx, const char *host, const char *port, int protofam, int socktype, int flags, resfunc_t *func, void *priv, FILE *f);

/* -------- */
/* record.c */
/* -------- */

int rrrdupdate(const char *file, const char *template, const char *fmt, ...);
void record(const char *ident, int unit, void *p, int len);

/* ------- */
/* shmem.c */
/* ------- */
void *shmem_alloc(u_int length, u_int tag);
void shmem_free(void *ptr);

/* -------- */
/* source.c */
/* -------- */

struct source;

typedef enum {
	SRC_INVALID = 1,
	SRC_LOS = 2,
	SRC_FREQUENCY = 3,
	SRC_FRACTION = 4,
	SRC_UTC = 5
} src_state_e;

typedef enum {
	SRC_UPDATE_RATE,
	SRC_LOS_TIMEOUT,
	SRC_VOTES,
	SRC_DISPERSION,
	SRC_DELAY,
	SRC_STRATUM
} src_param_e;

struct source *SourceAdd(const char *fmt, ...) __printflike(1, 2);
void SourceDel(struct source *sid);
void SourceDelta(struct source *sid, struct ts *ts, double delta, src_state_e state);
void SourceTick(struct source *sid, struct ts *ts, src_state_e state);
void SourceParam(struct source *sid, src_param_e param, double val);
void SourceRefid(struct source *sid, const void *refid);

extern u_char	sys_refid[4];
extern double	sys_delay;
extern double	sys_dispersion;
extern int	sys_precision;
extern int	sys_stratum;

#endif
