/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: load.c,v 1.3 2005/01/26 15:28:53 tom Exp $
 *
 * Load and unload modules
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <err.h>
#include <fcntl.h>
#include <errno.h>
#include <dlfcn.h>
#include <sys/queue.h>

#include "ntpns.h"
#include "internal.h"

static TAILQ_HEAD(, load) load_head =
	TAILQ_HEAD_INITIALIZER(load_head);

/*
 * This structure contains the status for each
 * loaded module
 */
struct load {
	TAILQ_ENTRY(load)	next;
	char			*name;
	void			*dlhandle;
};

static int
load_load(evContext ctx, FILE *f, void *priv __unused, char **tok, int flag)
{
	ref_init_t *func;
	struct load *ld;
	const char *p;
	char *q;
	int ret;

	if (flag & CFG_SHOW) {
		TAILQ_FOREACH(ld, &load_head, next) {
			fprintf(f, "load %s\n", ld->name);
		}
		return (0);
	}
	if ((flag & CFG_HELP) || tok[0] == NULL || tok[1] != NULL) {
		ConfigHelp(f, "load $filename");
		return (0);
	}
	if (flag & CFG_NO)
		return (EINVAL);

	ld = calloc(sizeof *ld, 1);
	if (ld == NULL)
		return (ENOMEM);
	ld->name = strdup(tok[0]);
	if (ld->name == NULL) {
		free(ld);
		return (ENOMEM);
	}
	TAILQ_INSERT_TAIL(&load_head, ld, next);
	ld->dlhandle = dlopen(tok[0], RTLD_NOW | RTLD_GLOBAL | RTLD_LOCAL);
	if (ld->dlhandle == NULL) {
		ret = errno;
		fprintf(f, "load failed, dlopen(%s) %s\n", tok[0], dlerror());
	} else {
		func = dlsym(ld->dlhandle, "init");
		if (func == NULL) {
			p = strrchr(tok[0], '.');
			if (p == NULL)
				p = tok[0];
			else
				p++;
			asprintf(&q, "%s_init", p);
			func = dlsym(ld->dlhandle, q);
			free(q);
		}
		if (func == NULL) {
			ret = errno;
			fprintf(f, "load failed, no \"init\" function\n");
		} else {
			p = func(ctx);
			/* XXX: is there a way to check validity of a pointer ? */
			if (p == NULL || memcmp(p, MAGIC, strlen(MAGIC))) {
				ret = EINVAL;
				fprintf(f, "load failed, module has wrong magic number.\n");
			} else {
				return (0);
			}
		}
	}
	free(ld->name);
	if (ld->dlhandle)
		dlclose(ld->dlhandle);
	TAILQ_REMOVE(&load_head, ld, next);
	free(ld);
	return (ret);
}

void
init_load(evContext ctx __unused)
{

	ConfigAdd(NULL, "load", load_load, "load extension module");
}
