/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: main.c,v 1.7 2005/01/26 15:28:53 tom Exp $
 *
 * This is the main entrance where we set up the event engine, parse the
 * options and set the ball rolling by starting events to read the config
 * file.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <err.h>
#include <errno.h>

#include "ntpns.h"
#include "internal.h"

ref_init_t	oncore_m12_init;
ref_init_t	dcf77_init;
ref_init_t	ntpv4_init;
ref_init_t	trimble_initialize;

int dopt, Dopt;

void
fhexdump (FILE *f, u_int len, const void *p)
{
	const u_char *s;
	u_int i;

	s = p;
	for(i = 0; i < len; i++)
		fprintf(f, "%02x", s[i]);
	fprintf(f, "\n");
}

static void
Usage(const char *progname)
{

	errx(1, "Usage: %s [-d] [-f config-file]", progname);
}

int
main(int argc, char **argv)
{
	int i;
	char *fopt;
	evContext ctx;
	FILE *fd;

	setbuf(stdout, NULL);
	fopt = NULL;
	while((i = getopt(argc, argv, "dDf:")) != -1) {
		switch (i) {
		case 'd':
			dopt++;
			break;
		case 'f':
			fopt = optarg;
			break;
		case 'D':
			Dopt++;
			break;
		default:
			Usage(argv[0]);
		}
	}

	if (getuid() != 0) {
		fprintf(stderr, "Running as non-root, setting '-d' and '-D'\n");
		dopt++;
		Dopt++;
	}

	i = evCreate(&ctx);
	if (i)
		err(1, "evCreate()");
	i = evSetOption(NULL, "monotime", 1);
	if (i)
		err(1, "evSetOption(NULL, monotime,1)");

	init_config(ctx);
	init_log(ctx);
	init_shmem(ctx);
	init_load(ctx);
	init_pll(ctx);
	init_pllmath(ctx);
	init_record(ctx);
	init_telnet(ctx);
	init_source(ctx);
#ifdef LEDS
	init_watch(ctx);
#endif /* LEDS */

	oncore_m12_init(ctx);
	dcf77_init(ctx);
	ntpv4_init(ctx);
	trimble_initialize(ctx);

	if (fopt != NULL) {
		fd = fopen(fopt, "r");
		if (fd == NULL)
			err(1, fopt);
		ConfigSource(ctx, stderr, fd, -1, 0);
	}

	if (dopt) {
		/*
		 * In debug mode we start an event on stdin as well.
 		 */
		ConfigSource(ctx, stderr, stdin, -1, CONFIG_PROMPT);
		if (dopt > 1)
			evSetDebug(ctx, dopt - 1, stdout);
	} else {
		daemon(1, 1);
	}


	run_shmem(ctx);

	i = evMainLoop(ctx);
	if (i && errno != ENOENT)
		err(1, "evMainLoop()");

	debug(1, "We're done...\n");

	return (0);
}
