/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: mumble.c,v 1.3 2005/01/26 15:28:53 tom Exp $
 *
 */

#include <stdarg.h>
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>

#include <ntpns.h>

static void
WarnV(FILE *f, int error, const char *fmt, va_list ap)
{

	if (fmt != NULL)
		vfprintf(f, fmt, ap);
	if (error != 0)
		fprintf(f, ": %s", strerror(error));
	fprintf(f, "\n");
}

int
Warn(FILE *f, int error, const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	WarnV(f, error, fmt, ap);
	va_end(ap);
	return (-1);
}

int
Err(FILE *f, int error, const char *fmt, ...)
{
	va_list ap;

	fprintf(f, "%s: ", getprogname());
	va_start(ap, fmt);
	WarnV(f, error, fmt, ap);
	va_end(ap);
	exit (1);
}

int
update_string(FILE *f, char **p, const char *q)
{

	*p = realloc(*p, strlen(q) + 1);
	if (*p == NULL)
		return Warn(f, errno, "malloc failed");
	strcpy(*p, q);
	return (0);
}
