/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: telnet.c,v 1.3 2005/01/26 15:28:53 tom Exp $
 *
 * Telnet protocol engine for OaM access.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <err.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/queue.h>
#include <sys/socket.h>
#include <sys/endian.h>
#include <math.h>
#include <netdb.h>

#include "ntpns.h"
#include "internal.h"

static int telnetsock = -1;
static evConnID cid;

static void
telnet_connect(evContext ctx, void *uap __unused, int fd, const void *la __unused, int lalen __unused, const void *ra __unused, int ralen __unused)
{
	
	/* XXX: log telnet connections src+dst IP */
	ConfigSource(ctx, NULL, NULL, fd, CONFIG_PROMPT);
}

static void
resolved(evContext ctx, void *priv __unused, int error, struct addrinfo *ai, FILE *f)
{
	int s, i;

	if (error) {
		fprintf(f, "failed to resolve address for record (%d)\n", error);
		return;
	}

	s = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
	if (s < 0) {
		warn("socket failed for record");
		return;
	}
	i = 1;
	i = setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &i, sizeof i);
	if (i < 0) 
		warn("setsockopt(SO_REUSEADDR) failed");
	i = bind(s, ai->ai_addr, ai->ai_addrlen);
	if (i < 0) {
		warn("bind failed");
		close (s);
		telnetsock = -1;
		return;
	}
	i = evListen(ctx, s, 10, telnet_connect, NULL, &cid);
	if (i < 0) {
		close (s);
		err(1, "evListen failed");
	} else {
		telnetsock = s;
	}
}


static int
telnet_telnet(evContext ctx, FILE *f, void *priv __unused, char **tok __unused, int flag)
{
	int i;

	if (flag & CFG_HELP) {
		ConfigHelp(f, "telnet");
		return (0);
	}
	if (flag & CFG_SHOW) {
		fprintf(f, "Telnet server on port 123 is %s\n",
		    telnetsock == -1 ? "disabled" : "enabled");
		return (0);
	}
	if (flag & CFG_NO) {
		evCancelConn(ctx, cid);
		close(telnetsock);
		telnetsock = -1;
		return (0);
	}
	i = resolver(ctx, NULL, "ntp", PF_INET, SOCK_STREAM,
	    AI_CANONNAME | AI_PASSIVE, resolved, NULL, f);
	if (i < 0)
		fprintf(f, "DNS resolver failed\n");

	return (0);
}

void
init_telnet(evContext ctx __unused)
{

	ConfigAdd(NULL, "telnet", telnet_telnet, "enable telnet server");
}
