/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: watch.c,v 1.3 2005/01/26 15:28:53 tom Exp $
 *
 * Watchdog stuff.
 *
 */

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include "internal.h"
#include <sys/types.h>
#include "ntpns.h"
#include "pll.h"

static evTimerID watch_tid;
static int fd_sec = -1;
static int fd_run = -1;
static int fd_src = -1;
static int fd_lock = -1;
static int fd_time = -1;

static void
watch_timer(evContext ctx __unused, void *uap __unused, struct timespec t0 __unused, struct timespec dt __unused)
{
	struct timespec t1;
	int i, j;

	/* Turn on/off the UTC led ASAP */
	if (clk_state > UTC_STEP)
		write(fd_sec, "1", 1);
	else
		write(fd_sec, "0", 1);

	/* Aim the timer for the next second */
	t1 = evUTCTime();
	j = 1000000000 - t1.tv_nsec;
	if (j < 1000000)
		j += 1000000000;
	i = evResetTimer(ctx, watch_tid, watch_timer, uap,
		t0, evConsTime(0, j));
	Assert(i == 0, "evResetTimer(watch_timer, %d)", j);

	/* Figure out how long time the UTC LED should be on */
	if (clk_state > UTC_STEP) {
		i = t1.tv_sec;
		if (t1.tv_nsec > 500000000)
			i++;
		if ((i % 60) == 0)
			write(fd_sec, "sDa.", 4);
		else
			write(fd_sec, "sAa.", 4);
	}

	/* Give the RUN Led a poke */
	write(fd_run, "sJAj.", 5);

	/* Give the source LED a poke */
	if (selsrc == NULL)
		write(fd_src, "1", 1);
	else
		write(fd_src, "sjaA.", 5);

	/* Give the time LED a poke */
	if (clk_state == UTC_LOCK) {
		write(fd_time, "sJAa.", 5);
		pll_locked = 1;
	} else if (clk_state == FRAC_LOCK) {
		write(fd_time, "sJAa.", 5);
		pll_locked = 1;
	} else if (clk_state == COAST) {
		write(fd_time, "saJa.", 5);
		pll_locked = 1;
	} else {
		write(fd_time, "0", 1);
		pll_locked = 0;
	}

	/* And the lock LED */
	if (pll_state.st >= 4 && clk_state >= UTC_LOCK)
		write(fd_lock, "sajA.", 5);
	else if (pll_state.st >= 4)
		write(fd_lock, "saA.", 4);
	else if (pll_state.st >= 3)
		write(fd_lock, "saAaA.", 6);
	else if (pll_state.st >= 2)
		write(fd_lock, "saAaAaA.", 8);
	else
		write(fd_lock, "1", 1);
}

static int
watch_watch(evContext ctx __unused, FILE *f, void *priv __unused, char **tok, int flag)
{

	if (tok[0] == NULL || (flag & CFG_HELP)) {
		ConfigHelp(f, "watch XXX");
		return (0);
	}
	if (flag & CFG_SHOW) {
		fprintf(f, "XXX show watch\n");
		return (0);
	}
	if (flag & CFG_NO) {
		
		return (0);
	}

	return (0);
}

void
init_watch(evContext ctx __unused)
{
	int i;

	if (0)
		ConfigAdd(NULL,
		    "watch", watch_watch, "Watchdog and announciator control");

	fd_run = open("/dev/led/gpio21", O_RDWR);
	if (fd_run < 0)
		LogErr(Lwarn, "Could not open RUN led");

	fd_src = open("/dev/led/gpio6", O_RDWR);
	if (fd_src < 0)
		LogErr(Lwarn, "Could not open SRC led");

	fd_lock = open("/dev/led/gpio7", O_RDWR);
	if (fd_lock < 0)
		LogErr(Lwarn, "Could not open LOCK led");

	fd_time = open("/dev/led/gpio8", O_RDWR);
	if (fd_time < 0)
		LogErr(Lwarn, "Could not open LOCK led");
	
	fd_sec = open("/dev/led/gpio5", O_RDWR);
	if (fd_sec < 0)
		LogErr(Lwarn, "Could not open UTC_SECOND led");


	i = evSetTimer(ctx, watch_timer, &watch_tid,
		evConsTime(0, 0), evConsTime(1, 0), &watch_tid);
	Assert(i == 0, "evSetTimer(watch_timer)");
	i = evConfigTimer(ctx, watch_tid, "rate", 0);
	Assert(i == 0, "evConfigTimer(watch_timer)");
}
