/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: ntpv4.c,v 1.2 2005/01/26 15:28:53 tom Exp $
 */

#include <err.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/endian.h>
#include <netinet/in.h>

#include <ntpns.h>
#include <ntpv4.h>

struct logh *lntpv4, *lsocket, *lpacket, *lpartner;

/**********************************************************************/

static TAILQ_HEAD(, instance)	instance_head =
    TAILQ_HEAD_INITIALIZER(instance_head);

/**********************************************************************/

static int
ntpv4_server(evContext ctx, FILE *f, void *priv __unused, char **tok, int flag)
{
	struct instance *ip;
	int i;
	u_int u;
	struct configured *cf;
	char *p;
	double a;

	ip = priv;

	if (flag & CFG_HELP) {
		ConfigHelp(f,"ntpv4 $instance server $dns_or_ip [poll $poll] [votes $votes]");
		return (0);
	}
	if (flag & CFG_SHOW) {
		fprintf(f, "XXX: ntpv4_server SHOW\n");
		return (0);
	}
	cf = FindConfigured(tok[0], flag & CFG_NO ? 0 : 1);
	if (cf == NULL)
		return (ENOMEM);
	cf->inp = ip;
	if (flag & CFG_NO) {
		DeleteConfigured(ctx, cf);
		return (0);
	}
	i = evSetTimer(ctx, config_timer, cf,
	     evConsTime(0, 0), evConsTime(0, 0), &cf->tid);
	if (i < 0)
		err(1, "evSetTimer() failed");

	cf->poll = 64;
	cf->votes = 1;

	for (tok++; tok[0] != NULL; tok++) {
		if (!strcmp(tok[0], "poll")) {
			if (tok[1] == NULL)
				fprintf(f,
				     "missing poll interval\n");
			u = strtoul(tok[1], &p, 0);
			if (*p != '\0' || u == 0)
				fprintf(f,
				     "non-numeric or illegal poll argument\n");
			else
				cf->poll = u;
			tok++;
			continue;
		}
		if (!strcmp(tok[0], "votes")) {
			if (tok[1] == NULL)
				fprintf(f,
				    "missing count of votes\n");
			a = strtod(tok[1], &p);
			if (*p != '\0')
				fprintf(f,
				     "non-numeric or illegal votes argument\n");
			else
				cf->votes = a;
			tok++;
			continue;
		}
		fprintf(f, "Illegal parameter \"%s\"\n", tok[0]);
		break;
	}
	
	return (0);
}

/**********************************************************************/

static struct configlist *ntpv4_cfl;

static struct instance *
mkinstance(evContext ctx, int u)
{
	struct instance *ip;

	ip = calloc(sizeof *ip, 1);
	if (ip == NULL) {
		nomem();
		return (NULL);
	}
	ip->unit = u;
	TAILQ_INSERT_TAIL(&instance_head, ip, next);
	OpenSocket(ctx, ip, NULL);
	return (ip);
}

static int
ntpv4_ntpv4(evContext ctx, FILE *f, void *priv, char **tok, int flag)
{
	u_long u;
	char *p;
	struct instance *ip;

	if (flag & CFG_SHOW) {
		ntpv4_show_partner(ctx, f, priv, tok, flag);
		return (0);
	}
	if (tok[0] == NULL) {
		fprintf(f, "Specify NTPv4 instance number\n");
		return (-1);
	}
	u = strtoul(tok[0], &p, 0);
	if (*p != '\0') {
		fprintf(f, "NTPv4 unit number is not numeric");
		return (0);
	}
	if (flag & CFG_NO) {
		fprintf(f, "XXX: delete instance %lu\n", u);
		return (-1);
	}
	TAILQ_FOREACH(ip, &instance_head, next) {
		if (ip->unit == u)
			break;
	}
	if (ip == NULL)
		ip = mkinstance(ctx, u);

	return (ConfigCall(ntpv4_cfl, ctx, f, ip, tok + 1, flag));
}

ref_init_t ntpv4_init;

const char *
ntpv4_init(evContext ctx)
{
	static int once = 0;

	if (once) {
		warnx("ntpv4 module already loaded");
		return (NULL);
	} else {
		once = 1;
	}

	ConfigAdd(NULL, "ntpv4", ntpv4_ntpv4, "NTP packet processing");
	ntpv4_cfl = ConfigCreate();
	ConfigAdd(ntpv4_cfl, "server", ntpv4_server, "Remote server instance");

	lntpv4 = LogNewHandle(Ldebug, "ntpv4");
	lsocket = LogNewHandle(lntpv4, "socket");
	lpacket = LogNewHandle(lntpv4, "packet");
	lpartner = LogNewHandle(lntpv4, "partner");

	mkinstance(ctx, 0);
	return (MAGIC);
}
