/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: ntpv4.h,v 1.1.1.1 2004/08/26 14:47:10 ttraffic Exp $
 */

/*
 * Our internal representation of a NTP packet
 */
struct ntp_packet {
	uint8_t		leap;
	uint8_t		version;
	uint8_t		mode;
	uint8_t		stratum;
	uint8_t		poll;
	int8_t		prec;
	double		delay;
	double		disp;
	uint8_t		refid[4];
	struct ts	ts[5];
};

struct instance {
	TAILQ_ENTRY(instance)	next;
	u_long			unit;

	/* packet.c */
	int			socket;
	evFileID		socket_fid;
	int			userland_ts;

};

struct partner;

struct configured {
	struct instance		*inp;
	LIST_ENTRY(configured)	next;
	char			*name;
	evTimerID		tid;
	LIST_HEAD(,partner)	partner;
	int			poll;
	double			votes;
};

#define TAG_PARTNER 0x50415254

struct partner {	/* ... in time */
	u_int			generation;
	struct instance		*inp;
	/* partner.c fields */
	LIST_ENTRY(partner)	hash;
	TAILQ_ENTRY(partner)	lru;
	uint32_t		ip;
	time_t			last;
	struct sockaddr_in	addr;
	evTimerID		tid;
	uint64_t		originate;

	struct source		*sid;

	void			*cookie;
	u_int			cookielen;

	int			flag;
#define P_F_UPDATE		1
#define P_F_NEVER		2

	LIST_ENTRY(partner)	config;
	struct configured	*configured;

	struct ntp_packet	packet;
};

extern struct logh *lntpv4, *lsocket, *lpacket, *lpartner;

/* packet.c */
int OpenSocket(evContext ctx, struct instance *ip, char **tok);
struct msghdr * socket_prepare(struct partner *p, void *buf, u_int len);

/* proto.c */
void RcvPacket(evContext ctx, struct instance *ip, void *packet, int len, struct sockaddr_in *s, struct ts *ts, void *cookie, u_int cookielen);
void poll_timer(evContext ctx, void *uap, struct timespec due, struct timespec inter);
void PrintPacket(FILE *f, struct ntp_packet *p);

/* partner.c */
struct partner *LookupPartner(evContext ctx, time_t sec, uint32_t ip);
void DeletePartner(evContext ctx, struct partner *);
struct configured *FindConfigured(char *name, int create);
void DeleteConfigured(evContext ctx, struct configured *cp);
void ntpv4_show_partner(evContext ctx, FILE *f, void **priv, char **tok, int flags);
void config_timer(evContext ctx, void *uap, struct timespec due, struct timespec inter);
