/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <phk@FreeBSD.ORG> wrote this file.  As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp
 * ----------------------------------------------------------------------------
 * $Id: oncore_m12.h,v 1.1.1.1 2004/08/26 14:47:10 ttraffic Exp $
 */

#include <sys/queue.h>

struct instance;
struct oncore_msg;
typedef void msgin_t(evContext ctx, struct instance *ip, struct oncore_msg *mp);

#define NSAT		33		/* 1-32 */
#define NCHAN		12
#define NALMANAC	34
#define LALMANAC	33

struct persat {
	/* from @@Bb */
	int	doppler;
	int	elevation;
	int	azimuth;
	int	health;
	/* from @@Ha */
	int	mode;
	int	signal_strength;
	int	iode;
	int	status;
	/* from @@Hn */
	int	offset;
};

enum oncore_state {
	ST_UNCONF = 0,
	ST_CONFIG,
	ST_START,
	ST_LISTEN,
	ST_CONTACT,
	ST_POLL_STATUS,
	ST_RESET,
	ST_RESET_WAIT,	
	ST_TEST,
	ST_TEST_WAIT,
	ST_SICK,
	ST_INIT,
	ST_SYNC_WAIT,
	ST_SYNC
};

struct instance {
	TAILQ_ENTRY(instance)	next;
	u_int			unit;

	enum oncore_state	state;
	evTimerID		reset_timer;

	struct source		*sid;

	char			*serial;
	int			serial_fd;
	evFileID		serial_fid;
	u_char			serial_buf[1024];
	int			serial_idx;

	struct ppsdev		*pps;

	struct persat	persat[NSAT];

	/* from @@Ha */
	struct tm	tm;
	time_t		t;
	int		fraction;
	int		flat, flon, fht;
	int		lat, lon, ht;
	int		dop, vsat, tsat;
	u_int		rcv_status;
	int		clock_bias;
	int		osc_offset;
	int		osc_temp;
	int		utc_status;
	int		lutc_status;
	int		utc_sign;
	int		utc_hour;
	int		utc_min;
	char		serno[7];
	struct ts	ts;
	double		delta;

	/* from @@Cb */
	enum {
	    ALMANAC_COLLECT = 0,
	    ALMANAC_LOAD,
	    ALMANAC_LOADED,
	    ALMANAC_SAVE,
	    ALMANAC_DONE
	 }		almanac_work;
	pthread_t	almanac_thread;

	char		*almanac_file;
	u_char		*almanac;
	u_char		*almanac_ptr;
	int		almanac_n;

};

extern struct logh *lm12_debug, *lm12_serial, *lm12_msg;

/* oncore_m12_serial.c */
conffunc_t m12_serial;
void m12_sendmsg(struct instance *ip, const void *msg, u_int len);
void oncore_initialize(struct instance *ip);

/* oncore_m12_msg.c */

struct oncore_msg {
	int		rq_len, rs_len;
	char const	*rq_format, *rs_format;
	msgin_t		*handler;
};

#define NMASH (256*256)
static __inline int
msg_mash(const void *s)
{
	const u_char *p = s;

	return ((p[2] << 8) | p[3]);
}

int init_oncore_tables(void);

extern char const *oncore_m12_response[];
extern char const *oncore_m12_request[];
extern struct oncore_msg **oncore_msgs;

#define DEFAULT_ALMANAC_FILE "/var/tmp/oncore_m12_almanac.bin"

/* oncore_m12_state.c */
void oncore_state(evContext ctx, struct instance *ip, enum oncore_state state);
void oncore_reset_timer(evContext ctx, struct instance *ip);
