/*
 * $Id: trimble.h,v 1.12 2004/09/23 12:46:34 ttraffic Exp $
 */

#ifndef __TRIMBLE_H

#define __TRIMBLE_H

#include <sys/queue.h>

enum trimble_state { ST_UNCONFIGURED,           // right after initialization of the driver
                     ST_SERIAL_SETUP_COMPLETE,  // serial port configured, ready for comm. with the trimble
		     ST_TRIMBLE_POS_FIX,        // trimble in position fix state
                     ST_TRIMBLE_CLOCK           // trimble in overdetermined clock mode
};

#define MAX_RPTBUF (256)

#define GPS_C     (299792458.)
#define GPS_PI    (3.1415926535898)
#define R2D	(180.0/GPS_PI)

// log a bulk of lines to clockstats every n seconds
#define CLOCKSTATS_LOG_FREQUENCY 64

// trimble heartbeat every 10 seconds
#define TRIMBLE_TIMER 10

struct instance {
   TAILQ_ENTRY(instance)   next;
   u_int unit;

   struct source *sid;

   enum trimble_state state;

   char *serial;
   int serial_fd;
   evFileID serial_fid;
   u_char serial_buf[1024];
   int serial_idx;

   struct ppsdev *pps;

   evTimerID trimble_timer;

   // do we have UTC lock?
   int utc_lock;

   double pps_cable_length;
};

/* TSIP packets have the following structure, whether report or command. */
typedef struct {
	unsigned short 
		counter, 	/* counter */
		len;		/* size of buf; < MAX_RPTBUF unsigned chars */
	unsigned char
		status,		/* TSIP packet format/parse status */
		code,		/* TSIP code */
		buf[MAX_RPTBUF];/* report or command string */
} TSIPPKT;

extern struct logh *trimble_debug, *trimble_serial;
extern struct logh *trimble_raw, *trimble_clockstats;
extern struct logh *trimble_pktid, *trimble_inith;

/* trimble_serial.c */
conffunc_t trimble_serconf;
void trimble_sendmsg(struct instance *ip, TSIPPKT *pkt);
void trimble_sendcode(struct instance *ip, unsigned char code);
void trimble_sendcode_subcode(struct instance *ip, unsigned char code, unsigned char subcode);
void trimble_hexdump(struct logh *loghdl, unsigned char *buffer, long len);

/* trimble_protocol.c */
void trimble_protocol_init(evContext ctx, struct instance *ip);
void trimble_do_protocol(evContext ctx, struct instance *ip, TSIPPKT *pkt);
void trimble_update_timer(evContext ctx, void *uap, struct timespec due, struct timespec inter);

/* trimble_utils.c */
void trimble_bPutShort (short *in, unsigned char *out);
void trimble_bPutULong (unsigned long *in, unsigned char *out);
void trimble_bPutFloat (float *in, unsigned char *out);
void trimble_bPutDouble (double *in, unsigned char *out);
unsigned short trimble_bGetUShort (unsigned char *bp);
long trimble_bGetLong (unsigned char *bp);
unsigned long trimble_bGetULong (unsigned char *bp);
float trimble_bGetSingle (unsigned char *bp);
double trimble_bGetDouble (unsigned char *bp);

#endif
