/*
   trimble_utils.c
   NTPns driver for the Trimble Acutime 2000 GPS Smart Antenna.

   Utility functions for the driver, i.e. convert floating point numbers for
   the message format etc.

   Author: Thomas Wana <tom@ripe.net>

   $Id: trimble_utils.c,v 1.4 2005/01/26 15:28:53 tom Exp $
 */

#include <err.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <termios.h>
#include <pthread.h>
#include <stdint.h>
#include <arpa/inet.h>

#include <ntpns.h>

#include "trimble.h"

/*
 * Definitions
 */

/* we are on little endian */
#define BYTESWAP

/*
 * Byte-reversal is necessary for little-endian (Intel-based) machines.
 * TSIP streams are Big-endian (Motorola-based).
 */
#ifdef BYTESWAP

void trimble_bPutFloat (float *in, unsigned char *out)
{
   unsigned char *inptr;

   inptr = (unsigned char*)in + 3;
   *out++ = *inptr--;
   *out++ = *inptr--;
   *out++ = *inptr--;
   *out = *inptr;
}

void trimble_bPutULong (unsigned long *in, unsigned char *out)
{
   unsigned char *inptr;

   inptr = (unsigned char*)in + 3;
   *out++ = *inptr--;
   *out++ = *inptr--;
   *out++ = *inptr--;
   *out = *inptr;
}

void trimble_bPutDouble (double *in, unsigned char *out)
{
   unsigned char *inptr;

   inptr = (unsigned char*)in + 7;
   *out++ = *inptr--;
   *out++ = *inptr--;
   *out++ = *inptr--;
   *out++ = *inptr--;
   *out++ = *inptr--;
   *out++ = *inptr--;
   *out++ = *inptr--;
   *out = *inptr;
}

#else /* not BYTESWAP */

void trimble_bPutShort (short *a, unsigned char *cmdbuf) {*(short*) cmdbuf = *a;}
void trimble_bPutULong (long *a, unsigned char *cmdbuf)  {*(long*) cmdbuf = *a;}
void trimble_bPutFloat (float *a, unsigned char *cmdbuf) {*(float*) cmdbuf = *a;}
void trimble_bPutDouble (double *a, unsigned char *cmdbuf){*(double*) cmdbuf = *a;}

#endif /* BYTESWAP */


unsigned short trimble_bGetUShort (unsigned char *bp)
{
   unsigned short outval;
   unsigned char *optr;

   optr = (unsigned char*)&outval + 1;
   *optr-- = *bp++;
   *optr = *bp;
   return outval;
}

long trimble_bGetLong (unsigned char *bp)
{
   long outval;
   unsigned char *optr;

   optr = (unsigned char*)&outval + 3;
   *optr-- = *bp++;
   *optr-- = *bp++;
   *optr-- = *bp++;
   *optr = *bp;
   return outval;
}

unsigned long trimble_bGetULong (unsigned char *bp)
{
   unsigned long outval;
   unsigned char *optr;

   optr = (unsigned char*)&outval + 3;
   *optr-- = *bp++;
   *optr-- = *bp++;
   *optr-- = *bp++;
   *optr = *bp;
   return outval;
}

float trimble_bGetSingle (unsigned char *bp)
{
   float outval;
   unsigned char *optr;

   optr = (unsigned char*)&outval + 3;
   *optr-- = *bp++;
   *optr-- = *bp++;
   *optr-- = *bp++;
   *optr = *bp;
   return outval;
}

double trimble_bGetDouble (unsigned char *bp)
{
   double outval;
   unsigned char *optr;

   optr = (unsigned char*)&outval + 7;
   *optr-- = *bp++;
   *optr-- = *bp++;
   *optr-- = *bp++;
   *optr-- = *bp++;
   *optr-- = *bp++;
   *optr-- = *bp++;
   *optr-- = *bp++;
   *optr = *bp;
   return outval;
}

