#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <err.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <netinet/in.h>
#include <math.h>
#include "shmem.h"
#include <sys/endian.h>
#include <isc/eventlib.h>
#include "ntpns.h"
#include "source.h"
#include "../ntpv4/ntpv4.h"

static int shmem_fd = -1;
static struct stat shmem_stat;
static void *shmem_map;
static struct ntpns_id *shmem_id;

#define ROW 8

static int
fp_comp(const void  *p1, const void *p2)
{
	double a, b;

	a = *(double *)p1;
	b = *(double *)p2;
	if (a > b)
		return 1;
	if (b > a)
		return -1;
	return 0;
}

static void
DoSource(struct source *sp, const char *file)
{
	int i, j, n;
	double y, sy, syy, b, c;
	double a[NTIMELINE];

	if (*sp->name != '1')
		return;
	printf("%s Source name: %-32s", file, sp->name);
	printf("%d %10.6e", sp->t[ROW].n, sp->t[ROW].s / sp->t[ROW].n);
	j = sy = syy = n = 0;
	for(i = 0; i < NTIMELINE; i++) {
		if (!sp->t[ROW].v[i])
			continue;
		y = sp->t[ROW].d[i];
		a[j++] = y;
		sy += y;
		syy += y * y;
		n++;
	}
	qsort(a, n, sizeof *a, fp_comp);

	if (0) {
		for(i = 0; i < n; i++) 
			printf(" %10.2e", a[i]);
	}
	b = fabs(a[0] - a[4]);
	c = fabs(a[3] - a[7]);
	if (b > c)
		printf(" n %d med %10.6e", n, a[4]);
	else
		printf(" n %d med %10.6e", n, a[3]);
	printf(" stddev: %10.6e", sqrt((syy - sy * sy / n) / (n - 1.0)));
	printf("\n");
}

static void
WalkShmem(const char *file)
{
	struct shmem *sp;
	u_char *shp;
	int i;

	shmem_id = NULL;
	shp = shmem_map;
	while (shp < (u_char *)shmem_map + shmem_stat.st_size) {
		sp = (void*)shp;
		if (0) {
			for (i = 0; i < 32; i++)
				printf("%02x", shp[i]);
			printf("\n");
			printf("%p TAG: %x length %d\n", sp, sp->tag, sp->length);
		}
		if (sp->tag == ID_TAG)
			shmem_id = (void *)(shp + sizeof *sp);
		if (sp->tag == SOURCE_TAG)
			DoSource((void*)(shp + sizeof *sp), file);
		shp += sp->length;
		if (sp->length == 0)
			break;
	}
}

static int
CheckShmem(const char *file)
{
	struct stat s1;
	static time_t t1, t2;

	/* If we cannot stat the shmem file, bail */
	if (stat(file, &s1) < 0) {
		warn("stat(%s)", file);
		if (shmem_fd >= 0)
			close(shmem_fd);
		shmem_fd = -1;
		return (-1);
	}

	/* Check that the shmem file have not changed under us */
	if (shmem_fd >= 0) {
		if (shmem_stat.st_dev != s1.st_dev ||
		    shmem_stat.st_ino != s1.st_ino ||
		    shmem_stat.st_size != s1.st_size) {
			close(shmem_fd);
			shmem_fd = -1;
		}
	}

	/* Open and map file */
	if (shmem_fd < 0) {
		shmem_fd = open(file, O_RDONLY);
		if (shmem_fd < 0) {
			warn("open %s", file);
			return (-1);
		}
		fstat(shmem_fd, &shmem_stat);
		shmem_map = mmap(shmem_map, shmem_stat.st_size, PROT_READ,
		    MAP_PRIVATE, shmem_fd, 0);
		if (shmem_map == MAP_FAILED) {
			warn("mmap %s", file);
			close(shmem_fd);
			shmem_fd = -1;
			return (-1);
		}

		/* Look for ID block */
		WalkShmem(file);
		time(&t1);
		if (shmem_id == NULL) {
			warn("no ID in shmem");
			close(shmem_fd);
			shmem_fd = -1;
			return (-1);
		}
	}

#if 0
	/* Check magic */
	if (strcmp(shmem_id->magic, MAGIC)) {
		warnx("Wrong magic");
		close(shmem_fd);
		shmem_fd = -1;
		return (-1);
	}
	/* Check that NTPns is still running */
	if (kill(shmem_id->pid, 0)) {
		warnx("could not ping with signal");
		close(shmem_fd);
		shmem_fd = -1;
		return (-1);
	}

	time(&t2);
	if (t2 > t1 + 10) {
		WalkShmem();
		t1 = t2;
	}
#endif
	

	/* Be happy */
	return (0);
}

int
main(int argc, char **argv)
{
	int i;

	for (i = 1; i < argc; i++)
		CheckShmem(argv[i]);
}
