#!/usr/bin/perl

use Math::Round qw(:all);

#$data_area = $ENV{"MCASTROOT"}."/data";
#$bin_area = $ENV{"MCASTROOT"}."/bin";

$data_area = "/home/franzsch/mcast-central/data/in";
$data_area_processed = "/home/franzsch/mcast-central/data/out";
$bin_area = "/home/franzsch/mcast-central/bin";

# First we join the files per group an hostname
%grouphost;
opendir (DATADIR, $data_area) || die "Cannot open $data_area";
foreach $filename (readdir (DATADIR)) {
   if( $filename =~ /^MTRC\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.(.*)\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.(\d*)$/ ) {
      # We have one of the relevant files
      $group = $1;
      $hostname = $2;
      $source = $3;
      $timestamp = $4;

      push @{ $grouphost{$group.".".$hostname.".".$timestamp} }, $filename;
   }
}
closedir(DATADIR);
foreach $key (keys %grouphost) {
   $filelist = "";
   foreach $filename (@{$grouphost{$key}}) {
      $filelist .= "$data_area/$filename ";
   }
   # Join the graph files
   `$bin_area/ccat $filelist > $data_area/MTRC.$key`;
   # And remove the old graph files
   `rm $filelist`;
   # And remove all duplicate entries edges from the file we just made
   #`mv $data_area/MTRC.$key $data_area/tmp`;
   #`$bin_area/rmdupes.pl $data_area/tmp > $data_area/MTRC.$key`;
   #`rm $data_area/tmp`;
}
# And now join the files per group
%group;
opendir (DATADIR, $data_area) || die "Cannot open $data_area";
foreach $filename (readdir (DATADIR)) {
   if( $filename =~ /^MTRC\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.(.*)\.(\d*)$/ ) {
      # We have one of the relevant files
      $group = $1;
      $hostname = $2;
      $timestamp = $3;

      push @timestamps, $timestamp;

      push @{ $group{$group.".".$timestamp} }, $filename;
   }
}
closedir(DATADIR);

# We will do a K-Means clustering algorithm here to get similar timestamps together
@timestamps = sort @timestamps;
$min = $timestamps[0];
$max = $timestamps[scalar(@timestamps)-1];
$clusNum = int(($max-$min)/900) + 1; # The estiminate number of clusters we will need
$next = $min;
# Generate our first estiminate set of clusters
for($i=0; $i<$clusNum; $i++) {
   push @clusters, $next;
   $next = $next + 900;
}
do {
   print "Next Iteration\n";
   %newGroup = ( );
   # First check where each group belongs to
   foreach $key (keys %group) {
      if( $key =~ /^(\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3})\.(\d*)\.?(\d*)?$/ ) {
         $group = $1;
         $ts = $2;
         # Lets see in which cluster it fits.
         $fit = -1;
         $distance = 9999999;
         $count = 0;
         foreach $cluster (@clusters) {
            if( abs($cluster-$ts) < $distance ) {
               $distance = abs($cluster-$ts);
               $fit = $count;
            }
            $count++;
         }
         #print localtime($ts)." fits in cluster ".$fit."\n";
         @{ $newGroup{$group.".".$ts.".".$fit} } = @{ $group{$key} };
      }
   }
   # Then recalculate the cluster mean
   $changed = 0;
   for($i = 0; $i < $clusNum; $i++) {
      $sum = 0;
      $count = 0;
      foreach $key (keys %newGroup) {
         if( $key =~ /^(\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3})\.(\d*)\.(\d*)$/ ) {
            $group = $1;
            $ts = $2;
            $fit = $3;
            if( $fit == $i ) {
               $sum += $ts;
               $count++;
            }
         }
      }
      if( $count > 0 ) {
        if( $clusters[$i] != int($sum/$count) ) {
            $clusters[$i] = int($sum/$count);
            $changed = 1;
         }
      }
   }
   %group = %newGroup;
} while( $changed == 1 );

foreach $key (keys %group) {
   if( $key =~ /^(\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3})\.(\d*)\.(\d*)$/ ) {
      $grp = $1;
      $ts = $2;
      $fit = $3;
      @{ $newGroup{$grp.".".$clusters[$fit]} } = @{ $group{$key} };
      foreach $filename (@{ $group{$key} }) {
         if( $filename =~ /^MTRC.(\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3})\.(.*)\.(\d*)$/ ) {
            # Round the timestamp to the next 10's
            # This is to make sure that if we get a file too late it will still be grouped to the same timestamp cluster.
            # Of course this is asuming that the file has been generated within 5 seconds of the predicted time. But this should
            # be okay, since hardly ever a file is only a second off.
            $clusters[$fit] = nearest( 10, $clusters[$fit] );
            # Then rename the file to our new filename if needed.
            $newFilename = "MTRC.".$1.".".$2.".".$clusters[$fit];
            if( $filename ne $newFilename ) {
               `mv $data_area/$filename $data_area/$newFilename`;
            }
            $filename = $newFilename;
         }
      }
   }
}

#Finally move all the processed data over to the webserver directory
`mv $data_area/* $data_area_processed/`;

