#!/usr/local/bin/perl -w

use POSIX;
use IO::Socket;
use Net::hostent;
use Time::Local;

#------------------------------------------------------------------------------
# Global parameters / enviroment variables
#------------------------------------------------------------------------------

my $mcastroot = "/home/franzsch/mcast-central";

my $data_area = $mcastroot."/data/rrd";
my $data_area_cust = $mcastroot."/data/rrd/cust";
my $bin_area = $mcastroot."/bin";
my $img_area = $mcastroot."/data/img";
my $rrdtool = "/usr/local/bin/rrdtool";

my $progname = "plotd.pl";

my $debug = 0;
my $service = 'plotd';

my $server = IO::Socket::INET->new( Proto  => 'tcp',
   LocalPort => 4261,
   Listen    => 10,	# max connections
   Reuse     => 1);

die "can't setup server: $!" unless $server;
print STDERR "[Server $progname accepting clients]\n";

# become daemon

if (my $pid = fork) {
   exit(0);
}

POSIX:setsid() or die "setsid(): Can't start a new session";

close STDIN;
close STDOUT;
open(STDERR, ">/tmp/podd.errlog");   # per machine

$SIG{'CHLD'} = sub {wait};

while () {
   if (my $socket = $server->accept()) {
      if (my $pid = fork()) {
         # the parent continues listening
         close($socket);
         next;
      }
      elsif (defined $pid) {  
         # $pid is zero here if defined; this is the child process.
         STDOUT->fdopen($socket, "w") or die "Can't reopen STDOUT on socket: $!";
         STDERR->fdopen($socket, "w") or die "Can't reopen STDERR on socket: $!";
         STDIN->fdopen($socket, "r") or die "Can't reopen STDIN on socket: $!";

         handle_request($socket);
      }
      else {
         print STDERR "$progname: cannot fork!? \n";
      }
   }
   else {
      sleep(1); # prevent runaway process monopolizing CPU
   }
}


#------------------------------------------------------------------------------
# Subroutine Header
#
# Purpose:	handle a plots-on-demand request
# Input:	socket file descriptor
# Output:	on socket: gif or ps image if succesfull, else error message
#------------------------------------------------------------------------------

sub handle_request {
   my ($socket) = @_;

   $socket->autoflush(1);

   $dest = "";

   while( <$socket> ) {
      chomp;
      if (("$_" !~ /^[a-zA-Z0-9_:\.\s\n]+$/) || ("$_" =~ /-o\s/) || "$_" =~ /^quit/) {
         # Someone is trying do hack us or they just don't want to play anymore...
         exit 0;
      }

      if( $_ =~ /^list/ || $_ =~ /^getImage/ || $_ =~ /^selDest/ ) {
         # Some sort of listing has to be done, check what files we got
         @files = ();
         %groups = ();

         if( $_ =~ /^selDest (tt\d{1,3})$/ ) {
            # Selecting destination for the next command
            $dest = $1;
            next;
         } elsif( $_ eq "listDest" ) {
            opendir( RRDDIR, "$data_area" );
            foreach $filename (readdir (RRDDIR)) {
               if( $filename =~ /^tt\d{1,3}$/ ) {
                  print $socket "$filename\n";
               }
            }
            print $socket "end\n";
            close $socket;
            exit;
         }

         # First the TTM rrds
         opendir( RRDDIR, "$data_area/$dest" );
         foreach $filename (readdir (RRDDIR)) {
            if( $filename =~ /([a-z]+\.)?(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.rrd/ ) {
               $lastInsert = `$rrdtool last $data_area/$dest/$filename`;
               chomp $lastInsert;
               if( time - $lastInsert < 604800 ) { # Only show it if the last insert was within 7 days
                  # Add to our files array and the groups hash
                  push @files, $filename;
                  $groups{$2} = 1;
               }
            }
         }
         closedir RRDDIR;

         # Then the custom listener rrds
         opendir( RRDDIR, "$data_area/$dest/cust" );
         foreach $filename (readdir (RRDDIR)) {
            if( $filename =~ /[a-z]+\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.\d+\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\.)?rrd/ ) {
               $lastInsert = `$rrdtool last $data_area/$dest/cust/$filename`;
               chomp $lastInsert;
               if( time - $lastInsert < 604800 ) { # Only show it if the last insert was within 7 days
                  # Add to our files array and the groups hash
                  push @files, "cust/$filename";
                  $groups{$1} = 1;
               }
            }
         }
         closedir RRDDIR;
      }

      if( $_ =~ /^list$/ ) {
         # List all available groups
         foreach $group (sort keys %groups) {
            print $socket "$group\n";
         }
         print $socket "end\n\0";
         close $socket;
         exit;
      }
      elsif( $_ =~ /^list (\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})$/ ) {
         # List all sources for a group
         $group = $1;
         %sources = ();

         foreach $file (@files) {
            if( $file =~ /cust\/[a-z]+\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.\d+\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.rrd/ ) {
               if( $group eq $1 ) {
                  $sources{$2} = 1;
               }
            }
         }

         foreach $source (sort keys %sources) {
            print $socket "$source\n";
         }
         print $socket "end\n";
         close $socket;
         exit;
      }
      elsif( $_ =~ /^listTypes (\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})$/ ) {
         # List types for a group without sources
         $group = $1;
         # Go through all files that we need
         foreach $file (@files) {
            if( $file =~ /(cust\/)?([a-z]+)\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.rrd/ ) {
               if( $group eq $3 ) {
                  # Group matches, print the type
                  print $socket "$2\n";
               }
            } elsif( $file =~ /(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.rrd/ ) {
               if( $group eq $1 ) {
                  # Group matches, we have a delay type
                  print $socket "delay\n";
               }
            }
         }
         print $socket "end\n";
         close $socket;
         exit;
      }
      elsif( $_ =~ /^listTypes (\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}) (\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})$/ ) {
         # List types for a group with source
         $group = $1;
         $source = $2;
         # Go through all files that we need
         foreach $file (@files) {
            if( $file =~ /(cust\/)([a-z]+)\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.\d+\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.rrd$/ ) {
               if( $group eq $3 && $source eq $4) {
                  # Group and source match, print the type
                  print $socket "$2\n";
               }
            }
         }
         print $socket "end\n";
         close $socket;
         exit;
      }
      elsif( $_ =~ /^getImage (\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}) ?(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})? ([a-z]+) ?(\d{8} \d{2}:\d{2})? ?(\d{8} \d{2}:\d{2})?$/ ) {
         $type = $3;
         $group = $1;
         if( defined($2) ) {
            $source = $2;
         } else {
            $source = "";
         }
         if( defined($4) ) {
            $startDate = $4;
         }
         if( defined($5) ) {
            $endDate = $5;
         }

         # We need to generate an image.
         # First we find us the rrd file
         $found = 0;
         foreach $file (@files) {
            if( $file =~ /^cust\/([a-z]+)\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.\d+\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.rrd$/) {
               # We need one of those files with a source
               if( $1 eq $type && $2 eq $group && $3 eq $source ) {
                  # We got our file!!
                  $found = 1;
                  $foundFile = $file;
                  last;
               }
            } elsif( $file =~ /^(cust\/)?([a-z]+)?\.?(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}).rrd$/ ) {
               if( defined($2) && $type eq $2 && $group eq $3 ) {
                  # We got our file!!
                  $found = 1;
                  $foundFile = $file;
                  last;
               } elsif( $type eq "delay" && $group eq $3 ) {
                  # We got our file!!
                  $found = 1;
                  $foundFile = $file;
                  last;
               }
            }
         }
         if( $found == 0 ) {
            # TODO: Return some no data found image
            exit;
         } else {
            # All fine, let's generate our image.
            # Set the unit
            if( $type eq "delay" ) { $unit = "ms"; $prType = "Delay"; }
            elsif( $type eq "loss" ) { $unit = "%"; $prType = "Loss"; }
            elsif( $type eq "dval" ) { $unit = "%"; $prType = "Defined Values"; }
            elsif( $type eq "psize" ) { $unit = "bytes"; $prType = "Packet Size"; }
            elsif( $type eq "rate" ) { $unit = "packets"; $prType = "Rate"; }
            $random = int(rand(1000));
            $imgFile = "$img_area/plotTemp$random.gif"; # Image filename
            $graphCmd  = "$rrdtool graph"; # RRD command for generating a graph
            $graphCmd .= " $imgFile"; # The place to store the temp image
            if( defined($startDate) ) {
               $startDate =~ /(\d{4})(\d{2})(\d{2}) (\d{2}):(\d{2})/;
               $ts = timegm( 0, $5, $4, $3, $2-1, $1 );
               $graphCmd .= " -s \"$ts\"";
               if( defined( $endDate ) ) {
                  $endDate =~ /(\d{4})(\d{2})(\d{2}) (\d{2}):(\d{2})/;
                  $ts = timegm( 0, $5, $4, $3, $2-1, $1 );
                  $graphCmd .= " -e \"$ts\"";
               }
            } else {
               $graphCmd .= " -s now-1day -e now";
            }
            if( defined($source) ) {
               if( defined($startDate) && defined($endDate) ) {
                  $startDate =~ s/(\d{4})(\d{2})(\d{2}) (\d{2}):(\d{2})/$1\/$2\/$3 $4:$5/;
                  $endDate =~ s/(\d{4})(\d{2})(\d{2}) (\d{2}):(\d{2})/$1\/$2\/$3 $4:$5/;
                  $graphCmd .= " --title \"Group: $group, Source: $source, Between $startDate and $endDate\""; # A nice title for the graph
               } else {
                  $graphCmd .= " --title \"Group: $group, Source: $source\""; # A nice title for the graph
               }
            } else {
               if( defined($startDate) && defined($endDate) ) {
                  $startDate =~ s/(\d{4})(\d{2})(\d{2}) (\d{2}):(\d{2})/$1\/$2\/$3 $4:$5/;
                  $endDate =~ s/(\d{4})(\d{2})(\d{2}) (\d{2}):(\d{2})/$1\/$2\/$3 $4:$5/;
                  $graphCmd .= " --title \"Group: $group, Between $startDate and $endDate\""; # A nice title for the graph
               } else {
                  $graphCmd .= " --title \"Group: $group\""; # A nice title for the graph
               }
            }
            $graphCmd .= " -h300 -w650"; # Height and Width
            $graphCmd .= " -l0.0"; # Whatever that may do (???)
            $graphCmd .= " -v \"$prType [$unit]\""; # Caption on the right
            $graphCmd .= " -c CANVAS#E0E0E0"; # Background color


            if( $type eq "loss" || $type eq "dval" ) {
               $graphCmd .= " DEF:val1=$data_area/$dest/$foundFile:loss:AVERAGE";
               $graphCmd .= " CDEF:val3=100,val1,*";
               $graphCmd .= " CDEF:val2=100,val3,-";
               $graphCmd .= " AREA:val2#FF0000:\"Failed \"";
               $graphCmd .= " GPRINT:val2:AVERAGE:\"%.2lf %% \"";
               $graphCmd .= "  -u105";
               $graphCmd .= "  --rigid";
               $graphCmd .= " STACK:val3#00CC00:\"Passed \"";
               $graphCmd .= " GPRINT:val3:AVERAGE:\"%.2lf %% ...\"";
            } else {
               # The DS name is delay also for psize and rate, which is not nice. Came from a mistake in the very beginning.
               $graphCmd .= " DEF:val1=$data_area/$dest/$foundFile:delay:MAX";
               $graphCmd .= " DEF:val2=$data_area/$dest/$foundFile:delay:AVERAGE";
               $graphCmd .= " DEF:val3=$data_area/$dest/$foundFile:delay:MIN";
               $graphCmd .= " AREA:val1#FF8800:\"Spread \"";
               $graphCmd .= " GPRINT:val3:MIN:\"%.2lf ...\"";
               $graphCmd .= " GPRINT:val1:MAX:\"%.2lf $unit \""; # TODO: bytes only for psize, also below
               $graphCmd .= " LINE:val2#0000FF:\"Average \"";
               $graphCmd .= " AREA:val3#E0E0E0";
               $graphCmd .= " GPRINT:val2:AVERAGE:\"%.2lf $unit\"";
            }
            `$graphCmd`;

            open(IMGFILE, "$imgFile");
            while(<IMGFILE>) {
               print $socket $_;
            }
            close IMGFILE;
            close $socket;
            `rm "$imgFile"`;
            exit;
         }
      } else {
         print $_;
      }
   }
}

