#!/usr/bin/perl

use POSIX;
use Time::Local;
use IO::Socket;
use CGI qw(:standard);

&page_header;

# Our CGI query object
$query = new CGI;


if( defined( $query->param('group') ) ) {
   $curGroup = $query->param('group');
}
if( defined( $query>param('dateStr') ) && defined( $query->param('timeStr') ) ) {
   if( $query->param('dateStr') =~ /^(\d{4})(\d{2})(\d{2})$/ ) {
      $year = $1;
      $mon = $2;
      $day = $3;
   } else {
      ($_,$_,$_,$day,$mon,$year,$_,$_,$_) = gmtime(time);
   }
   if( $query->param('timeStr') =~ /^(\d{2}):(\d{2})$/ ) {
      $hour = $1;
      $min = $2;
   } else {
      ($_,$min,$hour,$_,$_,$_,$_,$_,$_) = gmtime(time);
   }
   $current = timelocal(0,$min,$hour,$day,$mon-1,$year);
} elsif( defined( $query->param('time') ) && $query->param('time') =~ /\d+/ ) {
   $current = $query->param('time');
} else {
   $current = time;
}


#
# Front end for showing and merging graph files.
#

$ttroot = "/var/www/pub/projects/ttm/Plots/mcast";
$img_area = $ttroot."/img";


#
# Get a list of files from the server
#

# Open connection to server and send "list" command
my $socket = new IO::Socket::INET(
   Proto  => 'tcp',
   PeerAddr  => 'rimu.ripe.net',
   PeerPort  => '4260'
);
die "can't setup socket: $!" unless $socket;
$socket->autoflush(1);

print $socket "list\n";

%times = ( );
%groups = ( );
while (<$socket>) {
   if( $_ eq "end" ) {
      last;
   }
   if( $_ =~ /^MTRC\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.(.*)\.(\d*)$/ ) {
      chomp;
      if( !defined($curGroup) ) {
         $curGroup = $1;
      }
      $group = $1;
      $dest = $2;
      $timestamp = $3;
      $groups{$group} = 1;
      if( defined( $curGroup ) && $group eq $curGroup ) {
         push @{ $times{$timestamp} }, $_;
      }
   }
}
close $socket;

@groupList = sort keys %groups;

#
# Grab the 5 nearest timestamps
#
@closest = ();
foreach $ts (keys %times) {
   push @closest, $ts;
   # If we have more than 5 elements sort them by the difference to the current time, and pop away the highest.
   if( scalar(@closest) > 1 ) {
      @closest = sort {abs($a-$current) <=> abs($b-$current)} @closest;
      pop @closest;
   }
}

# Resort for output
@closest = sort @closest;
# And set our current time variable to the one in the middle
$current = $closest[0];

      print "<a name=\"graph\"></a>";
if( defined( $query->param('Compare') ) ) {
   print "<center><table border=1 style=\"text-align: left;width: 100%;\"><tr>";
} else {
   print "<center><table border=1 style=\"text-align: left;width: 770px;\"><tr>";
}
print "<tr><td style=\"text-align: center;\">";

# The list to join destinations
if( defined($curGroup) ) {
   print "<b>Group: $curGroup<br>".strftime( "%a %b %e %Y %H:%M ", gmtime($closest[0]) )."</b>";

   print $query->startform('GET');
   %destinations = ();
   @filenames = ();
   $ts = $closest[0];
   foreach $filename (@{ $times{$ts} }) {
      if( $filename =~ /^MTRC\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.(.*)\.(\d*)$/ ) {
         $destinations{$filename} = $2;
         push @filenames, $filename
      }
   }
   $query->param('time', $current);
   $query->param('group', $curGroup);
   print $query->hidden('time');
   print $query->hidden('group');
   print $query->scrolling_list(-name=>"join",
      -values=>\@filenames,
      -size=>5,
      -multiple=>'true',
      -labels=>\%destinations);
   print "<br>\n";
   print $query->checkbox(-name=>"Compact View",
      -value=>"compact"
      -label=>"compact");
   print "<br>\n";
   print $query->submit(-name=>'Submit',
      -value=>'Join Selected');
   print "<br>\n";
   print $query->submit(-name=>'Compare',
      -value=>'Compare with '.strftime( "%H:%M ", gmtime($closest[0]-900) ));
   print $query->endform;
}
print "</td><td>";
print "<p>Select the destinations you would like to see in the output graph. To select multiple destinations holt down the CTRL or SHIFT key. To show
a single graph click the \"Join Selected\" button. If you want to do a side to side comparisson with the previous graph click the \" Compare
with ...\" button.</p>";
print "<p>To navigate through the graph JavaScript has to be enabled in your browser. Use +/- to Zoom In/Out, the square symbol to fit to the window size
   or the 100% symbol to show the graph in its original size. To scroll around click and hold the left mouse button and drag the image around.
Double click will zoom in and center the image to the point under your cursor.</p>";
print "<p>Graphs are generated using the <i>mtrace</i> tool. Each multicast group that is configured for listening on a testbox is traced every
15 minutes. mtrace is called for every source that has been active in the last 30 days. The central processing mechanism at RIPE NCC collects this data
and joins all paths for each destination. If mtrace is unable to get a full path to the source after 5 retries, the path is discarded without further
notice. A path disappearing and reappearing in the next measurement is therefore most likely caused by this,
rather than an actual network problem. For more details you can always refer to the plots generated on the testboxes. All times are displayed in UTC.</p>";
print "</td>";
# We are not yet closing the table, since we need to insert the changes if there are any



#
# See if we have to show an image or join graph files.
#
@tojoin = ();
@tojoinCompare = ();
foreach $par ($query->param('join')) {
   if( $par =~ /^MTRC\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.(.*)\.(\d*)$/ ) {
      $group = $1;
      $dest = $2;
      $timestamp = $3;
      push @tojoin, $par;
      if( defined($query->param('Compare')) ) {
         $timestampCompare = $timestamp - 900;
         push @tojoinCompare, "MTRC.$group.$dest.$timestampCompare";
      }
   }
}

if( defined($query->param('join')) && scalar(@tojoin) > 0 ) {
   # Before connecting to the server generate and image filename
   $rand = int(rand(1000));
   $curTime = gmtime;
   # In case we have to show a single image
   if( !defined($query->param('Compare')) ) {
      # Open connection to server
      my $socket = new IO::Socket::INET(
         Proto  => 'tcp',
         PeerAddr  => 'rimu',
         PeerPort  => '4260'
      );
      die "can't setup socket: $!" unless $socket;
      $socket->autoflush(1);

      # Send join command, send a list of files to join and the end command
      if( defined($query->param('Compact View')) ) {
         print $socket "joinCompact\n";
      } else {
         print $socket "join\n";
      }
      foreach $join (@tojoin) {
         print $socket $join."\n";
      }
      print $socket "end\n";

      # First get a list of changes back
      @legend = ();
      while( <$socket> ) {
         chomp;
         if( $_ eq "next" ) {
            last;
         } else {
            push @legend, $_;
         }
      }

      # And get the filename back...
      $imageFileName = <$socket>;
      close $socket;

      $imageFile = "getImage.cgi?filename=$imageFileName";

      # Since we are showing a single file, there are no changes, we can close the navigation table
      # Show the legend still inside our navigation table.
      print "<td style=\"width: 15em;\">";
      print "<p><i>Legend</i><br><b>Ellipse:</b> Source<br><b>House:</b>Destination<br><b>Diamond</b>: RP/Core Router<br><b>Rectangle:</b> Ordinary Router<br><br>";
      foreach $line (@legend) {
         print $line;
      }
      # Show Related Data
      %destinations = ();
      foreach $par ($query->param('join')) {
         if( $par =~ /^MTRC\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.(.*)\.(\d*)$/ ) {
            $destinations{$2} = 1;
         }
      }
      print "<p><i>Related Data</i><br>";
      foreach $dest (keys %destinations) {
         if( $dest =~ /^([^\.]+)\.ripe\.net$/ ) {
            print "<a href=\"http://www.ripe.net/ttm/Plots/mcast/plots.cgi?dest=$1&group=$curGroup\">Multicast Plots ($1)</a><br>\n";
         }
      }
      print "</tr>";

      # And a final row for the timeform
      print "<tr><td colspan=3 style=\"text-align: center;\">";
      &time_form( $current, $curGroup );
      print "</td></tr>";
      print "</table></center>";

      print "Multicast Group: $group, ".strftime( "%a %b %e %Y %H:%M ", gmtime($timestamp) )."<br>\n";
      print "<a href=\"$imageFile\">Download Image (Use Right Click, Save As...)</a>";
      print '<iframe src="zoomer2.cgi?img='.$imageFile.'" width="100%" height="500px" name="zoomer" scrolling="no"></iframe>';
   } else {
      # Open connection to server
      my $socket = new IO::Socket::INET(
         Proto  => 'tcp',
         PeerAddr  => 'rimu',
         PeerPort  => '4260'
      );
      die "can't setup socket: $!" unless $socket;
      $socket->autoflush(1);

      # Send join command, send a list of files to join and the end command
      if( defined($query->param('Compact View')) ) {
         print $socket "compareCompact\n";
      } else {
         print $socket "compare\n";
      }
      foreach $join (@tojoinCompare) {
         print $socket $join."\n";
      }
      print $socket "next\n";
      foreach $join (@tojoin) {
         print $socket $join."\n";
      }
      print $socket "end\n";

      # First get a list of changes back
      @changes = ();
      while( <$socket> ) {
         chomp;
         if( $_ eq "next" ) {
            last;
         } else {
            push @changes, $_;
         }
      }
      # And get the filenames back...
      $imageFileName1 = <$socket>;
      $imageFileName2 = <$socket>;
      close $socket;
      $imageFile1 = "getImage.cgi?filename=$imageFileName1";
      $imageFile2 = "getImage.cgi?filename=$imageFileName2";

      # Show the legend and changes still inside our navigation table.
      print "<td style=\"width: 21em;\">";
      print "<p><i>Legend</i><br><b>Ellipse:</b> Source<br><b>House:</b>Destination<br><b>Diamond</b>: RP/Core Router<br><b>Rectangle:</b> Ordinary Router<br><br>";
      foreach $change (@changes) {
         print $change;
      }
      # Show Related Data
      %destinations = ();
      foreach $par ($query->param('join')) {
         if( $par =~ /^MTRC\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.(.*)\.(\d*)$/ ) {
            $destinations{$2} = 1;
         }
      }
      print "<p><i>Related Data</i><br>";
      foreach $dest (keys %destinations) {
         if( $dest =~ /^([^\.]+)\.ripe\.net$/ ) {
            print "<a href=\"http://www.ripe.net/ttm/Plots/mcast/plots.cgi?dest=$1&group=$curGroup\">Multicast Plots ($1)</a><br>\n";
         }
      }
      print "</td></tr>";
      # And a final row for the timeform
      print "<tr><td colspan=3 style=\"text-align: center;\">";
      &time_form( $current, $curGroup );
      print "</td></tr>";
      print "</table></center>";

      # At last the graphics
      print "<table width=100%><tr><td width=50%>";
      print "Multicast Group: $group, ".strftime( "%a %b %e %Y %H:%M ", gmtime($timestampCompare) )."<br>\n";
      print "<a href=\"$imageFile1\">Download Image (Use Right Click, Save As...)</a>";
      print '<iframe src="zoomer2.cgi?img='.$imageFile1.'" width="100%" height="500px" name="zoomer" scrolling="no"></iframe>';
      print "</td><td width=50% style=\"text-align: right;\">";
      print "Multicast Group: $group, ".strftime( "%a %b %e %Y %H:%M ", gmtime($timestamp) )."<br>\n";
      print "<a href=\"$imageFile2\">Download Image (Use Right Click, Save As...)</a>";
      print '<iframe src="zoomer2.cgi?img='.$imageFile2.'" width="100%" height="500px" name="zoomer" scrolling="no"></iframe>';
      print "</td></tr></table>";
   }
} else {
   # Since we are showing a single file, there are no changes, we can close the navigation table
   print "</tr>";
   # And a final row for the timeform
   print "<tr><td colspan=2 style=\"text-align: center;\">";
   &time_form( $current, $curGroup );
   print "</td></tr>";
   print "</table></center>";
}

&page_footer;

sub page_header {

   print "Content-type: text/html\n\n";
   print <<EOF;

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"><html xmlns="http://www.w3.org/1999/xhtml" lang="en"><head>


<title>Test Traffic Measurements: Multicast Traces</title>

<link rel="SHORTTCUT ICON" href="/favicon.ico" />
<meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1" />
<meta http-equiv="Content-Language" content="en" />
<meta http-equiv="Content-Style-Type" content="text/css" />
<meta http-equiv="Content-Script-Type" content="text/javascript" />
<meta name="Format" content="HTML" />
<meta name="Type" content="Web Page" />
<meta name="Function" content="Home Page" />
<meta name="Availability" content="Public - Members" />
<meta name="Description" content="The RIPE Network Co-ordination Centre (RIPE NCC) is a non profit organisation supporting the RIPE Community. The RIPE NCC is one of four Regional Internet Registries that manage Internet resource allocations such as IP addresses and Autonomous System numbers." />
<meta name="keywords" content="RIPE Network Coordination Centre, RIPE NCC, RIPE, Regional Registry, RIR, Whois Database,Address Supporting Organisation, ASO, IP numbers,Whois Database, RIPE Region Hostcount, RIPE Database, RIPE Whois Database, RIPE Meeting, IPv4, IPv6,IP Address Space,Local IR, RIR, ERX, ENUM, Reverse Delegation, ARIN, APNIC, LACNIC, ICANN, IANA" />

<meta name="author" content="RIPE NCC" />
<meta name="copyright" content="RIPE NCC" />
<meta name="expires" content="never" />
<meta name="revisted-after" content="7 days" />
<meta name="robots" content="all" />
<!--start stylesheet defenitions-->
<link rel="stylesheet" href="/css/ncc-ns4.css" type="text/css" />
<style type="text/css">
<!--
@import url("/css/ripe-ncc.css");
-->
</style>
<!--end of style sheet info-->
<script language="JavaScript1.2" src="/projects/ttm/ttm.js" type="text/javascript"></script>

        </head>

<!--end of head, start of body-->
<body onload="checkLicense();">

<table width="100%" border="0" cellpadding="0" cellspacing="0" class="bg-topnav">
  <tr> 
    <td align="left"><img src="/images/navigation/tabs-ncc-new.gif" alt="" width="443" height="24" border="0" usemap="#maintabs" /></td>
    <td align="right" nowrap="nowrap" ><a href="/info/ncc/about.html" class="topnav">About RIPE NCC</a> | 
   <a href="/info/ncc/contact.html" class="topnav">Contact</a>
   &nbsp;|&nbsp;<a href="/search/index.html" class="topnav">Search</a> |
   <a href="/sitemap.html" class="topnav">Sitemap</a> &nbsp; &nbsp; </td>

  </tr>
</table>
<table width="100%" border="0" cellpadding="0" cellspacing="0" summary="Main content">
  <!--DWLayoutTable-->
  <tr>
    <td colspan="2" rowspan="3" align="left" valign="top"><a href="/" title="Link to the RIPE NCC Homepage"><img src="/images/logo-ncc.gif" alt="Homepage RIPE NCC" title="Homepage RIPE NCC" width="176" height="101" border="0" /></a></td>
    <td width="1" height="62">&nbsp;</td>
    <td colspan="2" valign="top"  class="bg-header"><table width="100%" border="0" cellpadding="2" cellspacing="0" summary="markup table">

        <tr>

        <td width="100%" height="62" valign="bottom">
        <span class="titleheader">Multicast Traces</span></td>

        <td width="231" valign="bottom" nowrap="nowrap" align="right">
</td>
        </tr>
    </table>
    </td>
    <td>&nbsp;</td>

    <td>&nbsp;</td>
  </tr>

  <tr>
    <td height="18"></td>
    <td width="100%"></td>

    <td width="1"></td>
    <td></td>
    <td></td>

  </tr>
<!--create some gutter for breadcrumbs, use middle table-->

<tr>
<td height="21"></td>
<td valign="top" class="breadcrumbs"><img src="/images/spacer.gif" width="1" height="10" alt="Next Section" /><a href="/projects/ttm/index.html">Test Traffic Home Page</a></td>
<td></td>
<td>&nbsp;</td>
<td>&nbsp;</td>
</tr>
<tr>
<td width="33" height="330" valign="top"><img src="/images/spacer.gif" width="33" height="1" alt="Next Section" /></td>
    <td colspan="4" valign="top">
<!--end of left side, start of middle-->
<table width="97%" border="0" cellpadding="0" cellspacing="0" summary="Main content">
  <tr><td width="100%" rowspan="2" valign="top" align="left">

<script language="JavaScript1.2" type="text/javascript">function validate_regex(obj, regex, defval, message) {
        var re = new RegExp(regex);
        if (!re.test(obj.value))
        {
                obj.value=defval;
                alert(message);
                return false;
        }
        return true;
}
</script><script language="JavaScript1.2" type="text/javascript">function validate_range(obj, lowval, hival, defval, message) {
if ((obj.value < lowval) || (obj.value > hival))
        {
                obj.value=defval;
                alert(message);
                return false;
        }
        return true;
}
</script><script language="JavaScript1.2" type="text/javascript">function validate_equality(thelvalue, thevalue, theoperator, defval, message) {
      var code="'" + String(thelvalue) + "' " + String(theoperator) + " '" + String(thevalue) + "'";
        if (!eval(code))
        {
                thelvalue=defval;
                alert(message);
                return false;
        }
        return true;
}
</script>
EOF

}


sub page_footer {
   print <<EOF
</td>

        </tr>
            </table><!--end of related and whois-->
            </td>
                    <td width="16" rowspan="2" valign="top"><img src="/images/spacer.gif" alt="spacer" width="5" height="1" /></td>
                            <td width="1"><img src="/images/spacer.gif" alt="spacer" width="5" height="1" /></td>
                              </tr>
                              <!--end of right side-->

                                    <tr>
                                            <td height="18" valign="top"></td>

                                                    <td width="143" valign="top"></td>
                                                            <td></td>
                                                                    <td></td>
                                                                            <td></td>
                                                                                    <td></td>
                                                                                        </tr>
                                                                                        </table>



                                                                                        <br /> 

                                                                                        <table width="100%" border="0" cellspacing="0" cellpadding="0" summary="footer">
                                                                                          <!--DWLayoutTable-->
                                                                                            <tr> 
                                                                                                <td height="19" colspan="2" valign="top"><hr size="1" noshade="noshade" />
                                                                                                    </td>
                                                                                                      </tr>
                                                                                                        <tr> 
                                                                                                            <td width="33" height="18" valign="top"><img src="/images/spacer.gif" width="33" height="1" alt="spacer" /></td>

                                                                                                                <td width="100%"  align="left" valign="top"> &nbsp;&nbsp;&nbsp;<a href="/info/ncc/index.html" class="footer" title="About the RIPE NCC">About RIPE NCC</a> | 
                                                                                                                      <a href="/sitemap.html" class="footer" title="Sitemap">Site Map</a> | <a href="https://lirportal.ripe.net/" class="footer" title="LIR Portal">LIR Portal</a> | 
                                                                                                                             <a href="/ripe/about.html" class="footer" title="About RIPE">About RIPE</a> 
                                                                                                                                   | <a href="/info/ncc/contact.html" class="footer" title="Contact the RIPE NCC">Contact</a> | 
                                                                                                                                       <span class="footer">©&nbsp;RIPE NCC. All rights reserved.</span> </td>

                                                                                                                                         </tr>
                                                                                                                                           <tr>
                                                                                                                                               <td height="0"></td>
                                                                                                                                                   <td></td>
                                                                                                                                                     </tr>
                                                                                                                                                     </table>

                                                                                                                                                     <map name="maintabs" id="maintabs">
                                                                                                                                                     <area shape="rect" coords="174,7,257,24" href="/index.html" alt="home" />
                                                                                                                                                     <area shape="rect" coords="267,3,351,24" href="https://lirportal.ripe.net/" alt="lirportal"  />
                                                                                                                                                     <area shape="rect" coords="358,4,441,24" href="/ripe/index.html" alt="ripe" />
                                                                                                                                                     </map>


                                                                                                                                                     </body>
                                                                                                                                                     </html>

EOF

}

sub time_form {

   #
   # Print a drop down list of groups that are available.
   # Pass "current" timestamp and current group as parameters
   #
   print $query->start_form('GET');
   print "<i>Select a group:</i> ";
   print $query->hidden('time');
   print $query->popup_menu(-name=>'group',
      -values=>\@groupList,
      -default=>$_[1]
   );
   print $query->submit(-name=>'Select',
                        -value=>'Select');

   #
   # Time form
   #
   if( !defined($query->param('dateStr')) && defined($query->param('time')) ) {
      $dateStr = strftime( "%Y%m%d", gmtime($query->param('time')) );
   } elsif ( !defined($query->param('dateStr')) && !defined($query->param('time')) ) {
      $dateStr = strftime( "%Y%m%d", gmtime($_[0]) );
   }
   if( !defined($query->param('timeStr')) && defined($query->param('time')) ) {
      $timeStr = strftime( "%H:%M", gmtime($query->param('time')) );
   } elsif ( !defined($query->param('timeStr')) && !defined($query->param('time')) ) {
      $timeStr = strftime( "%H:%M", gmtime($_[0]) );
   }

   print $query->hidden('group');
   print " <i>Jump to</i> Date: ";
   print $query->textfield(-name=>'dateStr',
      -default=>$dateStr,
      -size=>8,
      -maxlength=>8);
   print " Time: ";
   print $query->textfield(-name=>'timeStr',
      -default=>$timeStr,
      -size=>5,
      -maxlength=>5);
   print $query->submit(-name=>'Submit',
      -value=>'Jump');
   print $query->endform;
   #print "&nbsp&nbsp&nbsp";
   $now = time;
   $rew = $current - 900;
   $frew = $current - 3600;
   $fwd = $current + 900;
   $ffwd = $current + 3600;
   $getString = "";
   # We have to rewrite the join parameter since its timestamp has to be changed.
   if( defined($query->param('join')) ) {
      $join = $query->param('join');
      $join =~ /^(MTRC\.\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.(.*)\.(\d*)$/;
      $rewJoin = "&join=".$1.".".$2.".".($3-900);
      $frewJoin = "&join=".$1.".".$2.".".($3-3600);
      $fwdJoin = "&join=".$1.".".$2.".".($3+900);
      $ffwdJoin = "&join=".$1.".".$2.".".($3+3600);
   }
   if( defined($query->param('Compact View')) ) {
      $getString .= "&Compact+View=".$query->param('Compact View');
   }
   if( defined($query->param('Compare')) ) {
      $getString .= "&Compare=1";
   }
   print $query->button(-name=>'now',
      -value=>'Now',
      -onClick=>"window.location.href=\"graph.cgi?time=$now&group=$curGroup\"");
   print "&nbsp&nbsp&nbsp";
   print $query->button(-name=>'frew',
      -value=>'<<',
      -onClick=>"window.location.href=\"graph.cgi?time=$frew&group=$curGroup".$getString.$frewJoin."#graph\"");
   print $query->button(-name=>'rew',
      -value=>'<',
      -onClick=>"window.location.href=\"graph.cgi?time=$rew&group=$curGroup".$getString.$rewJoin."#graph\"");
   print $query->button(-name=>'fwd',
      -value=>'>',
      -onClick=>"window.location.href=\"graph.cgi?time=$fwd&group=$curGroup".$getString.$fwdJoin."#graph\"");
   print $query->button(-name=>'ffwd',
      -value=>'>>',
      -onClick=>"window.location.href=\"graph.cgi?time=$ffwd&group=$curGroup".$getString.$ffwdJoin."#graph\"");
   print $query->end_form;
}
