#!/usr/bin/perl

use POSIX;
use Time::Local;
use IO::Socket;
use CGI qw(:standard);

&page_header;

# Our CGI query object
$query = new CGI;

if( defined( $query->param('dest') ) ) {
   $curDest = $query->param('dest');
}
if( defined( $query->param('group') ) ) {
   $curGroup = $query->param('group');
}
if( defined( $query->param('source') ) ) {
   $curSource = $query->param('source');
}
if( defined( $query->param('type') ) ) {
   $curType = $query->param('type');
}

if( defined( $query>param('stDateStr') ) && defined( $query->param('stTimeStr') ) ) {
   if( $query->param('stDateStr') =~ /^(\d{4})(\d{2})(\d{2})$/ ) {
      $year = $1;
      $mon = $2;
      $day = $3;
   } else {
      ($_,$_,$_,$day,$mon,$year,$_,$_,$_) = gmtime(time);
   }
   if( $query->param('stTimeStr') =~ /^(\d{2}):(\d{2})$/ ) {
      $hour = $1;
      $min = $2;
   } else {
      ($_,$min,$hour,$_,$_,$_,$_,$_,$_) = gmtime(time);
   }
}
if( defined( $query>param('enDateStr') ) && defined( $query->param('enTimeStr') ) ) {
   if( $query->param('enDateStr') =~ /^(\d{4})(\d{2})(\d{2})$/ ) {
      $year = $1;
      $mon = $2;
      $day = $3;
   } else {
      ($_,$_,$_,$day,$mon,$year,$_,$_,$_) = gmtime(time);
   }
   if( $query->param('enTimeStr') =~ /^(\d{2}):(\d{2})$/ ) {
      $hour = $1;
      $min = $2;
   } else {
      ($_,$min,$hour,$_,$_,$_,$_,$_,$_) = gmtime(time);
   }
}

#
# Front end for showing and merging graph files.
#

$ttroot = "/var/www/pub/projects/ttm/Plots/mcast";
$img_area = $ttroot."/img";


#
# Get a list of files from the server
#

sub listCommand {
   $command = @_[0];
   if( defined( @_[1] ) ) {
      $dest = @_[1];
   }
   # Open connection to server and send "list" command
   my $socket = new IO::Socket::INET(
      Proto  => 'tcp',
      PeerAddr  => 'rimu.ripe.net',
      PeerPort  => '4261'
   );
   die "can't setup socket: $!" unless $socket;
   $socket->autoflush(1);

   my @output = ();

   if(defined($dest)) {
      # We have a destination
      print $socket "selDest $dest\n";
   }
   print $socket "$command\n";
   while( <$socket> ) {
      chomp;
      if( $_ eq "end" ) {
         last;
      }
      push @output, $_;
   }
   close $socket;
   return @output;
}

%times = ( );
%groups = ( );


@destList = &listCommand("listDest");
print "<center>";
print $query->start_form('GET');
print "<b>Select a Destination:</b> ";
print $query->popup_menu(-name=>'dest',
   -values=>\@destList,
);
print $query->submit(-name=>'Select',
   -value=>'Select');
print $query->end_form;
print "<br>\n";
if( defined( $curDest ) ) {
   # Select our destination and move on
   @groupList = &listCommand("list", $curDest);
   print $query->start_form('GET');
   print $query->hidden('dest');
   print "<b>Select a Group:</b> ";
   print $query->popup_menu(-name=>'group',
      -values=>\@groupList,
   );
   print $query->submit(-name=>'Select',
      -value=>'Select');
   print $query->end_form;
   print "<br>\n";
   if( defined( $curGroup ) ) {
      @sourceList = &listCommand("list $curGroup", $curDest);
      if( scalar(@sourceList) > 0 ) {
         print $query->start_form('GET');
         print $query->hidden('dest');
         print $query->hidden('group');
         print "<b>Select a Source:</b> ";
         print $query->popup_menu(-name=>'source',
            -values=>\@sourceList,
         );
         print $query->submit(-name=>'Select',
            -value=>'Select');
         print $query->end_form;
      }
      if( defined( $curSource ) ) {
         @typeList = &listCommand("listTypes $curGroup $curSource", $curDest);
      } else {
         @typeList = &listCommand("listTypes $curGroup", $curDest);
      }


      if( scalar(@typeList) > 0 ) {
         print $query->start_form('GET');
         print $query->hidden('dest');
         print $query->hidden('group');
         print $query->hidden('source');
         print "<br><b>Select a Type:</b> ";
         print $query->popup_menu(-name=>'type',
            -values=>\@typeList,
         );
         print $query->submit(-name=>'Select',
            -value=>'Select');
         print $query->end_form;
      }


      if( defined( $curType ) ) {
         # Time Form
         if ( !defined($query->param('stDateStr')) ) {
            $stDateStr = strftime( "%Y%m%d", gmtime(time-604800) );
         } else {
            $stDateStr = $query->param('stDateStr');
         }
         if ( !defined($query->param('enDateStr')) ) {
            $enDateStr = strftime( "%Y%m%d", gmtime() );
         } else {
            $enDateStr = $query->param('enDateStr');
         }
         if( !defined($query->param('stTimeStr')) ) {
            $stTimeStr = strftime( "%H:%M", gmtime(time-604800) ); 
         } else {
            $stTimeStr = $query->param('stTimeStr');
         }
         if( !defined($query->param('enTimeStr')) ) {
            $enTimeStr = strftime( "%H:%M", gmtime() );
         } else {
            $enTimeStr = $query->param('enTimeStr');
         }

         print "<br>";
         print $query->start_form('GET');
         print $query->hidden('dest');
         print $query->hidden('group');
         print $query->hidden('source');
         print $query->hidden('type');
         print " <i>Start</i> Date: ";
         print $query->textfield(-name=>'stDateStr',
            -default=>$stDateStr,
            -size=>8,
            -maxlength=>8);
         print " Time: ";
         print $query->textfield(-name=>'stTimeStr',
            -default=>$stTimeStr,
            -size=>5,
            -maxlength=>5);
         print "<br>";
         print " <i>End</i> Date: ";
         print $query->textfield(-name=>'enDateStr',
            -default=>$enDateStr,
            -size=>8,
            -maxlength=>8);
         print " Time: ";
         print $query->textfield(-name=>'enTimeStr',
            -default=>$enTimeStr,
            -size=>5,
            -maxlength=>5);

         print $query->submit(-name=>'Select',
            -value=>'Select');
         print $query->end_form();
      }
   }
}

if( defined($enDateStr) && defined($enTimeStr) ) {
   print "<br>Related Data: [<a href=\"graph.cgi?group=$curGroup\">Multicast Traceroutes</a>] [<a href=\"../../plots-on-demand?SRC=$curDest\">Unicast Measurements</a>]<br>";
}


if( defined($curDest) && defined($curGroup) && defined($curSource) && defined($curType) ) {
   if( defined($stDateStr) && defined($stTimeStr) && defined($enDateStr) && defined($enTimeStr) ) {
      print "<img src=\"getPlot.cgi?dest=$curDest&group=$curGroup&source=$curSource&type=$curType&start=$stDateStr $stTimeStr&end=$enDateStr $enTimeStr\">";
   } else {
      print "<img src=\"getPlot.cgi?dest=$curDest&group=$curGroup&source=$curSource&type=$curType\">";
   }
} elsif( defined($curDest) && defined($curGroup) && defined($curType) ) {
   if( defined($stDateStr) && defined($stTimeStr) && defined($enDateStr) && defined($enTimeStr) ) {
      print "<img src=\"getPlot.cgi?dest=$curDest&group=$curGroup&type=$curType&start=$stDateStr $stTimeStr&end=$enDateStr $enTimeStr\">";
   } else {
      print "<img src=\"getPlot.cgi?dest=$curDest&group=$curGroup&type=$curType\">";
   }
}
print "</center>";

&page_footer;

sub page_header {

   print "Content-type: text/html\n\n";
   print <<EOF;

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"><html xmlns="http://www.w3.org/1999/xhtml" lang="en"><head>


<title>Test Traffic Measurements: Multicast Plots</title>

<link rel="SHORTTCUT ICON" href="/favicon.ico" />
<meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1" />
<meta http-equiv="Content-Language" content="en" />
<meta http-equiv="Content-Style-Type" content="text/css" />
<meta http-equiv="Content-Script-Type" content="text/javascript" />
<meta name="Format" content="HTML" />
<meta name="Type" content="Web Page" />
<meta name="Function" content="Home Page" />
<meta name="Availability" content="Public - Members" />
<meta name="Description" content="The RIPE Network Co-ordination Centre (RIPE NCC) is a non profit organisation supporting the RIPE Community. The RIPE NCC is one of four Regional Internet Registries that manage Internet resource allocations such as IP addresses and Autonomous System numbers." />
<meta name="keywords" content="RIPE Network Coordination Centre, RIPE NCC, RIPE, Regional Registry, RIR, Whois Database,Address Supporting Organisation, ASO, IP numbers,Whois Database, RIPE Region Hostcount, RIPE Database, RIPE Whois Database, RIPE Meeting, IPv4, IPv6,IP Address Space,Local IR, RIR, ERX, ENUM, Reverse Delegation, ARIN, APNIC, LACNIC, ICANN, IANA" />

<meta name="author" content="RIPE NCC" />
<meta name="copyright" content="RIPE NCC" />
<meta name="expires" content="never" />
<meta name="revisted-after" content="7 days" />
<meta name="robots" content="all" />
<!--start stylesheet defenitions-->
<link rel="stylesheet" href="/css/ncc-ns4.css" type="text/css" />
<style type="text/css">
<!--
@import url("/css/ripe-ncc.css");
-->
</style>
<!--end of style sheet info-->
<script language="JavaScript1.2" src="/projects/ttm/ttm.js" type="text/javascript"></script>

        </head>

<!--end of head, start of body-->
<body onload="checkLicense();">

<table width="100%" border="0" cellpadding="0" cellspacing="0" class="bg-topnav">
  <tr> 
    <td align="left"><img src="/images/navigation/tabs-ncc-new.gif" alt="" width="443" height="24" border="0" usemap="#maintabs" /></td>
    <td align="right" nowrap="nowrap" ><a href="/info/ncc/about.html" class="topnav">About RIPE NCC</a> | 
   <a href="/info/ncc/contact.html" class="topnav">Contact</a>
   &nbsp;|&nbsp;<a href="/search/index.html" class="topnav">Search</a> |
   <a href="/sitemap.html" class="topnav">Sitemap</a> &nbsp; &nbsp; </td>

  </tr>
</table>
<table width="100%" border="0" cellpadding="0" cellspacing="0" summary="Main content">
  <!--DWLayoutTable-->
  <tr>
    <td colspan="2" rowspan="3" align="left" valign="top"><a href="/" title="Link to the RIPE NCC Homepage"><img src="/images/logo-ncc.gif" alt="Homepage RIPE NCC" title="Homepage RIPE NCC" width="176" height="101" border="0" /></a></td>
    <td width="1" height="62">&nbsp;</td>
    <td colspan="2" valign="top"  class="bg-header"><table width="100%" border="0" cellpadding="2" cellspacing="0" summary="markup table">

        <tr>

        <td width="100%" height="62" valign="bottom">
        <span class="titleheader">Multicast Plots</span></td>

        <td width="231" valign="bottom" nowrap="nowrap" align="right">
</td>
        </tr>
    </table>
    </td>
    <td>&nbsp;</td>

    <td>&nbsp;</td>
  </tr>

  <tr>
    <td height="18"></td>
    <td width="100%"></td>

    <td width="1"></td>
    <td></td>
    <td></td>

  </tr>
<!--create some gutter for breadcrumbs, use middle table-->

<tr>
<td height="21"></td>
<td valign="top" class="breadcrumbs"><img src="/images/spacer.gif" width="1" height="10" alt="Next Section" /><a href="/projects/ttm/index.html">Test Traffic Home Page</a></td>
<td></td>
<td>&nbsp;</td>
<td>&nbsp;</td>
</tr>
<tr>
<td width="33" height="330" valign="top"><img src="/images/spacer.gif" width="33" height="1" alt="Next Section" /></td>
    <td colspan="4" valign="top">
<!--end of left side, start of middle-->
<table width="97%" border="0" cellpadding="0" cellspacing="0" summary="Main content">
  <tr><td width="100%" rowspan="2" valign="top" align="left">

<script language="JavaScript1.2" type="text/javascript">function validate_regex(obj, regex, defval, message) {
        var re = new RegExp(regex);
        if (!re.test(obj.value))
        {
                obj.value=defval;
                alert(message);
                return false;
        }
        return true;
}
</script><script language="JavaScript1.2" type="text/javascript">function validate_range(obj, lowval, hival, defval, message) {
if ((obj.value < lowval) || (obj.value > hival))
        {
                obj.value=defval;
                alert(message);
                return false;
        }
        return true;
}
</script><script language="JavaScript1.2" type="text/javascript">function validate_equality(thelvalue, thevalue, theoperator, defval, message) {
      var code="'" + String(thelvalue) + "' " + String(theoperator) + " '" + String(thevalue) + "'";
        if (!eval(code))
        {
                thelvalue=defval;
                alert(message);
                return false;
        }
        return true;
}
</script>
EOF

}


sub page_footer {
   print <<EOF
</td>

        </tr>
            </table><!--end of related and whois-->
            </td>
                    <td width="16" rowspan="2" valign="top"><img src="/images/spacer.gif" alt="spacer" width="5" height="1" /></td>
                            <td width="1"><img src="/images/spacer.gif" alt="spacer" width="5" height="1" /></td>
                              </tr>
                              <!--end of right side-->

                                    <tr>
                                            <td height="18" valign="top"></td>

                                                    <td width="143" valign="top"></td>
                                                            <td></td>
                                                                    <td></td>
                                                                            <td></td>
                                                                                    <td></td>
                                                                                        </tr>
                                                                                        </table>



                                                                                        <br /> 

                                                                                        <table width="100%" border="0" cellspacing="0" cellpadding="0" summary="footer">
                                                                                          <!--DWLayoutTable-->
                                                                                            <tr> 
                                                                                                <td height="19" colspan="2" valign="top"><hr size="1" noshade="noshade" />
                                                                                                    </td>
                                                                                                      </tr>
                                                                                                        <tr> 
                                                                                                            <td width="33" height="18" valign="top"><img src="/images/spacer.gif" width="33" height="1" alt="spacer" /></td>

                                                                                                                <td width="100%"  align="left" valign="top"> &nbsp;&nbsp;&nbsp;<a href="/info/ncc/index.html" class="footer" title="About the RIPE NCC">About RIPE NCC</a> | 
                                                                                                                      <a href="/sitemap.html" class="footer" title="Sitemap">Site Map</a> | <a href="https://lirportal.ripe.net/" class="footer" title="LIR Portal">LIR Portal</a> | 
                                                                                                                             <a href="/ripe/about.html" class="footer" title="About RIPE">About RIPE</a> 
                                                                                                                                   | <a href="/info/ncc/contact.html" class="footer" title="Contact the RIPE NCC">Contact</a> | 
                                                                                                                                       <span class="footer">©&nbsp;RIPE NCC. All rights reserved.</span> </td>

                                                                                                                                         </tr>
                                                                                                                                           <tr>
                                                                                                                                               <td height="0"></td>
                                                                                                                                                   <td></td>
                                                                                                                                                     </tr>
                                                                                                                                                     </table>

                                                                                                                                                     <map name="maintabs" id="maintabs">
                                                                                                                                                     <area shape="rect" coords="174,7,257,24" href="/index.html" alt="home" />
                                                                                                                                                     <area shape="rect" coords="267,3,351,24" href="https://lirportal.ripe.net/" alt="lirportal"  />
                                                                                                                                                     <area shape="rect" coords="358,4,441,24" href="/ripe/index.html" alt="ripe" />
                                                                                                                                                     </map>


                                                                                                                                                     </body>
                                                                                                                                                     </html>

EOF

}

