# $Id: Makefile.inc,v 1.12 2007/08/08 10:56:22 ruben Exp $

.include "osversion.mk"

# Set default destionation directory. overide in Makefile if you want another
# location, like /usr/local

TTROOT=	/usr/home/ttraffic
DESTDIR?=	${TTROOT}

# Create timestamp and binaries directory for install target here

TODAY!=	date +%Y%m%d
VERSION?=	${TODAY}
BINDIR?=        /SW/${VERSION}

NO_OBJ=		YES
BINOWN?=	ttraffic
BINGRP?=	wheel

WHOAMI!=	whoami

CTAGS?=	ctags

.if ${OSVERSION} < 600000
# NOMAN is depreciated in RELENG_6, but RELENG_4 doesn't know about it
# Define it on pre FreeBSD6
.if defined(NO_MAN)
NOMAN=yes
.endif
# Same for NO_OBJ
.if defined(NO_OBJ)
NOOBJ=yes
.endif
.endif


.if ${OSVERSION} >= 400000

# Level of compiler strictness, 0 to 6. 
# see /usr/share/mk/bsd.sys.mk 
# Don't enable -Werr by default (yet)
# Don't enable on pre FreeBSD4

WARNS?=5
.endif

.if defined(DEBUG)
CFLAGS+=	-g -DDEBUG
.endif

NO_WERROR?=	YES

.if defined(PROG) || defined(SCRIPTS)
# Create installation directory if it doesn't exist yet

beforeinstall:
	@test -d ${DESTDIR}${BINDIR} || ${ECHO} "===> Creating ${DESTDIR}${BINDIR}" && mkdir -p ${DESTDIR}${BINDIR} 

# display instructions

afterinstall:
	@${ECHO} "=============================================================================="
	@${ECHO} "New software has been installed in ${DESTDIR}${BINDIR}"
	@${ECHO}
	@${ECHO} "Place this software in "
	@${ECHO} "cedar:/ncc/ttpro/files/${HOST:S/.ripe.net//g}_generic${DESTDIR}${BINDIR}"
	@${ECHO} "So it will be available on the appropriate platform (proto_testbox_XXX)"
	@${ECHO} "directory and either update cf.Testboxes or cf.Testbox.root"
	@${ECHO} "depending on the value of DESTDIR"
	@${ECHO}
	@${ECHO} "This can be done on cedar as root like this:"
	@${ECHO}
	@${ECHO} "    sudo -u root -s"
	@${ECHO} "    cd /ncc/ttpro/files/${HOST:S/.ripe.net//g}_generic && \\"
	@${ECHO} "    ssh -i ~/.ssh/ssh_rsync_key \\"
	@${ECHO} "    ${HOST:S/.ripe.net//g} tar -cf - ${DESTDIR}${BINDIR} | tar -xpvf -"
	@${ECHO}
	@${ECHO} "To see which prototype is affected, do"
	@${ECHO} "   ls -ld /ncc/ttpro/files/${HOST:S/.ripe.net//g}_generic"
	@${ECHO} "=============================================================================="

# the beforeinstall/realinstall/afterinstall logic is broken on RELENG_2
# The user needs to do a couple of things manually to help make install

.if ${OSVERSION} < 400000

.if !defined(FORCE)
.if !exists(${DESTDIR}${BINDIR})

install:
	@${ECHO} "Installation is broken on RELENG_2 but there is a work around:"
	@${ECHO} "Run"
	@${ECHO} ""
	@${ECHO} "    ${MAKE} -DFORCE beforeinstall"
	@${ECHO} ""
	@${ECHO} "first, and then retry with:"
	@${ECHO} ""
	@${ECHO} "    ${MAKE} install"
.endif
.endif
.elif $(OSVERSION) <= 460000
.if !defined(FORCE) && (${BINOWN} != ${WHOAMI})
install:
	@${ECHO} "Installation is broken on RELENG_4_6 but there is a work around:"
	@${ECHO} "Run"
	@${ECHO} ""
	@${ECHO} "make install BINOWN=`whoami`"
	@${ECHO} ""
	@${ECHO} "or"
	@${ECHO} ""
	@${ECHO} "\"make -DFORCE install\" in order to force the old behavior anyway"
	@${ECHO} ""
	@${ECHO} "FreeBSD 4.6 and earlier don't pick up the BINOWN setting from Makefile.inc"
install:
.endif
.endif

.endif

release:
	@${ECHO} "bumping version to ${TODAY}"
	@${ECHO} "VERSION?=${TODAY}" > ${.CURDIR}/version.mk
	@${ECHO} "Please run \"cvs commit version.mk\" and \"make tag\""

tag: ${SRCS} ${SCRIPTS}
	@${ECHO} "Tagging release to REL_${VERSION}"
	@cvs tag -FR REL_${VERSION} . ${.ALLSRC}
