#!/usr/local/bin/perl5 -w -I/home/ttraffic/lib
# $Id: delayalarm.pl,v 1.24 2004/07/23 10:12:09 ttraffic Exp $
#
# Purpose: this script runs the STA code  and parse the output.
# It then sends mail to test-box hosts.
#
# Put an entry in the log file
#
use LogData;
use ConfigFile;
use Net::Syslog;
use POSIX;

$success = logdata( "$0 started as $$", "/tmp/localsocket", -1 );
if ( $success == 0 ) {

    #print "Debugging: No log file, $0 terminated\n";
    exit;
}

#
# Debug, set $debug to 1 for more output
#
$debug    = 0;
$rrd_area = "/data/rrd";
$whoami   = qx(hostname -s);

chomp($whoami);

# example input
#	TB 64 proto IPv4 at 1078492016 Status change old: No Short Data, new: Alarm
#	TB 64 proto IPv4 at 1078492016 ALARM SET   old: No Short Data, new: Alarm
#	TB 64 proto IPv4 at 1078492016 Long:  9643   3.0/  3.5/  5.0 Short:    13 298.0/298.5/299.0
#

MAIN: {
    @syslog         = ();
    @nonotify       = ();
    @ignore         = ();
    @nonotify6      = ();
    @ignore6        = ();
    @array          = ();
    @array_msg_self = ();
    $destlist       = "";
    $rrdlist        = "";
    $subject        = "";

    # read configuration
    #

    open( DELAYALARMCFG, "$ENV{'TTROOT'}/config/delayalarm.conf" )
      || die "Couldn't open delayalarm.conf: $!";

    while ( defined( my $input = <DELAYALARMCFG> ) ) {
        chomp($input);
        my ( $key, $fields ) = split( /:/, $input, 2 );
        my (@fields) = split( /[,\s]+/, $fields );

      SWITCH: {
            $key eq "syslog" && do {
                foreach my $field (@fields) {
                    if ( defined($field) && $field ) {
                        my ( $host, $port, $facility, $level ) = $field =~ /^(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}|[a-zA-Z]+[\w\-\.]+):(\d+)\/(kern|user|mail|daemon|auth|syslog|lpr|news|uucp|cron|authpriv|ftp|local0|local1|local2|local3|local4|local5|local6)\.(emerg|alert|crit|err|warning|notice|info|debug)$/;

                        push(
                            @syslog,
                            {
                                'host'     => $host,
                                'port'     => $port,
                                'facility' => $facility,
                                'level'    => $level
                            }
                        );
                    }
                }
                last SWITCH;
            };
            $key eq "ignore" && do {
                foreach my $field (@fields) {
                    push( @ignore, $field ) if ( defined($field) && $field );
                }
                last SWITCH;
            };
            $key eq "no-notify" && do {
                foreach my $field (@fields) {
                    push( @nonotify, $field ) if ( defined($field) && $field );
                }
                last SWITCH;
            };
            $key eq "ignore6" && do {
                foreach my $field (@fields) {
                    push( @ignore6, $field ) if ( defined($field) && $field );
                }
                last SWITCH;
            };
            $key eq "no6notify" && do {
                foreach my $field (@fields) {
                    push( @nonotify6, $field ) if ( defined($field) && $field );
                }
                last SWITCH;
            };

        }
    }
    close(DELAYALARMCFG);

    #
    # Run alarm code
    #
    @output = qx(/home/ttraffic/bin/sta);

    #
    # Put data in log file

    foreach $line (@output) { logdata("sta.pl $line"); }

    for ( $i = 0 ; $i < $#output ; $i++ ) {
        my ( $tb, $proto, $timestamp, $message ) = $output[$i] =~ /^TB\s+(\d+[a-f]?)\s+proto\s+(\w+)\s+at\s+(\d+)\s+(.*)$/;

        if (   ( $output[$i] =~ /ALARM SET/ )
            || ( $output[$i] =~ /ALARM CLEARED/ ) )
        {

            # XXX TODO Do not add ignored boxes, but save them somewhere else
            # as other people might be still interrested in them

            push( @array, $output[$i] );
            push( @array, $output[ $i + 1 ] );

            if (
                !(
                       ( $proto eq "IPv4" && grep { /^tt${tb}$/ } @ignore )
                    || ( $proto eq "IPv6" && grep { /^tt${tb}$/ } @ignore6 )
                )
              )
            {
                push( @array_msg_self, $output[$i] );
                push( @array_msg_self, $output[ $i + 1 ] );
            }
        }
    }

    #
    # See if we have to send mail
    #
    if ( $debug == 1 ) {
        printf("Parsing alarm output\n");
    }

    #
    # Extract TB numbers from the output
    #
    for ( $i = 0 ; $i < $#array ; $i += 2 ) {
        ( $head, $tbno, $head ) = split( /\s+/, $array[$i] );
        if ( $destlist ne "" ) { $destlist .= ", "; }

        $tbname =
          ( $array[$i] =~ /IPv6/ )
          ? sprintf( "tt%02d_IPv6", $tbno )
          : sprintf( "tt%02d",      $tbno );
        $tbaddr =
          ( $array[$i] =~ /IPv6/ )
          ? host_to_ip6($tbname)
          : host_to_ip4($tbname);

        # XXX Do not add boxes to destlist listed in @nonotify/@nonotify6

        if (
            !(
                grep { /^$tbname$/ }
                ( @nonotify, map { "${_}_IPv6" } @nonotify6 )
            )
          )
        {
            $destlist .= $tbname;
        }

        if ( ( $tbaddr ne "UNKNOWN" ) && ( $tbaddr ne "FAILED" ) ) {
            $rrdfile = $rrd_area . "/" . $tbaddr . ".rrd";
            if ( -e $rrdfile ) {
				# XXX FIXME Add IPv6 support here ?
                $rrdlist = $rrdlist . "&" . $tbname . "=" . $tbaddr;
            }
        }

        if ( ( $array[$i] =~ /ALARM SET/ ) ne "" ) {
            if ( $subject eq "" ) { $subject = "Testbox ALARM SET"; }
        }
    }
    if ( $subject eq "" ) { $subject = "Testbox ALARM CLEARED"; }

    #
    # Send mail
    #
    logdata("sta.pl testing destlist = $destlist");
    if ( $destlist ne "" ) {

        # skip
        chomp($destlist);
        logdata("sta.pl final destlist =  $destlist, $whoami");
        mail_a_message( "$destlist, $whoami", $subject, \@array, $rrdlist );
    }

	# XXX TODO tidy up here, we want the owner know what he is missing, more or less.
    # mail_a_message( $whoami, $subject, \@array_msg_self, $rrdlist );

    foreach $syslog (@syslog) {
        syslog_a_message( $syslog, \@array_msg_self );
    }

    #
    # End
    #
    if ( $debug == 1 ) {
        printf("Parsing alarm output DONE\n");
    }
    logdata("$0 finished");

    exit;

}

#
#---------------------------------------------------------------------------
#
sub syslog_a_message($$) {
    my ( $connection, $messages ) = @_;

    my $s = new Net::Syslog(
        SyslogHost => $connection->{'host'},
        SyslogPort => $connection->{'port'},
        Facility   => $connection->{'facility'},
        Priority   => $connection->{'level'}
    );
    foreach $line (@$messages) {
        $s->send( $line, Priority => $connection->{'level'} );
    }
}

sub mail_a_message {
    my ($dest) = $_[0];    # destination
    my ($subj) = $_[1];    # subject
    my ($mess) = $_[2];    # message
    my ($url)  = $_[3];    # url

    my ($host) = qx(hostname);
    chomp($host);

    my ( $sec, $min, $hour, $day, $month, $year ) = gmtime( time - 172800 );
    $stime = ( $year + 1900 ) * 10000 + ( $month + 1 ) * 100 + $day;
    $stime = "$stime%20$hour:$min";

    ( $sec, $min, $hour, $day, $month, $year ) = gmtime();
    my ($filename) = 10000 * ( $year + 1900 ) + 100 * ( $month + 1 ) + $day;
    $filename = "~/data/GENE." . $host . "." . $filename;

    open (MAIL, "|/usr/sbin/sendmail -t") || die "Cannot find mail program";

    print MAIL "From: ttraffic\@" . $host . "\n";
    print MAIL "To: henk\@ripe.net\n";
    print MAIL "Bcc: $dest\n";
    print MAIL "Subject: $subj on $host\n";
    print MAIL "Reply-To: ops-tt\@ripe.net\n";
    print MAIL "Precedence: bulk\n";
    print MAIL "\n";

    print MAIL "The testbox alarm program on $host found:\n\n";
    foreach $line (@$mess) {
        print MAIL $line;
    }
    print MAIL "\n\nThis message has been sent to: $dest\n";

    print MAIL "\n\nSatellite conditions on $host:\n";
    print MAIL `grep "Satellites seen" $filename | tail -1`;

    if ( $url ne "" ) {
        print MAIL "\nTo see how the delays developed in the last days, ";
        $url =
            "http://" . $host
          . ":10259/cgi-bin/multiple.cgi?"
          . $url
          . "&delay=delay&loss=loss&RRD_START="
          . $stime
          . "&RRD_END=start%2B2days";
        print MAIL "open this URL: $url\n\n";
        print MAIL "Access to this page is limited to the owner of $host,\n";
        print MAIL "please contact tt-ops\@ripe.net with any access errors\n\n";
    }

    print MAIL "For an explanation of this email please see ";
    print MAIL "http://www.ripe.net/test-traffic/General/alarm.html\n\n";
    close(MAIL);
}
