#ifndef _ALARM_H_
#define _ALARM_H_
/* $Id: alarm.h,v 1.7 2006/11/04 15:28:53 ruben Exp $ */

/* Assorted constants for the alarms */

/* Number of days in long term history */
#define DAYS_HISTORY       30

/* Number of intervals per day, run in 24/INTERVAL_DAY periods, starting 
   at 0:00  So 4 means 0-6 (=24/4), 6-12, 12-18, 18-24 */
#define INTERVAL_DAY        4

/* Maximum delay considered in ms, everything else is an overflow */
#define MAX_DELAY         300

/* Bin width of the delays in ms */
#define DELAY_BIN_WIDTH     1

/* Maximum number of boxes */
#define MAX_BOXES         100

/* Short term interval length in seconds */
#define SHORT_TERM_INTERVAL 1800

/* Minimum number of points in short and long term interval before this TB
   is considered (to avoid false alarms on only a few points */
#define MIN_POINTS_SHORT      10
#define MIN_POINTS_LONG      100

/* Percentiles for lower, median and upper limit */
#define LOWER_PERCENTILE     0.05
#define MEDIAN_PERCENTILE    0.50
#define HIGHER_PERCENTILE    0.95

#define LOWER_PERCENTILE_BIN     1
#define MEDIAN_PERCENTILE_BIN    2
#define HIGHER_PERCENTILE_BIN    3


/* Alarm codes, see sta.c for an ASCII translation.  These can have any value
   one likes as long as they are unique.  I've selected 0 for no alarm,
   single bits for alarms, all bits set (127) for an unidentified case.  */

#define STA_NOALARM        0
#define STA_ALARM          1
#define STA_NOLONG         2
#define STA_NOSHORT        4
#define STA_SPREAD         8
#define STA_IMPROVED      16
#define STA_GOUP          32
#define STA_GODOWN        64
#define STA_UNDEFINED    127


/* TRUE and FALSE */
#ifndef TRUE
#define TRUE (1)
#define FALSE (0)
#endif

/* Files used by the programs (without $TTROOT/tmp) */

#define FILE_ALARM_CNF "delayalarm.conf"
#define FILE_IN "lta_history.in"
#define FILE_IN6 "lta_history6.in"
#define FILE_OUT "lta_history.out"
#define FILE_OUT6 "lta_history6.out"
#define FILE_STAT "sta_history.status"
#define FILE_STAT6 "sta_history6.status"

#endif /* _ALARM_H_ */
