/*
 * vi:ts=4:sw=4:
 *
 * $Id: getppstime.c,v 1.4 2005/11/01 13:17:34 ruben Exp $
 *
 * Copyright (c) 1998-2003 RIPE NCC
 * 
 * All Rights Reserved
 * 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the author not be used in advertising or 
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.
 * 
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
 * AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
 */


/*

-------------------------------------------------------------------------------
Module Header
Filename          : getppstime.c
Purpose           : Provide getppstime like functionality if getppstime 
		    systemcall is not available.
Author            : Olaf M. Kolkman
Date              : $Date: 2005/11/01 13:17:34 $
Description       : getppstime sets the content of the argument 
		    (struct ppsval *). This argument can be used for 
		    differential correction of time wrt to the PPS.
		    See getppstime.h for the content of the struct

Language Version  : gcc version 2.7.2 
OSs Tested        : BSDI 
Command Line      : None.
Input Files       : None.
Output Files      : None.
External Programs : None
Problems          :
To Do             :
Comments          : This module is here to be able to test Test-Traffic 
		    programs on systems where getppstime is not implemented.
-------------------------------------------------------------------------------

*/

#include <time.h>
#include <string.h>
#include "getppstime.h"

static char const rcsid[] = "$Id: getppstime.c,v 1.4 2005/11/01 13:17:34 ruben Exp $";

int getppstime (struct pps_val *ppv)
{
	struct ntptimeval   now;
	struct timex        adj;

	/* get current time */
	ntp_gettime (&now);
	/* get timex structure */
	bzero (&adj, sizeof (adj));
	ntp_adjtime (&adj);

	/* fill in current time */
#if defined(NTP_API) && NTP_API >= 4

	TIMESPEC_TO_TIMEVAL(&(ppv->current_time),&(now.time));		/* XXX make this
															 * more NANO
															 * friendly */
#else
	ppv->current_time = now.time;
	
#endif /* defined(NTP_API) && NTP_API >= 4 */

	/* Time of the last pps timestamp We don't longer use this, because the PPS 
	 * is now controlled in the kernel with all the necessary filters and no
	 * longer by ntpd 
	 * For now, we just put the usecs to 0, but we might want to remove it
	 * totally 
	 * 
	 * marks@ripe 23-09-2002
	 */

	ppv->last_pps.tv_sec = now.time.tv_sec;
	ppv->last_pps.tv_usec = 0;
	ppv->pps_status = adj.status;
	ppv->pll_offset = adj.offset / 1000;	/* make this more NANO friendly */
	ppv->maxerror = adj.maxerror;
	ppv->esterror = adj.esterror;

	return (0);
}
