/*
 * vi:ts=4:sw=4:
 *
 * $Id: getppstime.h,v 1.5 2003/10/17 10:49:10 ruben Exp $
 *
 * Copyright (c) 1998-2003 RIPE NCC
 * 
 * All Rights Reserved
 * 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the author not be used in advertising or 
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.
 * 
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
 * AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
 */


/*

-------------------------------------------------------------------------------
Module Header
Filename          : getppstime.h
Purpose           : Provide getppstime like functionality if getppstime 
			is not defined
Author            : Olaf M. Kolkman
Date              :
Description       :
Language Version  :
OSs Tested        :
Command Line      :
Input Files       :
Output Files      :
External Programs :
Problems          :
To Do             :
Comments          :
-------------------------------------------------------------------------------

*/



#ifndef _TT_GETPPSTIME_H
#define _TT_GETPPSTIME_H


#ifdef __FreeBSD__
#include <sys/time.h>
#else
#include <time.h>
#endif

#ifdef __FreeBSD__
#include<sys/timex.h>
#endif /* __FreeBSD__ */



/* Only if the syscall is not declared (timex.h)*/

#ifndef _SYS_GETPPSTIME


/* stuct for syscal getppsval. For interaction with time and pps related paramenters. */
struct pps_val
{
	struct timeval      current_time;	/* Current timeofday as reported by
										 * the (NTP driven kernel clock) */
	struct timeval      last_pps;	/* time of the last PPS as set by *
									 * hardpps */
	int                 pps_status;	/* clock status bits (as defined in
									 * timex.h) Most imortant is that 0x0100 is 
									 * set. PPSSIGNAL pps signal present. */
	long                pll_offset;	/* NTPs idea of the offset to the real
									 * time. */
	long                maxerror;	/* PLL error estimates */
	long                esterror;

};

int getppstime (struct pps_val *);

#endif /* _SYS_GETPPSTIME */

#endif /* _TT_GETPPSTIME_H */
