/*
 * vi:ts=4:sw=4:
 *
 * $Id: if_utils.h,v 1.2 2006/06/21 13:25:03 ruben Exp $
 *
 * Copyright (c) 1998-2003 RIPE NCC
 * 
 * All Rights Reserved
 * 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the author not be used in advertising or 
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.
 * 
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
 * AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
 */


#ifndef _TT_IF_UTILS_H
#define _TT_IF_UTILS_H


#include <sys/types.h>
#include <sys/socket.h>

#ifndef _NETINET_IN_H_
#include <netinet/in.h>
#endif

#include <net/if.h>

#ifndef INET_ADDRSTRLEN
#define INET_ADDRSTRLEN		16
#endif /* !INET_ADDRSTRLEN */

#ifdef INET6_ADDRSTRLEN
# define _TTM_ADDRSTRLEN INET6_ADDRSTRLEN
#else
# define _TTM_ADDRSTRLEN INET_ADDRSTRLEN
#endif


struct etherifdata {
  char   ifname[IF_NAMESIZE];        /* The name of the interface */
  struct in_addr ethif_addr; /* The IP number it thinks it has */
  struct in6_addr ethif_addr6;
  struct if_data if_data;  /* See net/if.h for details...*/
  char	 ifaddr[_TTM_ADDRSTRLEN];
};

/* XXX ethif_addr should be sockaddr and maybe even a union of sockaddr and
 * sockaddr6 
 * Ultimate solution is to store just a char[_TTM_ADDRSTRLEN]
 * and include a portable getnameinfo(3) with the source
 */

int GetEthernetIFInfo(struct etherifdata *, char *);
int NextEthernetIFInfo(struct etherifdata *, char *, int);
void SetEthernetIFInfo(void);
void EndEthernetIFInfo(void);

/* Also return information about valid aliases */
#define ETHIFINFO_ALIASES_TOO	(1 << 0)


#endif /* _TT_IF_UTILS_H */
