#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <sys/param.h>
#include <sys/syslimits.h>

#include "testbox.h"
#include "libdelayalarm.h"


static char const   rcsid[] = "$Id: libdelayalarm.c,v 1.6 2006/06/21 13:25:03 ruben Exp $";

int rcdp_fname_cmp(const void*, const void*);
int int_cmp(const void*, const void*);
void getRCDPDataSlice(const char*, const time_t, const time_t, int, int (*)(char **, const void *), const void*);

int rcdp_fname_cmp(const void *a, const void *b)
{
        const char *stra=(const char *)a, *strb=(const char *)b;
        return strcmp(stra,strb);
}

int int_cmp(const void *a, const void *b)
{
	return	*(const int*)a == *(const int*)b ? 0 : *(const int*)a < *(const int*)b ? -1 : 1;
}

int ClocksOK (int SourceClock, int TargetClock)
{
	// report if both clocks are valid or not

	short               SourceOK,
	                    TargetOK;

	SourceOK = ((SourceClock & LEAPBITMASK) == GPSGOODCLOCK1) ||
		((SourceClock & LEAPBITMASK) == GPSGOODCLOCK2) || ((SourceClock & LEAPBITMASK) == CDMAGOODCLOCK);

	TargetOK = ((TargetClock & LEAPBITMASK) == GPSGOODCLOCK1) ||
		((TargetClock & LEAPBITMASK) == GPSGOODCLOCK2) || ((TargetClock & LEAPBITMASK) == CDMAGOODCLOCK);

	return (SourceOK && TargetOK);
}

int AnyClockOK (int SourceClock, int TargetClock)
{
	// report if any of the clocks is valid or not

	short               SourceOK,
	                    TargetOK;

	SourceOK = ((SourceClock & LEAPBITMASK) == GPSGOODCLOCK1) ||
		((SourceClock & LEAPBITMASK) == GPSGOODCLOCK2) || ((SourceClock & LEAPBITMASK) == CDMAGOODCLOCK);

	TargetOK = ((TargetClock & LEAPBITMASK) == GPSGOODCLOCK1) ||
		((TargetClock & LEAPBITMASK) == GPSGOODCLOCK2) || ((TargetClock & LEAPBITMASK) == CDMAGOODCLOCK);

	return (SourceOK || TargetOK);
}

void getRCDPDataSlice(const char* fname, const time_t starttime, const time_t duration, int family, int (*pckt_hook)(char **, const void *), const void *pckt_hook_data)
{
	/* Open file, and read data which is within limits */

	char rcdp_fname[PATH_MAX], rcdp_buffer[1024], *rcdp_buffer_p, *rcdp_fields[RCDP_FMT_MAXFIELDS], **rcdp_fields_p;
	FILE *rcdp_file;

	snprintf(rcdp_fname, PATH_MAX, "%s/%s/%s", getenv("TTROOT"), family == AF_INET ? "data" : "data6", fname); 

	if ((rcdp_file = fopen(rcdp_fname,"r")) != NULL)
	{
		int error=0;
			
		while (!error && (( rcdp_buffer_p = fgets(rcdp_buffer,1024,rcdp_file)) != NULL ))
		{
			for (rcdp_fields_p = rcdp_fields; (*rcdp_fields_p != rcdp_fields[RCDP_FMT_MAXFIELDS+1]) && (*rcdp_fields_p = strsep (&rcdp_buffer_p, " \t")) != NULL;)
			{
				if (**rcdp_fields_p != '\0')
					++rcdp_fields_p;
			}
				
			if ((*pckt_hook)(rcdp_fields,pckt_hook_data) == 0)
				error = 1;
		}

		if(!feof(rcdp_file) && !error)
			perror("fgets");
		
		fclose(rcdp_file);
	}
	else
	{
		perror("fopen");
	}
}

void getRCDPData(const char* fqdn_tbname, const time_t starttime, const time_t duration, int family,  int (*pckt_hook)(char **, const void *), const void *pckt_hook_data)
{
	/* Open directory, and fetch available files for reading, date ordered */
	
	DIR *rcdp_dir;
	char **rcdp_fnames=NULL;
	int rcdp_fnames_cnt=0;
	char pathbuffer[PATH_MAX];
	struct tm starttime_tm, endtime_tm;
	time_t endtime = starttime + duration;

	snprintf(pathbuffer, PATH_MAX, "%s/%s", getenv("TTROOT"), (family == AF_INET ? RCDPV4PATH : RCDPV6PATH)); 

	memcpy(&starttime_tm,gmtime(&starttime), sizeof(struct tm));
	memcpy(&endtime_tm,gmtime(&endtime), sizeof(struct tm));

	if ((rcdp_dir = opendir(pathbuffer)) != NULL)
	{
		struct dirent *thedirentry=NULL;
		while((thedirentry = readdir(rcdp_dir)) != NULL)
		{
			char date_pfx[9], /* YYYYMMDD\0 */
				rcdp_filename[PATH_MAX];

			int year,month,day;

			/* Want to probe for all files which matches the date prefix of the period requested */
			
			for (year=starttime_tm.tm_year; year <= endtime_tm.tm_year; year++)
			{
				for (month= ((year == starttime_tm.tm_year) ? starttime_tm.tm_mon : TM_FIRSTMONTH); month <= ((year == endtime_tm.tm_year) ? endtime_tm.tm_mon : TM_LASTMONTH); month++)
				{
					for (day= ((year == starttime_tm.tm_year && month == starttime_tm.tm_mon) ? starttime_tm.tm_mday : TM_FIRSTDAY); day <= ((year == endtime_tm.tm_year && month == endtime_tm.tm_mon) ? endtime_tm.tm_mday : TM_LASTDAY); day++)
					{
						struct tm current_tm;

						memset(&current_tm,0,sizeof(struct tm));
						
						current_tm.tm_mday = day;
						current_tm.tm_mon = month;
						current_tm.tm_year = year;
						
						strftime(date_pfx,9,"%Y%m%d", &current_tm);

						snprintf(rcdp_filename, FILENAME_MAX, "%s.%s.%s", family == AF_INET ? "RCDP" : "RCDP6", fqdn_tbname, date_pfx);

						/* TODO XXX should use gethostname to compare and select files */
						if (strstr(thedirentry->d_name,rcdp_filename) == thedirentry->d_name)
						{
							/* FOUND */
							char **rcdp_fnames_new;

							if ((rcdp_fnames_new = realloc(rcdp_fnames,(rcdp_fnames_cnt+1) * sizeof(char*))) == NULL)
							{
								perror("malloc");
								abort();
							}
							rcdp_fnames = rcdp_fnames_new;
							rcdp_fnames[rcdp_fnames_cnt] = strdup(thedirentry->d_name);
							rcdp_fnames_cnt++;
						}
					}
				}
			}
		}
		closedir(rcdp_dir);
	}
	else
	{
		printf("Couldn't open %s (%s)\n", pathbuffer,strerror(errno));
	}

	if (rcdp_fnames_cnt)
		qsort(rcdp_fnames,rcdp_fnames_cnt,sizeof(char*),rcdp_fname_cmp);

	if (rcdp_fnames_cnt)
	{
		int i;
		for (i=0; i<rcdp_fnames_cnt;i++)
		{
			getRCDPDataSlice(rcdp_fnames[i], starttime, duration, family, pckt_hook,pckt_hook_data);
		}
	}

	if (rcdp_fnames_cnt)
	{
		int i;
		for (i=0; i<rcdp_fnames_cnt;i++)
			free(rcdp_fnames[i]);
		free(rcdp_fnames);
	}
}


int *create_id2index(int family, size_t *size)
{
	size_t i, c;
	int *id2index, *id2index_uniq, previd;

	i=0;
	c=0;
	previd=-1;
	for (i=0;i < (number_of_boxes );i++)
	{
		if(test_box_list[i].family == family)
		{
			c++;
		}
	}
	*size = c;

	if( (id2index = malloc(*size * sizeof(int))) != NULL)
	{
		c=0;
		for (i=0;i < (number_of_boxes - 1);i++)
		{
			if(test_box_list[i].family == family)
			{
				id2index[c] = test_box_list[i].id;
				c++;
			}
		}
	}

	/* sort so bsearch can be used */

	qsort(id2index,*size,sizeof(int),int_cmp);

	/* 
	 * remove redundant id's due to historical IP addresses 
	 * saves memory in the lta memory resident data structures
	 */

	c=0;
	for (i=0;i < (*size);i++)
	{
		if(id2index[i] !=previd)
		{
			previd=id2index[i];
			c++;
		}
	}
	if( (id2index_uniq = malloc(c * sizeof(int))) != NULL)
	{
		c=0;
		previd=-1;
		for (i=0;i < (*size);i++)
		{
			if(id2index[i] !=previd)
			{
				previd=id2index[i];
				id2index_uniq[c] = id2index[i];
				c++;
			}
		}
		free(id2index);
		id2index = id2index_uniq;
		*size=c;
	}
	return id2index;
}

void free_id2index(int *idx)
{
	free(idx);
	idx=NULL;
}

int lookup_id2index(const int *id2index, int idx, int size)
{
	int *found;

	if ((found = bsearch(&idx, id2index,size,sizeof(int),int_cmp)) != NULL)
		return (found - id2index);
	else
		return -1;
}

static int decode_operator(const char *item, char *tb_name, alarmtype *alarm_type, alarmlevel *op_type, float *alert_value, int debug)
{
	const char *comp_op, *alert_value_str;
	int ret=0;
	if (((comp_op = strpbrk(item,"><!=")) != NULL) && ((alert_value_str = strpbrk(comp_op,".0123456789")) != NULL ))
	{
		char the_op[3];

		ret=1;

		strncpy(tb_name,item,comp_op-item);
		strncpy(the_op,comp_op,sizeof(the_op)-1);
		
		/* NUL terminate */
		the_op[2] = '\0';
		
		*alert_value = atof(alert_value_str);
		/* Decode operator */
		if(!strcmp(the_op,"=="))
			*op_type=EQUAL;
		else if(!strcmp(the_op,"!="))
			*op_type=NOT_EQUAL;
		else if(!strcmp(the_op,">="))
			*op_type=EQUAL_GREATER;
		else if(!strcmp(the_op,"<="))
			*op_type=EQUAL_LESS;
		else if(!strcmp(the_op,">"))
			*op_type=GREATER;
		else if(!strcmp(the_op,"<"))
			*op_type=LESS;
		else
			ret=0; /* XXX couldn't figure it out, sorry! */

		if (strchr(item,'%') != NULL)
			*alarm_type=PERCENT;	
		else
			*alarm_type=ABSOLUTE;
	}
	return ret;
}


int load_alarm_config(const char *path, struct alarm_config **config, const int *id2index, int size , int proto, int debug)
{
		int tb_id, tb_idx, ret=1;
		char *item, dac_buffer[32 * 1024];
		float alert_value;
		alarmtype alarm_type;
		alarmlevel op_type;

		FILE *dac_f;

		/* ... */

		*config=malloc(size * sizeof(struct alarm_config));
		memset(*config,0,(size * sizeof(struct alarm_config)));

		if ((dac_f = fopen(path,"r")) != NULL)
		{
			char *dac_buffer_p;
			while(( dac_buffer_p = fgets(dac_buffer,sizeof(dac_buffer), dac_f)) != NULL)	
			{
				/* Get key */
				item = strsep (&dac_buffer_p, " \t");
				if (strcmp(item,"alert-lvl:") == 0)
				{
					if (proto != AF_INET)
						continue;
				}
				else if (strcmp(item,"alert6lvl:") == 0)
				{
					/* XXX != AF_INET6. This allows compilation on non IPv6 systems without defines */
					if (proto == AF_INET)
						continue;
				}
				else
					continue;

				/* Get items */	

				while ((item = strsep (&dac_buffer_p, " \t,")) != NULL)
				{
					char tb_name[128];

					/* Skip empty fields */
					if (*item == '\0')
						continue;

					/* Get comparision operator */
					if (decode_operator(item,tb_name, &alarm_type, &op_type,&alert_value, debug))
					{
						/* Append .ripe.net for test_box_id() lookup */
						if (strstr(tb_name,".ripe.net") == NULL)
							strcat(tb_name,".ripe.net");
						
						if (((tb_id = test_box_id(tb_name)) != -1) && ((tb_idx = lookup_id2index(id2index,tb_id,size)) != -1) )
						{
								
							if (debug)
								printf("tb=%s,idx=%d,tbid=%d,op_type=%d, alarm_type=%d,alert_value=%f\n",tb_name,tb_idx,tb_id,op_type,alarm_type,alert_value);
							/* Found it, store it */
							if (proto == AF_INET)
							{
								(*config)[tb_idx].value = alert_value;
								(*config)[tb_idx].a_lvl = op_type;
								(*config)[tb_idx].a_type = alarm_type;
							}
							else
							{
								(*config)[tb_idx].value6 = alert_value;
								(*config)[tb_idx].a_lvl6 = op_type;
								(*config)[tb_idx].a_type6 = alarm_type;
							}
							
						}
					}
				}
			}
		}
		else
		{
			printf ("Cannot open alarmconfigfile\n");
		}
		return ret;
}

void free_alarm_config(struct alarm_config *config)
{
	if (config)
		free(config);
	config=NULL;
}

