#ifndef LIBALARM_H
#define LIBALARM_H

#include <sys/time.h>

/* $Id: libdelayalarm.h,v 1.6 2005/10/28 16:10:10 ruben Exp $ */

/*
 * Status codes (timex.status)
 */
#define STA_PLL         0x0001	/* enable PLL updates (rw) */
#define STA_PPSFREQ     0x0002	/* enable PPS freq discipline (rw) */
#define STA_PPSTIME     0x0004	/* enable PPS time discipline (rw) */
#define STA_FLL         0x0008	/* select frequency-lock mode (rw) */

#define STA_INS         0x0010	/* insert leap (rw) */
#define STA_DEL         0x0020	/* delete leap (rw) */
#define STA_UNSYNC      0x0040	/* clock unsynchronized (rw) */
#define STA_FREQHOLD    0x0080	/* hold frequency (rw) */

#define STA_PPSSIGNAL   0x0100	/* PPS signal present (ro) */
#define STA_PPSJITTER   0x0200	/* PPS signal jitter exceeded (ro) */
#define STA_PPSWANDER   0x0400	/* PPS signal wander exceeded (ro) */
#define STA_PPSERROR    0x0800	/* PPS signal calibration error (ro) */

#define STA_CLOCKERR    0x1000	/* clock hardware fault (ro) */
#define STA_NANO        0x2000	/* resolution (0 = us, 1 = ns) (ro) */

/* LEAPBITMASK: bitwise AND with NTP status masks the leap second insertion
 * bits// GOODCLOCK  : required value of status word after the AND with
 * LEAPBITMASK */

#define LEAPBITMASK   ~(STA_INS | STA_DEL)
#define GPSGOODCLOCK1  (STA_PLL | STA_PPSFREQ | STA_PPSTIME | STA_PPSSIGNAL)
#define GPSGOODCLOCK2  (STA_NANO | GPSGOODCLOCK1 )
#define CDMAGOODCLOCK  (STA_PLL)

int                 ClocksOK (int, int);	/* && */
int                 AnyClockOK (int, int);	/* || */

#define RCDPV4PATH "/data"
#define RCDPV6PATH "/data6"
#define RCDP_FMT_VERSION (2)
#define RCDP_FMT_MAXFIELDS (14)

#define TM_FIRSTMONTH (0)
#define TM_LASTMONTH (11)
#define TM_FIRSTDAY (1)
#define TM_LASTDAY (31)

typedef enum _alarmtype {
	UNDEFINED_TYPE=0,
	PERCENT,
	ABSOLUTE
} alarmtype;

typedef enum _alarmlevel {
	UNDEFINED_LVL=0,
	EQUAL,
	NOT_EQUAL,
	EQUAL_GREATER,
	EQUAL_LESS,
	GREATER,
	LESS
} alarmlevel;

struct alarm_config
{
	float value, value6;
	alarmlevel	a_lvl, a_lvl6;
	alarmtype	a_type, a_type6;
};

void                getRCDPData (const char *fqdn_tbname, const time_t starttime, const time_t duration, int family,
								 int (*pckt_hook) (char **, const void *), const void *pckt_hook_data);

int                *create_id2index (int family, size_t * size);
void                free_id2index (int *idx);
int                 lookup_id2index (const int *id2index, int idx, int size);

int load_alarm_config(const char *path, struct alarm_config **config, const int *id2index, int size , int proto, int debug);
void free_alarm_config(struct alarm_config *config);



#endif /* LIBALARM_H */
