/*
 * vi:ts=4:sw=4:
 *
 * $Id: packet.h,v 1.2 2003/10/17 10:49:10 ruben Exp $
 *
 * Copyright (c) 1998-2003 RIPE NCC
 * 
 * All Rights Reserved
 * 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the author not be used in advertising or 
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.
 * 
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
 * AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
 */

#ifndef _TT_PACKET_H
#define _TT_PACKET_H

#ifdef __FreeBSD__
#include <sys/time.h>
#else
#include <time.h>
#endif

#ifdef __FreeBSD__
#include	<sys/timex.h>
#endif /* __FreeBSD__ */

#ifndef _SYS_GETPPSTIME
#include 	"getppstime.h"
#endif

#ifndef  _SYS_TYPES_H_
#include	<sys/types.h>
#endif

#define DEFAULT_PORT    3000
#define DEFAULT_SERIAL  0X01
#define DEFAULT_LENGTH  512

#define ETHER_HEADER_SIZE sizeof(struct ether_header)
#define IP_HEADER_SIZE sizeof(struct ip)	/* N.B.  This is the header
											 * without additional options set
											 * ... */
#define IP6_HEADER_SIZE sizeof(struct ip6_hdr)
#define UDP_HEADER_SIZE sizeof(struct udphdr)

/* Max length of a packet still fitting in one Ethernet packet */
#define MAX_MTU_PACKET_LENGTH (ETHERMTU-IP_HEADER_SIZE-UDP_HEADER_SIZE)


#define FINGERPRINT    0x01234567L
#define PACKETVERSION  0		/* Binary representation of the version.  */


#define TEST_PACKET_MAXLENGTH    9160

					/* maximum packet size Values above this tend to give
					 * errors. The write systemcall will not write more than
					 * this on the socket. I think.... */


struct test_packet_header
{
	long                packet_id;	/* needed for later coupling */
	ushort              offset_to_data;	/* that is where the data goes * offset 
										 * relative to the end of the * packet 
										 * header */

};

#define OFFSET_TO_DATA_IS_UNSIGNED	/* Undefine this if for whatever reason you 
									 * want to start using signed types for the 
									 * offset_to_data */

#define TEST_PACKET_HEADERLENGTH sizeof(struct test_packet_header)



struct test_packet_data
{
	/* data byte is in network order... */
	ushort              version;
	long                fingerprint;

	/* Note that fingerprint is not NULL terminated */
	struct pps_val      timepars;	/* time parameters see getpps.h */
	long                serial_number;
};

#define TEST_PACKET_DATALENGTH sizeof(struct test_packet_data)

#endif /*_TT_PACKET_H*/
