#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/time.h>
#include <unistd.h>
#include <sys/param.h>
#include <netdb.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "testbox.h"

struct test_box_data *test_box_list=NULL;
size_t                 number_of_boxes, number_of_boxes_reserved;

/* grow with 10% steps */

#define TBLIST_GROWFACTOR (1.1)

int read_test_box_config (char *filename, int debug)
{

/*  ====================
    Process config file and fill struct with id, address and name of boxes sending
    to this testbox.
*/

    char lot_fname[FILENAME_MAX], lot_buffer[1024];
    int id=0;
    FILE *lot;

    size_t                 i;

    if (test_box_list)
    {
        free(test_box_list);
    }
    number_of_boxes = 0;        /* Clear existing entries */
    number_of_boxes_reserved = PREALLOC_TESTBOXES;

    if ((test_box_list = malloc(number_of_boxes_reserved * sizeof(struct test_box_data))) == NULL)
    {
        printf ("Out of memory while requesting storage for %d testboxes", number_of_boxes_reserved);
        return FALSE;
    }
    memset (test_box_list, 0, number_of_boxes_reserved * sizeof(struct test_box_data));

    snprintf(lot_fname, PATH_MAX, "%s/%s", PATH_CONFIG, LIST_OF_TESTBOXES); 

    if ((lot = fopen(filename == NULL ? lot_fname : filename,"r")) != NULL)
    {
        /* Loop over the config file */
        while(fgets(lot_buffer,1024,lot) != NULL)
        {
            char *token, *lot_buffer_p = lot_buffer, hostname[255] = "",addr[255] ="";
            id=0;

            /* truncate newline */
            if (lot_buffer[strlen(lot_buffer)-1] == '\n')
                lot_buffer[strlen(lot_buffer)-1] = '\0';

            for (token = strsep(&lot_buffer_p,"\t "); token; token = strsep(&lot_buffer_p,"\t "))
            {
                if(id && *hostname == '\0')
                    strncpy(hostname,token,255);
                else if(id && *addr == '\0')
                    strncpy(addr,token,255);
                else if (id == 0)
                    id = atoi(token);
            }
            if (id && *hostname != '\0' && *addr != '\0')
            {
                struct test_box_data *new;
                
                number_of_boxes++;
                if ((new = realloc(test_box_list,number_of_boxes * sizeof(struct test_box_data) )) != NULL)
                {
                    test_box_list = new;
                    test_box_list[number_of_boxes-1].id = id;
                    strncpy(test_box_list[number_of_boxes-1].name,hostname,25);

                    memset (test_box_list[number_of_boxes-1].ip_addr, 0, sizeof(test_box_list[number_of_boxes-1].ip_addr));
                    if (strchr(addr,':') == NULL)
                    {
                        /* IPv4 */
                        strncpy(test_box_list[number_of_boxes-1].ip_addr,addr,INET_ADDRSTRLEN);
                        test_box_list[number_of_boxes-1].family = AF_INET;
                    }
                    else
                    {
                        /* IPv6 */
                        strncpy(test_box_list[number_of_boxes-1].ip_addr,addr,INET6_ADDRSTRLEN);
                        test_box_list[number_of_boxes-1].family = AF_INET6;
                    }
                }
                else
                {
                    perror("malloc");
                    abort();
                }
            }
        }

        fclose (lot);
        if (debug == TRUE)
        {
            for (i = 0; i < number_of_boxes; i++)
            {
                printf ("%2d\t%s\t%s\t%d\n",
                        test_box_list[i].id, test_box_list[i].name, test_box_list[i].ip_addr, test_box_list[i].family);
            }
        }
    }
    else
    {
        printf ("Cannot open configfile\n");
        return FALSE;
    }

    return TRUE;
}

void free_test_box_config (void)
{
    if (test_box_list)
    {
        free(test_box_list);
    }
    test_box_list=NULL;
}


int test_box_id (char *test_box_name)
{

/*  ===========
    Returns ID for a testbox "test_box_name"
*/

    size_t                 i;

    for (i = 0; i < number_of_boxes; i++)
    {
        if (strcmp (test_box_list[i].name, test_box_name) == 0)
        {
            return test_box_list[i].id;
        }
    }
    printf ("Warning #1: Unknown testbox %s\n", test_box_name);
    return -1;
}

int test_box_ip_id (char *test_box_ip, int family)
{

/*  ==============
    Returns ID for a test-box with IPv4 or IPv6 address
*/

    size_t                 i;

    for (i = 0; i < number_of_boxes; i++)
    {
        if (test_box_list[i].family == family)
        {
            if (strcmp (test_box_list[i].ip_addr, test_box_ip) == 0)
            {
                return test_box_list[i].id;
            }
        }
    }

    printf ("Warning #2: Unknown testbox %s\n", test_box_ip);
    return -1;
}

const char *test_box_ip_name (char *test_box_ip, int family)
{

/*  ==============
    Returns Name for a test-box with IPv4 or IPv6 address
*/

    size_t                 i;

    for (i = 0; i < number_of_boxes; i++)
    {
        if (test_box_list[i].family == family)
        {
            if (strcmp (test_box_list[i].ip_addr, test_box_ip) == 0)
            {
                return test_box_list[i].name;
            }
        }
    }


    printf ("Warning #2: Unknown testbox %s\n", test_box_ip);
    return NULL;
}

static char const   rcsid[] = "$Id: testbox.c,v 1.10 2006/06/21 13:25:03 ruben Exp $";
