/*
 * vi:ts=4:sw=4:
 *
 * $Id: utils.h,v 1.11 2006/11/22 13:27:09 ruben Exp $
 *
 * Copyright (c) 1998-2003 RIPE NCC
 * 
 * All Rights Reserved
 * 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the author not be used in advertising or 
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.
 * 
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
 * AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
 */

#ifndef _TT_UTILS_H_
#define _TT_UTILS_H_

#define OUTPUT_TYPE_STDERR  0x0

#ifndef EPERM
#include <errno.h>
#endif

#ifdef __FreeBSD__
#include <sys/time.h>
#else
#include <time.h>
#endif

void                brandom (void *, size_t);
struct timeval      pps_diff_cor (struct timeval, struct timeval);
char               *timevaltoa (struct timeval);
void                HandleError (const char *, ...);
char               *copy_argv (char **);
void                sig_child (int);
char               *sys_err_str (void);

int                 writen (int, char *, int);
int                 readn (int, char *, int);

void ncc_timersub(struct timeval *, struct timeval *, struct timeval *);

/**
 * tvsprintf() is a macro that expands into a partial argument list
 * for printf(3) et al. Use the tvsprintf_fmt macro as the proper
 * format specifier within printf(3)
 * 
 * \param[in] tvp pointer to struct timeval to be used within printf(3)
 * 
 */

#define tvsprintf(tvp) (((tvp)->tv_usec < 0) ? '-' : ' '), (int)(tvp)->tv_sec, abs((tvp)->tv_usec)
#define tvsprintf_fmt "%c%d.%d"

#ifndef timercmp
#define	timercmp(tvp, uvp, cmp)					\
	(((tvp)->tv_sec == (uvp)->tv_sec) ?				\
	    ((tvp)->tv_usec cmp (uvp)->tv_usec) :			\
	    ((tvp)->tv_sec cmp (uvp)->tv_sec))
#endif
#ifndef timeradd
#define timeradd(tvp, uvp, vvp)						\
	do {								\
		(vvp)->tv_sec = (tvp)->tv_sec + (uvp)->tv_sec;		\
		(vvp)->tv_usec = (tvp)->tv_usec + (uvp)->tv_usec;	\
		if ((vvp)->tv_usec >= 1000000) {			\
			(vvp)->tv_sec++;				\
			(vvp)->tv_usec -= 1000000;			\
		}							\
	} while (0)
#endif
#ifndef timersub
#define timersub(tvp, uvp, vvp)						\
	do {								\
		(vvp)->tv_sec = (tvp)->tv_sec - (uvp)->tv_sec;		\
		(vvp)->tv_usec = (tvp)->tv_usec - (uvp)->tv_usec;	\
		if ((vvp)->tv_usec < 0) {				\
			(vvp)->tv_sec--;				\
			(vvp)->tv_usec += 1000000;			\
		}							\
	} while (0)
#endif


#endif /*_TT_UTILS_H_ */
