#ifndef MCAST_SEND_H
#define MCAST_SEND_H

#include	<stdio.h>
#include	<string.h>
#include 	<math.h>
#include	<stdlib.h>
#include	<netinet/in.h>
#include	<arpa/inet.h>

#include	<sys/socket.h>
#include	<net/if.h>

#include	<netinet/if_ether.h>

#include	<netinet/udp.h>

#define  FALSE (0)
#define  TRUE  (!FALSE)

#define MAXLINE 100

#define UDP_OPEN_OLDBEHAVIOUR NULL

/* Interface we should send our packets out on (set by install script) */
#define INTERFACE NULL

/* 
 * These flags are bits (from LSB to MSB), you can extend them if you need more
 * options in the future (e.g. RES1_FLAG 4, RES2_FLAG 8 ...)  [FF]
 */


#define V4_FLAG 1
#define V6_FLAG 2


/* For the new packet id code */
#define LASTUNIXTIME 1024488000	/* Wed Jun 19 12:00:00 2002 GMT */
#define TIMEZERO     1009843200	/* Tue Jan 1 00:00:00 2002 GMT */
#define CYCLEPERIOD  20000000	/* cycle period in seconds */
#define FREQUENCY    100		/* Max frequency */

/* Struct with the individual measurements */
/* ======================================= */

struct measurement
{
    struct measurement *next;   /* Pointer to the next element in the chain */
    char               *name;   /* Name of the testbox */
    int                 port;   /* Port to send to */
    int                 size;   /* Size of the packet */
    double              time;   /* Time to send the packet */
    int                 burst;  /* Number of packets in burst mode */
    int                 family; /* AF_INET or AF_INET6 */
    int                 cast;   /* 0 for uunicast, 1 for multicast */
    int                 ttl;    /* TTL for multicast packets */
    union
    {
        struct in_addr      addr;
        struct in6_addr     addr6;
    }
    target;
};

/* Struct with global measurement data */
/* =================================== */

struct global
{
    time_t              start;  /* Start time of measurements */
    time_t              period; /* Period length (seconds) */
    time_t              read;   /* Time when configfile was last read */
}
globaltime;

/* Struct with the measurements for each testbox */
/* ============================================= */

struct target
{
    struct target      *next;   /* Next element in the chain */
    char               *name;   /* Name of the testbox */
    int                 port;   /* Port to send to */
    int                 size;   /* Number of bytes in packet */
    int                 rate;   /* Rate (packets/period) */
    float              *time;   /* Pointer to an array of floats indicating the
                                 * time of each measurement */
    int                 seed;   /* Random seed */
    int                 burst;  /* Number of packets to send in burst mode */
    int                 linear; /* Linear (=TRUE) or Poisson (=False)
                                 * distribution */
    int                 family; /* AF_INET or AF_INET6 */
    int                 cast;   /* 0 for unicast, 1 for multicast */
    int                 ttl;    /* TTL for multicast packets */
    union
    {
        struct in_addr      addr;
        struct in6_addr     addr6;
    }
    target;
};


int                 read_configfile_mcast (int, char *);
char*               read_if_config (void);
int                 schedule (int, int, char*, time_t*, double*);
int                 SendPacket (int, struct measurement*, struct sockaddr*, socklen_t*);
int                 udp_open(char*, char*, int, int, int, struct measurement*);
char*               host_err_str (void);
void                create_logfile (char *, char *, int);
int                 record_pid (int);
int                 remove_pid (int);
void                exitcode (int);
struct 
test_packet_data*   SetupPacket (int, char*, u_int, long);

#endif
