/*
 * vi:ts=4:sw=4:
 *
 * $Id: sleep.c,v 1.3 2007/03/23 14:18:12 franzsch Exp $
 *
 * Copyright (c) 1998-2003 RIPE NCC
 * 
 * All Rights Reserved
 * 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the author not be used in advertising or 
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.
 * 
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
 * AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
 */

#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>

#include "sleep.h"
#include "poisson.h"

static char const   rcsid[] = "$Id: sleep.c,v 1.3 2007/03/23 14:18:12 franzsch Exp $";

/* Sleep until:
 * 
 * Call sleep/usleep such that the process will sleep until a specific time, if 
 * the time to sleep is 10 seconds or less, use usleep, otherwise use sleep.
 * Arguments:
 * 
 * int debug: TRUE/FALSE, more or no debug output double wakeup: when the
 * process should wake up again, should be a unix time as a double, with the
 * fraction expressing the fraction of a second.
 * 
 * Exact return time depends on the vagueries of the O/S.
 * 
 */

void sleep_until (int debug, double wakeup)
{

	time_t              t;
	long int            delay;

	delay = time (&t);
	delay = ((long int) wakeup) - delay;

	if (delay < 0)
	{
		delay = 0;
	}

	if (delay > 10)
	{
		if (debug)
		{
			printf ("sleep_until: Have to wait for %lu sec\n", delay);
		}
		sleep (delay - 10);
	}

	delay = time (&t);

	if (delay < wakeup)
	{
		delay = (((long int) wakeup) - delay) * 1000000;
		if (delay < MIN_INTERVAL * 1000)
		{
			delay = MIN_INTERVAL * 1000;
		}
	}
	else
	{
		delay = MIN_INTERVAL * 1000;
	}

	if (debug)
	{
		printf ("sleep_until: have to wait for %lu microsec\n", delay);
	}

	usleep (delay);

	return;
}
