// This software implements the low level functionality of the cutsom listener. It listens to
// a multicast stream and writes statistical information to a file in predefined intervalls
// TODO: Sampling??

#include "clistener.h"
#include "mcast_inc.h"

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <signal.h>
#include <string.h>
#include <netinet/in.h>
#include <unistd.h>
#include <limits.h>
#include <arpa/inet.h>
#include <netinet/udp.h>
#include <errno.h>
#include <syslog.h>
#include <time.h>

#define PNAME "CustomListener"
#define DEBUG 0

#define MAXLINE 100

// The line below will be replaced by the install script, depending weather or not the install target is a testbox or not.
#define GPSCLOCK 1

#ifdef GPSCLOCK
#include "../libtb/packet.h"
#include "../libtb/utils.h"
#endif

int     udp_client( const char*, const char*, void**, socklen_t* );
void    clistener( int, char** );
void    print_probes(void);
void    clear_probes(void);
void    write_stats(void);
int     check_dval( char*, char* , char*, int );
char*   hex2char( char*, int* );
int     get_ttm( char*, size_t, struct timeval* );
char*   read_if_config(void);


struct probe {
  struct timeval timeS;  // Timestamp from the packet
  struct timeval timeR;  // Recieve time
  int delay; // Stores the delay in case of dbeacon (u secs), otherwise 0.
  int dpsize;   // Difference between expected and actual packet size
  int dvalrate; // Percent of how many defined values match.
  struct sockaddr *from;
  socklen_t fromlen;
  struct probe *next;
};

#ifndef GPSCLOCK
/* stuct for syscal getppsval. For interaction with time and pps related paramenters. */
struct pps_val
{
        struct timeval      current_time;   /* Current timeofday as reported by
                                             * the (NTP driven kernel clock) */
        struct timeval      last_pps;   /* time of the last PPS as set by *
                                         * hardpps */
        int                 pps_status; /* clock status bits (as defined in
                                         * timex.h) Most imortant is that 0x0100 is
                                         * set. PPSSIGNAL pps signal present. */
        long                pll_offset; /* NTPs idea of the offset to the real
                                         * time. */
        long                maxerror;   /* PLL error estimates */
        long                esterror;
};

struct test_packet_header
{
	long                packet_id;	/* needed for later coupling */
	ushort              offset_to_data;	/* that is where the data goes * offset 
                                    		 * relative to the end of the * packet 
						 * header */

};

struct test_packet_data
{
	/* data byte is in network order... */
	ushort              version;
	long                fingerprint;

	/* Note that fingerprint is not NULL terminated */
	struct pps_val      timepars;	/* time parameters see getpps.h */
	long                serial_number;
};

void HandleError (const char *pname, ...)
{

    va_list             args;
    char               *fmt;
    char                message[_POSIX2_LINE_MAX];

    va_start (args, pname);

    if (pname != NULL)
    {
        fprintf (stderr, "%s: ", pname);
    }
    fmt = va_arg (args, char *);

    vsprintf (message, fmt, args);

#ifndef DEBUG
    syslog (LOG_NOTICE, "%s %s", message, strerror(errno));
#else
    fprintf (stderr, "%s %s\n", message, strerror(errno));
#endif

    va_end (args);

    fflush (stdout);
    fflush (stderr);

    return;
}

#endif
struct probe *first = NULL;
struct probe *current = NULL;

// Make these integers global since they are needed in the write_stats function
// which again has to be called via a signal
int exprate = 0;
int stat_int = 0;
int port = 0;
int multsrc = 0;
char *mcast; // The multicat adr we are listening to
char *datadir; // Directory to store logfiles
char *interface; // Interface name

static char const rcsid[] = "$Id: clistener.c,v 1.7 2007/11/08 11:50:22 franzsch Exp $";

int main( int argc, char **argv ) {
  interface = read_if_config();
  clistener(argc, argv);
  return 0;
}

/* Subtract the `struct timeval' values X and Y,
   storing the result in RESULT.
   Return 1 if the difference is negative, otherwise 0.  */

int
timeval_diff (result, x, y)
     struct timeval *result, *x, *y;
{
  /* Perform the carry for the later subtraction by updating y. */
  if (x->tv_usec < y->tv_usec) {
    int nsec = (y->tv_usec - x->tv_usec) / 1000000 + 1;
    y->tv_usec -= 1000000 * nsec;
    y->tv_sec += nsec;
  }
  if (x->tv_usec - y->tv_usec > 1000000) {
    int nsec = (x->tv_usec - y->tv_usec) / 1000000;
    y->tv_usec += 1000000 * nsec;
    y->tv_sec -= nsec;
  }

  /* Compute the time remaining to wait.
     tv_usec is certainly positive. */
  result->tv_sec = x->tv_sec - y->tv_sec;
  result->tv_usec = x->tv_usec - y->tv_usec;

  /* Return 1 if result is negative. */
  return x->tv_sec < y->tv_sec;
}

char* read_if_config() {
   char *file, *point, *ifname;
   char buffer[MAXLINE];
   FILE* fp;

   file = (char *) malloc (strlen (getenv ("TTROOT")) + strlen ("/config/mcast/interface.conf") + 1);
   file = strcpy (file, getenv ("TTROOT"));
   file = strcat (file, "/config/mcast/interface.conf");

   /* Read configfile */
   fp = fopen (file, "r");
   if (fp == NULL)
   {
      printf ("Cannot open %s\n", file);
      return NULL;
   }

   ifname = NULL;
   while ((point = fgets (buffer, MAXLINE, fp)) != NULL)
   {
      if (strncmp (point, "InterfaceName", 13) == 0) {
         strtok (point, " "); // Skip the first word
         ifname = strdup (strtok (NULL, "\n")); // Save the Interface Name
      }
   }
   // Return pointer to the interface name or null if we didn't find it or it's defined as 0.
   if( *ifname == '0' )
      return NULL;
   else
      return ifname;
}


void clistener(int argc, char **argv) {
  int i,tRe;

  int sockfd;
  const int on = 1;
  socklen_t salen;
  struct sockaddr *sa;
  char messagebuffer[512];

  char *tformat, *dvalues, *doffsets;
  int toffset, psize;

  int realsize;
  int count = 0;

  fd_set rset;

  struct timeval *timer, *countdown, *tp, *sFix1, *sFix2, *sFixDiff;
  
  struct probe *prb = NULL;

#ifdef GPSCLOCK
  struct pps_val gps_time;
#endif
 

  if (argc == 11 || argc == 12) {
      sockfd = udp_client (argv[1], argv[2], (void **) &sa, &salen);
  }
  else
  {
      printf ("usage: <mcast-addr> <port#> <rate#> <timeformat> <timeoffset#> <packetsize#> <definedvalue(s)> <dvaluesoffset(s)> <statinterval#> <datadir> [multiSource]\n");
      exit(1);
  }

  // Process the arguments
  mcast = malloc( strlen(argv[1]) );
  strcpy( mcast, argv[1] );

  port = atoi( argv[2] );

  exprate = atoi( argv[3] );

  tformat = strdup( argv[4] );

  toffset = atoi( argv[5] );

  psize = atoi( argv[6] );

  dvalues = strdup( argv[7] );
  doffsets = strdup( argv[8] );

  stat_int = atoi( argv[9] );

  datadir = strdup( argv[10] );

  if( argc == 12 ) {
    multsrc = atoi( argv[11] );
  }

  // Do networking stuff
  setsockopt( sockfd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on) );
  bind( sockfd, sa, salen );

  mcast_join( sockfd, sa, salen, interface, 0 );

  // Set up the timer and the selectFix timeval to see how long it took to wait for the next packet
  timer = malloc( sizeof( struct timeval ) );
  countdown = malloc( sizeof( struct timeval ) );
  countdown->tv_sec = stat_int+1;
  countdown->tv_usec = 0;
  sFix1 = malloc( sizeof( struct timeval ) );
  gettimeofday( sFix1, NULL );
  sFix2 = malloc( sizeof( struct timeval ) );
  sFixDiff = malloc( sizeof( struct timeval ) );


  // And allocate the timeval used for handling measurements
  tp = malloc( sizeof( struct timeval ) );

  for (;;) {
      ///
      FD_ZERO(&rset);
      FD_SET( sockfd, &rset );
      // From here until the select statement we fix the issues with select not updating the timeval.
      gettimeofday( sFix2, NULL );
      timeval_diff( sFixDiff, sFix2, sFix1 );
      // If timer got negative by this, we have to start write_stats
      if( timeval_diff( countdown, countdown, sFixDiff ) == 1 ) {
         countdown->tv_sec = stat_int+1;
         countdown->tv_usec = 0;
         write_stats();
         gettimeofday( sFix1, NULL );
         continue;
      }
      gettimeofday( sFix1, NULL );
      timer->tv_sec =  countdown->tv_sec;
      timer->tv_usec =  countdown->tv_usec;
      // End of select fix
      if( select( sockfd+1, &rset, NULL, NULL, timer ) == 0 ) {
         // We reached the timeout. Reset the timer values and call write_stats
         timer->tv_sec = stat_int+1;
         timer->tv_usec = 0;
         write_stats();
         continue;
      }
      // Otherwise we are ready to read from the socket our timer value should have been rewritten.
      realsize = recvfrom (sockfd, messagebuffer, 1024, 0, sa, &salen);

      // If the packet size doesn't match our expected size, ignore this package.
      if( psize != 0 && psize != realsize )
        continue;

      // Get time immediatly after we recieved the message
#ifdef GPSCLOCK
      getppstime( &gps_time );
#else
      if (gettimeofday(tp, NULL) != 0)
        HandleError(PNAME, "Error getting time.");
#endif
      count++;

      prb = malloc( sizeof(struct probe) );

      // Default delay (only used for dbeacon timestamps) to 0
      prb->delay = 0;

#ifdef GPSCLOCK
      memcpy(&(prb->timeR), &(gps_time.current_time), sizeof(struct timeval));
#else
      memcpy(&(prb->timeR), tp, sizeof(struct timeval));
#endif
      
      if( strncmp(tformat, "NoTimestamp", 11) == 0 )
      {
#ifdef GPSCLOCK
        memcpy(&(prb->timeS), &(gps_time.current_time), sizeof(struct timeval));
#else
        memcpy(&(prb->timeS), tp, sizeof(struct timeval));
#endif
      }
      else if( strncmp(tformat, "UnixTimestamp", 13) == 0 )
      {
        if( toffset + sizeof(struct timeval) > realsize )
        {
          // The Offset is too large to extract the timestamp
          if( DEBUG == 1 )
          {
            printf( "Timestampoffset too large for extraction, Delay will be 0\n" );
          }
#ifdef GPSCLOCK
          memcpy(&(prb->timeS), &(gps_time.current_time), sizeof(struct timeval));
#else
          memcpy(&(prb->timeS), tp, sizeof(struct timeval));
#endif
        } else {
          memcpy( &(prb->timeS), messagebuffer + toffset, sizeof(struct timeval) );
        }
      }
      else if( strncmp(tformat, "dbeaconTimestamp", 16) == 0 ) {
        if( realsize != 12 ) // dbeacon probe packets are always 12 bytes large.
        {
          // The Offset is too large to extract the timestamp
          if( DEBUG == 1 )
          {
            printf( "Packet has not the right size (%i) for timestamp extraction, delay will be 0.\n", realsize );
          }
        } else {

          // First grab the 32 bit u_int value used for timestamping by dbeacon
          i = ntohl( *((u_int *) (messagebuffer+8)) );
          // Then convert our time to the same format
          tRe = prb->timeR.tv_sec;
          tRe *= 1000;
          tRe += prb->timeR.tv_usec / 1000;

          // And calculate the delay 
          prb->delay = tRe-i;
          
          // ^^^ This is probably not the nicest way to to it since it is the only time the
          // delay has to be calculated before the write_stat function. This was however
          // necessary due to the timestamp format used by dbeacon.
        }
        // At last, in any case, make send time = recieve time
#ifdef GPSCLOCK
        memcpy(&(prb->timeS), &(gps_time.current_time), sizeof(struct timeval));
#else
        memcpy(&(prb->timeS), tp, sizeof(struct timeval));
#endif
      }
      else if( strncmp(tformat, "TTMTimestamp", 12) == 0 )
      {
        if( get_ttm( messagebuffer, realsize, &(prb->timeS) ) == 0 )
        {
          // Couldn't get the timestamp from the packet, use current timestamp -> delay of 0
          if( DEBUG == 1 )
            printf( "TTMTimestamp could not be extracted\n" );
          memcpy(&(prb->timeS), tp, sizeof(struct timeval));
        }
      }
      else
      {
        // If nothing wokrs, also go for current time.
#ifdef GPSCLOCK
        memcpy(&(prb->timeS), &(gps_time.current_time), sizeof(struct timeval));
#else
        memcpy(&(prb->timeS), tp, sizeof(struct timeval));
#endif

      }
      
      // Store the from address
      prb->from = malloc( salen );
      memcpy(prb->from, sa, salen);
      prb->fromlen = salen;

      // Store packet size
      prb->dpsize = realsize;

      // Check the defined values
      prb->dvalrate = check_dval(dvalues, doffsets, messagebuffer, realsize);

      prb->next = NULL;
      if (first == NULL)
      {
        first = prb;
        current = prb;
      } else {
        current->next = prb;
        current = prb;
      }

  }

  // Clean up and quit
  signal( SIGALRM, SIG_DFL );
  clear_probes();
  free(tformat);
  free(dvalues);
  free(doffsets);
  free(first);
  free(current);
  free(tp);
  exit (0);
}

// Function used to extract the TTM Timestamp
// Returns 0 on error, 1 on success
int get_ttm( char *message, size_t size, struct timeval* tp ) 
{
  struct test_packet_header *head;
  struct test_packet_data *data;
  char *temp;

  // First extract the header
  if( size >= sizeof(struct test_packet_header) )
    head = (struct test_packet_header *) message;
  else
    return 0;

  // And then locate the position of the timestamp and copy it over to our pointer
  if( size >= sizeof(struct test_packet_header) + ntohs(head->offset_to_data) + sizeof(struct test_packet_data))
  {
    temp = message + sizeof(struct test_packet_header) + ntohs(head->offset_to_data);
    data = (struct test_packet_data *) temp;
    memcpy( tp, &(data->timepars.current_time), sizeof(struct timeval) );
  }
  else
    return 0;

  // When everything went right, return 1
  return 1;
}

int check_dval(char *dval, char *offset, char *messagebuffer, int psize)
{
  char *val, *cval, *off, *valptr, *offptr, type;
  int len, ioff, pos, count, i, passed;


  // If nothing is defined, return all passed immediatly
  if( *dval == '0' && *offset == '0' )
    return 100;

  len = strlen( dval );

  // Using reentrant versions of strtok here since we have to parse two strings
  val = strtok_r(dval, ",", &valptr);
  off = strtok_r(offset, ",", &offptr);
  pos = 0;
  count = 0;
  
  do
  {
    count++;
    ioff = atoi( off );

    // Lets see if we have to do some special comparison
    if( *val == '&' || *val == '=' )
    {
      type = *val;
      // Get rid of the preceding character
      val++;
    } else {
      // Otherwise fall back to default comparison
      type = '=';
    }

    cval = hex2char( val, &len );
    
    if( psize < ioff + len ) // Offset too large, dont' check.
    {
      if( DEBUG == 1 )
      {
        printf("Offset of defined Value is too large, check skipped");
      }
      // Clean up and continue
      free( cval );
      continue;
    }
    
    // Everything safe, do the check.
    if( type == '=' ) // Do an ordinary memcmp
    {
      if(memcmp( messagebuffer + ioff, cval, len ) == 0)
      {
        pos++;
      }
    } 
    else if( type == '&' ) // We have to do a bitwise xor
    {
      passed = 1;
      // Check each character 
      for( i = 0; i++; i<len )
      {
        if( (*(cval+i) ^ *(messagebuffer + ioff + i)) != 0 )
          passed = 0; // We have at least one byte that didn't pass the test
      }
      if( passed == 1 )
        pos++;
    }

    // Parse the next value and offset
    val = strtok_r(NULL, ",", &valptr);
    off = strtok_r(NULL, ",", &offptr);

    // Clean up
    free( cval );
  } while( val != NULL && off != NULL );

  // Checked all values, calculate pass percentage and return
  return (pos*100)/count;
}

char* hex2char( char *hex, int *rlen )
{
  int flen,len, i;
  char *output;
  char temps[8];

  strncpy(temps, hex, 8);
 
  len = strlen( hex );
  // Calculate length of output string
  flen = len;
  if(len%2 == 1)
    flen = len + 1;
  flen = flen/2;
  *rlen = flen;  

  // Allocate this space
  output = malloc( sizeof(char) * (flen) );

  output[flen] = '\0';
  for (i=len-2; i>=-1; i=i-2)
  {
    if( i == -1 )
    {
      output[0] = (char) strtol(temps, (char **)NULL, 16);
    } else {
      output[flen-1] = (char) strtol(temps+i, (char **)NULL, 16);
      flen--;
      *(temps+i) = '\0';
    }
  }

  return output;
}

// Writes the Stats to a logfile, takes the number of packets and the expected rate as arguments.
// It also clears all probes.
void write_stats()
{
  struct probe *prb, *temp;
  time_t tp;
  int found;
  int dpsizeSum;
  int dvalrateSum;
  int count;
  int adjRate; // Expected rate (per minute) adjusted to intervall
  float rate; // Real rate (per minute)
  char filename[100];
  char* logfilestr;
  char timestring[9];
  char inet_adr[60];
  double ts, tr, delaySum;
  FILE* logfile;

  // A structure and some pointers to store sums for multiple sources.
  struct sum {
    struct sockaddr *from;
    int dpsize;
    int dvalrate;
    float delay;
    int count;
    struct sum *next;
  };
  struct sum *mSumFirst, *mSumIt, *mSumTmp;

  mSumFirst = NULL;
  mSumIt = NULL;
  mSumTmp = NULL;

  count = 0;
  adjRate = 0;
  dpsizeSum = 0;
  dvalrateSum = 0;
  delaySum = 0;

  if( multsrc == 1 ) {
    // First create our mSum linked list.
    prb = first;
    while( prb ) {
      mSumIt = mSumFirst;
      found = 0;
      // For each prb check if we already have that source address in our Sums linked list.
      while( mSumIt ) {
        mSumTmp = mSumIt; // Store in case we have to append something.
        if( memcmp( &((struct sockaddr_in*) prb->from)->sin_addr, &((struct sockaddr_in*) mSumIt->from)->sin_addr, sizeof(struct in_addr) ) == 0 ) {
          found = 1;
          break;
        } 
        mSumIt = mSumIt->next;
      }

      if( found == 1 ) { // Yes we have it in our Sums list and mSumIt already points to the right one.
          mSumTmp = mSumIt; // For later summing
      } else { // No it is not yet in our Sums list. mSumIt points to the last element or NULL
        if( mSumTmp == NULL ) { // This is our first element for the list
          mSumFirst = malloc( sizeof(struct sum) );
          mSumFirst->from = malloc( prb->from->sa_len ); // Malloc and copy over the from address
          memcpy( mSumFirst->from, prb->from, prb->from->sa_len );
          mSumFirst->dpsize = 0;
          mSumFirst->dvalrate = 0;
          mSumFirst->delay = 0;
          mSumFirst->count = 0;
          mSumFirst->next = NULL;
          mSumTmp = mSumFirst; // For later summing
        } else { // Not the first element we have to append it to the end of the list.
          mSumTmp->next = malloc( sizeof(struct sum) );
          mSumTmp->next->from = malloc( prb->from->sa_len ); // Malloc and copy over the from address
          memcpy( mSumTmp->next->from, prb->from, prb->from->sa_len );
          mSumTmp = mSumTmp->next; // For later summing
          mSumTmp->dpsize = 0;
          mSumTmp->dvalrate = 0;
          mSumTmp->delay = 0;
          mSumTmp->count = 0;
          mSumTmp->next = NULL;
        }
      }

      // Now do the actual summing
      // Calculate Delay
      // Check if we have a dbeacon timestamp (delay already in prb)
      if( prb->delay != 0 ) {
        mSumTmp->delay += (float)prb->delay/1000;
      } else { // Or we have to calculate it from our time_vals
        tr = (float)prb->timeR.tv_sec + (float)prb->timeR.tv_usec / 1000000;
        ts = (float)prb->timeS.tv_sec + (float)prb->timeS.tv_usec / 1000000;
        mSumTmp->delay += tr-ts;
      }

      // Sum the packet size
      mSumTmp->dpsize += prb->dpsize;
      // Sum the defined values rate  
      mSumTmp->dvalrate += prb->dvalrate;
      mSumTmp->count++;

      prb = prb->next;    
    }


  } else { // We only have to do one sum
    prb = first;
    while( prb != NULL )
    {
      // Calculate Delay
      // Check if we have a dbeacon timestamp (delay already in prb)
      if( prb->delay != 0 ) {
        ts = prb->delay;
        delaySum += ts/1000;
      } else { // Or we have to calculate it from our time_vals
        tr = (float)prb->timeR.tv_sec + (float)prb->timeR.tv_usec / 100000;
        ts = (float)prb->timeS.tv_sec + (float)prb->timeS.tv_usec / 100000;
        delaySum += tr-ts;
      }
    
      // Sum Data Package Size
      dpsizeSum += prb->dpsize;

      // Sum Defined Value pass percentage
      dvalrateSum += prb->dvalrate;

      count++;
      prb = prb->next;
    }
  }
  // Now clear all the probes. clear_probes is reimplemented here to avoid giving up the mutex when calling the function
  // This might not be the nicest way to do it, but I couldn't think of anything better
  prb = first;

  while( prb != NULL ) {
    temp = prb;
    prb = prb->next;
    free(temp->from);
    free(temp);
  }

  first = NULL;
  current = NULL;

  if( multsrc == 1 ) {
    // Write the multsrc logfiles
    time(&tp);
    mSumIt = mSumFirst;

    while( mSumIt ) {
      // Calculare rate 
      rate = (60 * (float) mSumIt->count) / (float) stat_int;

      if( exprate > 0 ) { // This is needed for calculating loss, so only do it if we have an expected Rate
        // Calculate adjusted expected rate (Expected Rate is specified in number of packets per 10 minutes,
        // the intervall in which stats are generated is usually not ten minutes, so we have to adjust)
        adjRate = (exprate * stat_int) / 60;
      } else {
        adjRate = 0;
      }

      logfilestr = malloc(sizeof(char)*100 + sizeof(datadir));

      strftime( timestring, sizeof (timestring), "%Y%m%d", gmtime (&tp) );
      if( mSumIt->from->sa_family == AF_INET ) {
        snprintf( filename, 70, "CRCP.%s.%i.%s.%s", mcast, port, 
                  inet_ntop( AF_INET, &( (struct sockaddr_in*) mSumIt->from )->sin_addr.s_addr, inet_adr, sizeof(inet_adr) ), timestring );
      } else {
        snprintf( filename, 70, "CRCP.%s.%i.%s.%s", mcast, port, 
                  inet_ntop( AF_INET6, &( (struct sockaddr_in6*) mSumIt->from )->sin6_addr.s6_addr, inet_adr, sizeof(inet_adr) ), timestring );
      }
        
      strcpy( logfilestr, datadir );
      strcat( logfilestr, "/" );
      strcat( logfilestr, filename );

      logfile = fopen( logfilestr , "a" );
      if( logfile == NULL ) {
        if(DEBUG == 1) {
          printf("Couldn't open logfile %s for writing", logfilestr);
        }
        return;
      }
    

      // Calculate averages and print to the file
      if( mSumIt->count > 0 ) {
        if(DEBUG == 1) {
          printf( "Source: %s Delay Av: %f Packet Size Av: %i Value Pass Perc Av: %i Loss Perc: %i Rate: %.4f\n", 
                   inet_ntop( AF_INET, &( (struct sockaddr_in*) mSumIt->from )->sin_addr.s_addr, inet_adr, sizeof(inet_adr) ),
                   (float) mSumIt->delay/mSumIt->count, mSumIt->dpsize/mSumIt->count,
                   mSumIt->dvalrate/mSumIt->count, exprate > 0 ? 100-((mSumIt->count)*100)/adjRate : 0, rate);
        }
        // Print out delay, packet size, defined value pass rate, loss, rate
        fprintf( logfile, "%.10f %i %i %i %.4f %i\n", (float) mSumIt->delay/mSumIt->count, mSumIt->dpsize/mSumIt->count,
                  mSumIt->dvalrate/mSumIt->count, exprate > 0 ? 100-(mSumIt->count*100)/adjRate : 0, rate, (int) tp);
      } else {
        if(DEBUG == 1) {
          printf( "Warning: Statistic interval might be too short or no packets are arriving!\n" );
        }
        // Print out delay, packet size, defined value pass rate, loss, rate. Most of this (except loss) is zero since we haven't recieved any
        // packets. This will show up in the graphs and trigger the alarm.
        // TODO: It might be nice to trigger the alarm directly from here... Think about that!
        fprintf( logfile, "%.10f %i %i %i %i %i\n", (float) 0, 0, 0, 100, 0, (int) tp);
      }
      fclose( logfile );
      free( logfilestr );
      mSumIt = mSumIt->next;
    }
   
  } else {
    // Make a nice filename CRCP.mcas_adr.yyyymmdd
    time(&tp);

    //Calculate real rate per minute
    rate = (60 * (float) count) / (float) stat_int;

    if( exprate > 0 ) { // This is needed for calculating loss, so only do it if we have an expected Rate
      // Calculate adjusted expected rate (Expected Rate is specified in number of packets per 10 minutes,
      // the intervall in which stats are generated is usually not ten minutes, so we have to adjust)
      adjRate = (exprate * stat_int) / 60;
    } else {
      adjRate = 0;
    }

    logfilestr = malloc(sizeof(char)*100 + sizeof(datadir));

    strftime (timestring, sizeof (timestring), "%Y%m%d", gmtime (&tp));
    snprintf( filename, 70, "CRCP.%s.%i.%s", mcast, port, timestring );
    strcpy( logfilestr, datadir );
    strcat( logfilestr, "/" );
    strcat( logfilestr, filename );


    logfile = fopen( logfilestr , "a" );
    if( logfile == NULL ) {
      if(DEBUG == 1) {
        printf("Couldn't open logfile %s for writing", logfilestr);
      }
      return;
    }
    

    // Calculate averages and print to the file
    if( count > 0 ) {
      if(DEBUG == 1) {
        printf( "Delay Av: %f Packet Size Av: %i Value Pass Perc Av: %i Loss Perc: %i Rate: %.4f\n", delaySum/count, dpsizeSum/count,
                 dvalrateSum/count, exprate > 0 ? 100-(count*100)/adjRate : 0, rate);
      }
      // Print out delay, packet size, defined value pass rate, loss, rate
      fprintf( logfile, "%.10f %i %i %i %.4f %i\n", delaySum/count, dpsizeSum/count,
                dvalrateSum/count, exprate > 0 ? 100-(count*100)/adjRate : 0, rate, (int) tp);
    } else {
      if(DEBUG == 1) {
        printf( "Warning: Statistic Interval might be too short or no packets are arriving!" );
      }
      // Print out delay, packet size, defined value pass rate, loss, rate. Most of this (except loss) is zero since we haven't recieved any
      // packets. This will show up in the graphs and trigger the alarm.
      // TODO: It might be nice to trigger the alarm directly from here... Think about that!
      fprintf( logfile, "%.10f %i %i %i %i %i\n", (float) 0, 0, 0, 100, 0, (int) tp);
    }

    fclose( logfile );
    free( logfilestr );
  }

  mSumIt = mSumFirst;
  while( mSumIt ) {
    mSumTmp = mSumIt;
    mSumIt = mSumIt->next;
    free( mSumTmp->from );
    free( mSumTmp );
  }

  return;
}

void clear_probes()
{
  struct probe *prb, *temp;

  prb = first;

  while( prb != NULL ) {
    temp = prb;
    prb = prb->next;
    free(temp->from);
    free(temp);
  }

  first = NULL;
  current = NULL;

  return;
}

void print_probes()
{
  struct probe *prb;

  prb = first;

  while( prb != NULL ) {
    printf( "Sent: %i, Recieved: %i, Size-diff: %i, Dval-perc: %i\n", (int) prb->timeS.tv_sec, (int) prb->timeR.tv_sec,
                                                                      prb->dpsize, prb->dvalrate );
    prb = prb->next;
  }
}

