/*
 * vi:ts=4:sw=4:
 *
 * $Id: DataHandling.h,v 1.3 2007/03/23 14:18:12 franzsch Exp $
 *
 * Copyright (c) 1998-2003 RIPE NCC
 * 
 * All Rights Reserved
 * 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the author not be used in advertising or 
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.
 * 
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
 * AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
 */

#ifndef _TT_DATAHANDLING_H
#define _TT_DATAHANDLING_H


#include "UDPProcessor.h"
#include "PacketCatcher.h"
#include "getppstime.h"
#include "if_utils.h"

#define FINAL_OUTPUT_ROOTNAME "data/output"
#define FINAL_OUTPUT_ROOTNAME6 "data6/output6"

/*
 * these pipes, set up by MainControlDaemon, are used by UDPProcessor and
 * PacketCatcher to communicate with the DataHandler 
 */

extern int          PC_pipe4[2],
                    UC_pipe4[2],
                    PC_pipe6[2],
                    UC_pipe6[2];	/* pipes used for IPC */

/*
 * pipe4 ... IPv4 pipe pipe6 ... IPv6 pipe 
 */

/*
 * UDP packets are thrown away if not all fragments are found within the
 * configured time limit (in seconds). This occures when bpf drops packets... 
 */

#define UDP_NOT_PROCESSED_TIMEOUT 10


/*
 * IP Fragments are thrown away if a corresponding UDP packet is not found
 * within the configured time limit (seconds). This occures when Fragments are
 * lost... 
 */

#define IPF_NOT_PROCESSED_TIMEOUT 15


/*
 * The structs below are used to controll the temporary storage of data. They
 * consists of a pointer to an array with the data and an integer indicating
 * the number of elements on that array.  The arrays will grow when unprocessed 
 * * data comes in and shrink when data is combined and written to output 
 */


/*
 * PC & UC dataStore now linked lists [RW 20020617] 
 */

/*
 * XXdataStoreY || | PC or UC | IP version 
 */

struct PCdataStore4
{
	struct packetcatchdata4 data;

	SLIST_ENTRY(PCdataStore4) entries;
};

struct PCdataStore6
{
	struct packetcatchdata6 data;

	SLIST_ENTRY(PCdataStore6) entries;
};


struct UCdataStore4
{
	struct udpportdata4 data;

	SLIST_ENTRY(UCdataStore4) entries;
};

struct UCdataStore6
{
	struct udpportdata6 data;

	SLIST_ENTRY(UCdataStore6) entries;
};

SLIST_HEAD(PCdataStore4List, PCdataStore4);
SLIST_HEAD(PCdataStore6List, PCdataStore6);
SLIST_HEAD(UCdataStore4List, UCdataStore4);
SLIST_HEAD(UCdataStore6List, UCdataStore6);

int                 DataHandler(struct DelayProbeList *, char *, char *);


/*
 * every IPv4 function has its IPv6 counterpart (keeps a nice overview :-) 
 */

void                PutOnPCStore(struct PCdataStore4List *, struct packetcatchdata4 *);
void                PutOnPCStore6(struct PCdataStore6List *, struct packetcatchdata6 *);

int                 ProcessPCStore(struct PCdataStore4List *);
int                 ProcessPCStore6(struct PCdataStore6List *);

int                 RemoveFromPCStore(struct PCdataStore4List *, int *, int);
int                 RemoveFromPCStore6(struct PCdataStore6List *, int *, int);

void                PutOnUCStore(struct UCdataStore4List *, struct udpportdata4 *);
void                PutOnUCStore6(struct UCdataStore6List *, struct udpportdata6 *);

int                 ProcessUCStore(struct UCdataStore4List *, struct PCdataStore4List *);
int                 ProcessUCStore6(struct UCdataStore6List *, struct PCdataStore6List *);

int                 RemoveFromUCStore(struct UCdataStore4List *, int *, int);
int                 RemoveFromUCStore6(struct UCdataStore6List *, int *, int);

int                 WriteOutput(struct udpportdata4, struct PCdataStore4List *);
int                 WriteOutput6(struct udpportdata6, struct PCdataStore6List *);

#endif							/* _TT_DATAHANDLING_H */
