/*
 * vi:ts=4:sw=4:
 *
 * $Id: PacketCatcher.h,v 1.3 2007/03/23 14:18:12 franzsch Exp $
 *
 * Copyright (c) 1998-2003 RIPE NCC
 * 
 * All Rights Reserved
 * 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the author not be used in advertising or 
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.
 * 
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
 * AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
 */

#ifndef _TT_PACKETCATCHER_H
#define _TT_PACKETCATCHER_H

#include <sys/types.h>
#include <pcap.h>
#include <netinet/ip6.h>


#include "packet.h"
#include "control_daemon.h"

#define PC_MORE_FRAGMENTS 0X01
#define PC_FRAGMENT_TO_SMALL 0X02


/* PCAP parameters */

/* 
 *  length to capture should at least contain the header
 *  of the test traffic data packet. 
 */

// Override Interface, set by mcast install script
#define DEVICE NULL

#ifndef PCAPDEVICE
#ifdef __FreeBSD__
#define PCAPDEVICE	      "ep0"
#else
#define PCAPDEVICE	      "ef0"

#endif /*__FreeBSD */
#endif

#define OPTIMIZE		 1
#define PROMISC_MODE_ON	  1
#define PROMISC_MODE_OFF	 0
#define TIMEOUT		  0
#define LOOP_UNTIL_ERROR	-1


/* declarations & globals */

/* IPv4 specific PC data */

struct packetcatchdata4
{
	u_int               flags;	/* Flags see below */
	struct pps_val      gps_times;	/* Times for differential corrections */
	struct timeval      bpftime;	/* bpf packet timestamp */
	u_short             ip_id;	/* ip_header ID */
	u_short             ip_offset;	/* IP datagram offset */
	u_short             ip_length;	/* IP datagram length */
	u_long              packet_id;	/* test packet ID when available */
	char                source_addr[INET_ADDRSTRLEN];	/* dotted quat + null */
	char                dest_addr[INET_ADDRSTRLEN];	/* dotted quat + null */
};

#define PC_DATA_SIZE sizeof(struct packetcatchdata4)


/* IPv6 specific PC data */

struct packetcatchdata6
{
	char                source_addr[INET6_ADDRSTRLEN];
	char                dest_addr[INET6_ADDRSTRLEN];
	long                flow_label;	/* Implemented but not used, perhaps in the 
									 * future? [FF] */
	u_short             p_length;	/* Datagram length without 40 byte IPv6
									 * header */
	struct pps_val      gps_times;	/* Times for differential corrections */
	struct timeval      bpftime;	/* bpf packet timestamp */
	u_long              packet_id;	/* test packet ID when available */
};

#define PC_DATA_SIZE6 sizeof(struct packetcatchdata6)


pid_t               control_pid;
struct etherifdata  EtherData;

int                 AddToChar (const char *, char **, char *);
int                 CreateTCPDUMPargument (struct DelayProbeList*, char **);
int                 PacketCatcher (struct DelayProbeList*);
int                 PCdataOutput (struct packetcatchdata4);
int                 PCdataOutput6 (struct packetcatchdata6);
void                ProcessPacket (u_char *, const struct pcap_pkthdr *, const u_char *);


#endif /* _TT_PACKETCATCHER_H */
