/*
 * vi:ts=4:sw=4:
 *
 * $Id: UDPProcessor.h,v 1.3 2007/03/23 14:18:12 franzsch Exp $
 *
 * Copyright (c) 1998-2003 RIPE NCC
 * 
 * All Rights Reserved
 * 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the author not be used in advertising or 
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.
 * 
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
 * AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
 */

#ifndef _TT_UDPPOCCESSOR_H
#define _TT_UDPPOCCESSOR_H

#include "packet.h"
#include "control_daemon.h"

/* type and function declarations */

/* IPv4 specific UC data */

struct udpportdata4
{

	/* except where code notes otherwise, longs and shorts are in network byte
	 * order */

	u_long              packet_id;	/* test packet ID when available */
	long                datasize;
	struct test_packet_data packet_data;
	char                source_addr[INET_ADDRSTRLEN];	/* IPv4 addresses */
	u_short             source_port;
	char                dest_addr[INET_ADDRSTRLEN];
	u_short             dest_port;
};

#define UC_DATA_SIZE sizeof(struct udpportdata4)


/* IPv6 specific UC data */

struct udpportdata6
{

	/* except where code notes otherwise, longs and shorts are in network byte
	 * order */

	u_long              packet_id;	/* test packet ID when available */
	long                datasize;
	struct test_packet_data packet_data;
	char                source_addr[INET6_ADDRSTRLEN];	/* IPv6 addresses */
	u_short             source_port;
	char                dest_addr[INET6_ADDRSTRLEN];
	u_short             dest_port;
};

#define UC_DATA_SIZE6 sizeof(struct udpportdata6)

void                HandleSpecificPort (int);
int                 UCdataOutput (struct udpportdata4);
int                 UCdataOutput6 (struct udpportdata6);
int                 UDPProcessor (struct DelayProbeList *);
void                JoinMulticasts (struct sockaddr_in *);
void                JoinMulticasts6 (struct sockaddr_in6 *);
#endif /* _TT_UDPPOCCESSOR_H */
