#!/usr/local/bin/perl5
use lib qw(/usr/local/rrd/lib/perl);

#
# This script triggers an alarm for monitored rrds which are behaving "unnaturally"
# using the LTA/STA algorithm. Alarms for multiple rrds can be configured. 

#
# Modules
#
use RRDs;
use POSIX;
use Time::Local;
use Sys::Hostname;

$myhost = hostname;
$reply_email = "bitbucket\@ripe.net";

$data_area = "/data/mcast";
$config_area = $ENV{"TTROOT"}."/config/mcast";
if( defined( $ARGV[0] ) && $ARGV[0] eq "-T" ) {
   # Only do TTM stuff
   push @rrd_areas, $data_area."/rrd";
} elsif( defined( $ARGV[0] ) && $ARGV[0] eq "-C" ) {
   # Only do Custom Listener stuff
   push @rrd_areas, $data_area."/rrd/cust";
} else {
   # Do everything
   push @rrd_areas, $data_area."/rrd";
   push @rrd_areas, $data_area."/rrd/cust";
}

$alarm_file = $config_area."/alarms.conf";


foreach $rrd_area (@rrd_areas) {
opendir (DATADIR, $rrd_area) || die "Cannot open $rrd_area";


foreach $filename (readdir (DATADIR)) {
  if( $filename !~ /^.+\.rrd$/ || $filename eq "dval.rrd"  || $filename eq "rate.rrd" 
                               || $filename eq "delay.rrd" || $filename eq "loss.rrd" || $filename eq "psize.rrd" ) {
    next;
  }
  # Parse config file
  $found = 0;
  open(ALARMCONFIG, $alarm_file) or die("Error opening $config_area/alarms.conf: $!\n");
  while(<ALARMCONFIG>) {
    chop ;
    if (substr ($_, 0, 1) eq "#") {
      # Ignore Comments
      next;
    } elsif (substr( $_, 0, 5 ) eq "email" ) {
      @line = split(/\s/, $_);
      $email = $line[1];
    } elsif ($_.".rrd" eq $filename) {
      $found = 1;
    }     
  }
  close (ALARMCONFIG);

  # Check if we really have to do this file
  if( $found == 0 ) {
    next;
  }


  #
  # Calculate Long Term Percentiles
  #
  my $end = time;
  my $start = $end - 60*60*24*30;

  @dt = ();
  my ($st,$sp,$nm,$dt) = RRDs::fetch( $rrd_area."/".$filename, "AVERAGE", "-s", $start, "-e", $end );

  @strip = ();
  @sorted = ();
  # Remove all NANs. There is probably a better way to deal this.
  foreach $line (@$dt) {
    if( defined( @$line[0] ) ) {
      push @strip, @$line[0];
    }
  }
  @sorted = sort {$a <=> $b} @strip;
  $lt95 = $sorted[int(scalar(@sorted)*0.95)];
  $lt05 = $sorted[int(scalar(@sorted)*0.05)];

  #
  # Calculate Short Term Percentiles
  #
  $end = time;
  $start = $end - 60*30;

  @dt = ();
  ($st,$sp,$nm,$dt) = RRDs::fetch( $rrd_area."/".$filename, "AVERAGE", "-s", $start, "-e", $end );

  @strip = ();
  @sorted = ();
  # Replace the NANs with -1.
  foreach $line (@$dt) {
    if( defined( @$line[0] ) ) {
      push @strip, @$line[0];
    } else {
      push @strip, -1;
    }
  }
  # Remove the last element since it appears to be always NAN. Could not find out why...
  pop @strip;
  @sorted = sort {$a <=> $b} @strip;
  $st95 = $sorted[int(scalar(@sorted)*0.95)];
  $st05 = $sorted[int(scalar(@sorted)*0.05)];

  # See if an alarm for this rrd is already active and store all the active alarms for later use.
  @curralarms = ();
  open(CURRALARM, $config_area."/current_alarms") or die("Error opening $config_area/current_alarms: $!\n");
  $found = 0;
  while(<CURRALARM>) {
    chop ;
    push @curralarms, $_;
    if( $_ eq $filename ) {
      $found = 1;
    }
  }
  close(CURRALARM);
  if( $st05 > $lt95 || $st95 < $lt05 ) {
    # ALARM!!!
    if( $found == 0 ) {
      # Alarm not yet in current alarms file
      open(CURRALARM, ">> ".$config_area."/current_alarms") or die("Error opening $config_area/current_alarms: $!\n");
      print CURRALARM $filename."\n";

      # Generate the url for the plots
      if( $filename =~ /^(.+)\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\.\d+\.\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.rrd$/ ) {
        $url = "http://$myhost:10259/mcast/multiple.cgi?$2=$2&$1=$1&RRD_START=now-1day&RRD_END=start%2B1day&RRD_W=-h100&RRD_H=-w350&RRD_L=0&RRD_R=1&RRD_RIGID=-l0&LFORMAT=BOTH&Draw=Draw";
      } elsif ( $filename = /^(.*)\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.rrd$/ ) {
        $url = "http://$myhost:10259/mcast/multiple.cgi?$2=$2&$1=$1&RRD_START=now-1day&RRD_END=start%2B1day&RRD_W=-h100&RRD_H=-w350&RRD_L=0&RRD_R=1&RRD_RIGID=-l0&LFORMAT=BOTH&Draw=Draw";
      }
      # Send new alarm email 
      $subject = "Subject: ALARM: $myhost, RIPE NCC Multicast Monitoring!\n";
      $body = "There was abnormal behaviour in the \"$filename\" RRD on $myhost you selected for monitoring.\n
               The URL below should take you to relevant plot data.\n
               $url";
      open(SENDMAIL, "|sendmail -t") or die "Unable to use sendmail command: $!";
      print SENDMAIL "Reply-to: $reply_email\n";
      print SENDMAIL "From: $reply_email\n";
      print SENDMAIL "To: ".$email."\n";
      print SENDMAIL $subject."\n";
      print SENDMAIL "Content-type: text/plain\n\n";
      print SENDMAIL $body;
      close(SENDMAIL);
    }
  } else {
    # No Alarm, everything okay
    if( $found == 1 ) {
      # Alarm was previously in the current alarms file, remove it from there.
      open(CURRALARM, "> ".$config_area."/current_alarms") or die("Error opening $config_area/current_alarms: $!\n");
      foreach $calarm (@curralarms) {
        if( $calarm ne $filename ) {
          print CURRALARM $calarm."\n";
        }
      }
      # Send alarm removed email
      $subject = "Subject: Alarm Removed, $myhost, RIPE NCC Multicast Monitoring";
      $body = "The previously triggered alarm for \"$filename\" on $myhost was removed again. Things went back to normal.\n";
      open(SENDMAIL, "|sendmail -t") or die "Unable to use sendmail command: $!";
      print SENDMAIL "Reply-to: $reply_email\n";
      print SENDMAIL "From: $reply_email\n";
      print SENDMAIL "To: ".$email."\n";
      print SENDMAIL $subject."\n";
      print SENDMAIL "Content-type: text/plain\n\n";
      print SENDMAIL $body;
      close(SENDMAIL);
    }
  }
}
}
