#!/usr/local/bin/perl5
use lib qw(/usr/local/rrd/lib/perl);

use RRDs;
use POSIX;
use Time::Local;

#
# Where to find stuff
#
$data_area  = "/data/mcast";
$data_area6 = "/data/mcast6";
$rrd_area   = "/data/mcast/rrd/cust";
#$configfile = $ENV{"TTROOT"}."/config/configfile";
#$mcastsfile = $ENV{"TTROOT"}."/config/mcasts";
$ttroot     = $ENV{"TTROOT"};
$config_area = $ttroot."/config/mcast";
$myname     = `hostname`;
chop ($myname);


sub getRRD {
  my ($logfile) = @_;
  if ($logfile =~ /.CRCP\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,5}\.\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\.\d{8}/) {
    return $1;
  } elsif ($logfile =~ /.CRCP\.(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,5})\.\d{8}/) {
    return $1;
  } elsif ($logfile =~ /.CRC6\.([0-9A-Fa-f]{0,4}:([0-9A-Fa-f]{0,4}:){1,6}[0-9A-Fa-f]{1,4}\.\d{1,5}\.[0-9A-Fa-f]{0,4}:([0-9A-Fa-f]{0,4}:){1,6}[0-9A-Fa-f]{1,4})\.\d{8}/) {
    return $1;
  } elsif ($logfile =~ /.CRC6\.([0-9A-Fa-f]{0,4}:([0-9A-Fa-f]{0,4}:){1,6}[0-9A-Fa-f]{1,4}\.\d{1,5})\.\d{8}/) {
    return $1;
  }

}


#
# Get all custom listener configs into a hash of arrays.
#
open (CUSTLIST, $config_area."/custom_listener.conf");
while (<CUSTLIST>) {
  chomp ;
  @line = split (/\s/, $_);
  if (substr ($_, 0, 1) eq "#" || substr($_, 0, 4) eq "conf") {
    next;
  } elsif ($_ =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/ || $_ =~ /^([0-9A-Fa-f]{0,4}:([0-9A-Fa-f]{0,4}:){1,6}[0-9A-Fa-f]{1,4})/) {
    push @listeners, $_;
  }
}
close (CUSTLIST);

my %stat = ();

foreach $lineStr (@listeners) {
  @line = split (/\s/, $lineStr); 
  #Let's see what statistics we have to generate
  $stat{'rate'} = 1; # We can always plot the rate
  $stat{'psize'} = 1; # Packet size as well
  if( $line[2] != 0 ) { # Check if loss has to be done
    $stat{'loss'} = 1;
  } else {
    $stat{'loss'} = 0;
  }
  if( $line[3] ne "NoTimestamp" ) { # Check if Delays have to be done
    $stat{'delay'} = 1;
  } else {
    $stat{'delay'} = 0;
  }
  if( $line[6] ne "0" ) { # Defined Values have to be done
    $stat{'dval'} = 1;
  } else {
    $stat{'dval'} = 0;
  }
  # Find most recent CRCP files (last two days we have)
  @mfile_list  = ( );
  @dateh = ();
  @dates = ();
  if (-e $data_area) {
    # Get the last two days we have logfiles for.
    opendir (DATADIR, $data_area) || die "Cannot open $data_area";
    foreach $filename (sort readdir (DATADIR)) {
       if( substr($filename, 0, 5) eq "CRCP." ) {
          $date = substr($filename, length($filename)-8, 8);
          if( $date ne $dates[0] && $date ne $dates[1] ) {
             $dateh{$date}++;
          }
       }
    }
    closedir DATADIR;
    @dates = sort {$b <=> $a} keys %dateh;

    # Push all the files that are from the last two days in the queue
    opendir (DATADIR, $data_area) || die "Cannot open $data_area";
    foreach $filename (sort readdir (DATADIR)) {
      if (substr($filename, 0, 5) eq "CRCP." && substr($filename, 5, length($line[0].".".$line[1])) eq $line[0].".".$line[1]) {
        $date = substr($filename, length($filename)-8, 8);
        if( $date eq $dates[0] || $date eq $dates[1] ) {
          push @mfile_list, $data_area."/".$filename;
        }
      }
    }
    close (DATADIR);
  }

  # Same only for ipv6, though not yet in use or implemented.
  @dates = ( );
  %dateh = ( );
  if (-e $data_area6) {
    opendir (DATADIR6, $data_area6) || die "Cannot open $data_area6";
    foreach $filename (sort readdir (DATADIR6)) {
       if( substr($filename, 0, 5) eq "CRCP." ) {
          $date = substr($filename, length($filename)-8, 8);
          if( $date ne $dates[0] && $date ne $dates[1] ) {
             $dateh{$date}++;
          }
       }
    }
    closedir DATADIR6;
    @dates = sort {$b <=> $a} keys %dateh;
    
    opendir (DATADIR6, $data_area6) || die "Cannot open $data_area6";
    foreach $filename (sort readdir (DATADIR6)) {
      if (substr($filename, 0, 5) eq "CRCP." && substr($filename, 5, length($line[0].".".$line[1])) eq $line[0].".".$line[1]) {
        $date = substr($filename, length($filename)-8, 8);
        if( $date eq $dates[0] || $date eq $dates[1] ) {
          push @mfile_list, $data_area6."/".$filename;
        }
      }
    }
    close (DATADIR6);
  }


  $statInterval = $line[8];
  # Process files
  foreach $mfile (@mfile_list) {
    open (IN, $mfile)  || die "Failure on opening $mfile";

    while (<IN>) {
      chop $_;
      @words = split (/\s+/, $_);

      # Do the loss data
      # Do we have RRD files for this source?
      if( $stat{'loss'} == 1 ) {
        $rrdfile = $rrd_area."/loss.".getRRD($mfile).".rrd";
        if( !(-e $rrdfile) ) {
          $start = $words[5] - 1;
          `$ttroot/bin/mc_makerrd.closs $rrdfile $start $statInterval`;
        }
        # Suppress error messages when attempting to insert data twice
        $last = RRDs::last ($rrdfile);
        $words[5] = int($words[5]);
        if ($words[5] > $last) {
          $result   = sprintf ("%d:%f", $words[5], 1 - $words[3]/100);
          RRDs::update ($rrdfile, $result);
          my $ERR=RRDs::error;
          die "ERROR while updating $rrdfile: $ERR\n" if $ERR;
        }
      }
      # Do delay data
      if( $stat{'delay'} == 1 ) {
        $rrdfile = $rrd_area."/delay.".getRRD($mfile).".rrd";
        if( !(-e $rrdfile) ) {
          $start = $words[5] - 1;
          `$ttroot/bin/mc_makerrd.delay $rrdfile $start $statInterval`;
        }
        # Suppress error messages when attempting to insert data twice
        $last = RRDs::last ($rrdfile);
        $words[5] = int($words[5]);
        if ($words[5] > $last) {
          $result   = sprintf ("%d:%f", $words[5], $words[0]*1000);
          RRDs::update ($rrdfile, $result);
          my $ERR=RRDs::error;
          die "ERROR while updating $rrdfile: $ERR\n" if $ERR;
        }
      }
      # Do do the rate data 
      if( $stat{'rate'} == 1 ) {
        $rrdfile = $rrd_area."/rate.".getRRD($mfile).".rrd";
        if( !(-e $rrdfile) ) {
          $start = $words[5] - 1;
          `$ttroot/bin/mc_makerrd.rate $rrdfile $start $statInterval`;
        }
        # Suppress error messages when attempting to insert data twice
        $last = RRDs::last ($rrdfile);
        $words[5] = int($words[5]);
        if ($words[5] > $last) {
          $result   = sprintf ("%d:%f", $words[5], $words[4]);
          RRDs::update ($rrdfile, $result);
          my $ERR=RRDs::error;
          die "ERROR while updating $rrdfile: $ERR\n" if $ERR;
        }
      }
      # Do the defined value pass percentage data
      # Do we have RRD files for this source?
      if( $stat{'dval'} == 1 ) {
        $rrdfile = $rrd_area."/dval.".getRRD($mfile).".rrd";
        if( !(-e $rrdfile) ) {
          $start = $words[5] - 1;
          `$ttroot/bin/mc_makerrd.dval $rrdfile $start $statInterval`;
        }
        # Suppress error messages when attempting to insert data twice
        $last = RRDs::last ($rrdfile);
        $words[5] = int($words[5]);
        if ($words[5] > $last) {
          $result   = sprintf ("%d:%f", $words[5], $words[2]/100);
          RRDs::update ($rrdfile, $result);
          my $ERR=RRDs::error;
          die "ERROR while updating $rrdfile: $ERR\n" if $ERR;
        }
      }
      # Do the packet size 
      # Do we have RRD files for this source?
      if( $stat{'psize'} == 1 ) {
        $rrdfile = $rrd_area."/psize.".getRRD($mfile).".rrd";
        if( !(-e $rrdfile) ) {
          $start = $words[5] - 1;
          `$ttroot/bin/mc_makerrd.psize $rrdfile $start $statInterval`;
        }
        # Suppress error messages when attempting to insert data twice
        $last = RRDs::last ($rrdfile);
        $words[5] = int($words[5]);
        if ($words[5] > $last) {
          $result   = sprintf ("%d:%f", $words[5], $words[1]);
          RRDs::update ($rrdfile, $result);
          my $ERR=RRDs::error;
          die "ERROR while updating $rrdfile: $ERR\n" if $ERR;
        }
      }
    }
  }
}

`$ttroot/bin/mc_alarm -C`;
