#!/usr/local/bin/perl5
use lib qw(/usr/local/rrd/lib/perl);
# 
# This script fills the RRD DB's with delay and loss data from the most
# recent RCDP files.  Command line arguments "-D" and "-L" determine
# what to do: -D for delays, -L for losses, "-D -L" (either way around)
# for both.
#

#
# Modules
#
use RRDs;
use POSIX;
use Time::Local;

#
# Some functions
#
sub host_to_ip4 {
                               # Arguments
   my ($hostname) = $_[0];     # Hostname

   use Socket;

   $hostname = inet_aton($_[0]);
   if (defined $hostname) {
      return inet_ntoa($hostname);
   } else {
        return "FAILED";
   }
   return "UNKNOWN";
}
sub host_to_ip6 {
                               # Arguments
   my ($hostname) = $_[0];     # Hostname

   use Socket6;

   $hostname = gethostbyname2($_[0], AF_INET6);

   if (defined $hostname) {
      return inet_ntop(AF_INET6, $hostname);
   } else {
        return "FAILED"
   }
   return "UNKNOWN";
}


#
# Where to find stuff
#
$data_area  = "/data/mcast";
$data_area6 = "/data/mcast6";
$rrd_area   = "/data/mcast/rrd/";
$configfile = $ENV{"TTROOT"}."/config/mcast/configfile";
$mcastsfile = $ENV{"TTROOT"}."/config/mcast/mcasts";
$ttroot     = $ENV{"TTROOT"};
$myname     = `hostname`;
chop ($myname);


#
# What to do?
#
$delays = 0;
$losses = 0;

for ($i = 0 ; $i <= $#ARGV ; $i++) {
  if    ($ARGV[$i] eq "-D") { $delays = 1;}
  elsif ($ARGV[$i] eq "-L") { $losses = 1;}
  else  {die "Unknown argument $ARGV[$i]\n";}
}

if ( ($delays + $losses) == 0) {
  die "Nothing to do\n";
}

#
# Get all multicast addresses that this testbox is a member of.
#
@mcasts = ();
open (MCONFIG, $mcastsfile) || die "McastConfigFile: Cannot open config file";
while (<MCONFIG>) {
  chop ;
  if (substr($_, 0, 1) ne "#") {
    @line = split (/\s/, $_);
    $mcasts[@mcasts+0] = $line[0]; 
  }
}

#
# Get incoming rates from configfile for loss calculation
# Storing multicast rates in a seperate hash, since these have to be treated differntly
#
if ($losses == 1) {
  %rates = ();
  open (CONFIG, $configfile) ||  die "ConfigFile: Cannot open config file";
  while (<CONFIG>) {  
    chop ;
    # Unicast IPv4
    if (substr($_, 0, 4) eq "SEND") {
      @tail = split (/\s+/, $_);
      if ($tail[3] eq $myname) {
        $rates{ host_to_ip4($tail[2]) } = $tail[7];
      }
    }
    #Unicast IPv6
    if (substr($_, 0, 4) eq "6SND") {
      @tail = split (/\s+/, $_);
      if ($tail[3] eq $myname) {
        $rates{ host_to_ip6($tail[2]) } = $tail[7];
      }
    }  
    #Multicast IPv4 
    if (substr($_, 0, 4) eq "MSND") {
      @tail = split (/\s+/, $_);
      foreach $mcast (@mcasts) {
        if ($tail[3] eq $mcast) {
          $rates{ host_to_ip4($tail[3]) } += $tail[7];
        }
      }
    }
    #Multicast IPv6
    if (substr($_, 0, 4) eq "MSN6") {
      @tail = split (/\s+/, $_);
      foreach $mcast (@mcasts) {
        if ($tail[3] eq $mcast) {
          $rates{ host_to_ip6($tail[3]) } += $tail[7];
        }
      }
    }
  }
  close (CONFIG);
# 
# Calculate time interval for losses
#
 ($sec, $min, $hour, $day, $month, $year) = gmtime();
 $sec = 0;
 $min = 15 * floor($min/15);
 $end = timelocal($sec, $min, $hour, $day, $month, $year);
 $start = $end - 3600;
}

#
# Find most recent RCDP files
# It seems as the two file lists for unicast are not really necessary, 
# didn't change it though, since I don't want to break anything. Also
# file_list6 seems to be filled, but not used anywhere?? (fs)
#
#Unicasts
@file_list  = ("EMPTY", "EMPTY", "EMPTY", "EMPTY");
#Multicasts
@mfile_list  = ("EMPTY", "EMPTY", "EMPTY", "EMPTY");

if (-e $data_area) {
  opendir (DATADIR, $data_area) || die "Cannot open $data_area";
  foreach $filename (sort readdir (DATADIR)) {
    if (substr($filename, 0, 4) eq "RCDP") { 
      $file_list[0] = $file_list[1] ;
      $file_list[1] = $data_area."/".$filename;
    }
    if (substr($filename, 0, 4) eq "MRCP") {
      $mfile_list[0] = $mfile_list[1] ;
      $mfile_list[1] = $data_area."/".$filename;
    }
  }
  close (DATADIR);
}

if (-e $data_area6) {
  opendir (DATADIR6, $data_area6) || die "Cannot open $data_area6";
  foreach $filename (sort readdir (DATADIR6)) {
    if (substr($filename, 0, 5) eq "RCDP6") { 
      $file_list[2] = $file_list[3] ;
      $file_list[3] = $data_area."/".$filename;
    }
    if (substr($filename, 0, 5) eq "MRCP6") {
      $mfile_list[2] = $mfile_list[3] ;
      $mfile_list[3] = $data_area."/".$filename;
    }
  }
  close (DATADIR6);
}


#
# Process files
#
%received = ();
for ($i=0 ; $i<4 ; $i++)   {

  if ($mfile_list[$i] ne "EMPTY") {
    open (IN, $mfile_list[$i])  || die "Failure on opening $mfile_list[$i]";

    while (<IN>) {
      chop $_;
      @words = split (/\s+/, $_);

      # 
      # Losses section
      #

      # Moved this line out of the if statement to have the same results in the delay section wheater or not
      # loss calculation is selected. (fs) 
      $words[7] = $words[7] - $words[8];
      if ($losses == 1) {
        if ($words[7] > $start && $words[7] <= $end) {
          $src = $words[5];
          $received{$src} ++;
        }
      }

      #
      # Delay section
      #
      if ($delays == 1) { 
        # 
        # Clock status
        #
        if ( ! ( ($words[9] ne "0X107") ||
                 ($words[9] ne "0X1")   ||
                 ($words[9] ne "0X2107") ) ) {next;}

        if ( ! ( ($words[10] ne "0X107") ||
                 ($words[10] ne "0X1")   ||
                 ($words[10] ne "0X2107") ) ) {next;}

        #
        # Is there a RRD file for this source?  If not, create
        #
        $dest = $words[5];
        $rrdfile = $rrd_area.$dest.".rrd";
        if (!(-e $rrdfile)) { 
          # print "Creating file $rrdfile\n"; 
          $start_d = int($words[7]) - 1;
          `$ttroot/bin/mc_makerrd $rrdfile $start_d`;
        }

        #
        # Suppress error messages when attempting to insert data twice
        #
        $last = RRDs::last ($rrdfile);

        $words[7] = int($words[7]); 
        if ($words[7] > $last) {
          $result   = sprintf ("%d:%f", $words[7], $words[8]*1000);
          RRDs::update ($rrdfile, $result);
          my $ERR=RRDs::error;
          die "ERROR while updating $rrdfile: $ERR\n" if $ERR;

        }
      }
    }
    close (IN);
  }
  if ($file_list[$i] ne "EMPTY") {
    open (IN, $file_list[$i])  || die "Failure on opening $file_list[$i]";

    while (<IN>) {
      chop $_;
      @words = split (/\s+/, $_);

      # 
      # Losses section
      #
      $words[7] = $words[7] - $words[8];
      if ($losses == 1) {
        if ($words[7] > $start && $words[7] <= $end) {
          $src = $words[3];
          $received{$src} ++;
        }
      }

      #
      # Delay section
      #
      if ($delays == 1) { 
        # 
        # Clock status
        #
        if ( ! ( ($words[9] ne "0X107") ||
                 ($words[9] ne "0X1")   ||
                 ($words[9] ne "0X2107") ) ) {next;}

        if ( ! ( ($words[10] ne "0X107") ||
                 ($words[10] ne "0X1")   ||
                 ($words[10] ne "0X2107") ) ) {next;}

        #
        # Is there a RRD file for this source?  If not, create
        #
        $dest = $words[3];
        $rrdfile = $rrd_area.$dest.".rrd";
        if (!(-e $rrdfile)) { 
          # print "Creating file $rrdfile\n"; 
          $start_d = int($words[7]) - 1;
          `$ttroot/bin/mc_makerrd $rrdfile $start_d`;
        }

        #
        # Suppress error messages when attempting to insert data twice
        #
        $last = RRDs::last ($rrdfile);

        $words[7] = int($words[7]); 
        if ($words[7] > $last) {
          $result    = sprintf ("%d:%f", $words[7], $words[8]*1000);
          RRDs::update ($rrdfile, $result);
          my $ERR=RRDs::error;
          die "ERROR while updating $rrdfile: $ERR\n" if $ERR;
        }
      }
    }
    close (IN);
  }
}


if ($losses == 1) {
  #
  # Divide by rate, and fill RRD file.
  #
  foreach $entry (keys %received) {
    $rrdfile =  $rrd_area."loss.".$entry.".rrd";
    # print $rrdfile, " ", $received{$entry}/$rates{$entry}, "\n";
 
    # 
    # Create RRD file if necessary 
    #
    if (!(-e $rrdfile)) {
      `$ttroot/bin/mc_makerrd.loss $rrdfile $start-900`;
    }
    $last = RRDs::last ($rrdfile); 
  
    if ($end > $last) {
      $lost = 0.0;  
      $lost = $received{$entry}/2;
      if ($lost < 0.0) { $lost = 0.0; }    
      if ($lost > 1.0) { $lost = 1.0; }    
    print "Loss: $end:$lost, $received{$entry}\/$rates{$entry}\n";
      RRDs::update ($rrdfile, $end.":".$lost);
      my $ERR=RRDs::error;
      die "ERROR while updating $rrdfile: $ERR\n" if $ERR;
    }
  }
}

`$ttroot/bin/mc_alarm -T`;
