#!/usr/bin/perl

#
# This script monitors the custom listener, if one dies, it will restart it automatically
#

$ttroot = $ENV{"TTROOT"};
$config_area = $ttroot."/config/mcast";

# First parse configfile...
open (CUSTLIST, $config_area."/custom_listener.conf");
while (<CUSTLIST>) {
  chomp ;
  @line = split (/\s/, $_);
  if (substr ($_, 0, 1) eq "#") {
    next;
  } elsif (substr ($_, 0, 4) eq "conf") {
    # Get rid of the 'conf' string and save the configuration to our hash
    shift @line;
    $confs{$line[0]} = [ @line ];
  } elsif ($_ =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/ || $_ =~ /^([0-9A-Fa-f]{0,4}:([0-9A-Fa-f]{0,4}:){1,6}[0-9A-Fa-f]{1,4})/) {
    $listeners{$line[0].":".$line[1]} = [ @line ];
  }
}
close (CUSTLIST);


foreach (sort keys %listeners) {
    if( -e "/proc/".$listeners{$_}[9] ) {
      # All Ok, listener is running
    } else {
      # We got a problem there, let's restart it
      @list = @{$listeners{$_}};
      # Fork a new process and start our listener in there 
      $pid = fork();
      die "fork() failed: $!" unless defined $pid;
      if ($pid == 0) # child
      {
         # Daemonise and exec listener
         setsid;
         chdir '/';
         umask 0;
         open STDIN, '/dev/null';
         open STDOUT, '>/dev/null';
         open STDERR, '>/dev/null';
         $srcstat = $list[10] eq "on" ? "1" : "0";
         exec($ttroot."/bin/McastCListener ".$list[0]." ".$list[1]." ".$list[2]." ".$list[3]." ".$list[4]." ".
                               $list[5]." ".$list[6]." ".$list[7]." ".$list[8]." "."/data/mcast ".$srcstat) || die "exec failed: $!";
      } else {
        # Listerner should be running again, now we need to rewrite the config file to mamke sure it still has the correct pid
        $listeners{$_}[9] = $pid;
        # And rewrite our config file. 
        open (CUSTLIST, ">".$config_area."/custom_listener.conf");
        foreach (sort keys %listeners) {
          print CUSTLIST $listeners{$_}[0]." ".$listeners{$_}[1]." ".$listeners{$_}[2]." ";
          print CUSTLIST $listeners{$_}[3]." ".$listeners{$_}[4]." ".$listeners{$_}[5]." ";
          print CUSTLIST $listeners{$_}[6]." ".$listeners{$_}[7]." ".$listeners{$_}[8]." ";
          print CUSTLIST $listeners{$_}[9]." ".$listeners{$_}[10]."\n";
        }
        foreach (sort keys %confs) {
          print CUSTLIST "conf ".$confs{$_}[0]." ".$confs{$_}[1]." ".$confs{$_}[2]." ".$confs{$_}[3]." ".$confs{$_}[4]." ";
          print CUSTLIST $confs{$_}[5]." ".$confs{$_}[6]." ".$confs{$_}[7]."\n";
        }
        close (CUSTLIST);
      }
   }
}

