#include <stdio.h>
#include <errno.h>
#include <termios.h>
#include <sys/types.h>
# include <sys/ioctl.h>
# include <sys/time.h>

int
setline(int fd, int speed)
{
	struct termios ttyb, *ttyp;
	u_long ltemp;
	struct timeval dcd_time;

	int debug = 0;

	ttyp = &ttyb;

	if (tcgetattr(fd, ttyp) < 0) {
		fprintf(stderr,
		    "fd %d tcgetattr %m", fd);
		return (0);
	}

	(void)cfsetispeed(&ttyb, speed);
	(void)cfsetospeed(&ttyb, speed);

	ttyp->c_iflag = IGNBRK;
	ttyp->c_oflag = 0;
	ttyp->c_cflag = CS8 | PARENB| PARODD | CLOCAL | CREAD;
	ttyp->c_lflag = 0;
	ttyp->c_cc[VERASE] = ttyp->c_cc[VKILL] = '\0';

	/*
	 * If we have modem control, check to see if modem leads are
	 * active; if so, set remote connection. This is necessary for
	 * the kernel pps mods to work.
	 */
	ltemp = 0;
	if (ioctl(fd, TIOCMGET, (char *)&ltemp) < 0)
		fprintf(stderr,
		    "setline: fd %d TIOCMGET failed: %m", fd);
	if (debug)
		printf("setline: fd %d modem status %lx\n",
		    fd, ltemp);
	if (ltemp & TIOCM_DSR)
		ttyp->c_cflag &= ~CLOCAL;
	if (tcsetattr(fd, TCSANOW, ttyp) < 0) {
		fprintf(stderr,
		    "setline: fd %d TCSANOW failed: %m", fd);
		return (0);
	}
	if (tcflush(fd, TCIOFLUSH) < 0) {
		fprintf(stderr,
			"setline: fd %d TCIOFLUSH failed: %m", fd);
		return(0);
	}

#ifdef TIOCDCDTIMESTAMP
	if(ioctl(fd, TIOCDCDTIMESTAMP, &dcd_time) < 0 ) {
		fprintf(stderr,
			"setline: fd %d TIOCDCDTIMESTAMP failed: %m", fd);
		return(0);
	}
#endif

	return (1);
}

static const char rcsid[] ="$Id: ttystuff.c,v 1.3 2006/11/04 15:28:54 ruben Exp $";
