/*
 * $Id: trimble.h,v 1.1.1.1 2004/12/09 17:01:48 tom Exp $
 */

#ifndef __TRIMBLE_H

#define __TRIMBLE_H

#include <sys/queue.h>
#include <sys/select.h>
#include <sys/timepps.h>
#include <refclocksync.h>

enum trimble_state { ST_UNCONFIGURED,           // right after initialization of the driver
                     ST_SERIAL_SETUP_COMPLETE,  // serial port configured, ready for comm. with the trimble
		     ST_TRIMBLE_POS_FIX,        // trimble in position fix state
                     ST_TRIMBLE_CLOCK           // trimble in overdetermined clock mode
};

#define MAX_RPTBUF (256)

#define GPS_C     (299792458.)
#define GPS_PI    (3.1415926535898)
#define R2D	(180.0/GPS_PI)

// log a bulk of lines to clockstats every n seconds
#define CLOCKSTATS_LOG_FREQUENCY 64

// trimble heartbeat every 10 seconds
#define TRIMBLE_TIMER 10

typedef struct {
   enum trimble_state state;

   // serial parameters
   char *serial;
   int serial_fd;
   int serial_idx;
   u_char serial_buf[1024];

   // PPS parameters
   double pps_cable_length;
   pps_handle_t ppsh;

   // do we have UTC lock?
   int utc_lock;

} trimble_t;

/* TSIP packets have the following structure, whether report or command. */
typedef struct {
	unsigned short 
		counter, 	/* counter */
		len;		/* size of buf; < MAX_RPTBUF unsigned chars */
	unsigned char
		status,		/* TSIP packet format/parse status */
		code,		/* TSIP code */
		buf[MAX_RPTBUF];/* report or command string */
} TSIPPKT;

/* trimble_serial.c */
int trimble_serconf(trimble_t *data);
void trimble_sendmsg(trimble_t *data, TSIPPKT *pkt);
void trimble_sendcode(trimble_t *data, unsigned char code);
void trimble_sendcode_subcode(trimble_t *data, unsigned char code, unsigned char subcode);
void trimble_hexdump(int logflags, unsigned char *buffer, long len);
void trimble_input_ready(struct __refclocksync_driver_t *driver, fd_set *rdfd);

/* trimble_protocol.c */
void trimble_protocol_init(trimble_t *data);
void trimble_do_protocol(trimble_t *data, TSIPPKT *pkt);
void trimble_alarm(struct __refclocksync_driver_t *driver);

/* trimble_utils.c */
void trimble_bPutShort (short *in, unsigned char *out);
void trimble_bPutULong (unsigned long *in, unsigned char *out);
void trimble_bPutFloat (float *in, unsigned char *out);
void trimble_bPutDouble (double *in, unsigned char *out);
unsigned short trimble_bGetUShort (unsigned char *bp);
long trimble_bGetLong (unsigned char *bp);
unsigned long trimble_bGetULong (unsigned char *bp);
float trimble_bGetSingle (unsigned char *bp);
double trimble_bGetDouble (unsigned char *bp);

/* trimble.c */
int trimble_initialize();

#endif
