#!/bin/sh
#
# $Id#
#

# check-datataking - check that data files are created as expected

# Version 0.01 - daily check of GENE.*

# Note: for final version will need a mechanism to tell
#    what the status of the machine is, i.e. what data
#    should be produced
#
#    Also some more intelligence: if file exists but has not been
#    updated in XX minutes, restart processes.

program=$0
PATH="/home/ttraffic/bin:/usr/local/bin:/bin:/usr/bin"
datadir=/data/ttraffic
today=`date +%Y%m%d`
genefile=GENE.`hostname`.$today

if [ ! -f ${datadir}/$genefile ]
then
	# group commands, so we can pipe all output to sendmail
	{

	# Mail header:
	# set From: such that replies can be delivered
	# tt-boxes don't accept mail, so make it tt-ops@ripe.net

	echo "From: `hostname` check-datataking <tt-ops@ripe.net>"
	echo "To:  tt-ops@ripe.net"
	echo "Date: `date`"
	echo "Subject: AUTO: data file rotation/creation problem"
	echo 
	echo 
	
	# Mail body:

	echo "[Note: this mail was generated by $program]" 
	echo
	echo "ERROR: couldn't see $genefile in $datadir"
	echo "trying to restart clock_monitor and log_status ..."
	echo

	clockmon_pid=`ps axwwu|grep clock_monitor|grep -v grep|awk '{print $2}'`
	if [ "$clockmon_pid" ]
	then
		# found one, try to kill it
		kill $clockmon_pid
		sleep 15
		clockmon_pid=`ps axwwu |\
			grep clock_monitor|grep -v grep|awk '{print $2}'`
		
		# still there? blast it out of existence
		[ "$clockmon_pid" ] && kill -9 $clockmon_pid
	fi


	logstat_pid=`ps axwwu |grep log_status|grep -v grep|awk '{print $2}'`
	if [ "$logstat_pid" ]
	then
		# found one, try to kill it
		kill $logstat_pid
		sleep 15
		logstat_pid=`ps axwwu |\
			grep log_status|grep -v grep|awk '{print $2}'`

		# still there? blast it out of existence
		[ "$logstat_pid" ] && kill -9 $logstat_pid
	fi

	# remove pid_list files
	rm -f /data/ttraffic/pid_list/clock_monitor 
	rm -f /data/ttraffic/pid_list/log_status  

	# restart processes, have to redir to /dev/null to detach
	# stdout of started processes from this shell

	start_processes.pl >/dev/null 2>&1
	
	tries=10
	while [ $tries -gt 0 ]
	do
		if [ -f ${datadir}/$genefile ]
		then
			break
		else
			sleep 15
		fi
		tries=`expr $tries - 1`
	done

	if [ -f ${datadir}/$genefile ]
	then
		echo "... seems ok now:"
		ls -l ${datadir}/$genefile 
	else
		echo "`date`: $genefile still non existant :-("
	fi

	} | /usr/sbin/sendmail tt-ops@ripe.net
fi
