#!/usr/local/bin/perl5
require 5.002;
use lib qw(/home/ttraffic/lib);
use lib qw(/usr/local/rrd/lib/perl);

#
# This script fills the RRD DB with delays from the most recent
# RCDP files.
#

use RRDs;
use POSIX;
use Time::Local;
use ConfigFile;

#
# Where to find stuff
#
$data_area  = $ENV{"TTROOT"}."/data";
#$data_area  = "/data/ttraffic";
$data_area6  = $ENV{"TTROOT"}."/data6";

$rrd_area   = "/data/rrd/";  
$configfile = $ENV{"TTROOT"}."/config/configfile";
$myname     = `hostname`;
chop ($myname);

#
# Get incoming rates from configfile
#
%rates = ();
open (CONFIG, $configfile) ||  die "ConfigFile: Cannot open config file";
while (<CONFIG>) {  
  chop ;
  if (substr($_, 0, 4) eq "SEND") {
    @tail = split (/\s+/, $_);
    if ($tail[3] eq $myname) {
      $rates{ host_to_ip4($tail[2]) } = $tail[7];
    }
  }

  if (substr($_, 0, 4) eq "6SND") {
    @tail = split (/\s+/, $_);
    if ($tail[3] eq $myname) {
      $rates{ host_to_ip6($tail[2]) } = $tail[7];
    }
  }
}
close (CONFIG);

#
# Find most recent RCDP files
#
@file_list = ("EMPTY", "EMPTY", "EMPTY") ;
if (-e $data_area) {
	opendir (DATADIR, $data_area) || die "Cannot open $data_area";

	foreach $filename (sort readdir (DATADIR)) {

	  if (substr($filename, 0, 4) ne "RCDP") { next ; }

	  $file_list[0] = $file_list[1] ;
	  $file_list[1] = $file_list[2] ;
	  $file_list[2] = $data_area."/".$filename;

	}
	close (DATADIR);
}

@file_list6 = ("EMPTY", "EMPTY", "EMPTY") ;
if (-e $data_area6) {
	opendir (DATADIR6, $data_area6) || die "Cannot open $data_area6";

	foreach $filename (sort readdir (DATADIR6)) {

	  if (substr($filename, 0, 5) ne "RCDP6") { next ; }

	  $file_list6[0] = $file_list6[1] ;
	  $file_list6[1] = $file_list6[2] ;
	  $file_list6[2] = $data_area6."/".$filename;

	}
	close (DATADIR6);
}



# 
# Calculate time interval
#
($sec, $min, $hour, $day, $month, $year) = gmtime();
$min = 15 * floor($min/15);
$end = timelocal(0, $min, $hour, $day, $month, $year);
$start = $end - 3600;

#
# Count # of packets from each TB
#
%received = ();
for ($i=0 ; $i<3 ; $i++)   {

  if ($file_list[$i] ne "EMPTY") {
    open (IN, $file_list[$i])  || die "Failure on opening $file_list[$i]";

    while (<IN>) {
      chop $_;
      @words = split (/\s+/, $_);

      $words[7] = $words[7] - $words[8];
      if ($words[7] > $start && $words[7] <= $end) {
        $src = $words[3];
        $received{$src} ++;
      }
    }
    close (IN);
  }
}

for ($i=0 ; $i<3 ; $i++)   {

  if ($file_list6[$i] ne "EMPTY") {
    open (IN6, $file_list6[$i])  || die "Failure on opening $file_list6[$i]";

    while (<IN6>) {
      chop $_;
      @words = split (/\s+/, $_);

      $words[7] = $words[7] - $words[8];
      if ($words[7] > $start && $words[7] <= $end) {
        $src = $words[3];
        $received{$src} ++;
      }
    }
    close (IN6);
  }
}


#
# Divide by rate, and fill RRD file.
#
foreach $entry (keys %received) {
  $rrdfile =  $rrd_area."loss.".$entry.".rrd";
  # print $rrdfile, " ", $received{$entry}/$rates{$entry}, "\n";
 
  # 
  # Create RRD file if necessary 
  #
  if (!(-e $rrdfile)) {
    `/home/ttraffic/bin/makerrd.loss $rrdfile $start-900`;
  }

  $last = RRDs::last ($rrdfile); 
  # print "$start $last $end\n";
  
  if ($end > $last) {
    $lost = 0.0;  
    if ($rates{$entry} != 0) { $lost = $received{$entry}/$rates{$entry} ;}
    if ($lost < 0.0) { $lost = 0.0; }    
    if ($lost > 1.0) { $lost = 1.0; }    
    RRDs::update ($rrdfile, $end.":".$lost);
    my $ERR=RRDs::error;
    die "ERROR while updating $rrdfile: $ERR\n" if $ERR;
  }
}


