/*
 * vi:ts=4:sw=4:
 *
 * $Id: measurement.c,v 1.7 2006/06/21 13:40:57 ruben Exp $
 *
 * Copyright (c) 1998-2003 RIPE NCC
 * 
 * All Rights Reserved
 * 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the author not be used in advertising or 
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.
 * 
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
 * AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
 */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <unistd.h>

#include "send_packet.h"
#include "measurement.h"
#include "utils.h"

static char const rcsid[] = "$Id: measurement.c,v 1.7 2006/06/21 13:40:57 ruben Exp $";

/* Create another entry in the array of measurements 

  Parameters: 
    struct target *params : Pointer to parameters for this measurement 
    int offset:             Offset into the array with times
    struct measurement:     Pointer to next entry in the array

  Returns:
    pointer to the data of this measurment 

*/
struct measurement *create_measurement (struct target *params, int offset, struct measurement *next_in_chain)
{
	struct measurement *new;

	if ((new = (struct measurement *) malloc (sizeof (struct measurement))) == NULL)
	{
		perror("create_measurement");
		abort();
	}
	memset(new, 0, sizeof(struct measurement *));

	new->next = next_in_chain;
	new->name = params->name;
	new->port = params->port;
	new->size = params->size;
	new->burst = params->burst;
	new->time = *(params->time + offset);
	new->family = params->family;

	memcpy (&new->target, &params->target, sizeof (params->target));

	return new;

}

/* 
 * Print all scheduled measurements
 * 
 * start = first measurement in the chain
 * 
 */
void list_measurement (struct measurement *start)
{
	while (start != NULL)
	{
		printf ("Dest %s Port %d Size %d Time %15.3f Burst %d Family %d\n",
				start->name, start->port, start->size, start->time, start->burst, start->family);
		start = start->next;
	}
	return;
}
