/*
 * vi:ts=4:sw=4:
 *
 * $Id: poisson.h,v 1.2 2003/10/17 10:49:10 ruben Exp $
 *
 * Copyright (c) 1998-2003 RIPE NCC
 * 
 * All Rights Reserved
 * 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the author not be used in advertising or 
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.
 * 
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
 * AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
 */
#ifndef POISSON_H
#define POISSON_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "send_packet.h"

 /* For poisson */
#define PI 3.141592654

 /* For ran1 */
#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

 /* TTM specific */
 /* All measurements should be done before a fraction ALL_DONE of the period
  * has elapsed */
#define ALL_DONE     0.99
 /* Minimum interval between two measurements, in ms */
#define MIN_INTERVAL 1.00

 /* Function templates */
float               poisson (float xm, long *idum);
int                 poisson_saveseed (int, char *, long);
long                poisson_getseed (int, char *);
float               gammaln (float xx);
float               ran1 (long *idum);
long                do_poisson (int, struct target *, int, long);

#endif
