/*
 * vi:ts=4:sw=4:
 *
 * $Id: send_packet.h,v 1.8 2003/10/17 10:49:10 ruben Exp $
 *
 * Copyright (c) 1998-2003 RIPE NCC
 * 
 * All Rights Reserved
 * 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the author not be used in advertising or 
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.
 * 
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
 * AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
 */
#ifndef SEND_PACKET_H
#define SEND_PACKET_H

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include "sleep.h"

#define  FALSE (0)
#define  TRUE  (!FALSE)

#define MAXLINE 100


/* 
 * These flags are bits (from LSB to MSB), you can extend them if you need more
 * options in the future (e.g. RES1_FLAG 4, RES2_FLAG 8 ...)  [FF]
 */


#define V4_FLAG 1
#define V6_FLAG 2


/* For the new packet id code */
#define LASTUNIXTIME 1024488000	/* Wed Jun 19 12:00:00 2002 GMT */
#define TIMEZERO     1009843200	/* Tue Jan 1 00:00:00 2002 GMT */
#define CYCLEPERIOD  20000000	/* cycle period in seconds */
#define FREQUENCY    100		/* Max frequency */


/* Struct with global measurement data */
/* =================================== */

struct global
{
	time_t              start;	/* Start time of measurements */
	time_t              period;	/* Period length (seconds) */
	time_t              read;	/* Time when configfile was last read */
}
globaltime;

/* Struct with the measurements for each testbox */
/* ============================================= */

struct target
{
	struct target      *next;	/* Next element in the chain */
	char               *name;	/* Name of the testbox */
	int                 port;	/* Port to send to */
	int                 size;	/* Number of bytes in packet */
	int                 rate;	/* Rate (packets/period) */
	float              *time;	/* Pointer to an array of floats indicating the
								 * time of each measurement */
	int                 seed;	/* Random seed */
	int                 burst;	/* Number of packets to send in burst mode */
	int                 linear;	/* Linear (=TRUE) or Poisson (=False)
								 * distribution */
	int                 family;	/* AF_INET or AF_INET6 */
	union
	{
		struct in_addr      addr;
		struct in6_addr     addr6;
	}
	target;
};


/* Struct with the individual measurements */
/* ======================================= */

struct measurement
{
	struct measurement *next;	/* Pointer to the next element in the chain */
	char               *name;	/* Name of the testbox */
	int                 port;	/* Port to send to */
	int                 size;	/* Size of the packet */
	double              time;	/* Time to send the packet */
	int                 burst;	/* Number of packets in burst mode */
	int                 family;	/* AF_INET or AF_INET6 */
	union
	{
		struct in_addr      addr;
		struct in6_addr     addr6;
	}
	target;
};

int                 read_configfile (int, char *);
void                create_logfile (char *, char *, int);
int                 record_pid (int);
int                 remove_pid (int);
void                exitcode (int);
char               *host_err_str (void);
int                 udp_open (char *, char *, int, int, int, struct measurement *);
struct test_packet_data *SetupPacket (int, char *, u_int, long);
int                 SendPacket (int, struct measurement *, struct sockaddr *, socklen_t *);


#endif
