#!/usr/local/bin/perl5 -w

# 
# This script fills the RRD DB's with delay and loss data from the most
# recent RCDP files.  Command line arguments "-D" and "-L" determine
# what to do: -D for delays, -L for losses, "-D -L" (either way around)
# for both.
#

#
# Modules
#
use RRDs;
use POSIX;
use Time::Local;
use ConfigFile;

#
# Where to find stuff
#
$data_area  = $ENV{"TTROOT"}."/data";
$data_area6 = $ENV{"TTROOT"}."/data6";
$rrd_area   = "/data/rrd/";
$configfile = $ENV{"TTROOT"}."/config/configfile";
$myname     = `hostname`;
chop ($myname);

#
# What to do?
#
$delays = 0;
$losses = 0;

for ($i = 0 ; $i <= $#ARGV ; $i++) {
  if    ($ARGV[$i] eq "-D") { $delays = 1;}
  elsif ($ARGV[$i] eq "-L") { $losses = 1;}
  else  {die "Unknown argument $ARGV[$i]\n";}
}

if ( ($delays + $losses) == 0) {
  die "Nothing to do\n";
}

#
# Get incoming rates from configfile for loss calculation
#
if ($losses == 1) {
  %rates = ();
  open (CONFIG, $configfile) ||  die "ConfigFile: Cannot open config file";
  while (<CONFIG>) {  
    chop ;
    if (substr($_, 0, 4) eq "SEND") {
      @tail = split (/\s+/, $_);
      if ($tail[3] eq $myname) {
        $rates{ host_to_ip4($tail[2]) } = $tail[7];
      }
    }
    if (substr($_, 0, 4) eq "6SND") {
      @tail = split (/\s+/, $_);
      if ($tail[3] eq $myname) {
        $rates{ host_to_ip6($tail[2]) } = $tail[7];
      }
    }
  }
  close (CONFIG);
# 
# Calculate time interval for losses
#
 ($sec, $min, $hour, $day, $month, $year) = gmtime();
 $sec = 0;
 $min = 15 * floor($min/15);
 $end = timelocal($sec, $min, $hour, $day, $month, $year);
 $start = $end - 3600;
}

#
# Find most recent RCDP files
#
@file_list  = ("EMPTY", "EMPTY", "EMPTY", "EMPTY");

if (-e $data_area) {
  opendir (DATADIR, $data_area) || die "Cannot open $data_area";
  foreach $filename (sort readdir (DATADIR)) {
    if (substr($filename, 0, 4) ne "RCDP") { next ; }
    $file_list[0] = $file_list[1] ;
    $file_list[1] = $data_area."/".$filename
  }
  close (DATADIR);
}

if (-e $data_area6) {
  opendir (DATADIR6, $data_area6) || die "Cannot open $data_area6";
  foreach $filename (sort readdir (DATADIR6)) {
    if (substr($filename, 0, 5) ne "RCDP6") { next ; }
    $file_list6[2] = $file_list6[3] ;
    $file_list6[3] = $data_area6."/".$filename;
  }
  close (DATADIR6);
}


#
# Process files
#

%received = ();
for ($i=0 ; $i<4 ; $i++)   {

  if ($file_list[$i] ne "EMPTY") {
    open (IN, $file_list[$i])  || die "Failure on opening $file_list[$i]";

    while (<IN>) {
      chop $_;
      @words = split (/\s+/, $_);

      # 
      # Losses section
      #

      if ($losses == 1) {
        $words[7] = $words[7] - $words[8];
        if ($words[7] > $start && $words[7] <= $end) {
          $src = $words[3];
          $received{$src} ++;
        }
      }

      #
      # Delay section
      #
      if ($delays == 1) { 
        # 
        # Clock status
        #
        if ( ! ( ($words[9] ne "0X107") ||
                 ($words[9] ne "0X1")   ||
                 ($words[9] ne "0X2107") ) ) {next;}

        if ( ! ( ($words[10] ne "0X107") ||
                 ($words[10] ne "0X1")   ||
                 ($words[10] ne "0X2107") ) ) {next;}

        #
        # Is there a RRD file for this source?  If not, create
        #
        $dest = $words[3];
        $rrdfile = $rrd_area.$dest.".rrd";
        if (!(-e $rrdfile)) { 
          # print "Creating file $rrdfile\n"; 
          $start = int($words[7]) - 1;
          `/usr/home/ttraffic/bin/makerrd $dest $start`;
        }

        #
        # Suppress error messages when attempting to insert data twice
        #
        $last = RRDs::last ($rrdfile);

        $words[7] = int($words[7]); 
        if ($words[7] > $last) {
          $dest    = sprintf ("%d:%f", $words[7], $words[8]*1000);
          RRDs::update ($rrdfile, $dest);
          my $ERR=RRDs::error;
          die "ERROR while updating $rrdfile: $ERR\n" if $ERR;
        }
      }
    }
    close (IN);
  }
}


if ($losses == 1) {
  #
  # Divide by rate, and fill RRD file.
  #
  foreach $entry (keys %received) {
    $rrdfile =  $rrd_area."loss.".$entry.".rrd";
    # print $rrdfile, " ", $received{$entry}/$rates{$entry}, "\n";
 
    # 
    # Create RRD file if necessary 
    #
    if (!(-e $rrdfile)) {
      `/home/ttraffic/bin/makerrd.loss $rrdfile $start-900`;
    }

    $last = RRDs::last ($rrdfile); 
  
    if ($end > $last) {
      $lost = 0.0;  
      if ( ( exists($rates{$entry}) )  && ( $rates{$entry} != 0)) {
        $lost = $received{$entry}/$rates{$entry} ;}
      if ($lost < 0.0) { $lost = 0.0; }    
      if ($lost > 1.0) { $lost = 1.0; }    
      # print "$start $last $end $lost \n";

      RRDs::update ($rrdfile, $end.":".$lost);
      my $ERR=RRDs::error;
      die "ERROR while updating $rrdfile: $ERR\n" if $ERR;
    }
  }
}
