#!/usr/bin/perl

# This is a user interface to configure custom multicast listeners
# TODO:  Add some help for the user

use POSIX qw(setsid);
use Sys::Hostname;

require "header.pl";
require "config.pl";
$config_area = $ttroot."/config/mcast";
$data_area = "/data/mcast";

# When we are already at it check if everything is still fine...
`TTROOT=$ttroot $ttroot/bin/mc_monitor`;

# Parse GET Strings
$buffer = "";
$buffer = $ENV{'QUERY_STRING'};

#
# Replace +-signs by spaces.
#
$_ = $buffer;
s/\+/ /g;
s/%3A/:/g;
s/%2F/\//g;
s/%2C/,/g;
s/%2B/+/g;
s/%40/@/g;
$buffer = $_;

@input_line = split (/&/, $buffer);

%fields = ();
foreach $entry (@input_line) {
  ($head, $tail) = split(/=/, $entry);
  $fields{$head} = $tail;
}

# Parse POST Strings
$buffer = "";
read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
#
# Replace +-signs by spaces.
#
$_ = $buffer;
s/\+/ /g;
s/%3A/:/g;
s/%2F/\//g;
s/%2C/,/g;
s/%2B/+/g;
s/%40/@/g;
$buffer = $_;

@input_line = split (/&/, $buffer);

foreach $entry (@input_line) {
  ($head, $tail) = split(/=/, $entry);
  $fields{$head} = $tail;
}

# Check if custom_listeners.conf exists, create it with default values if not
if( !(-e "$config_area/custom_listener.conf") ) {
   open( CONFIGFILE, ">$config_area/custom_listener.conf" );
   print CONFIGFILE "conf dbeacon 0 dbeaconTimestamp 8 0 beac 0 60\n";
   print CONFIGFILE "conf dbeaconSize 0 dbeaconTimestamp 8 12 beac 0 60\n";
   close( CONFIGFILE );
}

# First parse configfile...
open (CUSTLIST, $config_area."/custom_listener.conf");
while (<CUSTLIST>) {
  chomp ;
  @line = split (/\s/, $_);
  if (substr ($_, 0, 1) eq "#") {
    next;
  } elsif (substr ($_, 0, 4) eq "conf") {
    # Get rid of the 'conf' string and save the configuration to our hash
    shift @line;
    $confs{$line[0]} = [ @line ];
  } elsif ($_ =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/ || $_ =~ /^([0-9A-Fa-f]{0,4}:([0-9A-Fa-f]{0,4}:){1,6}[0-9A-Fa-f]{1,4})/) {
    $listeners{$line[0].":".$line[1]} = [ @line ];
  }
}
close (CUSTLIST);

@error;

if (exists $fields{'run'} || exists $fields{'runsave'}) {
  # One of the Submit buttons was pressed, let's validate the input
 
  # Check if IP is valid
  if ($fields{'mcast'} =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/) {
    $ipversion = 4;
  } elsif ($fields{'mcast'} =~ /^([0-9A-Fa-f]{0,4}:([0-9A-Fa-f]{0,4}:){1,6}[0-9A-Fa-f]{1,4})$/) {
    $ipversion = 6;
  } else {
    push @error, "Multicast Address not valid. Please supply an IPv4 or IPv6 address";
  }

  # Check if SSM IP is valid
  if ($fields{'ssm'} =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/ && $ipversion == 4 ) {
    ;
  } elsif ($fields{'ssm'} =~ /^([0-9A-Fa-f]{0,4}:([0-9A-Fa-f]{0,4}:){1,6}[0-9A-Fa-f]{1,4})$/ && $ipversion == 6) {
    ;
  } elsif ($fields{'ssm'} eq "") {
    ;
  } else {
    push @error, "SSM IP Address not valid. Please supply an IPv4 or IPv6 address";
  }

  # Check if port is valid
  if ($fields{'port'} !~ /^[0-9]+$/ || $fields{'port'} < 0) {
    push @error, "Port Number not valid.";
  }

  # Check if Listener is already there
  if ($fields{'mcast'} ne "" && $fields{'port'} ne "" && exists $listeners{ $fields{'mcast'}.":".$fields{'port'} } &&
      $listeners{ $fields{'mcast'}.":".$fields{'port'} }[0] eq $fields{'mcast'} && 
      $listeners{ $fields{'mcast'}.":".$fields{'port'} }[1] eq $fields{'port'}) {
    push @error, "Listener already configured.";
  }

  # Only check these if we have a new configuration, otherwise ignore them
  if ($fields{'predef'} eq "New Configuration") {
    # Limit configuration name to 20 chars
    $fields{'cfname'} = substr ($fields{'cfname'}, 0, 20);

    # Check if configuration name is valid
    if ($fields{'cfname'} =~ /\s/ || $fields{'cfname'} eq "") {
      push @error, "Configuration Name not valid (no whitespace allowed).";
    }
    if (exists $confs{$fields{'cfname'}}) {
      push @error, "Configuration already exists.";
    }    

    # Check if rate is valid
    if ($fields{'rate'} !~ /^[0-9]+$/ || $fields{'rate'} < 0) {
      push @error, "Rate not valid.";
    }
    
    # Check if timestamp offset is valid
    if ($fields{'timeoff'} !~ /^[0-9]+$/ || ($fields{'timeoff'} > $fields{'psize'} && $fields{'psize'} > 0)) {
      push @error, "Time Offset not valid.";
    }

    if ($fields{'psize'} !~ /^[0-9]+$/ || $fields{'psize'} < 0 || $fields{'psize'} > 1024) {
      push @error, "Packet Size not valid. (only 0-1024 allowed.)";
    }
    
    # Check if defined value and offset are valid
    if ($fields{'dval'} !~ /(^[=&]?[0-9a-fA-F]{1,8}?$)|(^[=&]?([0-9a-fA-F]{1,8}?\,)+[0-9a-fA-F]{1,8}?$)/) {
      push @error, "Defined Value(s) are not valid. (no whitespace, only 4 byte hex values seperated by \",\")";
    }

    if (length $fields{'dval'} > 200) {
      push @error, "Defined Value(s) field too long. (no more than 200 characters allowed.)";
    }
    
    if ($fields{'dvaloff'} !~ /(^[0-9]+$)|(^([0-9]+\,)+[0-9]+$)/) {
      push @error, "Defined Value(s) Offset(s) are not valid.";
    }
    
    if (length $fields{'dvaloff'} > 200) {
      push @error, "Defined Value(s) Offset(s) field too long. (no more than 200 characters allowed.)";
    }

    # Check if the number of defined values and offsets match.
    if ( ($fields{'dval'} =~ tr/,/,/) != ($fields{'dvaloff'} =~ tr/,/,/) ) {
      push @error, "Number of defined values and offsets must be the same.";
    }
    
    # Check if the Statistic Intervall is valid. TODO: Check if duration is ok
    if ($fields{'statint'} !~ /^[0-9]+$/ || $fields{'statint'} < 0) {
      push @error, "Statistic Intervall is not valid.";
    }
  }
  
  if (@error == 0) {
    # No Error found, so let's add the new listener to our list.
    $_[0] = $fields{'mcast'};
    $_[1] = $fields{'port'};
    $_[2] = 0;
    $_[3] = "NoTimestamp";
    $_[4] = 0;
    $_[5] = 0;
    $_[6] = "0";
    $_[7] = "0";
    $_[8] = 0;
    $_[9] = 0;
    $_[10] = $fields{'srcstat'};


    if ($fields{'predef'} eq "New Configuration") {
      # We have a new configuration, use values from form
      $_[2] = $fields{'rate'};
      $_[3] = $fields{'timeform'};
      $_[4] = $fields{'timeoff'};
      $_[5] = $fields{'psize'};
      $_[6] = $fields{'dval'};
      $_[7] = $fields{'dvaloff'};
      $_[8] = $fields{'statint'};

      if ($fields{'runsave'} ne "") {
        # We need to save the new configutration.
        $conf[0] = $fields{'cfname'};
        $conf[1] = $fields{'timeform'};
        $conf[2] = $fields{'timeoff'};
        $conf[3] = $fields{'psize'};
        $conf[4] = $fields{'dval'};
        $conf[5] = $fields{'dvaloff'};
        $conf[6] = $fields{'statint'};

        $confs{$conf[0]} = [ @conf ];
      }

    } elsif ($fields{'predef'} eq "TTM-Data") {
      # We are using standard TTM-Data
      $_[2] = 2;
      $_[3] = "TTMTimestamp";
      $_[4] = 0;
      $_[5] = 100;
      $_[6] = "0";
      $_[7] = "0";
      $_[8] = 300;
    } elsif ($fields{'predef'} eq "Auto-Configuration") {
      # Auto Configure, set all to 0
      $_[2] = 0;
      $_[3] = "NoTimestamp";
      $_[4] = 0;
      $_[5] = 0;
      $_[6] = "0";
      $_[7] = "0";
      $_[8] = 60;
    } else {
      # We are using an existing configuration
      $_[2] = $confs{$fields{'predef'}}[1];
      $_[3] = $confs{$fields{'predef'}}[2];
      $_[4] = $confs{$fields{'predef'}}[3];
      $_[5] = $confs{$fields{'predef'}}[4];
      $_[6] = $confs{$fields{'predef'}}[5];
      $_[7] = $confs{$fields{'predef'}}[6];
      $_[8] = $confs{$fields{'predef'}}[7];
      
    }
 
    # Fork a new process and start our listener in there 
    $pid = fork();
    die "fork() failed: $!" unless defined $pid;
    if ($pid == 0) # child
    {
        # Daemonise and exec listener
        chdir '/';
        umask 0;
	local $SIG{HUP} = 'IGNORE';
        open STDIN, '/dev/null';
        open STDOUT, '>/dev/null';
        setsid;
        open STDERR, '>/dev/null';
        $srcstat = $_[10] eq "on" ? "1" : "0";
 
        exec($ttroot."/bin/McastCListener ".$_[0]." ".$_[1]." ".$_[2]." ".$_[3]." ".$_[4]." ".
                              $_[5]." ".$_[6]." ".$_[7]." ".$_[8]." ".$data_area." ".$srcstat) || die "exec failed: $!";
    }

    # Store the pid and poplate the Listener hash of arrays
    $_[9] = $pid;
    $listeners{$_[0].":".$_[1]} = [ @_ ];

    # Populate the Configuration hash of arrays when necessary
    if ($fields{'runsave'} ne "") {
      $_[0] = $fields{'cfname'};
      $_[1] = $fields{'rate'};
      $_[2] = $fields{'timeform'};
      $_[3] = $fields{'timeoff'};
      $_[4] = $fields{'psize'};
      $_[5] = $fields{'dval'};
      $_[6] = $fields{'dvaloff'};
      $_[7] = $fields{'statint'};
      
      $confs{$_[0]} = [ @_ ];
    }
  }
}

# Look if there is something to be deleted
if (exists $fields{'deleteL'}) {
  # Kill the process and delete the hash entry
  system("kill ".$listeners{$fields{'deleteL'}}[9]);
  delete $listeners{$fields{'deleteL'}};
}
if (exists $fields{'deleteC'}) {
  delete $confs{$fields{'deleteC'}};
}

# And rewrite our config file. 
open (CUSTLIST, ">".$config_area."/custom_listener.conf");
foreach (sort keys %listeners) {
  # Alarm email address is no longer stored in this file
  print CUSTLIST $listeners{$_}[0]." ".$listeners{$_}[1]." ".$listeners{$_}[2]." ";
  print CUSTLIST $listeners{$_}[3]." ".$listeners{$_}[4]." ".$listeners{$_}[5]." ";
  print CUSTLIST $listeners{$_}[6]." ".$listeners{$_}[7]." ";
  print CUSTLIST $listeners{$_}[8]." ".$listeners{$_}[9]." ".$listeners{$_}[10]."\n";
}
foreach (sort keys %confs) {
  print CUSTLIST "conf ".$confs{$_}[0]." ".$confs{$_}[1]." ".$confs{$_}[2]." ".$confs{$_}[3]." ".$confs{$_}[4]." ";
  print CUSTLIST $confs{$_}[5]." ".$confs{$_}[6]." ".$confs{$_}[7]."\n";
}
close (CUSTLIST);
   
&page_header;
# Output the table of custom listeners. This is quiet hackish, not really nice. Should be improved at some point.
if (keys %listeners != 0) {
  print <<EOF;
    <br><table border=1 cellspacing=1 cellpadding=0 width=700 >
      <tr style="background-color:#FFCC00">
        <th> Multicast </th>
        <th> Port </th>
        <th> Source Stats</th>
        <th> Rate </th>
        <th> Time Format </th>
        <th> Time Offset </th>
        <th> Packet Size </th>
        <th> Defined Value(s) </th>
        <th> Value Offset(s) </th>
        <th> Stat. Intervall </th>
        <th> Delete? </th>
        <th> Status </th>
      </tr>
EOF
  $count = 0;
  foreach (sort keys %listeners) {
    if ($count % 2 == 0) { print "<tr style=\"background-color:#FFF2BF\">"; }
    else { print "<tr style=\"background-color:#FFE680\">"; }

    if ($listeners{$_}[3] eq "") { $ts = "&nbsp"; } else { $ts = $listeners{$_}[3]; }
    print "<td><center>".$listeners{$_}[0]."</center></td>\n";
    print "<td><center>".$listeners{$_}[1]."</center></td>\n";
   
    $sstats = $listeners{$_}[10] eq "on" ? "Yes" : "No";
    print "<td><center>".$sstats."</center></td>";
    
    print "<td><center>".$listeners{$_}[2]."</center></td>\n<td><center>";
    print $ts."</center></td>\n<td><center>".$listeners{$_}[4]."</center></td>\n<td><center>".$listeners{$_}[5]."</center></td>\n";
    $dval = $listeners{$_}[6] ne "" ? $listeners{$_}[6] : "&nbsp";
    $dvaloff = $listeners{$_}[7] ne "" ? $listeners{$_}[7] : "&nbsp"; 
    print "<td><center>".$dval."</center></td>\n";
    print "<td><center>".$dvaloff."</center></td>\n";
    print "<td><center>".$listeners{$_}[8]."</center></td>\n";
    print "<td><center><a href=\"custom_listener.cgi?deleteL=$_\">Delete</a></center></td>";
    if( -e "/proc/".$listeners{$_}[9] ) {
      print "<td><center><font color=green>Running</font></center></td></tr>";
    } else {
      print "<td><center><font color=red><blink>NOT RUNNING</blink></font></center></td></tr>";
    }
    $count++;
  }
  print "</table><br>";
  print "<IMG ID=\"SiteStyleLine6\" HEIGHT=4 WIDTH=700 SRC=\"../assets/images/Line.gif\" BORDER=0><br>";
} else {
  print "No Custom Listeners configured. <br>";
}

# Output the table of custom listeners configs. This is quiet hackish, not really nice. Should be improved at some point.
print <<EOF;
  <br><table border=1 cellspacing=1 cellpadding=0 width=700 >
    <tr style="background-color:#FFCC00">
      <th> Conf. Name </th>
      <th> Rate </th>
      <th> Time Format </th>
      <th> Time Offset </th>
      <th> Packet Size </th>
      <th> Defined Value(s) </th>
      <th> Value Offset(s) </th>
      <th> Stat. Intervall </th>
      <th> Delete? </th>
    </tr>
    <tr style="background-color:#FFF2BF">
      <td><center> TTM-Data </center></td>
      <td><center> 2 </center></td>
      <td><center> TTMTimestamp </center></td>
      <td><center> 0 </center></td>
      <td><center> 100 </center></td>
      <td><center> 0 </center></td>
      <td><center> 0 </center></td>
      <td><center> 300 </center></td>
      <td>&nbsp</td>
    </tr>
EOF
$count = 1;
foreach (sort keys %confs) {
  if ($count % 2 == 0) { print "<tr style=\"background-color:#FFF2BF\">"; }
  else { print "<tr style=\"background-color:#FFE680\">"; }
  if ($confs{$_}[2] eq "") { $ts = "&nbsp"; } else { $ts = $confs{$_}[2]; }
  print "<td><center>".$confs{$_}[0]."</center></td>\n<td><center>".$confs{$_}[1]."</center></td>\n<td><center>".$ts."</center></td>\n<td><center>";
  print $confs{$_}[3]."</center></td>\n<td><center>".$confs{$_}[4]."</center></td>\n";
  $dval = $confs{$_}[5] ne "" ? $confs{$_}[5] : "&nbsp";
  $dvaloff = $confs{$_}[6] ne "" ? $confs{$_}[6] : "&nbsp";
  print "<td><center>".$dval."</center></td>\n
         <td><center>".$dvaloff."</center></td>\n";
  print "<td><center>".$confs{$_}[7]."</center></td>\n";
  print "<td><center><a href=\"custom_listener.cgi?deleteC=$_\">Delete</a></center></td></tr>";
  $count++;
}
print "</table><br>";



# Here the Form to add new listeners is generated
print <<EOF;
  <IMG ID="SiteStyleLine6" HEIGHT=4 WIDTH=700 SRC="../assets/images/Line.gif" BORDER=0>
  <FORM NAME="CustomListener" ACTION="custom_listener.cgi" METHOD="POST" onchange="hide()">
    <TABLE BORDER=0 CELLSPACING=5 CELLPADDING=0 WIDTH=625>
      <TR>
        <TD COLSPAN=1>
          <FONT FACE="Arial,Helvetica,Univers,Zurich BT,sans-serif"><B>Add a Listener:</B>
        </TD>
      </TR>
      <TR>
        <TD>
          Multicast Address:
        </TD>
        <TD>
EOF
        if (@error != 0) {
          print "<INPUT TYPE=\"text\" NAME=\"mcast\" VALUE=\"$fields{'mcast'}\"/>";
        } else {
          print "<INPUT TYPE=\"text\" NAME=\"mcast\" />";
        }
        print "Source Stats? <input type=\"checkbox\" name=\"srcstat\" />";
print <<EOF;
        </TD>
      </TR>
      <TR>
        <TD>
          Port:
        </TD>
        <TD>
EOF
        if (@error != 0) {
          print "<INPUT TYPE=\"text\" NAME=\"port\" VALUE=\"$fields{'port'}\"/>";
        } else {
          print "<INPUT TYPE=\"text\" NAME=\"port\" />";
        }
print <<EOF;
        </TD>
      </TR>
      <TR>
        <TD>
          Predefined:
        </TD>
        <TD>
EOF

          print "<select name=\"predef\">";
          printf "<option %s >TTM-Data</option>", $fields{'predef'} eq "TTM-Data"?"selected":"";
          # Here should all the other configurations go.
          foreach (sort keys %confs) {
            print "<option %s >$_</option>", $fields{'predef'} eq $_?"selected":"";
          }
          printf "<option %s >New Configuration</option>", $fields{'predef'} eq "New Configuration"?"selected":"";
          print "</select>";

print <<EOF;
        </TD>
      </TR>
      <TR>
        <TD>
          Config Name:
        </TD>
        <TD>
EOF
        if (@error != 0) {
          print "<INPUT TYPE=\"text\" NAME=\"cfname\" VALUE=\"$fields{'cfname'}\" />";
        } else {
          print "<INPUT TYPE=\"text\" NAME=\"cfname\" />";
        }
print <<EOF;
        </TD>
      </TR>
      <TR>
        <TD>
          Expected Rate (Packets/Minute): 
        </TD>
        <TD>
EOF
        if (@error != 0) {
          print "<INPUT TYPE=\"text\" NAME=\"rate\" VALUE=\"$fields{'rate'}\" />";
        } else {
          print "<INPUT TYPE=\"text\" NAME=\"rate\" />";
        }
print <<EOF;
        </TD>
      </TR>
      <TR>
        <TD>
          Timestamp Format:
        </TD>
        <TD>
EOF
          # When adding new Options, don't use Whitespaces!!!
          print "<select name=\"timeform\">";
          printf "  <option %s>NoTimestamp</option>", $fields{'timeform'} eq "NoTimestamp"?"selected":"";
          printf "  <option %s>UnixTimestamp</option>", $fields{'timeform'} eq "UnixTimestamp"?"selected":"";
          printf "  <option %s>TTMTimestamp</option>", $fields{'timeform'} eq "TTMTimestamp"?"selected":"";
          printf "  <option %s>dbeaconTimestamp</option>", $fields{'timeform'} eq "dbeaconTimestamp"?"selected":"";
          print "</select>";
print <<EOF;
        </TD>
      </TR>
      <TR>
        <TD>
          Timestamp Offset:
        </TD>
        <TD>
EOF
        if (@error != 0) {
          print "<INPUT TYPE=\"text\" NAME=\"timeoff\" VALUE=\"$fields{'timeoff'}\" />";
        } else {
          print "<INPUT TYPE=\"text\" NAME=\"timeoff\" />";
        }
print <<EOF;
        </TD>
      </TR>
      <TR>
        <TD>
          Packet Payload:
        </TD>
        <TD>
EOF
        if (@error != 0) {
          print "<INPUT TYPE=\"text\" NAME=\"psize\" VALUE=\"$fields{'psize'}\" />";
        } else {
          print "<INPUT TYPE=\"text\" NAME=\"psize\" />";
        }
print <<EOF;
        </TD>
      </TR>
      <tr>
        <td>
          Defined Value(s) (hex):
        </td>
        <td>
EOF
        if (@error != 0) {
          print "<INPUT TYPE=\"text\" NAME=\"dval\" VALUE=\"$fields{'dval'}\" />";
        } else {
          print "<INPUT TYPE=\"text\" NAME=\"dval\" />";
        }
print <<EOF;
        </td>
      </tr>
      <tr>
        <td>
          Value Offset(s) (bytes):
        </td>
        <td>
EOF
        if (@error != 0) {
          print "<INPUT TYPE=\"text\" NAME=\"dvaloff\" VALUE=\"$fields{'dvaloff'}\" />";
        } else {
          print "<INPUT TYPE=\"text\" NAME=\"dvaloff\" />";
        }
print <<EOF;
        </td>
      </tr>
       <tr>
        <td>
        Statistic Intervall (sec):
        </td>
        <td>
EOF
        if (@error != 0) {
          print "<INPUT TYPE=\"text\" NAME=\"statint\" VALUE=\"$fields{'statint'}\" />";
        } else {
          print "<INPUT TYPE=\"text\" NAME=\"statint\" />";
        }
print <<EOF;
        </td>
      </tr>
     <TR>
        <TD>
          <INPUT TYPE="submit" NAME="run" VALUE="Start Listener" />
        </TD>
        <TD>
          <INPUT TYPE="submit" NAME="runsave" VALUE="Start Listener and Save Configuration"/>
        </TD>
      </TR>
      <TR>
        <TD COLSPAN=2>
EOF

if (@error != 0) {
  foreach (@error) {
    print "<FONT COLOR=\"RED\"> ERROR: $_ </FONT><br>\n";
  }
} elsif ($fields{'run'} ne "") { 
  print "<FONT COLOR=\"GREEN\"> Listener successfully started. </FONT>";
} elsif ($fields{'runsave'} ne "") {
  print "<FONT COLOR=\"GREEN\"> Listener successfully started and Configuration saved. </FONT>";
}

print <<EOF;
        <TD>
      </TR>
    </FORM>
  </TABLE>

EOF
      

&page_footer();



